@@ -965,7 +965,7 @@ double entry_get_numeric_value (GtkWidget *w, EntryValType t)
 
     if (get_local_decpoint() != '.') {
 	gretl_push_c_numeric_locale();
-	charsub(s, ',', '.');
+	gretl_charsub(s, ',', '.');
 	sub = 1;
     }
 
@@ -2036,7 +2036,7 @@ static int get_restriction_vxy (const char *s, int *vx, int *vy,
     *p = 0;
     p++;
 
-    if (sscanf(str, "%15s", test) != 1) {
+    if (gretl_scan_varname(str, test) != 1) {
 	err = 1;
     } else {
 	*vx = series_index(dataset, test);
@@ -2080,7 +2080,7 @@ static int get_restriction_vxy (const char *s, int *vx, int *vy,
     }
 
     if (!err) {
-	if (sscanf(p, "%15s", test) != 1) {
+	if (gretl_scan_varname(p, test) != 1) {
 	    err = 1;
 	} else {
 	    *vy = series_index(dataset, test);
@@ -2240,7 +2240,7 @@ static int var_is_ok (int i, int code)
 {
     int ret = 1;
 
-    if (var_is_hidden(dataset, i)) {
+    if (series_is_hidden(dataset, i)) {
 	ret = 0;
     } else if ((code == ONE_PROPN || code == TWO_PROPNS) &&
 	       !gretl_isdummy(dataset->t1, dataset->t2, dataset->Z[i])) {
@@ -2462,7 +2462,7 @@ static int n_ok_series (void)
 
     if (dataset != NULL) {
 	for (i=1; i<dataset->v; i++) {
-	    if (!var_is_hidden(dataset, i)) {
+	    if (!series_is_hidden(dataset, i)) {
 		nv++;
 	    }
 	}
@@ -2477,7 +2477,7 @@ static int n_ok_dummies (void)
 
     if (dataset != NULL) {
 	for (i=1; i<dataset->v; i++) {
-	    if (!var_is_hidden(dataset, i) &&
+	    if (!series_is_hidden(dataset, i) &&
 		gretl_isdummy(dataset->t1, dataset->t2, dataset->Z[i])) {
 		nv++;
 	    }
