@@ -31,6 +31,8 @@
 #include "winstack.h"
 #include "toolbar.h"
 #include "dlgutils.h"
+#include "gretl_bundle.h"
+#include "fncall.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -384,12 +386,12 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    }
 	    overwrite = 1;
 	    /* pick up on pre-registered compaction method? */
-	    if (COMPACT_METHOD(dataset, dbv) != COMPACT_NONE) {
-		method = COMPACT_METHOD(dataset, dbv);
+	    if (series_get_compact_method(dataset, dbv) != COMPACT_NONE) {
+		method = series_get_compact_method(dataset, dbv);
 	    }
 	}
 
-	if (!overwrite && dataset_add_series(1, dataset)) {
+	if (!overwrite && dataset_add_series(dataset, 1)) {
 	    nomem();
 	    return 1;
 	}
@@ -405,7 +407,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 				    &method, NULL, vwin->main);
 		if (method == COMPACT_NONE) {
 		    if (!overwrite) {
-			dataset_drop_last_variables(1, dataset);
+			dataset_drop_last_variables(dataset, 1);
 		    }
 		    return 0;
 		}
@@ -426,7 +428,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	if (xvec == NULL) {
 	    nomem();
 	    if (!overwrite) {
-		dataset_drop_last_variables(1, dataset);
+		dataset_drop_last_variables(dataset, 1);
 	    }
 	    return 1;
 	}
@@ -444,7 +446,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 
 	/* common stuff for adding a var */
 	strcpy(dataset->varname[dbv], sinfo->varname);
-	strcpy(VARLABEL(dataset, dbv), sinfo->descrip);
+	series_set_label(dataset, dbv, sinfo->descrip);
 	get_db_padding(sinfo, dataset, &pad1, &pad2);
 
 	if (pad1 > 0) {
@@ -503,7 +505,7 @@ static void add_dbdata (windata_t *vwin, DATASET *dbset,
 	    sinfo = &dw->sinfo[i-1];
 
 	    strcpy(dataset->varname[i], sinfo->varname);
-	    strcpy(VARLABEL(dataset, i), sinfo->descrip);
+	    series_set_label(dataset, i, sinfo->descrip);
 	    
 	    lib_command_sprintf("data %s", sinfo->varname);
 	    record_command_verbatim();
@@ -724,7 +726,7 @@ static void gui_get_db_series (windata_t *vwin, int cmd)
 	} 
 
 	strcpy(dbset->varname[i+1], sinfo->varname);
-	strcpy(VARLABEL(dbset, i+1), sinfo->descrip);
+	series_set_label(dbset, i+1, sinfo->descrip);
     }
 
     if (cmd == DB_DISPLAY) {
@@ -1122,9 +1124,9 @@ static int add_local_db_series_list (windata_t *vwin)
 	err = utf8_correct(line1);
 
 	tailstrip(line1);
-	charsub(line1, '\t', ' ');
+	gretl_charsub(line1, '\t', ' ');
 
-	if (sscanf(line1, "%15s", sername) != 1) {
+	if (gretl_scan_varname(line1, sername) != 1) {
 	    break;
 	}
 
@@ -1182,11 +1184,11 @@ static int add_remote_db_series_list (windata_t *vwin, char *buf)
 	}
 
 	tailstrip(line1);
-	charsub(line1, '\t', ' ');
+	gretl_charsub(line1, '\t', ' ');
 
 	err = utf8_correct(line1);
 
-	if (sscanf(line1, "%15s", sername) != 1) {
+	if (gretl_scan_varname(line1, sername) != 1) {
 	    break;
 	}
 
@@ -1486,13 +1488,13 @@ static dbwrapper *get_series_info (windata_t *vwin, int action)
 	row = rowlist[i+1];
 	tree_view_get_int(view, row, 3, &sinfo->offset);
 
-	*sinfo->varname = 0;
+	*sinfo->varname = '\0';
 	tree_view_get_string(view, row, 0, &tmp);
 	strncat(sinfo->varname, tmp, VNAMELEN - 1);
 	g_free(tmp);
 
 	tmp = NULL;
-	*sinfo->descrip = 0;
+	*sinfo->descrip = '\0';
 	tree_view_get_string(view, row, 1, &tmp);
 	if (tmp != NULL) {
 	    strncat(sinfo->descrip, tmp, MAXLABEL - 1);
@@ -2041,6 +2043,17 @@ int unzip_package_file (const char *zipname, const char *path)
 	return err;
     }
 
+#ifdef G_OS_WIN32
+    if (1) {
+	char *test;
+
+	fprintf(stderr, "unzip_package_file: unzipping in '%s'\n", dirname);
+	*dirname = '\0';
+	test = getcwd(dirname, FILENAME_MAX - 1);
+	fprintf(stderr, " check: getcwd() gives '%s'\n", test);
+    }
+#endif
+
     gretl_unzip_file = gui_get_plugin_function("gretl_unzip_file", 
 					       &handle);
     if (gretl_unzip_file == NULL) {
@@ -2049,25 +2062,27 @@ int unzip_package_file (const char *zipname, const char *path)
 
     err = (*gretl_unzip_file)(zipname, &gerr);
     if (gerr != NULL) {
-	fprintf(stderr, "gretl_unzip_file: '%s'\n", gerr->message);
+	gretl_errmsg_set(gerr->message);
 	if (!err) {
 	    err = 1;
 	}
 	g_error_free(gerr);
-    } else if (err) {
-	fprintf(stderr, "gretl_unzip_file: err = %d\n", err);
     }
 
     close_plugin(handle);
     gretl_remove(zipname);
 
+    if (err) {
+	fprintf(stderr, "gretl_unzip_file: err = %d\n", err);
+    }
+
     return err;
 }
 
 #define STATUS_COLUMN  4
 #define ZIPFILE_COLUMN 5
 
-/* note : 'vwin' here is the source viewer window displaying the
+/* note : @vwin here is the source viewer window displaying the
    remote file (database or datafiles package or function package)
    that is being installed onto the local machine.
 */
@@ -2138,15 +2153,17 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	show_network_error(NULL);
     } else {
 	windata_t *local = get_local_viewer(vwin->role);
-
+	
 	if (vwin->role == REMOTE_FUNC_FILES) {
-	    infobox(_("Installed"));
+	    if (!maybe_handle_pkg_menu_option(path)) {
+		infobox(_("Installed"));
+	    }
 	    list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
 				  vwin->active_var, STATUS_COLUMN,
 				  _("Up to date"));
 	    if (local != NULL) {
 		populate_filelist(local, NULL);
-	    } 
+	    }
 	} else if (vwin->role == REMOTE_DATA_PKGS) {
 	    fprintf(stderr, "downloaded '%s'\n", path);
 	    err = unpack_book_data(path);
@@ -2368,7 +2385,7 @@ static void get_local_object_status (const char *fname, int role,
 	} 
     }
 
-    free_strings_array(dirs, n_dirs);
+    strings_array_free(dirs, n_dirs);
 
     if (found) {
 	double dt;
@@ -2668,7 +2685,7 @@ static int get_addon_info (xmlNodePtr node, xmlDocPtr doc, char **S)
 	char *path = gretl_function_package_get_path(S[0], PKG_SUBDIR);
 
 	if (path != NULL) {
-	    S[3] = gretl_strdup(_("Installed"));
+	    S[3] = installed_addon_status_string(path, S[1]);
 	    free(path);
 	} else {
 	    S[3] = gretl_strdup(_("Not installed"));
@@ -2963,7 +2980,7 @@ gint populate_dbfilelist (windata_t *vwin)
 	}
     }  
 
-    free_strings_array(dirnames, n_dirs);
+    strings_array_free(dirnames, n_dirs);
 
     if (ndb == 0) {
 	errbox(_("No database files found"));
@@ -3000,8 +3017,8 @@ static void set_compact_info_from_default (int method)
     int i;
 
     for (i=1; i<dataset->v; i++) {
-	if (COMPACT_METHOD(dataset, i) == COMPACT_NONE) {
-	    COMPACT_METHOD(dataset, i) = method;
+	if (series_get_compact_method(dataset, i) == COMPACT_NONE) {
+	    series_set_compact_method(dataset, i, method);
 	}
     }
 }
