@@ -157,7 +157,7 @@ static void prep_spreadsheet (GtkWidget *widget, dialog_t *dlg)
     for (t=0; t<dataset->n; t++) {
 	dataset->Z[1][t] = NADBL;
     }
-    *(VARLABEL(dataset, 1)) = 0;
+    series_set_label(dataset, 1, "");
 
     show_spreadsheet(SHEET_NEW_DATASET);
 }
@@ -165,12 +165,16 @@ static void prep_spreadsheet (GtkWidget *widget, dialog_t *dlg)
 static void maybe_start_editing (void)
 {
     int cancel = 0;
+    gchar *msg;
 
-    blocking_edit_dialog(CREATE_DATASET, _("gretl: name variable"), 
-			 _("Enter name for first variable\n"
-			   "(max. 15 characters)"),
-			 NULL, prep_spreadsheet, NULL, 
+    msg = g_strdup_printf(_("Enter name for first variable\n"
+			    "(max. %d characters)"),
+			  VNAMELEN - 1);
+
+    blocking_edit_dialog(CREATE_DATASET, _("gretl: name variable"),
+			 msg, NULL, prep_spreadsheet, NULL, 
 			 VARCLICK_NONE, NULL, &cancel);
+    g_free(msg);
 
     if (cancel) {
 	/* accept the default blank dataset */
@@ -992,7 +996,7 @@ static void panelvar_candidates (GList *vlist, int *uid, int *tid)
     for (i=0; list != NULL; i++) {
 	vname = (const char *) list->data;
 	strcpy(vtest, vname);
-	lower(vtest);
+	gretl_lower(vtest);
 	if (*tid < 0) {
 	    if (!strcmp(vtest, "time") || 
 		!strcmp(vtest, "year") ||
