@@ -1328,9 +1328,11 @@ blocking_edit_dialog (int ci, const char *title,
 	active_edit_text = d->edit;
     } 
 
-    if (click == VARCLICK_NONE && helpcode == 0) {
+    if (click != VARCLICK_NONE || helpcode > 0) {
+	gtk_window_set_keep_above(GTK_WINDOW(d->dialog), FALSE);
+    } else {
 	gtk_window_set_keep_above(GTK_WINDOW(d->dialog), TRUE);
-    } 
+    }
 
     gtk_widget_grab_focus(d->edit);
 
@@ -1407,20 +1409,6 @@ gchar *entry_box_get_trimmed_text (GtkWidget *w)
     return ret;
 }
 
-GList *get_list_of_listnames (void)
-{
-    GList *llist = NULL;
-    const char *lname;
-    int i, n = n_saved_lists();
-
-    for (i=0; i<n; i++) {
-	lname = get_list_name_by_index(i);
-	llist = g_list_append(llist, (gpointer) lname);
-    }
-
-    return llist;
-}
-
 void set_combo_box_strings_from_list (GtkWidget *box, GList *list)
 {
     GList *mylist = list;
