@@ -23,11 +23,9 @@
 #include "selector.h"
 #include "gretl_func.h"
 #include "monte_carlo.h"
-#include "usermat.h"
+#include "uservar.h"
 #include "cmd_private.h"
 #include "gretl_www.h"
-#include "gretl_string_table.h"
-#include "gretl_scalar.h"
 #include "gretl_bundle.h"
 #include "gretl_xml.h"
 #include "database.h"
@@ -65,6 +63,7 @@ struct call_info_ {
     char rettype;        /* its return type */
     gchar **args;        /* its arguments */
     gchar *ret;          /* return assignment name */
+    gchar *label;        /* the function's label */
 };
 
 #define scalar_arg(t) (t == GRETL_TYPE_DOUBLE || t == GRETL_TYPE_SCALAR_REF)
@@ -75,6 +74,7 @@ static GtkWidget *open_fncall_dlg;
 static gboolean close_on_OK = TRUE;
 
 static void fncall_exec_callback (GtkWidget *w, call_info *cinfo);
+static void unregister_package (const gchar *pkgname);
 
 static gchar **glib_str_array_new (int n)
 {
@@ -144,6 +144,7 @@ static call_info *cinfo_new (fnpkg *pkg, windata_t *vwin)
     cinfo->ret = NULL;
 
     cinfo->dreq = 0;
+    cinfo->label = NULL;
 
     return cinfo;
 }
@@ -186,6 +187,7 @@ static void cinfo_free (call_info *cinfo)
 	g_list_free(cinfo->ssels);
     }
 
+    g_free(cinfo->label);
     free(cinfo->publist);
     free(cinfo);
 }
@@ -239,7 +241,8 @@ static GtkWidget *label_hbox (call_info *cinfo, GtkWidget *w,
     lbl = gtk_label_new(NULL);
     gtk_label_set_markup(GTK_LABEL(lbl), buf);
     g_free(buf);
-    g_free(label);
+
+    cinfo->label = label;
 
     gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 5);
     gtk_widget_show(lbl);
@@ -320,7 +323,7 @@ static gboolean update_arg (GtkComboBox *combo,
 
     if (s != NULL && fn_param_type(cinfo->func, i) == GRETL_TYPE_DOUBLE) {
 	if (isdigit(*s) || *s == '-' || *s == '+' || *s == ',') {
-	    charsub(s, ',', '.');
+	    gretl_charsub(s, ',', '.');
 	}
     }
 
@@ -359,27 +362,46 @@ static int probably_stochastic (int v)
 
 static GList *add_list_names (GList *list)
 {
-    int i, n = n_saved_lists();
-    const char *name;
+    GList *llist = user_var_names_for_type(GRETL_TYPE_LIST);
+    GList *tail = llist;
 
-    for (i=0; i<n; i++) {
-	name = get_list_name_by_index(i);
-	list = g_list_append(list, (gpointer) name);
+    while (tail != NULL) {
+	list = g_list_append(list, tail->data);
+	tail = tail->next;
     }
 
+    g_list_free(llist);
+
     return list;
 }
 
 static GList *add_matrix_names (GList *list)
 {
-    int i, n = n_user_matrices();
-    const char *name;
+    GList *mlist = user_var_names_for_type(GRETL_TYPE_MATRIX);
+    GList *tail = mlist;
 
-    for (i=0; i<n; i++) {
-	name = get_matrix_name_by_index(i);
-	list = g_list_append(list, (gpointer) name);
+    while (tail != NULL) {
+	list = g_list_append(list, tail->data);
+	tail = tail->next;
     }
 
+    g_list_free(mlist);
+
+    return list;
+}
+
+static GList *add_scalar_names (GList *list)
+{
+    GList *mlist = user_var_names_for_type(GRETL_TYPE_DOUBLE);
+    GList *tail = mlist;
+
+    while (tail != NULL) {
+	list = g_list_append(list, tail->data);
+	tail = tail->next;
+    }
+
+    g_list_free(mlist);
+
     return list;
 }
 
@@ -388,7 +410,7 @@ static GList *add_series_names (GList *list)
     int i;
 
     for (i=1; i<dataset->v; i++) {
-	if (!var_is_hidden(dataset, i)) {
+	if (!series_is_hidden(dataset, i)) {
 	    list = g_list_append(list, (gpointer) dataset->varname[i]);
 	} 
     }
@@ -398,19 +420,6 @@ static GList *add_series_names (GList *list)
     return list;
 }
 
-static GList *add_scalar_names (GList *list)
-{
-    int i, n = n_saved_scalars();
-    const char *name;
-
-    for (i=0; i<n; i++) {
-	name = gretl_scalar_get_name(i);
-	list = g_list_append(list, (gpointer) name);
-    }
-
-    return list;
-}
-
 static GList *get_selection_list (int type)
 {
     GList *list = NULL;
@@ -445,9 +454,8 @@ static void fncall_help (GtkWidget *w, call_info *cinfo)
     char *pdfname = NULL;
 
     if (function_package_has_PDF_doc(cinfo->pkg, &pdfname)) {
-	FILE *fp;
+	FILE *fp = gretl_fopen(pdfname, "r");
 
-	fp = gretl_fopen(pdfname, "r");
 	if (fp != NULL) {
 	    fclose(fp);
 	    gretl_show_pdf(pdfname);
@@ -604,7 +612,7 @@ static int do_make_list (selector *sr)
 	return 1;
     } 
 
-    nl = n_saved_lists();
+    nl = n_user_lists();
 
     if (buf == NULL || *buf == '\0') {
 	int resp;
@@ -649,7 +657,7 @@ static int do_make_list (selector *sr)
     if (!err) {
 	gtk_widget_hide(selector_get_window(sr));
 	if (cinfo != NULL) {
-	    if (n_saved_lists() > nl) {
+	    if (n_user_lists() > nl) {
 		update_combo_selectors(cinfo, aux, GRETL_TYPE_LIST);
 	    } 
 	} else {
@@ -686,7 +694,7 @@ static void launch_matrix_maker (GtkWidget *button, call_info *cinfo)
 {
     int n = n_user_matrices();
 
-    gui_new_matrix();
+    gui_new_matrix(cinfo->dlg);
 
     if (n_user_matrices() > n) {
 	GtkWidget *sel = g_object_get_data(G_OBJECT(button), "combo");
@@ -967,6 +975,19 @@ static int already_set_as_default (call_info *cinfo,
     return ret;
 }
 
+static int has_single_series_arg (call_info *cinfo)
+{
+    int i, ns = 0;
+
+    for (i=0; i<cinfo->n_params; i++) {
+	if (fn_param_type(cinfo->func, i) == GRETL_TYPE_SERIES) {
+	    ns++;
+	}
+    }
+
+    return ns == 1;
+}
+
 /* Try to be somewhat clever in selecting the default values to show
    in function-argument drop-down "combo" selectors.
 
@@ -976,6 +997,10 @@ static int already_set_as_default (call_info *cinfo,
    unlikely that the user wants to select the same named variable in
    more than one argument slot, so we try to avoid setting duplicate
    defaults.
+
+   Special case: the function has exactly one series argument, and
+   a single series is selected in the main gretl window: in that
+   case we pre-select that series.
 */
 
 static void arg_combo_set_default (call_info *cinfo,
@@ -984,13 +1009,24 @@ static void arg_combo_set_default (call_info *cinfo,
 				   int ptype)
 {
     GList *mylist = g_list_first(list);
+    const char *targname = NULL;
     int i, k = 0;
 
+    if (ptype == GRETL_TYPE_SERIES && has_single_series_arg(cinfo)) {
+	int vsel = mdata_active_var();
+
+	if (vsel > 0) {
+	    targname = dataset->varname[vsel];
+	}
+    }
+
     for (i=0; mylist != NULL; i++) {
 	gchar *name = mylist->data;
 	int ok = 0;
 
-	if (series_arg(ptype)) {
+	if (targname != NULL) {
+	    ok = strcmp(name, targname) == 0;
+	} else if (series_arg(ptype)) {
 	    int v = current_series_index(dataset, name);
 
 	    if (v > 0 && probably_stochastic(v)) {
@@ -1225,10 +1261,15 @@ static void function_call_dialog (call_info *cinfo)
 				     (desc != NULL)? desc :
 				     parname);
 	} else {
-	    argtxt = g_strdup_printf("%s (%s)",
-				     (desc != NULL)? desc :
-				     parname,
-				     gretl_arg_type_name(ptype));
+	    const char *astr = gretl_arg_type_name(ptype);
+
+	    if (desc != NULL && strstr(desc, astr)) {
+		argtxt = g_strdup_printf("%s", desc);
+	    } else {
+		argtxt = g_strdup_printf("%s (%s)",
+					 (desc != NULL)? desc :
+					 parname, astr);
+	    }
 	}
 
 	label = gtk_label_new(argtxt);
@@ -1439,7 +1480,9 @@ static int maybe_add_amp (call_info *cinfo, int i, PRN *prn, int *add)
 	    if (m == NULL) {
 		err = E_ALLOC;
 	    } else {
-		err = add_or_replace_user_matrix(m, s);
+		err = user_var_add_or_replace(s,
+					      GRETL_TYPE_MATRIX,
+					      m);
 	    }
 	    if (!err) {
 		pprintf(prn, "? matrix %s\n", s);
@@ -1536,8 +1579,8 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
     char *tmpname = NULL;
     const char *funname;
     gretl_bundle *bundle = NULL;
-    int attach_bundle = 0;
-    int no_print = 0;
+    int grab_bundle = 0;
+    int show = 1;
     int orig_v = dataset->v;
     int i, err = 0;
 
@@ -1553,10 +1596,10 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	/* make a special arrangement to grab the returned
 	   bundle for GUI purposes 
 	*/
-	tmpname = get_bundle_temp_name();
+	tmpname = temp_name_for_bundle();
 	strcat(fnline, tmpname);
 	strcat(fnline, " = ");
-	attach_bundle = 1;
+	grab_bundle = 1;
     }	
 
     strcat(fnline, funname);
@@ -1573,7 +1616,7 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 
     strcat(fnline, ")");
 
-    if (!attach_bundle) {
+    if (!grab_bundle && strncmp(funname, "GUI", 3)) {
 	pprintf(prn, "? %s\n", fnline);
     }
 
@@ -1611,38 +1654,51 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
     /* destroy any "ARG" vars or matrices that were created? */
 
     if (!err && cinfo->rettype == GRETL_TYPE_BUNDLE) {
-	if (attach_bundle) {
-	    bundle = get_gretl_bundle_by_name(tmpname);
+	if (grab_bundle) {
+	    bundle = get_bundle_by_name(tmpname);
 	    if (bundle != NULL && gretl_bundle_get_n_keys(bundle) == 0) {
-		/* got a useless empty bundle */
+		/* we got a useless empty bundle */
 		gretl_bundle_pull_from_stack(tmpname, &err);
 		gretl_bundle_destroy(bundle);
 		bundle = NULL;
 	    }
-	    free(tmpname);
 	} else if (cinfo->ret != NULL) {
-	    bundle = get_gretl_bundle_by_name(cinfo->ret);
+	    bundle = get_bundle_by_name(cinfo->ret);
 	}
     }
 
-    no_print = user_func_is_noprint(cinfo->func);
+    show = !user_func_is_noprint(cinfo->func);
 
-    if (!err && bundle != NULL && no_print) {
+    if (!err && bundle != NULL && !show) {
 	gretl_print_reset_buffer(prn);
 	if (try_exec_bundle_print_function(bundle, prn)) {
-	    /* flag printing of bundled result */
-	    no_print = 0;
-	} 
+	    /* flag the fact that we do have something to show */
+	    show = 1;
+	}
     }
 
-    if (!err && no_print) {
+    if (grab_bundle && bundle != NULL) {
+	/* If the user specified an assignment of a returned bundle,
+	   we should leave it in the user_vars stack; but if we added
+	   the assignment automatically, we should pull it out of the
+	   stack, leaving the saving (or not) up to the user.
+	*/
+	gretl_bundle_pull_from_stack(tmpname, &err);
+    }
+
+    if (!err && !show) {
 	gretl_print_destroy(prn);
     } else {
-	view_buffer(prn, 80, 400, funname, 
+	const char *title = cinfo->label != NULL ?
+	    cinfo->label : funname;
+
+	view_buffer(prn, 80, 400, title, 
 		    (bundle == NULL)? PRINT : VIEW_BUNDLE,
 		    bundle);
     }
 
+    free(tmpname);
+
     if (err) {
 	gui_errmsg(err);
     } 
@@ -1693,7 +1749,7 @@ static void select_interface (call_info *cinfo, int npub)
 	} else {
 	    cinfo->iface = -1;
 	}
-	free_strings_array(opts, nopts);
+	strings_array_free(opts, nopts);
     } else {
 	resp = combo_selector_dialog(ilist, "select function", 
 				     0, cinfo->dlg);
@@ -1908,18 +1964,17 @@ void function_call_cleanup (void)
 }
 
 /* Execute the plotting function made available by the function
-   package that produced the bundle represented by @ptr, possibly
-   inflected by an integer option -- if an option is present it's
-   packed into @aname, after a colon. 
+   package that produced bundle @b, possibly inflected by an 
+   integer option -- if an option is present it's packed into 
+   @aname, after a colon. 
 */
 
 int exec_bundle_plot_function (gretl_bundle *b, const char *aname)
 {
     ufunc *func;
     fnargs *args = NULL;
-    const char *bname;
     char funname[32];
-    int iopt = -1;
+    int argc, iopt = -1;
     int err = 0;
 
     if (strchr(aname, ':') != NULL) {
@@ -1931,39 +1986,39 @@ int exec_bundle_plot_function (gretl_bundle *b, const char *aname)
     }
 
     func = get_user_function_by_name(funname);
-    bname = gretl_bundle_get_name(b);
 
-    if (func == NULL || bname == NULL) {
+    if (func == NULL) {
 	err = E_DATA;
     } else {
-	args = fn_args_new();
+	argc = iopt >= 0 ? 2 : 1;
+	args = fn_args_new(argc);
 	if (args == NULL) {
 	    err = E_ALLOC;
 	}
     }
 
     if (!err) {
+	const char *bname = user_var_get_name_by_data(b);
 #if 1
 	fprintf(stderr, "bundle plot: using bundle %p (%s)\n",
 		(void *) b, bname);
 #endif
-	err = push_fn_arg(args, GRETL_TYPE_BUNDLE_REF, (void *) bname);
+	err = push_fn_arg(args, bname, GRETL_TYPE_BUNDLE_REF, b);
 
 	if (!err && iopt >= 0) {
-	    /* add option flag to args */
+	    /* add the option flag, if any, to args */
 	    double minv = fn_param_minval(func, 1);
 
 	    if (!na(minv)) {
 		iopt += (int) minv;
-		err = push_fn_arg(args, GRETL_TYPE_INT, &iopt);
+		err = push_fn_arg(args, NULL, GRETL_TYPE_INT, &iopt);
 	    }
 	}
     }
 
     if (!err) {
-	/* note that the function may need a non-NULL prn for
-	   use with printing redirection; also we may want to
-	   see error output if something goes amiss
+	/* Note that the function may need a non-NULL prn for
+	   use with printing redirection (outfile).
 	*/
 	PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
 
@@ -2035,7 +2090,7 @@ int try_exec_bundle_print_function (gretl_bundle *b, PRN *prn)
     funname = get_bundle_special_function(b, BUNDLE_PRINT);
 
     if (funname != NULL) {
-	const char *name = gretl_bundle_get_name(b);
+	const char *name = user_var_get_name_by_data(b);
 	char fnline[MAXLINE];
 	ExecState state;
 	int err;
@@ -2091,7 +2146,6 @@ int query_addons (void)
     }
 
     if (!err) {
-	/* make use of info! */
 	infobox(buf);
     }
 
@@ -2243,6 +2297,39 @@ static addon_info *retrieve_addon_info (const gchar *pkgname)
     return NULL;
 }
 
+int package_is_available_for_menu (const gchar *pkgname,
+				   const char *path)
+{
+    int present = 0;
+    int i, ret = 0;
+
+    for (i=0; i<n_addons && !present; i++) {
+	if (!strcmp(pkgname, addons[i].pkgname)) {
+	    present = 1;
+	}
+    }
+
+    if (!present) {
+	/* not already present in menus: can it be added? */
+	gchar *mpath = NULL;
+	fnpkg *pkg;
+	int err = 0;
+
+	pkg = get_function_package_by_filename(path, &err);
+	if (!err) {
+	    err = function_package_get_properties(pkg,
+						  "menu-attachment", &mpath,
+						  NULL);
+	    if (mpath != NULL) {
+		ret = 1;
+		g_free(mpath);
+	    }
+	}
+    }
+
+    return ret;
+}
+
 /* Callback for a menu item representing a "addon" function package, 
    whose name (e.g. "gig") is attached to @action. We first see if 
    we can find the full path to the corresponding gfn file; if so we
@@ -2278,6 +2365,8 @@ static void gfn_menu_callback (GtkAction *action, windata_t *vwin)
 
     if (addon->filepath != NULL) {
 	call_function_package(addon->filepath, vwin, NULL);
+    } else {
+	unregister_package(pkgname);
     }
 }
 
@@ -2369,8 +2458,12 @@ static int read_addons_info (void)
     err = real_read_packages_file(fname, &n, 0);
 
     if (!err) {
+	struct stat buf;
+
 	sprintf(fname, "%sfunctions%cpackages.xml", gretl_dotdir(), SLASH);
-	err = real_read_packages_file(fname, &n, 1);
+	if (gretl_stat(fname, &buf) == 0) {
+	    err = real_read_packages_file(fname, &n, 1);
+	}
     }
 
     if (err) {
@@ -2382,12 +2475,12 @@ static int read_addons_info (void)
     return err;
 }
 
-/* For recognized "addon" function packages: given the internal
+/* For recognized function packages: given the internal
    package name (e.g. "gig") and a menu path where we'd like
    it to appear (e.g. "/menubar/Model/TSModels" -- see the
    ui definition file gui2/gretlmain.xml), construct the
    appropriate menu item and connect it to gfn_menu_callback(), 
-   which see above.
+   for which see above.
 */
 
 static void add_package_to_menu (addon_info *addon,
@@ -2398,6 +2491,11 @@ static void add_package_to_menu (addon_info *addon,
     };
     GtkActionGroup *actions;
 
+#if 0
+    fprintf(stderr, "add_package_to_menu:\n pkgname='%s', menupath='%s'\n",
+	    addon->pkgname, addon->menupath);
+#endif
+
     pkg_item.name = addon->pkgname;
     if (addon->label != NULL) {
 	pkg_item.label = _(addon->label);
@@ -2423,7 +2521,7 @@ static void add_package_to_menu (addon_info *addon,
 
 static int precheck_error (ufunc *func, windata_t *vwin)
 {
-    fnargs *args = fn_args_new();
+    fnargs *args = fn_args_new(0);
     PRN *prn;
     double check_err = 0;
     int err = 0;
@@ -2547,3 +2645,388 @@ void maybe_add_packages_to_model_menus (windata_t *vwin)
 	}
     }
 }
+
+/* Below: apparatus for activation when the user installs a function
+   package (other than a known addon) from the gretl server.
+
+   We check to see if (a) the package offers a menu attachment, and if
+   so (b) that the package is not already "registered" in the user's
+   packages.xml file. If both of these conditions are met we put up a
+   dialog asking if the user wants to add the package to the menu
+   system. If the answer is Yes we write an appropriate entry into
+   packages.xml, or write this file from scratch if it doesn't yet
+   exist.
+*/
+
+/* Find out where the package is supposed to attach: the
+   @mpath string (which gets into the gfn file from its
+   associated spec file) should look something like
+
+   MODELWIN/Analysis or
+   MAINWIN/Model
+
+   The first portion just tells us in which window it should
+   appear.
+*/
+
+static const gchar *pkg_get_attachment (const gchar *mpath,
+					int *modelwin)
+{
+    const gchar *relpath = mpath;
+
+    if (!strncmp(mpath, "MAINWIN/", 8)) {
+	relpath = mpath + 7;
+    } else if (!strncmp(mpath, "MODELWIN/", 9)) {
+	relpath = mpath + 8;
+	*modelwin = 1;
+    }
+
+    return relpath;
+}
+
+static int pkg_attach_dialog (const gchar *name,
+			      const gchar *label,
+			      const gchar *mpath)
+{
+    const gchar *relpath = NULL;
+    int modelwin = 0;
+    int resp = -1;
+
+    relpath = pkg_get_attachment(mpath, &modelwin);
+
+    if (relpath != NULL && *relpath != '\0') {
+	const gchar *window_names[] = {
+	    N_("main window"),
+	    N_("model window")
+	};	
+	gchar *msg;
+
+	msg = g_strdup_printf(_("The package %s can be attached to the "
+				"gretl menus\n"
+				"as \"%s/%s\" in the %s.\n"
+				"Do you want to do this?"),
+			      name, relpath, _(label),
+			      modelwin ? _(window_names[1]) :
+			      _(window_names[0]));
+	resp = yes_no_dialog(NULL, msg, 0);
+	g_free(msg);
+    }
+
+    return resp;
+}
+
+static void print_pkg_line (const gchar *name, 
+			    const gchar *label, 
+			    const gchar *mpath, 
+			    int toplev, 
+			    FILE *fp)
+{
+    const gchar *relpath;
+    int modelwin = 0;
+
+    relpath = pkg_get_attachment(mpath, &modelwin);
+
+    fprintf(fp, "<package name=\"%s\" label=\"%s\"", name, label);
+    if (modelwin) {
+	fputs(" model-window=\"true\"", fp);
+    }
+    if (!strncmp(relpath, "/menubar", 8)) {
+	fprintf(fp, " path=\"%s\"", relpath);
+    } else {
+	fprintf(fp, " path=\"/menubar%s\"", relpath);
+    }
+    if (toplev) {
+	fputs(" toplev=\"true\"/>\n", fp);
+    } else {
+	fputs("/>\n", fp);
+    }
+}
+
+/* When a package nominally appears in a menu but in fact
+   it is not found, remove it from packages.xml.
+*/
+
+static void unregister_package (const gchar *pkgname)
+{
+    gchar *pkgxml;
+    FILE *fp;
+    int err = 0;
+
+    pkgxml = g_strdup_printf("%sfunctions%cpackages.xml", 
+			     gretl_dotdir(), SLASH);
+
+    fp = gretl_fopen(pkgxml, "r");
+
+    if (fp != NULL) {
+	gchar *tmpfile;
+	char line[512];
+	FILE *fnew;
+
+	tmpfile = g_strdup_printf("%sfunctions%cpackages.xml.new", 
+				  gretl_dotdir(), SLASH);
+	fnew = gretl_fopen(tmpfile, "w");
+
+	if (fnew == NULL) {
+	    err = E_FOPEN;
+	} else {
+	    char *p, *q, *targ;
+	    int found = 0;
+
+	    while (fgets(line, sizeof line, fp)) {
+		int skip = 0;
+
+		if (!found && (p = strstr(line, "<package name=\""))) {
+		    p += 15;
+		    q = strchr(p, '"');
+		    targ = gretl_strndup(p, q - p);
+		    if (!strcmp(pkgname, targ)) {
+			found = skip = 1;
+		    }
+		    free(targ);
+		}
+		if (!skip) {
+		    fputs(line, fnew);
+		}
+	    }
+	    fclose(fnew);
+	}
+	fclose(fp);
+	if (!err) {
+	    err = gretl_copy_file(tmpfile, pkgxml);
+	    gretl_remove(tmpfile);
+	}
+	g_free(tmpfile);
+    } 
+
+    g_free(pkgxml);
+}
+
+static int register_package (const gchar *name,
+			     const gchar *label,
+			     const gchar *mpath,
+			     int uses_subdir)
+{
+    int toplev = !uses_subdir;
+    gchar *pkgxml;
+    FILE *fp;
+    int err = 0;
+
+    pkgxml = g_strdup_printf("%sfunctions%cpackages.xml", 
+			     gretl_dotdir(), SLASH);
+
+    fp = gretl_fopen(pkgxml, "r");
+
+    if (fp == NULL) {
+	/* starting a new packages.xml */
+	fp = gretl_fopen(pkgxml, "w");
+	if (fp == NULL) {
+	    err = E_FOPEN;
+	} else {
+	    fputs("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n", fp);
+	    fputs("<gretl-package-info>\n", fp);
+	    print_pkg_line(name, label, mpath, toplev, fp);
+	    fputs("</gretl-package-info>\n", fp);
+	    fclose(fp);
+	}
+    } else {
+	/* adding to an existing packages.xml, open as @fp */
+	gchar *tmpfile;
+	char line[512];
+	FILE *fnew;
+
+	tmpfile = g_strdup_printf("%sfunctions%cpackages.xml.new", 
+				  gretl_dotdir(), SLASH);
+	fnew = gretl_fopen(tmpfile, "w");
+
+	if (fnew == NULL) {
+	    err = E_FOPEN;
+	} else {
+	    while (fgets(line, sizeof line, fp)) {
+		if (strstr(line, "</gretl-package-info>")) {
+		    /* reached the last line */
+		    print_pkg_line(name, label, mpath, toplev, fnew);
+		}
+		fputs(line, fnew);
+	    }
+	    fclose(fnew);
+	}
+	fclose(fp);
+	if (!err) {
+	    err = gretl_copy_file(tmpfile, pkgxml);
+	    gretl_remove(tmpfile);
+	}
+	g_free(tmpfile);
+    } 
+
+    g_free(pkgxml);
+
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	gchar *upath = get_user_menu_string(mpath);
+
+	if (upath != NULL) {
+	    gchar *tmp = g_strdup_printf(_("Adding %s under %s."), 
+					 label, upath);
+
+	    infobox("%s\n%s", tmp, 
+		    _("This change will take effect when you restart gretl"));
+	    g_free(upath);
+	    g_free(tmp);
+	} else {
+	    infobox(_("This change will take effect when you restart gretl"));
+	}
+    }
+
+    return err;
+}
+
+/* see if a package is already "registered" in the user's
+   packages.xml file
+*/
+
+static int package_already_registered (const char *name)
+{
+    gchar *pkgxml;
+    FILE *fp;
+    int found = 0;
+
+    pkgxml = g_strdup_printf("%sfunctions%cpackages.xml", 
+			     gretl_dotdir(), SLASH);
+    fp = gretl_fopen(pkgxml, "r");
+
+    if (fp != NULL) {
+	char *s, line[512];
+	int pos;
+
+	while (fgets(line, sizeof line, fp) && !found) {
+	    s = strstr(line, "<package name=");
+	    if (s != NULL) {
+		s += 15;
+		pos = gretl_charpos('"', s);
+		if (pos > 0 && strncmp(s, name, pos) == 0) {
+		    found = 1;
+		}
+	    }
+	}
+	fclose(fp);
+    }
+
+    g_free(pkgxml);
+
+    return found;
+}
+
+int gui_add_package_to_menu (const char *path, gboolean prechecked)
+{
+    fnpkg *pkg;
+    int err = 0, ret = 0;
+
+    pkg = get_function_package_by_filename(path, &err);
+
+    if (!err) {
+	gchar *name = NULL, *mpath = NULL, *label = NULL;
+	int registered = 0, uses_subdir = 0;
+	int addit = 0;
+
+	err = function_package_get_properties(pkg,
+					      "name", &name,
+					      "label", &label,
+					      "menu-attachment", &mpath,
+					      "lives-in-subdir", &uses_subdir,
+					      NULL);
+
+	if (!err && !prechecked && name != NULL) {
+	    registered = package_already_registered(name);
+	}
+
+	if (!err && !registered && name != NULL && 
+	    label != NULL && mpath != NULL) {
+	    if (prechecked) {
+		/* go right ahead */
+		addit = 1;
+	    } else {
+		/* not "prechecked": put up a dialog */
+		int resp = pkg_attach_dialog(name, label, mpath);
+
+		if (resp >= 0) {
+		    ret = 1;
+		}
+		if (resp == GRETL_YES) {
+		    addit = 1;
+		}
+	    }
+	}
+
+	if (addit) {
+	    register_package(name, label, mpath, uses_subdir);
+	}
+
+	g_free(name);
+	g_free(label);
+	g_free(mpath);
+    }
+
+    return ret;
+}
+
+/* returns non-zero if we put up a dialog box */
+
+int maybe_handle_pkg_menu_option (const char *path)
+{
+    return gui_add_package_to_menu(path, FALSE);
+}
+
+char *installed_addon_status_string (const char *path,
+				     const char *svstr)
+{
+    fnpkg *pkg;
+    int err = 0;
+    char *ret = NULL;
+
+    pkg = get_function_package_by_filename(path, &err);
+
+    if (pkg != NULL) {
+	int current = 0;
+	int update_ok = 0;
+	char reqstr[8] = {0};
+	gchar *ivstr = NULL;
+	int minver = 0;
+
+	/* @ivstr = installed package version string
+	   @svstr = package version string from server
+	*/
+	err = function_package_get_properties(pkg, 
+					      "version", &ivstr, 
+					      "min-version", &minver,
+					      NULL);
+	if (!err) {
+	    double svnum = dot_atof(svstr);
+	    double ivnum = dot_atof(ivstr);
+
+	    current = ivnum >= svnum;
+	    g_free(ivstr);
+
+	    if (!current) {
+		/* Not current, but can the addon be updated?  It may
+		   be that the running instance of gretl is too old.
+		*/
+		update_ok = package_version_ok(minver, reqstr);
+	    }
+
+	    if (current) {
+		ret = gretl_strdup(_("Up to date"));
+	    } else if (update_ok) {
+		ret = gretl_strdup(_("Not up to date"));
+	    } else if (*reqstr != '\0') {
+		ret = gretl_strdup_printf(_("Requires gretl %s"), reqstr);
+	    }
+	}
+    }
+
+    if (ret == NULL) {
+	ret = gretl_strdup(_("Error reading package"));
+    }
+
+    return ret;
+}
