@@ -23,6 +23,7 @@
 #include "gretl_func.h"
 #include "gretl_xml.h"
 #include "libset.h"
+#include "uservar.h"
 #include "gretl_bundle.h"
 
 #include "treeutils.h"
@@ -209,6 +210,11 @@ static void edit_package_callback (GtkAction *action, gpointer p)
     file_selector(OPEN_GFN, FSEL_DATA_NONE, NULL);
 }
 
+static void new_matrix_callback (GtkAction *action, gpointer p)
+{
+    gui_new_matrix(mdata->main);
+}
+
 static void new_package_callback (GtkAction *action, gpointer p)
 {
     if (no_user_functions_check()) {
@@ -856,7 +862,7 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
     return FALSE;
 }
 
-static int lagvar_get_parent_iter (int pv, GtkTreeIter *parent)
+static int series_get_parent_iter (int pv, GtkTreeIter *parent)
 {
     GtkTreeModel *model = 
 	gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox));
@@ -914,6 +920,32 @@ static int get_line_pos (GtkTreeModel *mod)
     return pos;
 }
 
+static int get_lag_or_dummy_parent (int v)
+{
+    const char *vname = dataset->varname[v];
+    int i = 0, pv = 0;
+
+    if (series_get_lag(dataset, v) != 0) {
+	pv = series_get_parent_id(dataset, v);
+    } else if (series_get_transform(dataset, v) == DUMMIFY) {
+	pv = series_get_parent_id(dataset, v);
+    } else if (!strncmp(vname, "dt_", 3)) {
+	if (sscanf(vname + 3, "%d", &i) && i > 1) {
+	    pv = current_series_index(dataset, "dt_1");
+	}
+    } else if (!strncmp(vname, "du_", 3)) {
+	if (sscanf(vname + 3, "%d", &i) && i > 1) {
+	    pv = current_series_index(dataset, "du_1");
+	}
+    }
+
+    if (pv < 0 || pv >= dataset->v) {
+	pv = 0;
+    }
+    
+    return pv;
+}
+
 /* populate the list of series in the main gretl window */
 
 void populate_varlist (void)
@@ -939,24 +971,28 @@ void populate_varlist (void)
     for (i=0; i<dataset->v; i++) {
 	int pv = 0;
 
-	if (var_is_hidden(dataset, i)) {
+	if (series_is_hidden(dataset, i)) {
 	    continue;
 	}
-	if (i > 0 && (is_standard_lag(i, dataset, &pv) ||
-		      is_dummy_child(i, dataset, &pv))) {
-	    if (pv > 0) {
-		GtkTreeIter child_iter, parent_iter;
 
-		if (lagvar_get_parent_iter(pv, &parent_iter)) {
-		    gtk_tree_store_insert_before(store, &child_iter, 
-						 &parent_iter, NULL);
-		    sprintf(id, "%d", i);
-		    gtk_tree_store_set(store, &child_iter, 
-				       0, id, 
-				       1, dataset->varname[i],
-				       2, VARLABEL(dataset, i),
-				       -1);	
-		}	
+	if (i > 0) {
+	    pv = get_lag_or_dummy_parent(i);
+	}
+
+	if (pv > 0) {
+	    GtkTreeIter child_iter, parent_iter;
+
+	    if (series_get_parent_iter(pv, &parent_iter)) {
+		gtk_tree_store_insert_before(store, &child_iter, 
+					     &parent_iter, NULL);
+		sprintf(id, "%d", i);
+		gtk_tree_store_set(store, &child_iter, 
+				   0, id, 
+				   1, dataset->varname[i],
+				   2, series_get_label(dataset, i),
+				   -1);
+	    } else {
+		pv = 0;
 	    }
 	}
 
@@ -966,7 +1002,7 @@ void populate_varlist (void)
 	    gtk_tree_store_set(store, &iter, 
 			       0, id, 
 			       1, dataset->varname[i],
-			       2, VARLABEL(dataset, i),
+			       2, series_get_label(dataset, i),
 			       -1);
 	}
     } 
@@ -1088,9 +1124,7 @@ static void real_select_list (const int *list)
 
 static void mdata_select_list (void)
 {
-    int nl = n_saved_lists();
-
-    if (nl == 0) {
+    if (n_user_lists() == 0) {
 	warnbox(_("No lists are currently defined"));
 	return;
     } else {
@@ -1492,7 +1526,7 @@ GtkActionEntry main_entries[] = {
     { "TimeDums", NULL, N_("_Time dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
     { "dummify", NULL, N_("Dummies for selected _discrete variables"), NULL, NULL, 
       G_CALLBACK(logs_etc_callback) },
-    { "NewMatrix", NULL, N_("_Define matrix..."), NULL, NULL, G_CALLBACK(gui_new_matrix) },
+    { "NewMatrix", NULL, N_("_Define matrix..."), NULL, NULL, G_CALLBACK(new_matrix_callback) },
 
     /* Sample */
     { "Sample", NULL, N_("_Sample"), NULL, NULL, NULL },
@@ -1618,6 +1652,59 @@ GtkActionEntry main_entries[] = {
     { "About", GTK_STOCK_ABOUT, N_("_About gretl"), NULL, NULL, G_CALLBACK(about_dialog) }
 };
 
+gchar *get_user_menu_string (const gchar *mpath)
+{
+    gchar *ret = NULL;
+    gchar **S;
+
+    if (!strncmp(mpath, "/menubar/", 9)) {
+	mpath += 8;
+    } else if (!strncmp(mpath, "MAINWIN/", 8)) {
+	mpath += 8;
+    } else if (*mpath == '/') {
+	mpath += 1;
+    } else if (!strncmp(mpath, "MODELWIN/", 9)) {
+	return NULL;
+    }
+
+    fprintf(stderr, "get_user_menu_string: mpath='%s'\n", mpath);
+
+    S = g_strsplit(mpath, "/", 0);
+
+    if (S != NULL && S[0] != NULL) {
+	int i, j, nmain = G_N_ELEMENTS(main_entries);
+	const gchar *p1 = NULL, *p2 = NULL;
+
+	for (i=0; i<nmain && !p1; i++) {
+	    if (main_entries[i].callback == NULL && 
+		!strcmp(S[0], main_entries[i].name)) {
+		p1 = main_entries[i].label;
+		if (S[1] != NULL) {
+		    for (j=i+1; j<nmain && !p2; j++) {
+			if (main_entries[j].callback == NULL &&
+			    !strcmp(S[1], main_entries[j].name)) {
+			    p2 = main_entries[j].label;
+			}
+		    }
+		}
+	    }
+	}
+	if (p2 != NULL) {
+	    ret = g_strdup_printf("/%s/%s", p1, p2);
+	} else if (p1 != NULL) {
+	    ret = g_strdup_printf("/%s", p1);
+	}
+    }
+	
+    g_strfreev(S);
+
+    if (ret != NULL) {
+	gretl_delchar('_', ret);
+    }
+
+    return ret;
+}
+
 static void add_conditional_items (windata_t *vwin)
 {
     GtkUIManager *ui = vwin->ui;
@@ -1982,8 +2069,8 @@ mdata_handle_drag  (GtkWidget *widget,
 
     /* there may be multiple files: we ignore all but the first */
     *tmp = 0;
-    if ((pos = charpos('\r', dfname)) > 0 || 
-	(pos = charpos('\n', dfname) > 0)) {
+    if ((pos = gretl_charpos('\r', dfname)) > 0 || 
+	(pos = gretl_charpos('\n', dfname) > 0)) {
 	strncat(tmp, dfname + skip, pos - skip);
     } else {
 	strcat(tmp, dfname + skip);
