@@ -8,7 +8,6 @@ enum extra_cmds {
     SMPLDUM,
     SMPLBOOL,
     SMPLRAND,
-    MARKERS,
     STORE_MODEL,
     VAR_SUMMARY,
     ALL_SUMMARY,
@@ -220,7 +219,7 @@ enum browser_codes {
 enum pref_tabs {
     TAB_NONE = 0,
     TAB_MAIN,
-    TAB_DBS,
+    TAB_NET,
     TAB_PROGS,
     TAB_VCV,
     TAB_MAN,
