@@ -29,6 +29,8 @@
 
 #include <gdk/gdkwin32.h>
 #include <dirent.h>
+
+#include <windows.h>
 #include <mapi.h>
 #include <shlobj.h>
 #include <shellapi.h>
@@ -67,36 +69,6 @@ void redirect_io_to_console (void)
     setvbuf(stderr, NULL, _IONBF, 0);
 }
 
-static void ws_cleanup (void)
-{
-    WSACleanup();
-}
-
-static int ws_startup (void)
-{
-    WORD requested;
-    WSADATA data;
-
-    requested = MAKEWORD(1, 1);
-
-    if (WSAStartup(requested, &data)) {
-	fprintf(stderr, I_("Couldn't find usable socket driver\n"));
-	return 1;
-    }
-
-    if (LOBYTE (requested) < 1 || (LOBYTE (requested) == 1 &&
-				   HIBYTE (requested) < 1)) {
-	fprintf(stderr, I_("Couldn't find usable socket driver\n"));
-	WSACleanup();
-	return 1;
-    } else {
-	/* arrange for cleanup on exit */
-	atexit(ws_cleanup);
-    }
-
-    return 0;
-}
-
 int real_create_child_process (char *prog, int showerr) 
 { 
     PROCESS_INFORMATION proc_info; 
@@ -324,7 +296,9 @@ static void wimp_init (void)
 {
     char tmp[4] = {0};
 
-    /* Are we using the XP theme (as opposed to "classic")? */
+    /* Are we using the XP theme (as opposed to "classic")?
+       In that case we'll make use of libwimp the default.
+    */
 
     read_reg_val(HKEY_CURRENT_USER, 
 		 "Microsoft\\Windows\\CurrentVersion\\ThemeManager", 
@@ -362,17 +336,20 @@ void gretl_win32_debug_init (int debug)
 }
 
 /* carry out some Windows-specific start-up tasks, and
-   call read_rc to get configuration info */
+   call read_rc to get configuration info 
+*/
 
 void gretl_win32_init (const char *progname, int debug)
 {
     set_gretlnet_filename(progname);
     wimp_init();
+    if (debug) {
+	fprintf(stderr, "after wimp_init: use_wimp = %d\n", use_wimp);
+    }    
     read_win32_config(debug);
     set_gretl_startdir();
-    ws_startup();
     if (debug) {
-	fprintf(stderr, "gretl_win32_init: done\n");
+	fprintf(stderr, "gretl_win32_init: done (use_wimp = %d)\n", use_wimp);
     }
 }
 
@@ -777,3 +754,85 @@ void win32_font_selector (char *fontname, int flag)
 	*fontname = '\0';
     }
 }
+
+static BOOL running_as_admin (void)
+{
+    SID_IDENTIFIER_AUTHORITY auth = {SECURITY_NT_AUTHORITY};
+    PSID admin_group = NULL;
+    BOOL ok, ret = FALSE;
+
+    ok = AllocateAndInitializeSid(&auth, 2,
+				  SECURITY_BUILTIN_DOMAIN_RID, 
+				  DOMAIN_ALIAS_RID_ADMINS, 
+				  0, 0, 0, 0, 0, 0, 
+				  &admin_group);
+
+    if (ok) {
+	ok = CheckTokenMembership(NULL, admin_group, &ret);
+    }
+
+    if (!ok) {
+	fprintf(stderr, "running_as_admin: the check failed\n");
+    }
+ 
+    if (admin_group != NULL) {
+	FreeSid(admin_group);
+    }
+ 
+    return ret;
+}
+
+/* Note: at some point MS will scrap the virtual store
+   apparatus (supposedly), in which case we'll want to
+   add a further condition in this function -- that is,
+   dwMajorVersion < something.
+*/
+
+int windows_uses_virtual_store (void)
+{
+    OSVERSIONINFO osvi;
+    int ret;
+
+    ZeroMemory(&osvi, sizeof osvi);
+    osvi.dwOSVersionInfoSize = sizeof osvi;
+
+    GetVersionEx(&osvi);
+    /* VirtualStore came in with Vista */
+    ret = osvi.dwMajorVersion > 5;
+    
+    if (ret && running_as_admin()) {
+	ret = 0;
+    }
+
+    return ret;
+}
+
+int win32_rename_dir (const char *oldname, const char *newname)
+{
+    char *oldtmp = NULL, *newtmp = NULL;
+    int len, err;
+
+    /* trim trailing slash for non-root dirs */
+
+    len = strlen(oldname);
+    if (len > 1 && oldname[len - 1] == '\\' && oldname[len - 2] != ':') {
+	oldtmp = gretl_strndup(oldname, len - 1);
+	oldname = oldtmp;
+    } 
+
+    len = strlen(newname);
+    if (len > 1 && newname[len - 1] == '\\' && newname[len - 2] != ':') {
+	newtmp = gretl_strndup(newname, len - 1);
+	newname = newtmp;
+    }    
+
+    err = gretl_rename(oldname, newname);
+
+    if (oldtmp!= NULL || newtmp != NULL) {
+	free(oldtmp);
+	free(newtmp);
+    }
+
+    return err;
+}
+
