@@ -30,7 +30,7 @@
 #include "guiprint.h"
 #include "graph_page.h"
 
-#include "gretl_scalar.h"
+#include "uservar.h"
 #include "libset.h"
 
 #ifdef G_OS_WIN32
@@ -749,6 +749,28 @@ rtfprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
 	printf_rtf(summ->xkurt[i], prn, 1);
     }
 
+    if (summ->list[0] > 1) pprintf(prn, "\\intbl \\qc %s\\cell",
+				   A_("Variable"));
+
+    pprintf(prn, 
+	    " \\qc %s\\cell"
+	    " \\qc %s\\cell"
+	    " \\qc %s\\cell"
+	    " \\qc %s\\cell"
+	    " \\intbl \\row\n",
+	    A_("5% Perc."), A_("95% Perc."), A_("IQ range"), A_("Missing obs."));
+
+    for (i=0; i<summ->list[0]; i++) {
+	vi = summ->list[i + 1];
+	if (summ->list[0] > 1) {
+	    pprintf(prn, "\\intbl \\qc %s\\cell ", pdinfo->varname[vi]);
+	}
+	printf_rtf(summ->perc05[i], prn, 0);
+	printf_rtf(summ->perc95[i], prn, 0);
+	printf_rtf(summ->iqr[i], prn, 0);
+	printf_rtf(summ->missing[i], prn, 1);
+    }
+
     pputs(prn, "}}\n");
 }
 
@@ -1041,6 +1063,38 @@ texprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
 	printf_tex(summ->cv[i], prn, 0);
 	printf_tex(summ->skew[i], prn, 0);
 	printf_tex(summ->xkurt[i], prn, 1);
+	if (i == summ->list[0] - 1) {
+	    pputs(prn, "[10pt]\n\n");
+	} else {
+	    pputc(prn, '\n');
+	}
+    }
+
+    if (summ->list[0] > 1) {
+	pprintf(prn, "%s & ", A_("Variable"));
+    }
+
+    pprintf(prn, " \\multicolumn{2}{c}{%s}%%\n"
+	    " & \\multicolumn{2}{c}{%s}%%\n"
+	    "  & \\multicolumn{2}{c}{%s}%%\n"
+	    "   & \\multicolumn{2}{c}{%s} \\\\[1ex]\n",
+	    /* xgettext:no-c-format */
+	    A_("5\\% perc."),
+	    /* xgettext:no-c-format */
+	    A_("95\\% perc."), 
+	    A_("IQ Range"), 
+	    A_("Missing obs."));
+
+    for (i=0; i<summ->list[0]; i++) {
+	vi = summ->list[i + 1];
+	if (summ->list[0] > 1) {
+	    tex_escape(vname, pdinfo->varname[vi]);
+	    pprintf(prn, "%s & ", vname);
+	}
+	printf_tex(summ->perc05[i], prn, 0);
+	printf_tex(summ->perc95[i], prn, 0);
+	printf_tex(summ->iqr[i], prn, 0);
+	printf_tex(summ->missing[i], prn, 1);
 	pputc(prn, '\n');
     }
 
@@ -1899,11 +1953,12 @@ void special_print_confints (const CoeffIntervals *cf, PRN *prn)
 
 int scalars_to_prn (PRN *prn)
 {
+    GList *slist, *tail;
     char decpoint = get_data_export_decpoint();
     char delim = get_data_export_delimiter();
     const char *sname;
     double sval;
-    int i, n = n_saved_scalars();
+    user_var *u;
 
     if (delim == ',' && ',' == decpoint) {
 	errbox(_("You can't use the same character for "
@@ -1911,20 +1966,26 @@ int scalars_to_prn (PRN *prn)
 	return 1;
     }
 
+    tail = slist = user_var_list_for_type(GRETL_TYPE_DOUBLE);
+
     if (decpoint != ',') {
 	gretl_push_c_numeric_locale();
     }
 
-    for (i=0; i<n; i++) {
-	sname = gretl_scalar_get_name(i);
-	sval = gretl_scalar_get_value_by_index(i);
+    while (tail) {
+	u = tail->data;
+	sname = user_var_get_name(u);
+	sval = user_var_get_scalar_value(u);
 	if (na(sval)) {
 	    pprintf(prn, "%s%cNA\n", sname, delim);
 	} else {
 	    pprintf(prn, "%s%c%.15g\n", sname, delim, sval);
 	}
+	tail = tail->next;
     }
 
+    g_list_free(slist);
+
     if (decpoint != ',') {
 	gretl_pop_c_numeric_locale();
     }
@@ -2039,7 +2100,7 @@ static void matrix_print_as_csv (const gretl_matrix *m, PRN *prn)
 	    x = gretl_matrix_get(m, i, j);
 	    sprintf(numstr, "%.*g", DBL_DIG, x);
 	    if (decpoint != '.') {
-		charsub(numstr, '.', decpoint);
+		gretl_charsub(numstr, '.', decpoint);
 	    }
 	    pputs(prn, numstr);
 	    if (j < m->cols - 1) {
@@ -2078,7 +2139,7 @@ int scalars_to_clipboard_as_csv (GtkWidget *parent)
 {
     int err = 0;
 
-    if (n_saved_scalars() == 0) {
+    if (n_user_scalars() == 0) {
 	warnbox(_("No scalar variables are currently defined"));
     } else {
 	int resp = csv_options_dialog(COPY_CSV, GRETL_OBJ_SCALARS,
@@ -2375,7 +2436,7 @@ static void view_or_save_latex (PRN *bprn, const char *fname, int saveit)
 	return;
     }
 
-    dot = dotpos(texfile);
+    dot = gretl_dotpos(texfile);
     *texbase = 0;
     strncat(texbase, texfile, dot);
 
