@@ -101,7 +101,6 @@ static struct gui_help_item gui_help_items[] = {
     { GR_FBOX,        "boxplots" },
     { GR_3D,          "3-D" },
     { ONLINE,         "online" },
-    { MARKERS,        "markers" },
     { EXPORT,         "export" },
     { SMPLBOOL,       "sampling" },
     { SMPLDUM,        "sampling" },
@@ -1896,7 +1895,7 @@ enum {
     GRETL_KEYS
 };
 
-static int get_writable_path (char *path, const char *fname, int code)
+static int get_writable_doc_path (char *path, const char *fname)
 {
     static int sysdoc_writable = -1;
     static int userdoc_writable = -1;
@@ -1913,6 +1912,13 @@ static int get_writable_path (char *path, const char *fname, int code)
 	return 0;
     }
 
+#ifdef G_OS_WIN32
+    if (sysdoc_writable < 0 && windows_uses_virtual_store()) {
+	/* don't write to virtualized location */
+	sysdoc_writable = 0;
+    }
+#endif
+
     if (sysdoc_writable < 0) {
 	sysdoc_writable = 0;
 	sprintf(path, "%sdoc", gretldir);
@@ -2045,7 +2051,7 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
     }	
 
     if (!gotit) {
-	/* or maybe in user dir? */
+	/* or maybe in user's dotdir? */
 	sprintf(fullpath, "%sdoc%c%s", gretl_dotdir(), SLASH, fname);
 	fp = gretl_fopen(fullpath, "r");
 	if (fp != NULL) {
@@ -2056,7 +2062,7 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 
     if (!gotit) {
 	/* check for download location */
-	err = get_writable_path(fullpath, fname, code);
+	err = get_writable_doc_path(fullpath, fname);
 
 	/* do actual download */
 	if (!err) {
