@@ -48,7 +48,7 @@
 #include "gretl_func.h"
 #include "matrix_extra.h"
 #include "cmd_private.h"
-#include "gretl_scalar.h"
+#include "uservar.h"
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -86,7 +86,8 @@ typedef struct gui_obj_ gui_obj;
 
 enum {
     SESSION_CHANGED    = 1 << 0,
-    SESSION_SAVED      = 1 << 1
+    SESSION_SAVED      = 1 << 1,
+    SESSION_OPEN       = 1 << 2
 };
 
 struct SESSION_ {
@@ -181,7 +182,8 @@ static char *graph_items[] = {
     N_("Display"),
     N_("Edit plot commands"),
     N_("Rename"),
-    N_("Delete")
+    N_("Delete"),
+    N_("Copy")
 };
 
 static char *dataset_items[] = {
@@ -205,7 +207,8 @@ static char *matrix_items[] = {
     N_("Properties"),
     N_("Copy as CSV..."),
     N_("Rename"),
-    N_("Delete")
+    N_("Delete"),
+    N_("Copy")
 };
 
 static char *bundle_items[] = {
@@ -260,10 +263,11 @@ static gboolean session_view_click (GtkWidget *widget,
 static int real_delete_model_from_session (SESSION_MODEL *model);
 static void make_short_label_string (char *targ, const char *src);
 static gui_obj *get_gui_obj_by_data (void *finddata);
-static void add_user_matrix_callback (const char *mname);
-static void add_bundle_callback (gretl_bundle *bundle);
+static int gui_user_var_callback (const char *name, GretlType type,
+				  int action);
 
 static int session_graph_count;
+static int session_bundle_count;
 static int commands_recorded;
 
 int session_is_modified (void)
@@ -271,27 +275,37 @@ int session_is_modified (void)
     return session.status & SESSION_CHANGED;
 }
 
+int session_is_open (void)
+{
+    return session.status & SESSION_OPEN;
+}
+
 void mark_session_changed (void)
 {
     iconview_menubar_state(TRUE);
-    session.status = SESSION_CHANGED;
+    session.status &= ~SESSION_SAVED;
+    session.status |= SESSION_CHANGED;
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, TRUE);
-	flip(mdata->ui, "/menubar/File/SessionFiles/SaveSession", TRUE);
-	if (*session.name != '\0') {
-	    set_main_window_title(session.name, TRUE);
-	}
+    }
+    flip(mdata->ui, "/menubar/File/SessionFiles/SaveSession", TRUE);
+    flip(mdata->ui, "/menubar/File/SessionFiles/SaveSessionAs", TRUE);
+    if (*session.name != '\0') {
+	set_main_window_title(session.name, TRUE);
     }
 }
 
 static void mark_session_saved (void)
 {
-    session.status = SESSION_SAVED;
+    session.status &= ~SESSION_CHANGED;
+    session.status |= SESSION_SAVED;
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, FALSE);
-	flip(mdata->ui, "/menubar/File/SessionFiles/SaveSession", FALSE);
-	set_main_window_title(session.name, FALSE);
     }
+    flip(mdata->ui, "/menubar/File/SessionFiles/SaveSession", FALSE);
+    flip(mdata->ui, "/menubar/File/SessionFiles/SaveSessionAs", 
+	 session.status & SESSION_OPEN);
+    set_main_window_title(session.name, FALSE);
 }
 
 void set_commands_recorded (void)
@@ -696,9 +710,9 @@ void save_output_as_text_icon (windata_t *vwin)
     if (err) {
 	return;
     } else if (iconlist != NULL) {
-	    SESSION_TEXT * text = get_session_text_by_name(tname);
+	SESSION_TEXT * text = get_session_text_by_name(tname);
 
-	    session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
+	session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
 	view_session();
     }
@@ -851,7 +865,6 @@ int gui_add_graph_to_session (char *fname, char *fullname, int type)
     }
 
     gretl_chdir(gretl_dotdir());
-
     make_graph_name(shortname, graphname);
     session_file_make_path(fullname, shortname);
 
@@ -1107,29 +1120,26 @@ void bundle_add_as_icon (GtkAction *action, gpointer p)
     windata_t *vwin = (windata_t *) p;
     gretl_bundle *bundle = vwin->data;
     char vname[VNAMELEN];
-    char *defname;
-    const char *blurb;
+    gchar *blurb;
     int *pshow, show = 0;
     int resp;
 
-    defname = get_bundle_default_name();
-    strcpy(vname, defname);
-    free(defname);
-    blurb = "Save bundle\nName (max. 15 characters):";
+    sprintf(vname, "bundle%d", ++session_bundle_count);
+    blurb = g_strdup_printf("Save bundle\nName (max. %d characters):",
+			    VNAMELEN - 1);
     pshow = (iconlist == NULL)? &show : NULL;
     resp = object_name_entry_dialog(vname, GRETL_TYPE_BUNDLE, 
 				    blurb, pshow, vwin->main);
+    g_free(blurb);
 
-    if (!canceled(resp)) {    
-	int err = gretl_bundle_set_name(bundle, vname);
+    if (!canceled(resp)) { 
+	int err = user_var_add(vname, GRETL_TYPE_BUNDLE, bundle);
 	int flipit = 1;
 
 	if (err) {
 	    gui_errmsg(err);
 	} else {
-	    if (iconlist != NULL) {
-		session_add_icon(bundle, GRETL_OBJ_BUNDLE, ICON_ADD_SINGLE);
-	    } else if (autoicon_on() || show) {
+	    if (autoicon_on() || show) {
 		view_session();
 	    }
 	    mark_session_changed();
@@ -1401,28 +1411,17 @@ void do_open_session (void)
 	goto bailout;
     }
 
-    session_file_make_path(fname, "matrices.xml");
-    err = maybe_read_matrix_file(fname);
-
-    session_file_make_path(fname, "bundles.xml");
-    err = maybe_read_bundles_file(fname);
-
-    session_file_make_path(fname, "scalars.xml");
-    err = maybe_read_scalars_file(fname);
+    err = deserialize_user_vars(session.dirname);
 
     session_file_make_path(fname, "functions.xml");
     err = maybe_read_functions_file(fname);
 
-    session_file_make_path(fname, "lists.xml");
-    err = maybe_read_lists_file(fname);
-
     session_file_make_path(fname, "settings.inp");
     err = maybe_read_settings_file(fname);
 
     if (!nodata) {
 	if (sinfo.resample_n > 0) {
-	    err = dataset_resample(sinfo.resample_n, sinfo.seed, 
-				   dataset);
+	    err = dataset_resample(dataset, sinfo.resample_n, sinfo.seed); 
 	} else if (sinfo.mask != NULL) {
 	    err = restrict_sample_from_mask(sinfo.mask, dataset,
 					    OPT_NONE);
@@ -1459,6 +1458,8 @@ void do_open_session (void)
 	strcpy(sessionfile, tryfile);
 	mkfilelist(FILE_LIST_SESSION, sessionfile);
 
+	session.status = SESSION_OPEN;
+
 	/* sync gui with session */
 	session_menu_state(TRUE);
 
@@ -1520,11 +1521,7 @@ void session_init (void)
     *session.name = '\0';
     *session.dirname = '\0';
 
-    set_matrix_add_callback(add_user_matrix_callback);
-    set_matrix_delete_callback(session_matrix_destroy_by_name);
-
-    set_bundle_add_callback(add_bundle_callback);
-    set_bundle_delete_callback(session_bundle_destroy_by_name);
+    set_user_var_callback(gui_user_var_callback);
 }
 
 void free_session (void)
@@ -1659,13 +1656,11 @@ void close_session (gretlopt opt)
     fprintf(stderr, "close_session: starting cleanup\n");
 #endif
 
-    if (!(opt & OPT_O)) {
-	if (dataset != NULL && dataset->v > 0) {
-	    logcode = LOG_CLOSE;
-	    session_clear_data(dataset); 
-	}
+    if (dataset != NULL && dataset->v > 0) {
+	logcode = LOG_CLOSE;
+	session_clear_data(dataset); 
     }
-
+ 
     free_session();
 
     clear_model_table(1, NULL);
@@ -1679,7 +1674,7 @@ void close_session (gretlopt opt)
 	gtk_widget_destroy(iconview);
     }
 
-    session.status = 0;
+    session.status = 0; /* not saved, changed or open */
     session.show_notes = 0;
     commands_recorded = 0;
 
@@ -1690,13 +1685,12 @@ void close_session (gretlopt opt)
 
     if (opt & OPT_P) {
 	libgretl_session_cleanup(SESSION_CLEAR_DATASET);
-    } else if (opt & OPT_O) {
-	libgretl_session_cleanup(SESSION_CLEAR_OTHER);
     } else {
 	libgretl_session_cleanup(SESSION_CLEAR_ALL);
     }
 
     session_graph_count = 0;
+    session_bundle_count = 0;
     reset_plot_count();
 
     set_session_log(NULL, logcode);
@@ -1767,8 +1761,6 @@ static int real_save_session_dataset (const char *dname)
 	/* flag the fact that the data are saved */
 	data_status &= ~MODIFIED_DATA;
     }
-
-    fprintf(stderr, "real_save_session_dataset: err=%d\n", err);
     
     return err;
 }
@@ -1835,23 +1827,34 @@ static void make_session_dataname (char *datname)
     }
 }
 
+#define SAVE_DEBUG 1
+
 int save_session (char *fname) 
 {
+    char *dirbak = NULL;
     char datname[MAXLEN];
     char dirname[MAXLEN];
     void *handle;
     int (*gretl_make_zipfile) (const char *, const char *, GError **);
     int len, err = 0;
+    int log_code = LOG_SAVE;
     GError *gerr = NULL;
 
-    fprintf(stderr, "save_session: fname='%s', sessionfile='%s'\n",
-	    fname, sessionfile);
-
     if (fname == NULL) {
-	/* saving session 'as is' */
+	/* re-saving session 'as is' */
 	fname = sessionfile;
     } 
 
+#if SAVE_DEBUG
+    if (fname == sessionfile) {
+	fprintf(stderr, "save_session:\n sessionfile='%s'\n",
+		sessionfile);
+    } else {
+	fprintf(stderr, "save_session as:\n current session='%s'\n"
+		" save filename='%s'\n", sessionfile, fname);
+    }
+#endif
+
     if (!session_dir_ok()) {
 	errbox("Couldn't make session directory");
 	return 1;
@@ -1866,31 +1869,62 @@ int save_session (char *fname)
 	strcat(fname, ".gretl");
     }
 
-    /* paths below are relative to this */
-    gretl_chdir(gretl_dotdir());
+#if SAVE_DEBUG
+    fprintf(stderr, " save dirname = '%s'\n", dirname);
+    fprintf(stderr, " current session.dirname = '%s'\n", session.dirname);
+    fprintf(stderr, " doing chdir to '%s'\n", gretl_dotdir());
+#endif
+
+    /* note: paths below are relative to this */
+    err = gretl_chdir(gretl_dotdir());
+    if (err) {
+	fprintf(stderr, " chdir to dotdir failed\n");
+	gui_errmsg(err);
+	return 1;
+    }
 
     if (strcmp(dirname, session.dirname)) {
-	/* rename session directory */
-	gretl_rename(session.dirname, dirname);
-	strcpy(session.dirname, dirname);
+	/* have to rename the session directory */
+	maybe_suspend_session_log();
+	log_code = LOG_SAVE_AS;
+	dirbak = gretl_strdup(session.dirname);
+#ifdef G_OS_WIN32
+	err = win32_rename_dir(session.dirname, dirname);
+#else
+	err = gretl_rename(session.dirname, dirname);
+#endif
+	if (err) {
+	    fprintf(stderr, " failed to rename session dir\n");
+	    gui_errmsg(err);
+	} else {
+	    fprintf(stderr, " renamed session dir OK\n");
+	    strcpy(session.dirname, dirname);
+	}
+    }  
+
+    if (!err) {
+	*datname = '\0';
+	if (data_status) {
+	    make_session_dataname(datname);
+	} else {
+	    strcpy(datname, "none");
+	}
+
+	err = write_session_xml(datname);
+#if SAVE_DEBUG
+	fprintf(stderr, " write_session_xml: err = %d\n", err);
+#endif
     }
 
-    *datname = '\0';
-
-    if (data_status) {
-	make_session_dataname(datname);
-    } else {
-	strcpy(datname, "none");
-    }
-
-    write_session_xml(datname);
-
-    if (data_status) {
+    if (!err && data_status) {
 	err = real_save_session_dataset(datname);
+#if SAVE_DEBUG
+	fprintf(stderr, " real_save_session_dataset: err = %d\n", err);
+#endif
     }
 
     if (!err) {
-	session_switch_log_location(LOG_SAVE);
+	session_switch_log_location(log_code);
     }
     
     if (!err) {
@@ -1902,22 +1936,30 @@ int save_session (char *fname)
 
 	/* make zipfile containing session files */
 	err = (*gretl_make_zipfile)(fname, dirname, &gerr);
-	fprintf(stderr, "gretl_make_zipfile: err = %d\n", err);
+	fprintf(stderr, " gretl_make_zipfile: err = %d\n", err);
 	close_plugin(handle);
+
+	if (gerr != NULL) {
+	    errbox(gerr->message);
+	    g_error_free(gerr);
+	} else {
+	    mkfilelist(FILE_LIST_SESSION, fname);
+	    if (fname != sessionfile) {
+		session_name_from_session_file(session.name, fname);
+		strcpy(sessionfile, fname);
+		data_status |= SESSION_DATA; /* FIXME? */
+		set_sample_label(dataset);
+	    }
+	    mark_session_saved();
+	}
     }
 
-    if (gerr != NULL) {
-	errbox(gerr->message);
-	g_error_free(gerr);
-    } else {
-	mkfilelist(FILE_LIST_SESSION, fname);
-	if (fname != sessionfile) {
-	    session_name_from_session_file(session.name, fname);
-	    strcpy(sessionfile, fname);
-	    data_status |= SESSION_DATA; /* FIXME? */
-	    set_sample_label(dataset);
+    if (dirbak != NULL) {
+	if (err) {
+	    /* restore original name on error */
+	    strcpy(session.dirname, dirbak);
 	}
-	mark_session_saved();
+	free(dirbak);
     }
 
     fprintf(stderr, "save_session; returning %d\n", err);
@@ -2128,19 +2170,29 @@ void session_model_callback (void *ptr, int action)
 
 static void open_matrix (gui_obj *obj)
 {
-    user_matrix *u = (user_matrix *) obj->data;
-    const char *name = user_matrix_get_name(u);
+    user_var *u = (user_var *) obj->data;
+    const char *name = user_var_get_name(u);
 
-    edit_user_matrix_by_name(name);
+    edit_user_matrix_by_name(name, iconview);
 }
 
 static void open_bundle (gui_obj *obj)
 {
-    gretl_bundle *b = (gretl_bundle *) obj->data;
-    const char *name = gretl_bundle_get_name(b);
+    user_var *u = (user_var *) obj->data;
+    const char *name = user_var_get_name(u);
+    gretl_bundle *b = user_var_get_value(u);
     PRN *prn = NULL;
     int done = 0;
 
+    if (maybe_raise_object_window(b)) {
+	return;
+    }
+
+    if (gretl_bundle_get_n_keys(b) == 0) {
+	warnbox(_("Bundle is empty"));
+	return;
+    }
+
     if (bufopen(&prn)) {
 	return;
     }
@@ -2293,10 +2345,9 @@ static int delete_session_object (gui_obj *obj)
 	real_delete_graph_from_session(obj->data);
     } else if (obj->sort == GRETL_OBJ_TEXT) { 
 	real_delete_text_from_session(obj->data);
-    } else if (obj->sort == GRETL_OBJ_MATRIX) { 
-	user_matrix_destroy(obj->data);
-    } else if (obj->sort == GRETL_OBJ_BUNDLE) {
-	gretl_bundle_delete_by_name(obj->name, NULL);
+    } else if (obj->sort == GRETL_OBJ_MATRIX ||
+	       obj->sort == GRETL_OBJ_BUNDLE) {
+	user_var_delete(obj->data);
     }
 
     session_delete_icon(obj);
@@ -2329,11 +2380,17 @@ static void maybe_delete_session_object (gui_obj *obj)
 	    SESSION_MODEL *mod = obj->data;
 
 	    p = mod->ptr;
+	} else if (obj->sort == GRETL_OBJ_BUNDLE) {
+	    p = user_var_get_value((user_var *) obj->data);
 	} else {
 	    p = obj->data;
 	}
 
-	busy = (p != NULL && get_viewer_for_data(p) != NULL);
+	if (obj->sort == GRETL_OBJ_MATRIX) {
+	    busy = (p != NULL && get_window_for_data(p) != NULL);
+	} else {
+	    busy = (p != NULL && get_viewer_for_data(p) != NULL);
+	}
     }
 
     if (busy) {
@@ -2349,6 +2406,7 @@ static void maybe_delete_session_object (gui_obj *obj)
     }
 }
 
+#if 0 /* FIXME? */
 static gui_obj *get_gui_obj_by_name_and_sort (const char *name,
 					      int sort)
 {
@@ -2365,6 +2423,7 @@ static gui_obj *get_gui_obj_by_name_and_sort (const char *name,
 
     return NULL;
 }
+#endif
 
 static gui_obj *get_gui_obj_by_data (void *targ)
 {
@@ -2382,53 +2441,35 @@ static gui_obj *get_gui_obj_by_data (void *targ)
     return NULL;
 }
 
-/* This callback is invoked when a named matrix is deleted via script
-   or console.  Note that we call user_matrix_destroy() below, and NOT
-   user_matrix_destroy_by_name() -- calling the latter would lead to a
-   loop back to here.  
-*/
-
-int session_matrix_destroy_by_name (const char *name)
+static GretlObjType get_obj_type (GretlType type)
 {
-    user_matrix *u = get_user_matrix_by_name(name);
+    if (type == GRETL_TYPE_MATRIX) {
+	return GRETL_OBJ_MATRIX;
+    } else if (type == GRETL_TYPE_BUNDLE) {
+	return GRETL_OBJ_BUNDLE;
+    } else {
+	return GRETL_TYPE_NONE;
+    }
+}
+
+/* called from DELEET case in library.c */
+
+int session_user_var_destroy_by_name (const char *name, 
+				      GretlObjType type)
+{
+    user_var *u = get_user_var_by_name(name);
     int err;
 
     if (u == NULL) {
 	err = E_UNKVAR;
     } else {
-	maybe_close_window_for_data(u, GRETL_OBJ_MATRIX);
+	maybe_close_window_for_user_var(u, type);
 	if (iconlist != NULL) {
 	    gui_obj *obj = get_gui_obj_by_data(u);
 
 	    session_delete_icon(obj);
 	}
-	err = user_matrix_destroy(u);
-    } 
-    
-    return err;
-}
-
-/* This callback is invoked when a named bundle is deleted via script
-   or console. As with session_matrix_destroy_by_name() we invoke
-   user_bundle_destroy() rather than gretl_bundle_delete_by_name()
-   to avoid looping back.
-*/
-
-int session_bundle_destroy_by_name (const char *name)
-{
-    gretl_bundle *b = get_gretl_bundle_by_name(name);
-    int err;
-
-    if (b == NULL) {
-	err = E_UNKVAR;
-    } else {
-	maybe_close_window_for_data(b, GRETL_OBJ_BUNDLE);
-	if (iconlist != NULL) {
-	    gui_obj *obj = get_gui_obj_by_data(b);
-
-	    session_delete_icon(obj);
-	}
-	err = user_bundle_destroy(b);
+	err = user_var_delete(u);
     } 
     
     return err;
@@ -2488,7 +2529,7 @@ static int rename_session_object (gui_obj *obj, const char *newname)
 	    rename_session_graph(sg, newname);
 	}
     } else if (obj->sort == GRETL_OBJ_MATRIX) {
-	user_matrix_set_name(obj->data, newname);
+	user_var_set_name(obj->data, newname);
     } else if (obj->sort == GRETL_OBJ_BUNDLE) {
 	gretl_bundle_set_name(obj->data, newname);
     } else if (obj->sort == GRETL_OBJ_TEXT) {
@@ -2514,6 +2555,108 @@ static int rename_session_object (gui_obj *obj, const char *newname)
     return err;
 }
 
+static int copy_session_object (gui_obj *obj, const char *cpyname)
+{
+    void *oldp = NULL;
+    void *p = NULL;
+    int ptype = 0;
+    int err = 0;
+
+    /* Only graphs and matrices are supported for copying */
+
+    if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
+	oldp = get_session_graph_by_name(cpyname);
+    } else if (obj->sort == GRETL_OBJ_MATRIX) {
+	oldp = get_user_var_by_name(cpyname);
+    } 
+
+    if (oldp != NULL) {
+	errbox(_("'%s': there is already an object of this name"), cpyname);
+    } else if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
+	SESSION_GRAPH *g0 = obj->data;
+
+	errno = 0;
+	if (!session_dir_ok()) {
+	    err = 1;
+	} else {
+	    char fname1[MAXSAVENAME];
+	    char path0[FILENAME_MAX];
+	    char path1[FILENAME_MAX];
+
+	    gretl_chdir(gretl_dotdir());
+	    make_graph_filename(fname1);
+	    session_file_make_path(path0, g0->fname);
+	    session_file_make_path(path1, fname1);
+	    err = copyfile(path0, path1);
+	    if (!err) {
+		p = session_append_graph(cpyname, fname1, g0->type);
+		ptype = g0->type;
+		err = (p == NULL);
+	    }
+	}
+    } else if (obj->sort == GRETL_OBJ_MATRIX) {
+	user_var *u = obj->data;
+
+	err = copy_matrix_as(user_var_get_value(u), cpyname, 0);
+	if (!err) {
+	    p = get_user_var_by_name(cpyname);
+	    ptype = GRETL_OBJ_MATRIX;
+	    err = (p == NULL);
+	}
+    }
+
+    if (!err) {
+	session_add_icon(p, ptype, ICON_ADD_SINGLE);
+    }
+
+    return err;
+}
+
+static void copy_object_callback (GtkWidget *widget, dialog_t *dlg) 
+{
+    gui_obj *obj = (gui_obj *) edit_dialog_get_data(dlg);
+    const gchar *cpyname;
+    int err = 0;
+
+    cpyname = edit_dialog_get_text(dlg);
+
+    if (cpyname != NULL && *cpyname != '\0') {
+	err = copy_session_object(obj, cpyname);
+	if (err) {
+	    errbox(_("Failed to copy object"));
+	} else {
+	    mark_session_changed();
+	} 
+    }
+
+    if (!err) {
+	edit_dialog_close(dlg);
+    }
+}
+
+static void copy_object_dialog (gui_obj *obj) 
+{
+    int maxlen = MAXSAVENAME - 1;
+    gchar *tmp;
+
+    if (obj->sort != GRETL_OBJ_MATRIX &&
+	obj->sort != GRETL_OBJ_GRAPH &&
+	obj->sort != GRETL_OBJ_PLOT) {
+	dummy_call();
+    }
+
+    if (obj->sort == GRETL_OBJ_MATRIX || obj->sort == GRETL_OBJ_BUNDLE) {
+	maxlen = VNAMELEN - 1;
+    }
+
+    tmp = g_strdup_printf(_("Enter new name\n(max. %d characters)"),
+			  maxlen);
+    edit_dialog(0, _("gretl: copy object"), tmp, obj->name, 
+		copy_object_callback, obj, 
+		VARCLICK_NONE, iconview);
+    g_free(tmp);
+}
+
 static void rename_object_callback (GtkWidget *widget, dialog_t *dlg) 
 {
     gui_obj *obj = (gui_obj *) edit_dialog_get_data(dlg);
@@ -2745,38 +2888,39 @@ static void batch_pack_icons (void)
     }
 }
 
-static void add_user_matrix_callback (const char *mname)
+static int gui_user_var_callback (const char *name, GretlType type,
+				  int action)
 {
-    if (iconview != NULL) {
-	user_matrix *u = get_user_matrix_by_name(mname);
+    GretlObjType otype = get_obj_type(type);
+    user_var *u;
+    int err = 0;
 
-	if (u != NULL) {
-	    session_add_icon(u, GRETL_OBJ_MATRIX, ICON_ADD_SINGLE);
+    if (action == UVAR_ADD) {
+	/* variable has been added, GUI sync wanted */
+	if (iconview != NULL) {
+	    u = get_user_var_of_type_by_name(name, type);
+	    if (u != NULL) {
+		session_add_icon(u, otype, ICON_ADD_SINGLE);
+	    }
+	} else if (autoicon_on()) {
+	    view_session();
 	}
-    } else if (autoicon_on()) {
-	view_session();
-    }
-
-    mark_session_changed();
-}
-
-static void add_bundle_callback (gretl_bundle *bundle)
-{
-    if (iconview != NULL) {
-	/* first check for replacement of an existing bundle */
-	const char *name = gretl_bundle_get_name(bundle);
-	gui_obj *obj = get_gui_obj_by_name_and_sort(name, GRETL_OBJ_BUNDLE);
-
-	if (obj != NULL) {
-	    obj->data = bundle;
+	mark_session_changed();
+    } else if (action == UVAR_DELETE) {
+	/* variable not yet deleted (deferred to GUI) */
+	u = get_user_var_of_type_by_name(name, type);
+	if (u == NULL) {
+	    err = E_UNKVAR;
 	} else {
-	    session_add_icon(bundle, GRETL_OBJ_BUNDLE, ICON_ADD_SINGLE);
+	    maybe_close_window_for_user_var(u, otype);
+	    if (iconlist != NULL) {
+		session_delete_icon(get_gui_obj_by_data(u));
+	    }
+	    err = user_var_delete(u);
 	}
-    } else if (autoicon_on()) {
-	view_session();
-    }
-
-    mark_session_changed();
+    }	
+    
+    return err;
 }
 
 int have_session_objects (void)
@@ -2792,16 +2936,22 @@ int have_session_objects (void)
     }
 
     if (n == 0) {
-	n = n_user_matrices() + n_user_bundles() + n_saved_scalars();
+	n = n_user_matrices() + n_user_bundles() + n_user_scalars();
     }
 
     return n > 0;
 }
 
+static void add_user_var_icon (gpointer data, gpointer intp)
+{
+    session_add_icon(data, GPOINTER_TO_INT(intp), ICON_ADD_BATCH);
+}
+
 static void add_all_icons (void) 
 {
+    GList *list = NULL;
     int show_graph_page = check_for_prog(latex);
-    int i, n;
+    int i;
 
     active_object = NULL;
 
@@ -2845,17 +2995,13 @@ static void add_all_icons (void)
 	session_add_icon(session.texts[i], GRETL_OBJ_TEXT, ICON_ADD_BATCH);
     }
 
-    n = n_user_matrices();
-    for (i=0; i<n; i++) {
-	session_add_icon(get_user_matrix_by_index(i), GRETL_OBJ_MATRIX, 
-			 ICON_ADD_BATCH);
-    }
+    list = user_var_list_for_type(GRETL_TYPE_MATRIX);
+    g_list_foreach(list, add_user_var_icon, GINT_TO_POINTER(GRETL_OBJ_MATRIX));
+    g_list_free(list);
 
-    n = n_user_bundles();
-    for (i=0; i<n; i++) {
-	session_add_icon(get_gretl_bundle_by_index(i), GRETL_OBJ_BUNDLE, 
-			 ICON_ADD_BATCH);
-    }    
+    list = user_var_list_for_type(GRETL_TYPE_BUNDLE);
+    g_list_foreach(list, add_user_var_icon, GINT_TO_POINTER(GRETL_OBJ_BUNDLE));
+    g_list_free(list);
 
     batch_pack_icons();
 }
@@ -2948,31 +3094,11 @@ static void graph_page_save_wrapper (void)
     }
 }
 
-static gboolean session_view_click (GtkWidget *widget, 
+static gboolean session_icon_click (GtkWidget *icon, 
 				    GdkEventButton *event,
 				    gpointer data)
 {
-    gui_obj *obj;
-    GdkModifierType mods;
-
-    if (event == NULL || (in_icon && data == NULL)) {
-	return FALSE;
-    }
-
-    mods = widget_get_pointer_mask(widget);
-
-    if (!in_icon) {
-	if (RIGHT_CLICK(mods)) {
-	    /* right-click on window background */
-	    gtk_menu_popup(GTK_MENU(global_popup), NULL, NULL, NULL, NULL,
-			   event->button, event->time);
-	    return TRUE;
-	} else {
-	    return FALSE;
-	}
-    }
-
-    obj = (gui_obj *) data;
+    gui_obj *obj = (gui_obj *) data;
 
     if (event->type == GDK_2BUTTON_PRESS) {
 	switch (obj->sort) {
@@ -3020,19 +3146,39 @@ static gboolean session_view_click (GtkWidget *widget,
 	    break;
 	}
 	return TRUE;
+    } else {
+	GdkModifierType mods = widget_get_pointer_mask(icon);
+
+	if (RIGHT_CLICK(mods)) {
+	    if (obj->sort == GRETL_OBJ_EQN  || obj->sort == GRETL_OBJ_GRAPH || 
+		obj->sort == GRETL_OBJ_TEXT || obj->sort == GRETL_OBJ_DSET || 
+		obj->sort == GRETL_OBJ_INFO || obj->sort == GRETL_OBJ_GPAGE ||
+		obj->sort == GRETL_OBJ_PLOT || obj->sort == GRETL_OBJ_MODTAB ||
+		obj->sort == GRETL_OBJ_VAR  || obj->sort == GRETL_OBJ_SYS ||
+		obj->sort == GRETL_OBJ_MATRIX || obj->sort == GRETL_OBJ_BUNDLE ||
+		obj->sort == GRETL_OBJ_SCALARS) {
+		object_popup_show(obj, (GdkEventButton *) event);
+	    }
+	    return TRUE;
+	}
     }
 
-    if (RIGHT_CLICK(mods)) {
-	if (obj->sort == GRETL_OBJ_EQN  || obj->sort == GRETL_OBJ_GRAPH || 
-	    obj->sort == GRETL_OBJ_TEXT || obj->sort == GRETL_OBJ_DSET || 
-	    obj->sort == GRETL_OBJ_INFO || obj->sort == GRETL_OBJ_GPAGE ||
-	    obj->sort == GRETL_OBJ_PLOT || obj->sort == GRETL_OBJ_MODTAB ||
-	    obj->sort == GRETL_OBJ_VAR  || obj->sort == GRETL_OBJ_SYS ||
-	    obj->sort == GRETL_OBJ_MATRIX || obj->sort == GRETL_OBJ_BUNDLE ||
-	    obj->sort == GRETL_OBJ_SCALARS) {
-	    object_popup_show(obj, (GdkEventButton *) event);
+    return FALSE;
+}
+
+static gboolean session_view_click (GtkWidget *widget, 
+				    GdkEventButton *event,
+				    gpointer data)
+{
+    if (!in_icon) {
+	GdkModifierType mods = widget_get_pointer_mask(widget);
+
+	if (RIGHT_CLICK(mods)) {
+	    /* right-click on iconview background */
+	    gtk_menu_popup(GTK_MENU(global_popup), NULL, NULL, NULL, NULL,
+			   event->button, event->time);
+	    return TRUE;
 	}
-	return TRUE;
     }
 
     return FALSE;
@@ -3044,7 +3190,7 @@ static void global_popup_callback (GtkWidget *widget, gpointer data)
 
     if (!strcmp(item, _("Save session"))) {
 	if (sessionfile[0]) {
-	    if (session.status == SESSION_CHANGED) {
+	    if (session.status & SESSION_CHANGED) {
 		save_session(NULL);
 	    } else {
 		mark_session_saved();
@@ -3070,30 +3216,36 @@ static void matrix_popup_callback (GtkWidget *widget, gpointer data)
 {
     gchar *item = (gchar *) data;
     gui_obj *obj = active_object;
-    user_matrix *u = (user_matrix *) obj->data;
-    const char *name = user_matrix_get_name(u);
+    user_var *u = (user_var *) obj->data;
+    const char *name = user_var_get_name(u);
     gretl_matrix *m;
 
     if (!strcmp(item, _("View"))) {
 	PRN *prn;
 
-	m = user_matrix_get_matrix(u);
+	m = user_var_get_value(u);
 	if (m != NULL && bufopen(&prn) == 0) {
 	    gretl_matrix_print_to_prn(m, name, prn);
 	    view_buffer(prn, 78, 400, name, PRINT, NULL);
 	} 
     } else if (!strcmp(item, _("Edit"))) {
-	edit_user_matrix_by_name(name);
+	edit_user_matrix_by_name(name, iconview);
     } else if (!strcmp(item, _("Properties"))) {
-	m = user_matrix_get_matrix(u);
+	m = user_var_get_value(u);
 	view_matrix_properties(m, name);
     } else if (!strcmp(item, _("Copy as CSV..."))) {
-	m = user_matrix_get_matrix(u);
-	matrix_to_clipboard_as_csv(m, iconview);
+	m = user_var_get_value(u);
+	if (gretl_is_null_matrix(m)) {
+	    warnbox("matrix is null");
+	} else {
+	    matrix_to_clipboard_as_csv(m, iconview);
+	}
     } else if (!strcmp(item, _("Rename"))) {
 	rename_object_dialog(obj);
     } else if (!strcmp(item, _("Delete"))) {
 	maybe_delete_session_object(obj);
+    } else if (!strcmp(item, _("Copy"))) {
+	copy_object_dialog(obj);
     }
 }
 
@@ -3217,6 +3369,10 @@ static void object_popup_callback (GtkWidget *widget, gpointer data)
 	if (obj->sort == GRETL_OBJ_GPAGE) {
 	    graph_page_save_wrapper();
 	}
+    } else if (!strcmp(item, _("Copy"))) {
+	if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
+	    copy_object_dialog(obj);
+	}
     }
 }
 
@@ -3380,10 +3536,8 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
 	name = g_strdup(_("Graph page"));
 	break;
     case GRETL_OBJ_MATRIX:
-	name = g_strdup(user_matrix_get_name((user_matrix *) data));
-	break;
     case GRETL_OBJ_BUNDLE:
-	name = g_strdup(gretl_bundle_get_name((gretl_bundle *) data));
+	name = g_strdup(user_var_get_name((user_var *) data));
 	break;
     default:
 	break;
@@ -3454,6 +3608,9 @@ static GtkWidget *create_pop_item (GtkWidget *popup, char *str,
 
     if (!strcmp(str, _("Save session"))) {
 	save_item = item;
+	if (session.status == 0) {
+	    gtk_widget_set_sensitive(item, FALSE);
+	}
     }
 
     return item;
@@ -3593,7 +3750,7 @@ static void iconview_connect_signals (GtkWidget *iconview)
 
 void view_session (void)
 {
-    GtkWidget *hbox, *scroller;
+    GtkWidget *ebox, *scroller;
     gchar *title;
 
     if (iconview != NULL) {
@@ -3614,31 +3771,24 @@ void view_session (void)
 
     session_build_popups();
 
-    hbox = gtk_hbox_new(FALSE,0);
-    gtk_container_add(GTK_CONTAINER(iconview), hbox);
-    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
+    ebox = gtk_event_box_new();
+    gtk_container_set_border_width(GTK_CONTAINER(ebox), 5);
+    gtk_container_add(GTK_CONTAINER(iconview), ebox);
+    g_signal_connect(G_OBJECT(ebox), "button-press-event",
+		     G_CALLBACK(session_view_click), NULL);
 
     scroller = gtk_scrolled_window_new(NULL, NULL);
     gtk_container_set_border_width(GTK_CONTAINER(scroller), 0);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    g_signal_connect(G_OBJECT(scroller), "button-press-event",
-		     G_CALLBACK(session_view_click), NULL);
-
-    gtk_box_pack_start(GTK_BOX(hbox), scroller, TRUE, TRUE, 0); 
+    gtk_container_add(GTK_CONTAINER(ebox), scroller);
 
     icon_table = gtk_table_new(2, ICONVIEW_MIN_COLS, FALSE);
-
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroller), 
 					  icon_table);
 
     add_all_icons();
-
-    gtk_widget_show(icon_table);
-    gtk_widget_show(scroller);
-    gtk_widget_show(hbox);
-    gtk_widget_show(iconview);
-
+    gtk_widget_show_all(iconview);
     window_list_add(iconview, OPEN_SESSION);
 
     gtk_container_foreach(GTK_CONTAINER(scroller), 
@@ -3687,7 +3837,7 @@ static void create_gobj_icon (gui_obj *obj, const char **xpm)
     g_object_ref(obj->label);
 
     g_signal_connect(G_OBJECT(obj->icon), "button-press-event",
-		     G_CALLBACK(session_view_click), obj);
+		     G_CALLBACK(session_icon_click), obj);
     g_signal_connect(G_OBJECT(obj->icon), "enter-notify-event",
 		     G_CALLBACK(icon_entered), obj);
     g_signal_connect(G_OBJECT(obj->icon), "leave-notify-event",
@@ -3813,7 +3963,10 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
     pprintf(prn, _("Properties of matrix %s"), (name != NULL)? name : "");
     pputs(prn, "\n\n");
 
-    if (m->rows == 1 && m->cols == 1) {
+    if (m->rows == 0 || m->cols == 0) {
+	pprintf(prn, _("Null matrix, %d x %d\n"), m->rows, m->cols);
+	goto done;
+    } else if (m->rows == 1 && m->cols == 1) {
 	pprintf(prn, _("Scalar matrix, value %g\n"), m->val[0]);
 	goto done;
     } else if (gretl_is_identity_matrix(m)) {
@@ -3922,5 +4075,3 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
     view_buffer(prn, 78, 400, title, PRINT, NULL);
     g_free(title);
 }
-
-
