@@ -489,51 +489,6 @@ read_session_xml (const char *fname, struct sample_info *sinfo)
     return err;
 }
 
-static int maybe_read_matrix_file (const char *fname) 
-{
-    FILE *fp;
-
-    fp = gretl_fopen(fname, "r");
-    if (fp == NULL) {
-	/* nothing to be read */
-	return 0;
-    }
-
-    fclose(fp);
-
-    return load_user_matrix_file(fname);
-}
-
-static int maybe_read_bundles_file (const char *fname) 
-{
-    FILE *fp;
-
-    fp = gretl_fopen(fname, "r");
-    if (fp == NULL) {
-	/* nothing to be read */
-	return 0;
-    }
-
-    fclose(fp);
-
-    return load_user_bundle_file(fname);
-}
-
-static int maybe_read_scalars_file (const char *fname) 
-{
-    FILE *fp;
-
-    fp = gretl_fopen(fname, "r");
-    if (fp == NULL) {
-	/* nothing to be read */
-	return 0;
-    }
-
-    fclose(fp);
-
-    return load_user_scalars_file(fname);
-}
-
 static int maybe_read_functions_file (const char *fname) 
 {
     FILE *fp;
@@ -549,21 +504,6 @@ static int maybe_read_functions_file (const char *fname)
     return read_session_functions_file(fname);
 }
 
-static int maybe_read_lists_file (const char *fname) 
-{
-    FILE *fp;
-
-    fp = gretl_fopen(fname, "r");
-    if (fp == NULL) {
-	/* nothing to be read */
-	return 0;
-    }
-
-    fclose(fp);
-
-    return load_user_lists_file(fname);
-}
-
 static int maybe_read_settings_file (const char *fname) 
 {
     FILE *fp;
@@ -616,78 +556,12 @@ static SavedObjectFlags model_save_flags (const void *ptr,
     return flags;
 }
 
-static int maybe_write_matrix_file (char *fullname)
-{
-    FILE *fp;
-
-    if (n_user_matrices() == 0) {
-	return 0;
-    }
-
-    session_file_make_path(fullname, "matrices.xml");
-    fp = gretl_fopen(fullname, "w");
-    if (fp == NULL) {
-	return E_FOPEN;
-    }
-
-    write_matrices_to_file(fp);
-    fclose(fp);
-
-    return 0;
-}
-
-static int maybe_write_bundle_file (char *fullname)
-{
-    FILE *fp;
-
-    if (n_user_bundles() == 0) {
-	return 0;
-    }
-
-    session_file_make_path(fullname, "bundles.xml");
-    fp = gretl_fopen(fullname, "w");
-    if (fp == NULL) {
-	return E_FOPEN;
-    }
-
-    write_bundles_to_file(fp);
-    fclose(fp);
-
-    return 0;
-}
-
-static int maybe_write_scalars_file (char *fullname)
-{
-    FILE *fp;
-
-    if (n_saved_scalars() == 0) {
-	return 0;
-    }
-
-    session_file_make_path(fullname, "scalars.xml");
-    fp = gretl_fopen(fullname, "w");
-    if (fp == NULL) {
-	return E_FOPEN;
-    }
-
-    write_scalars_to_file(fp);
-    fclose(fp);
-
-    return 0;
-}
-
 static int maybe_write_function_file (char *fullname)
 {
     session_file_make_path(fullname, "functions.xml");
     return write_session_functions_file(fullname);
 }
 
-static int maybe_write_lists_file (char *fullname)
-{
-    session_file_make_path(fullname, "lists.xml");
-    return gretl_serialize_lists(fullname);
-}
-
 static int write_settings_file (char *fullname)
 {
     session_file_make_path(fullname, "settings.inp");
@@ -722,12 +596,13 @@ static int write_session_xml (const char *datname)
     int i, modnum;
     int err = 0;
 
-    gretl_chdir(gretl_dotdir());
+    /* we should be in dotdir already when this is called */
 
     sprintf(fname, "%s%csession.xml", session.dirname, SLASH);
     fp = gretl_fopen(fname, "w");
 
     if (fp == NULL) {
+	fprintf(stderr, " write_session_xml: failed on '%s'\n", fname);
 	file_write_errbox(fname);
 	return E_FOPEN;
     }
@@ -903,11 +778,9 @@ static int write_session_xml (const char *datname)
 
     fclose(fp);
 
-    maybe_write_matrix_file(tmpname);
-    maybe_write_bundle_file(tmpname);
-    maybe_write_scalars_file(tmpname);
+    serialize_user_vars(session.dirname);
+
     maybe_write_function_file(tmpname);
-    maybe_write_lists_file(tmpname);
     write_settings_file(tmpname);
 
     return 0;
