@@ -31,6 +31,7 @@
 
 #include "libset.h"
 #include "texprint.h"
+#include "uservar.h"
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -57,7 +58,7 @@
 #endif
 
 static char rcfile[FILENAME_MAX];
-static char dbproxy[21];
+static char http_proxy[128];
 static int use_proxy;
 
 static ConfigPaths paths;
@@ -142,14 +143,14 @@ typedef struct {
    type BOOLSET and not part of a radio group, then a non-zero value
    for len will link the var's toggle button with the sensitivity of
    the preceding rc_var's entry field.  For example, the "use_proxy"
-   button controls the sensitivity of the "dbproxy" entry widget.
+   button controls the sensitivity of the "http_proxy" entry widget.
 */
 
 RCVAR rc_vars[] = {
     { "gretldir", N_("Main gretl directory"), NULL, paths.gretldir, 
-      MACHSET | BROWSER, MAXLEN, TAB_MAIN, NULL },
+      MACHSET | BROWSER, sizeof paths.gretldir, TAB_MAIN, NULL },
     { "userdir", N_("User's gretl directory"), NULL, paths.workdir, 
-      INVISET, MAXLEN, TAB_MAIN, NULL },
+      INVISET, sizeof paths.workdir, TAB_MAIN, NULL },
     { "updater", N_("Tell me about gretl updates"), NULL, &updater, 
       BOOLSET, 0, TAB_MAIN, NULL },
 #ifndef G_OS_WIN32
@@ -216,38 +217,36 @@ RCVAR rc_vars[] = {
       USERSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #ifdef HAVE_X12A
     { "x12a", N_("Path to x12arima"), NULL, paths.x12a, 
-      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
+      MACHSET | BROWSER, sizeof paths.x12a, TAB_PROGS, NULL },
 #endif
 #ifdef HAVE_TRAMO
     { "tramo", N_("Path to tramo"), NULL, paths.tramo, 
-      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
+      MACHSET | BROWSER, sizeof paths.tramo, TAB_PROGS, NULL},
 #endif
 #ifdef USE_RLIB
     { "Rlib", N_("Path to R library"), NULL, paths.rlibpath, 
-      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
+      MACHSET | BROWSER, sizeof paths.rlibpath, TAB_PROGS, NULL},
 #endif
     { "ox", N_("Path to oxl executable"), NULL, paths.oxlpath, 
-      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
+      MACHSET | BROWSER, sizeof paths.oxlpath, TAB_PROGS, NULL},
     { "octave", N_("Path to octave executable"), NULL, paths.octpath, 
-      MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL},
-    { "ratsbase", N_("RATS data directory"), NULL, paths.ratsbase, 
-      USERSET | BROWSER, MAXLEN, TAB_DBS, NULL },
+      MACHSET | BROWSER, sizeof paths.octpath, TAB_PROGS, NULL},
     { "dbhost", N_("Database server name"), NULL, paths.dbhost, 
-      USERSET, 32, TAB_DBS, NULL },
-    { "dbproxy", N_("HTTP proxy (ipnumber:port)"), NULL, dbproxy, 
-      USERSET, 21, TAB_DBS, NULL },
+      USERSET, sizeof paths.dbhost, TAB_NET, NULL },
+    { "dbproxy", N_("HTTP proxy"), NULL, http_proxy, 
+      USERSET, sizeof http_proxy, TAB_NET, NULL },
     { "useproxy", N_("Use HTTP proxy"), NULL, &use_proxy, 
-      BOOLSET, 1, TAB_DBS, NULL },
+      BOOLSET, 1, TAB_NET, NULL },
     { "Fixed_font", N_("Fixed font"), NULL, fixedfontname, 
-      USERSET, MAXLEN, TAB_NONE, NULL },
+      USERSET, sizeof fixedfontname, TAB_NONE, NULL },
     { "App_font", N_("Menu font"), NULL, appfontname, 
-      USERSET, MAXLEN, TAB_NONE, NULL },
+      USERSET, sizeof appfontname, TAB_NONE, NULL },
     { "DataPage", "Default data page", NULL, datapage, 
       INVISET, sizeof datapage, TAB_NONE, NULL },
     { "ScriptPage", "Default script page", NULL, scriptpage, 
       INVISET, sizeof scriptpage, TAB_NONE, NULL },    
     { "Png_font", N_("PNG graph font"), NULL, paths.pngfont, 
-      INVISET, 32, TAB_NONE, NULL },
+      INVISET, sizeof paths.pngfont, TAB_NONE, NULL },
     { "Gp_colors", N_("Gnuplot colors"), NULL, gpcolors, 
       INVISET, sizeof gpcolors, TAB_NONE, NULL },
     { "tabwidth", "spaces per tab", NULL, &tabwidth, 
@@ -313,6 +312,8 @@ int autoicon_on (void)
 {
     if (dataset != NULL && dataset->v > 0) {
 	return autoicon;
+    } else if (n_user_matrices() > 0) {
+	return autoicon;
     } else {
 	return 0;
     }
@@ -498,6 +499,7 @@ void set_gretl_startdir (void)
 static void get_pkg_save_dir (char *dirname, int action)
 {
     const char *subdir = NULL;
+    int try_sysdir = 1;
     int ok = 0;
 
     if (action == SAVE_FUNCTIONS) {
@@ -508,10 +510,18 @@ static void get_pkg_save_dir (char *dirname, int action)
 	return;
     }
 
-    /* try 'system' location first */
+#ifdef G_OS_WIN32
+    if (windows_uses_virtual_store()) {
+	/* don't write to virtualized location */
+	try_sysdir = 0;
+    }
+#endif
 
-    sprintf(dirname, "%s%s", gretl_home(), subdir);
-    ok = write_OK(dirname);
+    if (try_sysdir) {
+	/* try 'system' location first */
+	sprintf(dirname, "%s%s", gretl_home(), subdir);
+	ok = write_OK(dirname);
+    }
 
     if (!ok) {
 	/* try user's filespace */
@@ -537,9 +547,8 @@ void get_default_dir (char *s, int action)
     if (action == SAVE_FUNCTIONS || action == SAVE_DATA_PKG) {
 	get_pkg_save_dir(s, action);
     } else if (action == SAVE_REMOTE_DB) {
+	/* FIXME Windows Vista and higher? */
 	sprintf(s, "%sdb", gretl_home());
-    } else if (action == OPEN_RATS_DB) {
-	strcpy(s, gretl_ratsbase());
     } else {
 	strcpy(s, gretl_workdir());
     }
@@ -899,7 +908,7 @@ int options_dialog (int page, const char *varname, GtkWidget *parent)
     gtk_widget_show(notebook);
 
     make_prefs_tab(notebook, TAB_MAIN);
-    make_prefs_tab(notebook, TAB_DBS);
+    make_prefs_tab(notebook, TAB_NET);
     make_prefs_tab(notebook, TAB_PROGS);
     make_prefs_tab(notebook, TAB_VCV);
     make_prefs_tab(notebook, TAB_MAN);
@@ -1168,8 +1177,8 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 
     if (tab == TAB_MAIN) {
 	w = gtk_label_new(_("General"));
-    } else if (tab == TAB_DBS) {
-	w = gtk_label_new(_("Databases"));
+    } else if (tab == TAB_NET) {
+	w = gtk_label_new(_("Network"));
     } else if (tab == TAB_PROGS) {
 	w = gtk_label_new(_("Programs"));
     } else if (tab == TAB_VCV) {
@@ -1602,7 +1611,7 @@ static void apply_changes (GtkWidget *widget, gpointer data)
     maybe_revise_tramo_x12a_status();
 #endif
 
-    if (use_proxy && *dbproxy == '\0') {
+    if (use_proxy && *http_proxy == '\0') {
 	/* fix inconsistency */
 	use_proxy = 0;
     }
@@ -1620,7 +1629,7 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
     selector_register_hc_choice();
 
-    gretl_www_init(paths.dbhost, dbproxy, use_proxy);
+    gretl_www_init(paths.dbhost, http_proxy, use_proxy);
 }
 
 static void boolvar_to_str (void *b, char *s)
@@ -1783,7 +1792,7 @@ static int common_read_rc_setup (void)
 	set_gretl_alarm(0);
     }
 
-    gretl_www_init(paths.dbhost, dbproxy, use_proxy);
+    gretl_www_init(paths.dbhost, http_proxy, use_proxy);
     set_tex_use_pdf(latex);
 
 #if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
@@ -1869,7 +1878,7 @@ static int get_network_settings (void)
 
 	    if (sscanf(p, "%31s", key) == 1) {
 		strcpy(linevar, p + strlen(key) + 3); 
-		chopstr(linevar); 
+		gretl_chopstr(linevar); 
 		gotvar = 0;
 		for (j=0; rc_vars[j].key != NULL; j++) {
 		    if (!strcmp(key, rc_vars[j].key)) {
@@ -1938,8 +1947,8 @@ static void win32_read_gretlrc (void)
 	    /* note: don't take gretldir from here */
 	    if (strcmp(key, "gretldir")) {
 		strcpy(linevar, line + strlen(key) + 3); 
-		chopstr(linevar); 
-		if (*linevar) {
+		gretl_chopstr(linevar); 
+		if (*linevar != '\0') {
 		    find_and_set_rc_var(key, linevar);
 		}
 	    }
@@ -2066,7 +2075,7 @@ static int read_gretlrc (void)
 	    if (sscanf(line, "%31s", key) == 1) {
 		*linevar = '\0';
 		strncat(linevar, line + strlen(key) + 3, MAXLEN-1); 
-		chopstr(linevar); 
+		gretl_chopstr(linevar); 
 		if (*linevar != '\0') {
 		    find_and_set_rc_var(key, linevar);
 		}
@@ -2186,15 +2195,15 @@ static void font_selection_ok (GtkWidget *w, GtkFontChooser *fc)
 
 void font_selector (GtkAction *action)
 {
-    static GtkWidget *fontsel = NULL;
+    static GtkWidget *fc = NULL;
     GtkFontFilterFunc filter;
     GtkWidget *button;
     int which = fontsel_code(action);
     char *title = NULL;
     const char *fontname = NULL;
  
-    if (fontsel != NULL) {
-	gtk_window_present(GTK_WINDOW(fontsel));
+    if (fc != NULL) {
+	gtk_window_present(GTK_WINDOW(fc));
         return;
     }
 
@@ -2210,34 +2219,38 @@ void font_selector (GtkAction *action)
 
     gretl_font_filter_init();
 
-    fontsel = gtk_font_chooser_dialog_new(title, GTK_WINDOW(mdata->main));
-    gtk_font_chooser_set_font(GTK_FONT_CHOOSER(fontsel), 
+    fc = gtk_font_chooser_dialog_new(title, GTK_WINDOW(mdata->main));
+    gtk_font_chooser_set_font(GTK_FONT_CHOOSER(fc), 
 			      fontname); 
-    gtk_font_chooser_set_filter_func(GTK_FONT_CHOOSER(fontsel),
+    gtk_font_chooser_set_filter_func(GTK_FONT_CHOOSER(fc),
 				     filter, NULL, NULL);
-    gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
+    gtk_window_set_position(GTK_WINDOW(fc), GTK_WIN_POS_MOUSE);
 
-    g_signal_connect(G_OBJECT(fontsel), "destroy",
+    if (which == FIXED_FONT_SELECTION) {
+	widget_set_int(fc, "mono", 1);
+    }
+
+    g_signal_connect(G_OBJECT(fc), "destroy",
 		     G_CALLBACK(gtk_widget_destroyed),
-		     &fontsel);
+		     &fc);
 
-    button = gtk_dialog_get_widget_for_response(GTK_DIALOG(fontsel),
+    button = gtk_dialog_get_widget_for_response(GTK_DIALOG(fc),
 						GTK_RESPONSE_OK);
     if (button != NULL) {
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(font_selection_ok),
-			 fontsel);
+			 fc);
     }
 
-    button = gtk_dialog_get_widget_for_response(GTK_DIALOG(fontsel),
+    button = gtk_dialog_get_widget_for_response(GTK_DIALOG(fc),
 						GTK_RESPONSE_CANCEL);
     if (button != NULL) {
 	g_signal_connect(G_OBJECT(button), "clicked",
 			 G_CALLBACK(close_font_chooser),
-			 fontsel);
+			 fc);
     }
 
-    gtk_widget_show(fontsel);
+    gtk_widget_show(fc);
 }
 
 #else /* GTK, not using GtkFontChooser */
