@@ -35,7 +35,7 @@
 #include "tabwin.h"
 #include "toolbar.h"
 
-#include "usermat.h"
+#include "uservar.h"
 
 #ifdef G_OS_WIN32
 # include "gretlwin32.h"
@@ -71,6 +71,10 @@
 #include "../pixmaps/mini.page.xpm"
 #include "../pixmaps/mini.tools.xpm"
 
+/* for plot bar */
+#include "../pixmaps/upsize.xpm"
+#include "../pixmaps/downsize.xpm"
+
 enum {
     SAVE_ITEM = 1,
     SAVE_AS_ITEM,
@@ -135,7 +139,9 @@ void gretl_stock_icons_init (void)
 	{ mini_gretl_xpm, GRETL_STOCK_GRETL},
 	{ mini_table_xpm, GRETL_STOCK_TABLE},
 	{ mini_page_xpm, GRETL_STOCK_PAGE},
-	{ mini_tools_xpm, GRETL_STOCK_TOOLS}
+	{ mini_tools_xpm, GRETL_STOCK_TOOLS},
+	{ upsize_xpm, GRETL_STOCK_BIGGER},
+	{ downsize_xpm, GRETL_STOCK_SMALLER}
     };
     int n = G_N_ELEMENTS(stocks);
 
@@ -348,7 +354,9 @@ static void add_matrix_callback (GtkWidget *w, windata_t *vwin)
 	    if (m == NULL) {
 		nomem();
 	    } else {
-		err = add_or_replace_user_matrix(m, mname);
+		err = user_var_add_or_replace(mname,
+					      GRETL_TYPE_MATRIX,
+					      m);
 		if (err) {
 		    gretl_matrix_free(m);
 		    gui_errmsg(err);
