@@ -26,6 +26,8 @@
 #include "winstack.h"
 #include "varinfo.h"
 
+#include "uservar.h"
+
 #define VSET_MAX_FIELDS 6
 
 /* symbolic names for the things which can be set here */
@@ -141,8 +143,8 @@ static void varinfo_set_field_changed (gui_varinfo *vset, int i,
 
 static int formula_ok (int v)
 {
-    if (var_is_generated(dataset, v)) {
-	const char *s = VARLABEL(dataset, v);
+    if (series_is_generated(dataset, v)) {
+	const char *s = series_get_label(dataset, v);
 	int n = strlen(s);
 
 	/* note: the test for a trailing dot here is to check
@@ -193,7 +195,7 @@ static void show_varinfo_changes (int v)
     if (iptr != NULL) {
         gtk_tree_store_set(GTK_TREE_STORE(model), iptr, 
                            1, dataset->varname[v],
-                           2, VARLABEL(dataset, v),
+                           2, series_get_label(dataset, v),
                            -1);
     }
 }
@@ -263,7 +265,7 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
 	if (vset->use_formula) {
 	    err = try_regenerate_var(v, newstr);
 	} else {
-	    var_set_description(dataset, v, newstr);
+	    series_record_label(dataset, v, newstr);
 	}
 	g_free(newstr);
     }
@@ -275,24 +277,29 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
 		      "contain double quotes"));
 	    err = E_DATA;
 	} else {
-	    var_set_display_name(dataset, v, newstr);
+	    series_record_display_name(dataset, v, newstr);
 	}
 	g_free(newstr);
     }
 
     if (!err && vset->changed[VSET_COMPACT]) {
+	int orig = series_get_compact_method(dataset, v);
+
 	ival = gtk_combo_box_get_active(GTK_COMBO_BOX(vset->compaction_menu));
-	var_set_compact_method(dataset, v, ival);
+	if (ival != orig) {
+	    series_set_compact_method(dataset, v, ival);
+	    set_dataset_is_changed();
+	}
     }
 
     if (!err && vset->changed[VSET_LINEWIDTH]) {
 	ival = spinner_get_int(vset->line_spin);
-	var_set_linewidth(dataset, v, ival);
+	series_set_linewidth(dataset, v, ival);
     }
 
     if (!err && vset->changed[VSET_DISCRETE]) {
 	ival = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vset->discrete_check));
-	set_var_discrete(dataset, v, ival);
+	series_set_discrete(dataset, v, ival);
 	if (ival) {
 	    lib_command_sprintf("setinfo %s --discrete", dataset->varname[v]);
 	} else {
@@ -489,22 +496,22 @@ static void varinfo_insert_info (gui_varinfo *vset, int v)
     }
 
     gtk_entry_set_text(GTK_ENTRY(vset->label_entry), 
-		       VARLABEL(dataset, v));
+		       series_get_label(dataset, v));
 
     if (vset->display_entry != NULL) {
 	gtk_entry_set_text(GTK_ENTRY(vset->display_entry), 
-			   DISPLAYNAME(dataset, v));
+			   series_get_display_name(dataset, v));
     }
 
     if (vset->line_spin != NULL) { 
 	gtk_combo_box_set_active(GTK_COMBO_BOX(vset->compaction_menu), 
-				 COMPACT_METHOD(dataset, v));
+				 series_get_compact_method(dataset, v));
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(vset->line_spin), 
-				  var_get_linewidth(dataset, v));
+				  series_get_linewidth(dataset, v));
     } 
 
     if (vset->discrete_check != NULL) {
-	int d2 = 0, d1 = var_is_discrete(dataset, v);
+	int d2 = 0, d1 = series_is_discrete(dataset, v);
 
 	if (!d1) {
 	    d2 = gretl_isdiscrete(0, dataset->n - 1, dataset->Z[v]);
@@ -577,7 +584,7 @@ static void varinfo_apply (GtkButton *b, gui_varinfo *vset)
 static void varinfo_discrete_changed (GtkToggleButton *button, gui_varinfo *vset)
 {
     int d = gtk_toggle_button_get_active(button);
-    int orig = var_is_discrete(dataset, vset->varnum);
+    int orig = series_is_discrete(dataset, vset->varnum);
 
     varinfo_set_field_changed(vset, VSET_DISCRETE, d != orig);
 }
@@ -585,7 +592,7 @@ static void varinfo_discrete_changed (GtkToggleButton *button, gui_varinfo *vset
 static void varinfo_linewidth_changed (GtkSpinButton *spin, gui_varinfo *vset)
 {
     int lw = gtk_spin_button_get_value_as_int(spin);
-    int orig = var_get_linewidth(dataset, vset->varnum);
+    int orig = series_get_linewidth(dataset, vset->varnum);
 
     varinfo_set_field_changed(vset, VSET_LINEWIDTH, lw != orig);
 }
@@ -593,7 +600,7 @@ static void varinfo_linewidth_changed (GtkSpinButton *spin, gui_varinfo *vset)
 static void varinfo_compact_changed (GtkComboBox *box, gui_varinfo *vset)
 {
     int method = gtk_combo_box_get_active(box);
-    int orig = COMPACT_METHOD(dataset, vset->varnum);
+    int orig = series_get_compact_method(dataset, vset->varnum);
 
     varinfo_set_field_changed(vset, VSET_COMPACT, method != orig);
 }
@@ -607,10 +614,10 @@ static void varinfo_text_changed (GtkEditable *e, gui_varinfo *vset)
     gboolean s;
 
     if (w == vset->label_entry) {
-	orig = VARLABEL(dataset, vset->varnum);
+	orig = series_get_label(dataset, vset->varnum);
 	f = VSET_LABEL;
     } else if (w == vset->display_entry) {
-	orig = DISPLAYNAME(dataset, vset->varnum);
+	orig = series_get_display_name(dataset, vset->varnum);
 	f = VSET_DISPLAY;
     }
 
@@ -828,7 +835,7 @@ void varinfo_dialog (int varnum)
     vset->label_entry = gtk_entry_new();
     gtk_entry_set_max_length(GTK_ENTRY(vset->label_entry), MAXLABEL-1);
     gtk_entry_set_text(GTK_ENTRY(vset->label_entry), 
-		       VARLABEL(dataset, varnum));
+		       series_get_label(dataset, varnum));
     g_signal_connect(G_OBJECT(vset->label_entry), "changed", 
 		     G_CALLBACK(varinfo_text_changed), vset);
     gtk_box_pack_start(GTK_BOX(hbox), vset->label_entry, TRUE, TRUE, 5);
@@ -853,7 +860,7 @@ void varinfo_dialog (int varnum)
     gtk_entry_set_width_chars(GTK_ENTRY(vset->display_entry), 
 			      MAXDISP+4);
     gtk_entry_set_text(GTK_ENTRY(vset->display_entry), 
-		       DISPLAYNAME(dataset, varnum));
+		       series_get_display_name(dataset, varnum));
     g_signal_connect(G_OBJECT(vset->display_entry), "changed", 
 		     G_CALLBACK(varinfo_text_changed), vset);
     gtk_box_pack_start(GTK_BOX(hbox), 
@@ -881,7 +888,7 @@ void varinfo_dialog (int varnum)
 	}
 	
 	gtk_combo_box_set_active(GTK_COMBO_BOX(vset->compaction_menu), 
-				 COMPACT_METHOD(dataset, varnum));
+				 series_get_compact_method(dataset, varnum));
 	g_signal_connect(G_OBJECT(vset->compaction_menu), "changed",
 			 G_CALLBACK(varinfo_compact_changed), vset);
 	gtk_box_pack_start(GTK_BOX(hbox), vset->compaction_menu, FALSE, FALSE, 5);
@@ -901,7 +908,7 @@ void varinfo_dialog (int varnum)
 	gtk_widget_show(tmp);
 
 	tmp = gtk_spin_button_new_with_range(1, 8, 1);
-	w = var_get_linewidth(dataset, varnum);
+	w = series_get_linewidth(dataset, varnum);
 	if (w > 1) {
 	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(tmp), w);
 	}
@@ -917,7 +924,7 @@ void varinfo_dialog (int varnum)
 
     if (1) {
 	/* mark variable as discrete or not */
-	int d = var_is_discrete(dataset, varnum);
+	int d = series_is_discrete(dataset, varnum);
 
 	hbox = gtk_hbox_new(FALSE, 5);
 	tmp = gtk_check_button_new_with_label(_("Treat this variable "
