@@ -166,7 +166,7 @@ adf_prepare_vars (int order, int varno, int nseas, int *d0,
 	int test = (opt & OPT_T)? UR_TREND : UR_CONST;
 	int t, v = dset->v;
 
-	err = dataset_add_series(1, dset);
+	err = dataset_add_series(dset, 1);
 	if (!err) {
 	    int T, offset = 0;
 
@@ -459,7 +459,8 @@ print_adf_results (int order, int pmax, double DFt, double pv,
 }
 
 static int auto_adjust_order (int *list, int order_max,
-			      DATASET *dset, PRN *prn)
+			      DATASET *dset, int *err,
+			      PRN *prn)
 {
     MODEL kmod;
     double tstat, pval;
@@ -467,9 +468,13 @@ static int auto_adjust_order (int *list, int order_max,
 
     for (k=order_max; k>0; k--) {
 	kmod = lsq(list, dset, OLS, OPT_A);
+	if (!kmod.errcode && kmod.dfd == 0) {
+	    kmod.errcode = E_DF;
+	}
 	if (kmod.errcode) {
 	    fprintf(stderr, "auto_adjust_order: k = %d, err = %d\n", k,
 		    kmod.errcode);
+	    *err = kmod.errcode;
 	    clear_model(&kmod);
 	    k = -1;
 	    break;
@@ -626,7 +631,7 @@ static int gettrend (DATASET *dset, int square)
 	return idx;
     }
 
-    if (dataset_add_series(1, dset)) {
+    if (dataset_add_series(dset, 1)) {
 	return 0; /* error: valid value cannot == 0 */
     }
 
@@ -637,10 +642,10 @@ static int gettrend (DATASET *dset, int square)
 
     if (square) {
 	strcpy(dset->varname[v], "timesq");
-	strcpy(VARLABEL(dset, v), _("squared time trend variable"));
+	series_set_label(dset, v, _("squared time trend variable"));
     } else {
 	strcpy(dset->varname[v], "time");
-	strcpy(VARLABEL(dset, v), _("time trend variable"));
+	series_set_label(dset, v, _("time trend variable"));
     }
 	    
     return idx;
@@ -816,9 +821,8 @@ static int real_adf_test (int varno, int order, int niv,
     skipdet:
 
 	if (auto_order) {
-	    order = auto_adjust_order(list, order_max, dset, prn);
-	    if (order < 0) {
-		err = 1;
+	    order = auto_adjust_order(list, order_max, dset, &err, prn);
+	    if (err) {
 		clear_model(&dfmod);
 		goto bailout;
 	    }
@@ -829,6 +833,10 @@ static int real_adf_test (int varno, int order, int niv,
 #endif
 
 	dfmod = lsq(list, dset, OLS, df_mod_opt);
+	if (!dfmod.errcode && dfmod.dfd == 0) {
+	    /* we can't have an exact fit here */
+	    dfmod.errcode = E_DF;
+	}
 	if (dfmod.errcode) {
 	    fprintf(stderr, "adf_test: dfmod.errcode = %d\n", 
 		    dfmod.errcode);
@@ -904,7 +912,7 @@ static int real_adf_test (int varno, int order, int niv,
 	free(biglist);
     }
 
-    dataset_drop_last_variables(dset->v - orig_nvars, dset);
+    dataset_drop_last_variables(dset, dset->v - orig_nvars);
 
     return err;
 }
@@ -1751,7 +1759,7 @@ int kpss_test (int order, const int *list, DATASET *dset,
     dset->t2 = save_t2;
 
     /* added 2012-03-22 for consistency with adf test */
-    dataset_drop_last_variables(dset->v - orig_nvars, dset);
+    dataset_drop_last_variables(dset, dset->v - orig_nvars);
 
     return err;
 }
@@ -1990,7 +1998,7 @@ int engle_granger_test (int order, const int *list, DATASET *dset,
     printmodel(&cmod, dset, OPT_NONE, prn);
 
     /* add residuals from cointegrating regression to data set */
-    err = dataset_add_allocated_series(cmod.uhat, dset);
+    err = dataset_add_allocated_series(dset, cmod.uhat);
     if (err) {
 	goto bailout;
     }
