@@ -427,7 +427,7 @@ static PLOTGROUP *plotgroup_new (const int *list,
 	    grp->nplots = list[0];
 	}
 	if ((opt & OPT_P) && dset != NULL) {
-	    strcpy(grp->ylabel, VARLABEL(dset, 1));
+	    strcpy(grp->ylabel, series_get_label(dset, 1));
 	    strcpy(grp->xlabel, _("group"));
 	}
     }
@@ -726,7 +726,7 @@ static int factorized_boxplot_check (const int *list,
     } else {
 	int v2 = list[2];
 	
-	if (!var_is_discrete(dset, v2) &&
+	if (!series_is_discrete(dset, v2) &&
 	    !gretl_isdiscrete(dset->t1, dset->t2, dset->Z[v2])) {
 	    gretl_errmsg_set(_("You must supply two variables, the second of "
 			       "which is discrete"));
@@ -737,6 +737,19 @@ static int factorized_boxplot_check (const int *list,
     return err;
 }
 
+static int all_missing (int t1, int t2, const double *x)
+{
+    int t;
+
+    for (t=t1; t<=t2; t++) {
+	if (!na(x[t])) {
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
 /* build the boxplot group data structure, then pass it to 
    gnuplot_do_boxplot to write the command file */
 
@@ -763,6 +776,8 @@ static int real_boxplots (const int *list, char **bools,
 	if (gretl_isconst(dset->t1, dset->t2, dset->Z[v])) {
 	    gretl_errmsg_sprintf(_("%s is a constant"), dset->varname[v]);
 	    return E_DATA;
+	} else if (all_missing(dset->t1, dset->t2, dset->Z[v])) {
+	    return E_MISSDATA;
 	}
     }
 
@@ -880,7 +895,7 @@ static int panel_group_boxplots (int vnum, const DATASET *dset,
     nv = nunits + 1;
     u0 = dset->t1 / T;
 
-    gdset = create_auxiliary_dataset(nv, T);
+    gdset = create_auxiliary_dataset(nv, T, 0);
     if (gdset == NULL) {
 	return E_ALLOC;
     }
@@ -894,7 +909,7 @@ static int panel_group_boxplots (int vnum, const DATASET *dset,
     s0 = dset->t1 * dset->pd;
 
     /* record the name of the original variable */
-    strcpy(VARLABEL(gdset, 1), dset->varname[vnum]);
+    series_set_label(gdset, 1, dset->varname[vnum]);
 
     for (i=0; i<nunits; i++) {
 	sprintf(gdset->varname[i+1], "%d", u0+i+1);
@@ -1114,7 +1129,7 @@ int boolean_boxplots (const char *line, DATASET *dset, gretlopt opt)
     free(s);
 
     if (nbool) {
-	dataset_drop_last_variables(nbool, dset);
+	dataset_drop_last_variables(dset, nbool);
     }
     
     return err;
@@ -1253,7 +1268,7 @@ int boxplot_numerical_summary (const char *fname, PRN *prn)
     char yvar[VNAMELEN];
     char xvar[VNAMELEN];
     char vname[VNAMELEN];
-    char line[512];
+    char fmt[16], line[512];
     FILE *fp;
     int n = 0;
     int err = 0;
@@ -1264,6 +1279,7 @@ int boxplot_numerical_summary (const char *fname, PRN *prn)
     }
 
     *yvar = *xvar = *vname = '\0';
+    sprintf(fmt, "%%%d[^\\\"]", VNAMELEN-1);
 
     /* first pass: count the plots, using labels as heuristic */
 
@@ -1272,11 +1288,11 @@ int boxplot_numerical_summary (const char *fname, PRN *prn)
 	    /* reached first data block */
 	    break;
 	} else if (!strncmp(line, "set ylabel ", 11)) {
-	    sscanf(line + 12, "%15[^\"]", yvar);
+	    sscanf(line + 12, fmt, yvar);
 	} else if (!strncmp(line, "set xlabel ", 11)) {
-	    sscanf(line + 12, "%15[^\"]", xvar);
+	    sscanf(line + 12, fmt, xvar);
 	} else if (!strncmp(line, "set title ", 10)) {
-	    sscanf(line + 11, "%15[^\"]", vname);
+	    sscanf(line + 11, fmt, vname);
 	} else if (!strncmp(line, "set label ", 10)) {
 	    if (!strncmp(line + 10, "\"(", 2)) {
 		; /* boolean specifier */
