@@ -891,7 +891,7 @@ static int add_residual_to_dataset (MODEL *pmod, DATASET *dset)
 {
     int err = 0;
 
-    if (dataset_add_series(1, dset)) {
+    if (dataset_add_series(dset, 1)) {
 	err = E_ALLOC;
     } else {
 	int t, v = dset->v - 1;
@@ -901,7 +901,7 @@ static int add_residual_to_dataset (MODEL *pmod, DATASET *dset)
 	}
 
 	strcpy(dset->varname[v], "uhat");
-	strcpy(VARLABEL(dset, v), _("residual"));
+	series_set_label(dset, v, _("residual"));
     }
 
     return err;
@@ -1050,7 +1050,7 @@ int nonlinearity_test (MODEL *pmod, DATASET *dset, ModelAuxCode aux,
     }
 	
     /* trash any extra variables generated (squares, logs) */
-    dataset_drop_last_variables(dset->v - orig_nvar, dset);
+    dataset_drop_last_variables(dset, dset->v - orig_nvar);
 
     /* put back into dset what was there on input */
     dset->t1 = save_t1;
@@ -1223,7 +1223,7 @@ int add_test_full (MODEL *orig, MODEL *pmod, const int *addvars,
     } 
 
     /* put dset back as it was on input */
-    dataset_drop_last_variables(dset->v - orig_nvar, dset);
+    dataset_drop_last_variables(dset, dset->v - orig_nvar);
     dset->t1 = save_t1;
     dset->t2 = save_t2;
 
@@ -1779,7 +1779,7 @@ int reset_test (MODEL *pmod, DATASET *dset,
 	for (i=1; i<=pmod->list[0]; i++) {
 	    newlist[i] = pmod->list[i];
 	}
-	if (dataset_add_series(addv, dset)) {
+	if (dataset_add_series(dset, addv)) {
 	    err = E_ALLOC;
 	}
     }
@@ -1867,7 +1867,7 @@ int reset_test (MODEL *pmod, DATASET *dset,
     }
 
     free(newlist);
-    dataset_drop_last_variables(addv, dset); 
+    dataset_drop_last_variables(dset, addv); 
     clear_model(&aux); 
 
     dset->t1 = save_t1;
@@ -2080,7 +2080,7 @@ static int ivreg_autocorr_test (MODEL *pmod, int order,
     free(addlist);
     free(testlist);
 
-    dataset_drop_last_variables(dset->v - v, dset); 
+    dataset_drop_last_variables(dset, dset->v - v); 
     clear_model(&aux); 
 
     /* reset sample as it was */
@@ -2090,6 +2090,59 @@ static int ivreg_autocorr_test (MODEL *pmod, int order,
     return err;
 }
 
+static int lb_autocorr_test (MODEL *pmod, int order, 
+			     gretlopt opt, PRN *prn)
+{
+    double lb, pval;
+    int npq, df = order;
+    int err = 0;
+
+    npq = arma_model_get_n_arma_coeffs(pmod);
+    if (npq < df) {
+	df -= npq; /* FIXME? */
+    }
+
+    lb = ljung_box(order, pmod->t1, pmod->t2, pmod->uhat, &err);
+
+    if (err) {
+	lb = pval = NADBL;
+    } else {
+	pval = chisq_cdf_comp(order, lb);
+	if (na(pval)) {
+	    err = E_DATA;
+	} 
+    }
+
+    if (err) {
+	gretl_errmsg_set(_("Error calculating Ljung-Box statistic"));
+    } else {
+	pputc(prn, '\n');
+	pprintf(prn, _("Test for autocorrelation up to order %d"), 
+		order);
+	pputs(prn, "\n\n");
+	pprintf(prn, "Ljung-Box Q' = %g,\n", lb);
+	pprintf(prn, "%s = P(%s(%d) > %g) = %#.4g\n", _("with p-value"), 
+		_("Chi-square"), df, lb, chisq_cdf_comp(df, lb));
+	pputc(prn, '\n');
+	record_test_result(lb, pval, _("autocorrelation"));
+    }
+
+    if (!err && (opt & OPT_S)) {
+	ModelTest *test = model_test_new(GRETL_TEST_AUTOCORR);
+
+	if (test != NULL) {
+	    model_test_set_teststat(test, GRETL_STAT_LB_CHISQ);
+	    model_test_set_dfn(test, df);
+	    model_test_set_order(test, order);
+	    model_test_set_value(test, lb);
+	    model_test_set_pvalue(test, pval);
+	    maybe_add_test_to_model(pmod, test);
+	}	    
+    }
+
+    return err;
+}
+
 /**
  * autocorr_test:
  * @pmod: pointer to model to be tested.
@@ -2122,6 +2175,10 @@ int autocorr_test (MODEL *pmod, int order, DATASET *dset,
 	return ivreg_autocorr_test(pmod, order, dset, opt, prn);
     }
 
+    if (pmod->ci == ARMA) {
+	return lb_autocorr_test(pmod, order, opt, prn);
+    }
+
     if (pmod->ci != OLS && pmod->ci != VAR) { 
 	return E_NOTIMP;
     }
@@ -2157,7 +2214,7 @@ int autocorr_test (MODEL *pmod, int order, DATASET *dset,
 	for (i=2; i<=pmod->list[0]; i++) {
 	    newlist[i] = pmod->list[i];
 	}
-	if (dataset_add_series(1 + order, dset)) {
+	if (dataset_add_series(dset, 1 + order)) {
 	    err = E_ALLOC;
 	}
     }
@@ -2173,7 +2230,7 @@ int autocorr_test (MODEL *pmod, int order, DATASET *dset,
 	    }
 	}
 	strcpy(dset->varname[v], "uhat");
-	strcpy(VARLABEL(dset, v), _("residual"));
+	series_set_label(dset, v, _("residual"));
 	/* then order lags of same */
 	for (i=1; i<=order; i++) {
 	    int s, lv = v + i;
@@ -2263,7 +2320,7 @@ int autocorr_test (MODEL *pmod, int order, DATASET *dset,
     }
 
     free(newlist);
-    dataset_drop_last_variables(dset->v - v, dset); 
+    dataset_drop_last_variables(dset, dset->v - v); 
     clear_model(&aux); 
 
     /* reset sample as it was */
@@ -2303,7 +2360,7 @@ make_chow_list (const MODEL *pmod, DATASET *dset,
 	return NULL;
     }
 
-    if (dataset_add_series(newvars, dset)) {
+    if (dataset_add_series(dset, newvars)) {
 	*err = E_ALLOC;
     } else {
 	chowlist = gretl_list_new(pmod->list[0] + ninter + 1);
@@ -2328,7 +2385,7 @@ make_chow_list (const MODEL *pmod, DATASET *dset,
 		dset->Z[v][t] = (double) (t >= split); 
 	    }
 	    strcpy(dset->varname[v], "splitdum");
-	    strcpy(VARLABEL(dset, v), _("dummy variable for Chow test"));
+	    series_set_label(dset, v, _("dummy variable for Chow test"));
 	}
 
 	chowlist[l0 + 1] = (dumv > 0)? dumv : v;
@@ -2721,7 +2778,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
  bailout:
 
     /* clean up extra variables */
-    dataset_drop_last_variables(dset->v - origv, dset);
+    dataset_drop_last_variables(dset, dset->v - origv);
     free(chowlist);
 
     dset->t1 = save_t1;
@@ -3196,14 +3253,14 @@ int comfac_test (MODEL *pmod, DATASET *dset,
 
     nadd = 1 + pmod->ncoeff - pmod->ifc;
 
-    err = dataset_add_series(nadd, dset);
+    err = dataset_add_series(dset, nadd);
     if (err) {
 	free(biglist);
 	return err;
     }
 
     /* add lags of the dependent variable and all regressors: some of
-       these may be redundant but we'll just let the redundat terms be
+       these may be redundant but we'll just let the redundant terms be
        eliminated automatically via gretl's collinearity checking.
     */
 
@@ -3227,8 +3284,9 @@ int comfac_test (MODEL *pmod, DATASET *dset,
 	    err = E_ALLOC;
 	    break;
 	}
-	lag = is_standard_lag(src, dset, &parent);
-	if (lag && parent) {
+	lag = series_get_lag(dset, src);
+	parent = series_get_parent_id(dset, src);
+	if (lag > 0 && parent > 0) {
 	    char tmp[8];
 
 	    sprintf(tmp, "_%d", lag + 1);
@@ -3308,7 +3366,7 @@ int comfac_test (MODEL *pmod, DATASET *dset,
     /* delete the added variables and restore the original
        sample range */
 
-    dataset_drop_last_variables(nadd, dset);   
+    dataset_drop_last_variables(dset, nadd);   
     free(biglist);
 
     dset->t1 = save_t1;
@@ -3381,7 +3439,7 @@ int add_leverage_values_to_dataset (DATASET *dset, gretl_matrix *m,
 	return 0;
     }
 
-    if (dataset_add_series(addvars, dset)) {
+    if (dataset_add_series(dset, addvars)) {
 	return E_ALLOC;
     }
 
@@ -3402,7 +3460,7 @@ int add_leverage_values_to_dataset (DATASET *dset, gretl_matrix *m,
 	}
 	strcpy(dset->varname[v], "lever");
 	make_varname_unique(dset->varname[v], v, dset);
-	strcpy(VARLABEL(dset, v), "leverage values");
+	series_set_label(dset, v, "leverage values");
     }
 
     /* add influence? */
@@ -3419,7 +3477,7 @@ int add_leverage_values_to_dataset (DATASET *dset, gretl_matrix *m,
 	}	
 	strcpy(dset->varname[v], "influ");
 	make_varname_unique(dset->varname[v], v, dset);
-	strcpy(VARLABEL(dset, v), "influence values");
+	series_set_label(dset, v, "influence values");
     }
 
     /* add DFFITS? */
@@ -3446,7 +3504,7 @@ int add_leverage_values_to_dataset (DATASET *dset, gretl_matrix *m,
 	}	
 	strcpy(dset->varname[v], "dffits");
 	make_varname_unique(dset->varname[v], v, dset);
-	strcpy(VARLABEL(dset, v), "DFFITS values");
+	series_set_label(dset, v, "DFFITS values");
     }
 
     return 0;
