@@ -18,7 +18,7 @@
  */
 
 #include "libgretl.h"
-#include "gretl_string_table.h"
+#include "uservar.h"
 #include "dbwrite.h"
 #include "libset.h"
 #include "gretl_xml.h"
@@ -346,7 +346,8 @@ static int read_esl_hdr (const char *hdrfile, DATASET *dset, int *byvar)
     *str = '\0';
     fscanf(fp, "%s", str);
     if (skip_esl_comments(fp, str)) {
-        safecpy(dset->varname[i], str, VNAMELEN - 1);
+	*dset->varname[i] = '\0';
+	strncat(dset->varname[i], str, VNAMELEN - 1);
 	err = check_varname(dset->varname[i++]);
     } else {
 	descrip = 1; /* comments were found */
@@ -356,12 +357,13 @@ static int read_esl_hdr (const char *hdrfile, DATASET *dset, int *byvar)
         fscanf(fp, "%s", str);
 	n = strlen(str);
 	if (str[n-1] != ';') {
-            safecpy(dset->varname[i], str, VNAMELEN - 1);
+	    *dset->varname[i] = '\0';
+            strncat(dset->varname[i], str, VNAMELEN - 1);
 	    err = check_varname(dset->varname[i++]);
         } else {
 	    if (n > 1) {
-		safecpy(dset->varname[i], str, n-1);
-		dset->varname[i][n] = '\0';
+		*dset->varname[i] = '\0';
+		strncat(dset->varname[i], str, n-1);
 		err = check_varname(dset->varname[i]);
 	    }
 	    break;
@@ -416,7 +418,7 @@ static int read_esl_hdr (const char *hdrfile, DATASET *dset, int *byvar)
 	rewind(fp);
 	dbuf = get_esl_comment_lines(fp);
 	if (dbuf != NULL) {
-	    delchar('\r', dbuf);
+	    gretl_delchar('\r', dbuf);
 	    dset->descrip = gretl_strdup(dbuf);
 	    free(dbuf);
 	}
@@ -727,7 +729,7 @@ char *ntodate (char *datestr, int t, const DATASET *dset)
     } else if (dataset_is_daily(dset) || 
 	       dataset_is_weekly(dset)) {
 	/* undated time series */
-	x = date(t, 1, dset->sd0);
+	x = date_as_double(t, 1, dset->sd0);
 	sprintf(datestr, "%d", (int) x);
 	return datestr;
     } else if (dataset_is_decennial(dset)) {
@@ -739,7 +741,7 @@ char *ntodate (char *datestr, int t, const DATASET *dset)
 	return datestr;
     }
 
-    x = date(t, dset->pd, dset->sd0);
+    x = date_as_double(t, dset->pd, dset->sd0);
 
     if (dset->pd == 1) {
         sprintf(datestr, "%d", (int) x);
@@ -799,7 +801,7 @@ int get_subperiod (int t, const DATASET *dset, int *err)
 	ret = t % dset->pd;
     } else {
 	/* quarterly, monthly, hourly... */
-	double x = date(t, dset->pd, dset->sd0);
+	double x = date_as_double(t, dset->pd, dset->sd0);
 	int i, d = ceil(log10(dset->pd));
 
 	x -= floor(x);
@@ -873,7 +875,7 @@ int get_info (const char *hdrfile, PRN *prn)
 	do {
 	    if (fgets(s, MAXLEN-1, hdr) != NULL && strncmp(s, "*)", 2)) {
 #ifndef WIN32
-		delchar('\r', s);
+		gretl_delchar('\r', s);
 #endif
 		pputs(prn, s);
 		i++;
@@ -902,7 +904,7 @@ static int write_esl_lbl (const char *lblfile, const int *list,
 
     for (i=1; i<=list[0]; i++) {
 	vi = list[i];
-	if (vi != 0 && strlen(VARLABEL(dset, vi)) > 2) {
+	if (vi != 0 && strlen(series_get_label(dset, vi)) > 2) {
 	    labels = 1;
 	    break;
 	}
@@ -917,9 +919,9 @@ static int write_esl_lbl (const char *lblfile, const int *list,
 	    /* spit out varnames and labels */
 	    for (i=1; i<=list[0]; i++) {
 		vi = list[i];
-		if (vi != 0 && strlen(VARLABEL(dset, vi)) > 2) {
+		if (vi != 0 && strlen(series_get_label(dset, vi)) > 2) {
 		    fprintf(fp, "%s %s\n", dset->varname[vi],
-			    VARLABEL(dset, vi));
+			    series_get_label(dset, vi));
 		}
 	    }
 	    fclose(fp);
@@ -1209,7 +1211,7 @@ static void csv_data_out (const DATASET *dset, const int *list,
 		    sprintf(tmp, "%.*f", pmax[i-1], xt);
 		}
 		if (dotsub) {
-		    charsub(tmp, '.', ',');
+		    gretl_charsub(tmp, '.', ',');
 		}
 		fputs(tmp, fp);
 	    }
@@ -1511,7 +1513,7 @@ int write_data (const char *fname, int *list, const DATASET *dset,
 	fputs("/*\n", fp);
 	for (i=1; i<=list[0]; i++) {
 	    v = list[i];
-	    fprintf(fp, " %s: %s\n", dset->varname[v], VARLABEL(dset, v));
+	    fprintf(fp, " %s: %s\n", dset->varname[v], series_get_label(dset, v));
 	}
 	fputs("*/\n", fp);
 	date_maj_min(dset->t1, dset, &maj, &min);
@@ -1569,11 +1571,11 @@ static int no_case_series_index (const DATASET *dset,
 
     *s1 = '\0';
     strncat(s1, vname, VNAMELEN - 1);
-    lower(s1);
+    gretl_lower(s1);
 
     for (i=1; i<dset->v; i++) {
 	strcpy(s2, dset->varname[i]);
-	lower(s2);
+	gretl_lower(s2);
 	if (strcmp(s1, s2) == 0) {
 	    return i;
 	}
@@ -1612,8 +1614,7 @@ static void read_esl_labels (const char *lblfile, DATASET *dset)
 	if (v > 0 && v < dset->v) {
 	    p = line + strlen(varname);
 	    p += strspn(p, " \t");
-	    VARLABEL(dset, v)[0] = '\0';
-	    strncat(VARLABEL(dset, v), p, MAXLABEL - 1);
+	    series_set_label(dset, v, p);
 	} else {
 	    fprintf(stderr, I_("extraneous label for var '%s'\n"), varname);
 	}
@@ -1622,8 +1623,6 @@ static void read_esl_labels (const char *lblfile, DATASET *dset)
     fclose(fp);
 }
 
-
-
 /**
  * is_gzipped:
  * @fname: filename to examine.
@@ -1680,7 +1679,7 @@ static int import_esl (const char *datafile, char *auxfile,
     pprintf(prn, I_("\nReading header file %s\n"), auxfile);
 
     /* allocate data storage */
-    err = allocate_Z(tmpset);
+    err = allocate_Z(tmpset, 0);
 
     if (!err && tmpset->markers) {
 	/* deal with case where first column of the data file contains
@@ -1852,21 +1851,23 @@ int open_nulldata (DATASET *dset, int data_status, int length,
     }
 
     /* allocate dataset */
-    if (allocate_Z(dset)) {
+    if (allocate_Z(dset, 0)) {
 	return E_ALLOC;
     }
 
     /* add an index var */
     strcpy(dset->varname[1], "index");
-    strcpy(VARLABEL(dset, 1), _("index variable"));
+    series_set_label(dset, 1, _("index variable"));
     for (t=0; t<dset->n; t++) {
 	dset->Z[1][t] = (double) (t + 1);
     }
 
-    /* print out basic info */
-    pprintf(prn, A_("periodicity: %d, maxobs: %d\n"
-	   "observations range: %s-%s\n"), dset->pd, dset->n,
-	   dset->stobs, dset->endobs);
+    if (gretl_messages_on()) {
+	/* print out basic info */
+	pprintf(prn, A_("periodicity: %d, maxobs: %d\n"
+			"observations range: %s-%s\n"), 
+		dset->pd, dset->n, dset->stobs, dset->endobs);
+    }
 
     /* Set sample range to entire length of data-set by default */
     dset->t1 = 0; 
@@ -2042,7 +2043,7 @@ static int panel_append_special (int addvars,
     int i, j, s, p, t;
     int err = 0;
 
-    if (addvars > 0 && dataset_add_series(addvars, dset)) {
+    if (addvars > 0 && dataset_add_series(dset, addvars)) {
 	merge_error(_("Out of memory!\n"), prn);
 	err = E_ALLOC;
     }
@@ -2261,7 +2262,7 @@ static int merge_data (DATASET *dset, DATASET *addset,
 	int k = dset->v;
 	int i, t;
 
-	if (addvars > 0 && dataset_add_series(addvars, dset)) {
+	if (addvars > 0 && dataset_add_series(dset, addvars)) {
 	    merge_error(_("Out of memory!\n"), prn);
 	    err = E_ALLOC;
 	}
@@ -2418,10 +2419,10 @@ int add_obs_markers_from_file (DATASET *dset, const char *fname)
     }
 
     if (err) {
-	free_strings_array(S, dset->n);
+	strings_array_free(S, dset->n);
     } else {
 	if (dset->S != NULL) {
-	    free_strings_array(dset->S, dset->n);
+	    strings_array_free(dset->S, dset->n);
 	} 
 	dset->markers = REGULAR_MARKERS;
 	dset->S = S;
@@ -2445,13 +2446,13 @@ int dataset_has_var_labels (const DATASET *dset)
 
     if (dset->v > 1) {
 	if (!strcmp(dset->varname[1], "index") &&
-	    !strcmp(VARLABEL(dset, 1), _("index variable"))) {
+	    !strcmp(series_get_label(dset, 1), _("index variable"))) {
 	    imin = 2;
 	}
     }
 
     for (i=imin; i<dset->v; i++) {
-	label = VARLABEL(dset, i);
+	label = series_get_label(dset, i);
 	if (*label != '\0') {
 	    return 1;
 	}
@@ -2482,7 +2483,7 @@ int save_var_labels_to_file (const DATASET *dset, const char *fname)
 	err = E_FOPEN;
     } else {
 	for (i=1; i<dset->v; i++) {
-	    fprintf(fp, "%s\n", VARLABEL(dset, i));
+	    fprintf(fp, "%s\n", series_get_label(dset, i));
 	}
 	fclose(fp);
     }
@@ -2508,7 +2509,6 @@ int add_var_labels_from_file (DATASET *dset, const char *fname)
 {
     FILE *fp;
     char line[256], label[MAXLABEL];
-    char *targ;
     int nlabels = 0;
     int i, err = 0;
 
@@ -2523,14 +2523,10 @@ int add_var_labels_from_file (DATASET *dset, const char *fname)
 	} else if (sscanf(line, "%127[^\n\r]", label) != 1) {
 	    continue;
 	} else {
-	    targ = VARLABEL(dset, i);
 	    g_strstrip(label);
-	    *targ = '\0';
-	    strncat(targ, label, MAXLABEL - 1);
-	    err = check_imported_string(targ, i+1, MAXLABEL);
-	    if (err) {
-		*targ = '\0';
-	    } else {
+	    err = check_imported_string(label, i+1, MAXLABEL);
+	    if (!err) {
+		series_set_label(dset, i, label);
 		nlabels++;
 	    }
 	}
@@ -2546,20 +2542,31 @@ int add_var_labels_from_file (DATASET *dset, const char *fname)
 
 int read_or_write_var_labels (gretlopt opt, DATASET *dset, PRN *prn)
 {
-    const char *fname;
+    const char *fname = NULL;
     int err;
 
-    err = incompatible_options(opt, OPT_T | OPT_F); 
+    err = incompatible_options(opt, OPT_D | OPT_T | OPT_F); 
     if (err) {
 	return err;
     }
 
-    fname = get_optval_string(LABELS, opt);
-    if (fname == NULL) {
-	return E_BADOPT;
+    if (opt & (OPT_T | OPT_F)) {
+	fname = get_optval_string(LABELS, opt);
+	if (fname == NULL) {
+	    return E_BADOPT;
+	} else {
+	    fname = gretl_maybe_switch_dir(fname);
+	}
     }
 
-    if (opt & OPT_T) {
+    if (opt & OPT_D) {
+	/* delete */
+	int i;
+
+	for (i=1; i<dset->v; i++) {
+	    series_set_label(dset, i, "");
+	}	
+    } else if (opt & OPT_T) {
 	/* to-file */
 	if (!dataset_has_var_labels(dset)) {
 	    pprintf(prn, "No labels are available for writing\n");
@@ -2581,6 +2588,69 @@ int read_or_write_var_labels (gretlopt opt, DATASET *dset, PRN *prn)
     return err;
 }
 
+static int save_obs_markers_to_file (DATASET *dset, const char *fname)
+{
+    FILE *fp = gretl_fopen(fname, "w");
+    int err = 0;
+
+    if (fp == NULL) {
+	err = E_FOPEN;
+    } else {
+	int i;
+
+	for (i=0; i<dset->n; i++) {
+	    fprintf(fp, "%s\n", dset->S[i]);
+	}
+	fclose(fp);
+    }
+
+    return err;
+}
+
+int read_or_write_obs_markers (gretlopt opt, DATASET *dset, PRN *prn)
+{
+    const char *fname = NULL;
+    int err;
+
+    err = incompatible_options(opt, OPT_D | OPT_T | OPT_F); 
+    if (err) {
+	return err;
+    }
+
+    if (opt & (OPT_T | OPT_F)) {
+	fname = get_optval_string(MARKERS, opt);
+	if (fname == NULL) {
+	    return E_BADOPT;
+	} else {
+	    fname = gretl_maybe_switch_dir(fname);
+	}
+    }
+
+    if (opt & OPT_D) {
+	/* delete */
+	dataset_destroy_obs_markers(dset);
+    } else if (opt & OPT_T) {
+	/* to-file */
+	if (dset->S == NULL) {
+	    gretl_errmsg_set(_("No markers are available for writing"));
+	    err = E_DATA;
+	} else {
+	    err = save_obs_markers_to_file(dset, fname);
+	    if (!err && gretl_messages_on() && !gretl_looping_quietly()) {
+		pprintf(prn, "Markers written OK\n");
+	    }
+	}
+    } else if (opt & OPT_F) {
+	/* from-file */
+	err = add_obs_markers_from_file(dset, fname);
+	if (!err && gretl_messages_on() && !gretl_looping_quietly()) {
+	    pprintf(prn, "Markers loaded OK\n");
+	}	
+    }
+
+    return err;
+}
+
 static void 
 octave_varname (char *name, const char *s, int nnum, int v)
 {
@@ -2657,7 +2727,7 @@ static int import_octave (const char *fname, DATASET *dset,
     DATASET *octset = NULL;
     FILE *fp = NULL;
     char *line = NULL;
-    char tmp[8], name[32];
+    char tmp[8], fmt[16], name[32];
     int nrows = 0, ncols = 0, nblocks = 0;
     int brows = 0, bcols = 0, oldbcols = 0;
     int maxlen, got_type = 0, got_name = 0;
@@ -2767,12 +2837,14 @@ static int import_octave (const char *fname, DATASET *dset,
     i = 1;
     t = 0;
 
+    sprintf(fmt, "# name: %%%ds", VNAMELEN - 1);
+
     while (fgets(line, maxlen, fp) && !err) {
 	char *s = line;
 	int j;
 
 	if (*s == '#') {
-	    if (sscanf(line, "# name: %15s", name) == 1) {
+	    if (sscanf(line, fmt, name) == 1) {
 		;
 	    } else if (sscanf(line, "# rows: %d", &brows) == 1) {
 		t = 0;
@@ -3268,7 +3340,7 @@ int transpose_data (DATASET *dset)
 
 	    *targ = '\0';
 	    strncat(targ, dset->S[t], VNAMELEN - 1);
-	    charsub(targ, ' ', '_');
+	    gretl_charsub(targ, ' ', '_');
 	    err = check_varname(targ);
 	    if (err) {
 		sprintf(targ, "v%d", k);
