@@ -62,6 +62,34 @@ typedef enum {
     DS_GRAB_VALUES
 } DataCopyFlag;
 
+/**
+ * CompactMethod:
+ * @COMPACT_NONE: no data compaction
+ * @COMPACT_SUM: take sum of higher frequency data
+ * @COMPACT_AVG: take mean of higher frequency data
+ * @COMPACT_SOP: use start-of-period value
+ * @COMPACT_EOP: use end-of-period value
+ * @COMPACT_WDAY: use a specified day of the week
+ * @COMPACT_MAX: sentinel value
+ *
+ * Symbolic codes for various methods of compacting data
+ * series (i.e. converting from a higher to a lower
+ * frequency). %COMPACT_WDAY is applicable only when
+ * converting from daily to weekly frequency.
+ */
+
+typedef enum {
+    COMPACT_NONE,
+    COMPACT_SUM,
+    COMPACT_AVG,
+    COMPACT_SOP,
+    COMPACT_EOP,
+    COMPACT_WDAY,
+    COMPACT_MAX
+} CompactMethod;
+
+typedef struct _series_table series_table;
+
 /**
  * dataset_is_cross_section:
  * @p: pointer to data information struct.
@@ -213,73 +241,6 @@ typedef enum {
  */
 #define dataset_has_markers(p) (p != NULL && p->markers && p->S != NULL)
 
-/**
- * var_is_discrete:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Determine whether a variable should be treated as discrete
- * or not.
- */
-#define var_is_discrete(p, i) (p->varinfo[i]->flags & VAR_DISCRETE)
-
-/**
- * var_is_hidden:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Determine whether or not a variable is hidden.
- */
-#define var_is_hidden(p, i) (p->varinfo[i]->flags & VAR_HIDDEN)
-
-/**
- * var_is_generated:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Determine whether or not a variable was generated using
- * a formula or transformation function.
- */
-#define var_is_generated(p, i) (p->varinfo[i]->flags & VAR_GENERATED)
-
-/**
- * var_is_listarg:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Determine whether or not a variable has been marked as
- * belonging to a list argument to a function.
- */
-#define var_is_listarg(p, i) (p->varinfo[i]->flags & VAR_LISTARG)
-
-/**
- * set_var_listarg:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Set the "listarg" flag on the given variable.
- */
-#define set_var_listarg(p, i) (p->varinfo[i]->flags |= VAR_LISTARG)
-
-/**
- * unset_var_listarg:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- *
- * Remove the "listarg" flag from the given variable.
- */
-#define unset_var_listarg(p, i) (p->varinfo[i]->flags &= ~VAR_LISTARG)
-
-/**
- * series_set_flag:
- * @p: pointer to data information struct.
- * @i: index number of variable.
- * @f: flag to set.
- *
- * Set the given flag on the given (series) variable.
- */
-#define series_set_flag(p, i, f) (p->varinfo[i]->flags |= f)
-
 /**
  * sample_size:
  * @p: pointer to data information struct.
@@ -320,13 +281,13 @@ DATASET *create_new_dataset (int nvar,     /* number of variables */
 			     int markers   /* case markers or not? */
 			     );
 
-DATASET *create_auxiliary_dataset (int nvar, int nobs);
+DATASET *create_auxiliary_dataset (int nvar, int nobs, gretlopt opt);
 
 void destroy_dataset (DATASET *dset);
 
 void clear_datainfo (DATASET *dset, int code);
 
-int allocate_Z (DATASET *dset);
+int allocate_Z (DATASET *dset, gretlopt opt);
 
 int dataset_allocate_varnames (DATASET *dset);
 
@@ -344,30 +305,27 @@ void set_sorted_markers (DATASET *dset, int v, char **S);
 
 void dataset_set_regular_markers (DATASET *dset);
 
-int start_new_Z (DATASET *dset, int resample);
+int start_new_Z (DATASET *dset, gretlopt opt);
 
 int is_trend_variable (const double *x, int n);
 
 int is_periodic_dummy (const double *x, const DATASET *dset);
 
-int dataset_add_observations (int newobs, DATASET *dset,
-			      gretlopt opt);
+int dataset_add_observations (DATASET *dset, int n, gretlopt opt);
 
-int dataset_drop_observations (int n, DATASET *dset);
+int dataset_drop_observations (DATASET *dset, int n);
 
 int dataset_shrink_obs_range (DATASET *dset);
 
-int dataset_add_series (int newvars, DATASET *dset);
+int dataset_add_series (DATASET *dset, int newvars);
 
 int dataset_add_NA_series (DATASET *dset);
 
-int dataset_add_allocated_series (double *x, DATASET *dset);
+int dataset_add_allocated_series (DATASET *dset, double *x);
 
-int dataset_add_series_as (double *x, const char *newname,
-			   DATASET *dset);
+int dataset_add_series_as (DATASET *dset, double *x, const char *name);
 
-int dataset_copy_variable_as (int v, const char *newname,
-			      DATASET *dset);
+int dataset_copy_series_as (DATASET *dset, int v, const char *name);
 
 int overwrite_err (const char *name);
 
@@ -386,7 +344,7 @@ int dataset_drop_variable (int v, DATASET *dset);
 
 int dataset_destroy_hidden_variables (DATASET *dset, int vmin);
 
-int dataset_drop_last_variables (int delvars, DATASET *dset);
+int dataset_drop_last_variables (DATASET *dset, int delvars);
 
 int dataset_renumber_variable (int v_old, int v_new, 
 			       DATASET *dset);
@@ -399,39 +357,36 @@ int maybe_prune_dataset (DATASET **pdset, void *p);
 int dataset_stack_variables (const char *vname, const char *line,
 			     DATASET *dset, PRN *prn);
 
-int dataset_sort_by (const int *list, DATASET *dset, gretlopt opt);
+int dataset_sort_by (DATASET *dset, const int *list, gretlopt opt);
 
-int is_log_variable (int i, const DATASET *dset, char *parent);
+int dataset_set_matrix_name (DATASET *dset, const char *name);
 
-void set_var_discrete (DATASET *dset, int i, int s);
+const char *dataset_get_matrix_name (const DATASET *dset);
 
-void set_var_scalar (DATASET *dset, int i, int s);
+int series_is_log (const DATASET *dset, int i, char *parent);
 
-void set_var_hidden (DATASET *dset, int i);
+void series_set_discrete (DATASET *dset, int i, int s);
 
-void var_set_linewidth (DATASET *dset, int i, int w);
+void series_set_linewidth (DATASET *dset, int i, int w);
 
-int var_get_linewidth (const DATASET *dset, int i);
+int series_get_linewidth (const DATASET *dset, int i);
 
-int var_set_display_name (DATASET *dset, int i,
+int series_record_display_name (DATASET *dset, int i,
 			  const char *s); 
 
-int var_set_description (DATASET *dset, int i,
+int series_record_label (DATASET *dset, int i,
 			 const char *s); 
 
-int var_set_compact_method (DATASET *dset, int i,
-			    int method);
-
-const char *var_get_graph_name (const DATASET *dset, int i);
+const char *series_get_graph_name (const DATASET *dset, int i);
 
 unsigned int get_resampling_seed (void);
 
-int dataset_resample (int n, unsigned int seed, DATASET *dset);
+int dataset_resample (DATASET *dset, int n, unsigned int seed);
 
 int dataset_op_from_string (const char *s);
 
-int modify_dataset (int op, const int *list, const char *s, 
-		    DATASET *dset, PRN *prn);
+int modify_dataset (DATASET *dset, int op, const int *list, 
+		    const char *s, PRN *prn);
 
 int dataset_get_structure (const DATASET *dset);
 
@@ -445,4 +400,76 @@ int check_dataset_is_changed (void);
 
 void set_dataset_is_changed (void);
 
+int dataset_set_time_series (DATASET *dset, int pd, 
+			     int yr0, int minor0);
+
+int series_is_discrete (const DATASET *dset, int i);
+
+int series_is_hidden (const DATASET *dset, int i);
+
+int series_is_generated (const DATASET *dset, int i);
+
+int series_is_listarg (const DATASET *dset, int i);
+
+void series_set_flag (DATASET *dset, int i, int flag);
+
+void series_unset_flag (DATASET *dset, int i, int flag);
+
+void series_zero_flags (DATASET *dset, int i);
+
+const char *series_get_label (const DATASET *dset, int i);
+
+const char *series_get_display_name (const DATASET *dset, int i);
+
+const char *series_get_parent_name (const DATASET *dset, int i);
+
+int series_get_parent_id (const DATASET *dset, int i);
+
+int series_get_compact_method (const DATASET *dset, int i);
+
+int series_get_stack_level (const DATASET *dset, int i);
+
+int series_get_transform (const DATASET *dset, int i);
+
+int series_get_lag (const DATASET *dset, int i);
+
+void series_set_label (DATASET *dset, int i, 
+		       const char *s);
+
+void series_set_display_name (DATASET *dset, int i, 
+			      const char *s);
+
+void series_set_compact_method (DATASET *dset, int i, 
+				int method);
+
+void series_set_parent (DATASET *dset, int i, 
+			const char *parent);
+
+void series_set_transform (DATASET *dset, int i, 
+			   int transform);
+
+void series_set_lag (DATASET *dset, int i, int lag);
+
+void series_set_stack_level (DATASET *dset, int i, int level);
+
+void series_increment_stack_level (DATASET *dset, int i);
+
+void series_decrement_stack_level (DATASET *dset, int i);
+
+void series_attach_string_table (DATASET *dset, int i, void *ptr);
+
+int series_has_string_table (const DATASET *dset, int i);
+
+series_table *series_get_string_table (const DATASET *dset, int i);
+
+const char *series_get_string_val (const DATASET *dset, int i, int t);
+
+double series_decode_string (const DATASET *dset, int i, const char *s);
+
+const char **series_get_string_vals (const DATASET *dset, int i,
+				     int *n_strs);
+
+int steal_string_table (DATASET *l_dset, int lvar,
+			DATASET *r_dset, int rvar);
+
 #endif /* DATASET_H */
