@@ -23,8 +23,9 @@
 #include "swap_bytes.h"
 #include "gretl_www.h"
 #include "libset.h"
-#include "gretl_string_table.h"
+#include "uservar.h"
 #include "matrix_extra.h"
+#include "usermat.h"
 #include "dbread.h"
 
 #include <glib.h>
@@ -159,7 +160,6 @@ static FILE *open_binfile (const char *dbbase, int code, int offset, int *err)
  * @sinfo:
  * @Z: data array.
  *
- *
  * Returns: 0 on success, non-zero code on failure.
  */
 
@@ -196,13 +196,59 @@ int get_native_db_data (const char *dbbase, SERIESINFO *sinfo,
     return err;
 }
 
+static int db_row_wanted (const gretl_matrix *mask, 
+			  int masklen, int t)
+{
+    if (t >= masklen) {
+	return 0;
+    } else {
+	return gretl_vector_get(mask, t) != 0;
+    }
+}
+
+static int get_native_db_data_masked (const char *dbbase, 
+				      SERIESINFO *sinfo,
+				      double **dbZ,
+				      const gretl_matrix *mask)
+{
+    char numstr[32];
+    FILE *fp;
+    dbnumber x;
+    int s, t, masklen;
+    int err = 0;
+
+    fp = open_binfile(dbbase, GRETL_NATIVE_DB, sinfo->offset, &err);
+    if (err) {
+	return err;
+    }
+
+    masklen = gretl_vector_get_length(mask);
+
+    s = 0;
+    for (t=0; t<=sinfo->nobs && !err; t++) {
+	if (fread(&x, sizeof x, 1, fp) != 1) {
+	    err = DB_PARSE_ERROR;
+	} else if (db_row_wanted(mask, masklen, t)) {
+	    sprintf(numstr, "%.7g", (double) x);
+	    dbZ[1][s] = atof(numstr);
+	    if (dbZ[1][s] == DBNA) {
+		dbZ[1][s] = NADBL;
+	    }
+	    s++;
+	}
+    }
+
+    fclose(fp);
+
+    return err;
+}
+
 /**
  * get_remote_db_data:
  * @dbbase:
  * @sinfo:
  * @Z: data array.
  *
- *
  * Returns:  0 on success, non-zero code on failure.
  */
 
@@ -435,7 +481,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
     FILE *fp = NULL;
     char sername[VNAMELEN];
     char s1[256], s2[72];
-    char stobs[11], endobs[11];
+    char stobs[16], endobs[16];
     char pdc;
     int offset = 0;
     int gotit = 0, err = 0;
@@ -452,7 +498,7 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 	    continue;
 	}
 
-	if (sscanf(s1, "%15s", sername) != 1) {
+	if (gretl_scan_varname(s1, sername) != 1) {
 	    break;
 	}
 
@@ -501,7 +547,7 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
     char *buf = NULL;
     char sername[VNAMELEN];
     char s1[256], s2[72];
-    char stobs[11], endobs[11];
+    char stobs[16], endobs[16];
     char pdc;
     int gotit = 0;
     int err = 0;
@@ -520,7 +566,7 @@ get_remote_series_info (const char *series, SERIESINFO *sinfo)
 	    continue;
 	}
 
-	if (sscanf(s1, "%15s", sername) != 1) {
+	if (gretl_scan_varname(s1, sername) != 1) {
 	    break;
 	}
 
@@ -609,6 +655,7 @@ get_pcgive_series_info (const char *series, SERIESINFO *sinfo)
     FILE *fp;
     char dbidx[MAXLEN];
     char line[1024];
+    char fmt[24];
     char *p;
     int y0, p0, y1, p1;
     int nf, gotit = 0;
@@ -632,13 +679,14 @@ get_pcgive_series_info (const char *series, SERIESINFO *sinfo)
 	return E_FOPEN;
     }
 
+    sprintf(fmt, "%%%ds %%d %%d %%d %%d %%d %%d", VNAMELEN - 1);
+
     while (fgets(line, sizeof line, fp) && !gotit) {
 	if (*line == '>') {
 	    *sinfo->varname = 0;
 	    *sinfo->descrip = 0;
-	    nf = sscanf(line + 1, "%15s %d %d %d %d %d %d",
-			sinfo->varname, &y0, &p0, &y1, &p1, 
-			&sinfo->pd, &sinfo->offset);
+	    nf = sscanf(line + 1, fmt, sinfo->varname, &y0, &p0, 
+			&y1, &p1, &sinfo->pd, &sinfo->offset);
 	    fprintf(stderr, "in7: varname='%s'\n", sinfo->varname);
 	    if (!strcmp(sinfo->varname, series)) {
 		gotit = 1;
@@ -652,7 +700,7 @@ get_pcgive_series_info (const char *series, SERIESINFO *sinfo)
 			int rem = MAXLABEL - strlen(sinfo->descrip) - 1;
 
 			if (rem > 0) {
-			    chopstr(line);
+			    gretl_chopstr(line);
 			    strncat(sinfo->descrip, line + 1, rem);
 			}
 		    } else {
@@ -859,9 +907,9 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
     fseek(fp, 4L, SEEK_CUR); /* skip two shorts */
     fread(&rdir.first_data, sizeof(RECNUM), 1, fp);
     fread(rdir.series_name, NAMELENGTH, 1, fp);  
-    rdir.series_name[15] = '\0';
+    rdir.series_name[NAMELENGTH-1] = '\0';
 
-    chopstr(rdir.series_name);
+    gretl_chopstr(rdir.series_name);
 
 #if DB_DEBUG
     fprintf(stderr, "read_rats_directory: name='%s'\n", rdir.series_name);
@@ -904,7 +952,7 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
 	    memset(rdir.comments[i], 0, 80);
 	    fread(rdir.comments[i], 80, 1, fp);
 	    rdir.comments[i][79] = '\0';
-	    chopstr(rdir.comments[i]);
+	    gretl_chopstr(rdir.comments[i]);
 	} else {
 	    rdir.comments[i][0] = 0;
 	    fseek(fp, 80, SEEK_CUR);
@@ -1039,16 +1087,19 @@ static int read_in7_series_info (FILE *fp, dbwrapper *dw)
     char line[1024];
     char sname[VNAMELEN];
     char desc[MAXLABEL];
+    char fmt[24];
     int y0, p0, y1, p1;
     int pd, offset, pos;
     int i, nf;
     int err = 0;
 
+    sprintf(fmt, "%%%ds %%d %%d %%d %%d %%d %%d", VNAMELEN - 1);
+
     i = 0;
     while (fgets(line, sizeof line, fp) && !err) {
 	if (*line == '>') {
-	    nf = sscanf(line + 1, "%15s %d %d %d %d %d %d",
-			sname, &y0, &p0, &y1, &p1, &pd, &offset);
+	    nf = sscanf(line + 1, fmt, sname, &y0, &p0, &y1, 
+			&p1, &pd, &offset);
 	    if (nf == 7 && y0 > 0 && p0 > 0 && y1 > 0 && p1 > 0 &&
 		pd >= 1 && offset > 0) {
 		*desc = 0;
@@ -1059,7 +1110,7 @@ static int read_in7_series_info (FILE *fp, dbwrapper *dw)
 			int rem = MAXLABEL - strlen(desc) - 1;
 
 			if (rem > 0) {
-			    chopstr(line);
+			    gretl_chopstr(line);
 			    strncat(desc, line + 1, rem);
 			}
 			pos = ftell(fp);
@@ -1086,10 +1137,13 @@ static int count_in7_series (FILE *fp, int *err)
 {
     char line[1024];
     char sname[VNAMELEN];
+    char fmt[24];
     int y0, p0, y1, p1;
     int pd, offset;
     int nf, i = 0, nseries = 0;
 
+    sprintf(fmt, "%%%ds %%d %%d %%d %%d %%d %%d", VNAMELEN - 1);
+
     while (fgets(line, sizeof line, fp)) {
 	if (i == 0 && strncmp(line, "pcgive 700", 10)) {
 	    *err = 1;
@@ -1097,8 +1151,8 @@ static int count_in7_series (FILE *fp, int *err)
 	    return 0;
 	}
 	if (*line == '>') {
-	    nf = sscanf(line + 1, "%15s %d %d %d %d %d %d",
-			sname, &y0, &p0, &y1, &p1, &pd, &offset);
+	    nf = sscanf(line + 1, fmt, sname, &y0, &p0, &y1, 
+			&p1, &pd, &offset);
 	    if (nf < 7 || y0 < 0 || p0 < 0 || y1 < 0 || p1 < 0 ||
 		pd < 1 || offset < 0) {
 		fprintf(stderr, "Error reading series info\n");
@@ -1298,7 +1352,8 @@ static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp,
  * data were found but there were some missing values.
  */
 
-int get_rats_db_data (const char *fname, SERIESINFO *sinfo, double **Z)
+int get_rats_db_data (const char *fname, SERIESINFO *sinfo, 
+		      double **Z)
 {
     FILE *fp;
     int ret = 0;
@@ -1612,23 +1667,18 @@ int set_db_name (const char *fname, int filetype, PRN *prn)
 
     fp = gretl_fopen(saved_db_name, "rb");
 
-    if (fp == NULL && !g_path_is_absolute(saved_db_name)) {
+    if (fp == NULL && !g_path_is_absolute(saved_db_name) &&
+	filetype == GRETL_NATIVE_DB) {
 	/* try looking a bit more */
-	const char *path = NULL;
+	const char *path = gretl_binbase();
 
-	if (filetype == GRETL_NATIVE_DB) {
-	    path = gretl_binbase();
-	} else if (filetype == GRETL_RATS_DB) {
-	    path = gretl_ratsbase();
-	}
-
-	if (path != NULL) {
+	if (path != NULL && *path != '\0') {
 	    build_path(saved_db_name, path, fname, NULL);
 	    fp = gretl_fopen(saved_db_name, "rb");
 	}
 
 #ifdef OSX_BUILD
-	if (fp == NULL && filetype == GRETL_NATIVE_DB) {
+	if (fp == NULL) {
 	    gchar *tmp = g_build_path(SLASHSTR, gretl_app_support_dir(), "db",
 				      fname, NULL);
 
@@ -1696,7 +1746,7 @@ static void ODBC_info_clear_read (void)
     doubles_array_free(gretl_odinfo.X, gretl_odinfo.nvars);
     gretl_odinfo.X = NULL;
 
-    free_strings_array(gretl_odinfo.S, gretl_odinfo.nrows);
+    strings_array_free(gretl_odinfo.S, gretl_odinfo.nrows);
     gretl_odinfo.S = NULL;
 
     for (i=0; i<ODBC_OBSCOLS; i++) {
@@ -1704,7 +1754,7 @@ static void ODBC_info_clear_read (void)
     }
 
     if (gretl_odinfo.fmts != NULL) {
-	free_strings_array(gretl_odinfo.fmts, gretl_odinfo.obscols);
+	strings_array_free(gretl_odinfo.fmts, gretl_odinfo.obscols);
 	gretl_odinfo.fmts = NULL;
     }
 
@@ -2089,7 +2139,7 @@ static char **odbc_get_varnames (char **line, int *err)
     }
 
     if (*err) {
-	free_strings_array(vnames, nv);
+	strings_array_free(vnames, nv);
 	vnames = NULL;
     } else {	
 	gretl_odinfo.nvars = nv;
@@ -2103,6 +2153,7 @@ static char **odbc_get_varnames (char **line, int *err)
 static int odbc_transcribe_data (char **vnames, DATASET *dset, 
 				 int vmin, int newvars)
 {
+    char label[MAXLABEL];
     int nv = gretl_odinfo.nvars;
     int n = gretl_odinfo.nrows;
     int nrepl = nv - newvars;
@@ -2123,7 +2174,8 @@ static int odbc_transcribe_data (char **vnames, DATASET *dset,
 	if (v < 0) {
 	    v = vmin++;
 	    strcpy(dset->varname[v], vnames[i]);
-	    sprintf(VARLABEL(dset, v), "ODBC series %d", i + 1);
+	    sprintf(label, "ODBC series %d", i + 1);
+	    series_set_label(dset, v, label);
 	} else {
 	    vnew = 0;
 	}
@@ -2268,7 +2320,7 @@ static int odbc_get_series (char *line, DATASET *dset,
 	    newvars = odbc_count_new_vars(vnames, nv, dset);
 	    vmin = dset->v;
 	    if (newvars > 0) {
-		err = dataset_add_series(newvars, dset);
+		err = dataset_add_series(dset, newvars);
 	    }
 	}
 
@@ -2277,12 +2329,73 @@ static int odbc_get_series (char *line, DATASET *dset,
 	}
     }
 
-    free_strings_array(vnames, gretl_odinfo.nvars);
+    strings_array_free(vnames, gretl_odinfo.nvars);
     ODBC_info_clear_read();
 
     return err;
 }
 
+static int db_get_row_mask (const gretl_matrix **pmat)
+{
+    const char *rows;
+    int err = 0;
+
+    rows = get_optval_string(DATA, OPT_M);
+    if (rows == NULL || *rows == '\0') {
+	return E_PARSE;
+    }   
+
+    *pmat = get_matrix_by_name(rows);
+    if (*pmat == NULL) {
+	gretl_errmsg_sprintf(_("'%s': no such matrix"), rows);
+	err = E_DATA;
+    } else if (gretl_vector_get_length(*pmat) == 0) {
+	err = E_NONCONF;
+    }
+
+    return err;
+}
+
+static int db_n_from_row_mask (const gretl_matrix *mask,
+			       int n_max)
+{
+    int mlen = gretl_vector_get_length(mask);
+    int i, n = 0;
+
+    for (i=0; i<mlen && i<n_max; i++) {
+	if (gretl_vector_get(mask, i) != 0) {
+	    n++;
+	}
+    }
+
+    return n;
+}
+
+/* when a row mask is applied in reading from a gretl
+   database, we rejig the series information such that
+   it's flat (undated) with the observations index
+   running from 1 to the number of rows selected
+*/
+
+static int update_sinfo_masked (SERIESINFO *sinfo, int nobs)
+{
+    int err = 0;
+
+    sinfo->nobs = nobs;
+    sinfo->t1 = 0;
+    sinfo->t2 = nobs - 1;
+
+    if (sinfo->pd != 1) {
+	err = E_PDWRONG;
+    } else if (strcmp(sinfo->stobs, "1")) {
+	err = E_DATA;
+    } else {
+	sprintf(sinfo->endobs, "%d", nobs);
+    }
+
+    return err;
+}
+
 /* main function for getting a series out of a database, using the
    command-line client or in script or console mode
 */
@@ -2290,6 +2403,7 @@ static int odbc_get_series (char *line, DATASET *dset,
 int db_get_series (char *line, DATASET *dset, 
 		   gretlopt opt, PRN *prn)
 {
+    const gretl_matrix *rowmask = NULL;
     char series[VNAMELEN];
     CompactMethod method;
     SERIESINFO sinfo;
@@ -2300,6 +2414,13 @@ int db_get_series (char *line, DATASET *dset,
 	return odbc_get_series(line, dset, prn);
     }
 
+    if (opt & OPT_M) {
+	err = db_get_row_mask(&rowmask);
+	if (err) {
+	    return err;
+	}
+    }    
+
 #if DB_DEBUG
     fprintf(stderr, "db_get_series: line='%s', dset=%p\n", 
 	    line, (void *) dset);
@@ -2317,14 +2438,14 @@ int db_get_series (char *line, DATASET *dset,
 
     while ((line = get_word_and_advance(line, series, VNAMELEN-1)) && !err) {
 	int v, this_var_method = method; 
-	int newdata = 0;
+	int nobs, newdata = 0;
 
 	series_info_init(&sinfo);
 
 	/* see if the series is already in the dataset */
 	v = series_index(dset, series);
 	if (v < dset->v && method == COMPACT_NONE) {
-	    this_var_method = COMPACT_METHOD(dset, v);
+	    this_var_method = series_get_compact_method(dset, v);
 	}
 
 #if DB_DEBUG
@@ -2349,8 +2470,14 @@ int db_get_series (char *line, DATASET *dset,
 	    return 1;
 	}
 
+	if (rowmask != NULL) {
+	    nobs = db_n_from_row_mask(rowmask, sinfo.nobs);
+	} else {
+	    nobs = sinfo.nobs;
+	}
+
 	/* temporary dataset */
-	dbZ = new_dbZ(sinfo.nobs);
+	dbZ = new_dbZ(nobs);
 	if (dbZ == NULL) {
 	    gretl_errmsg_set(_("Out of memory!"));
 	    return 1;
@@ -2358,7 +2485,7 @@ int db_get_series (char *line, DATASET *dset,
 
 #if DB_DEBUG
 	fprintf(stderr, "db_get_series: offset=%d, nobs=%d\n", 
-		sinfo.offset, sinfo.nobs);
+		sinfo.offset, nobs);
 #endif
 
 	if (saved_db_type == GRETL_RATS_DB) {
@@ -2367,6 +2494,9 @@ int db_get_series (char *line, DATASET *dset,
 	    err = get_pcgive_db_data(saved_db_name, &sinfo, dbZ);
 	} else if (saved_db_type == GRETL_NATIVE_DB_WWW) {
 	    err = get_remote_db_data(saved_db_name, &sinfo, dbZ);
+	} else if (rowmask != NULL) {
+	    err = get_native_db_data_masked(saved_db_name, &sinfo, dbZ, 
+					    rowmask);
 	} else {
 	    err = get_native_db_data(saved_db_name, &sinfo, dbZ);
 	} 
@@ -2380,6 +2510,10 @@ int db_get_series (char *line, DATASET *dset,
 	    err = 0;
 	}
 
+	if (!err && rowmask != NULL) {
+	    err = update_sinfo_masked(&sinfo, nobs);
+	}
+
 	if (!err) {
 	    err = cli_add_db_data(dbZ, &sinfo, dset, 
 				  this_var_method, v, 
@@ -2553,7 +2687,7 @@ static int db_delete_series (char *line, const int *list,
     }
 
     if (snames != NULL) {
-	free_strings_array(snames, ns);
+	strings_array_free(snames, ns);
     }
 
  bailout:
@@ -2694,7 +2828,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    gretl_errmsg_set(_("Out of memory!"));
 	    return 1;
 	}
-    } else if (new && dataset_add_series(1, dset)) {
+    } else if (new && dataset_add_series(dset, 1)) {
 	gretl_errmsg_set(_("Out of memory!"));
 	return 1;
     }
@@ -2711,7 +2845,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    sinfo->pd != 12) {
 	    gretl_errmsg_set(_("Sorry, can't handle this conversion yet!"));
 	    if (new) {
-		dataset_drop_last_variables(1, dset);
+		dataset_drop_last_variables(dset, 1);
 	    }
 	    return 1;
 	}
@@ -2725,7 +2859,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	if (xvec == NULL) {
 	    gretl_errmsg_set(_("Out of memory!"));
 	    if (new) {
-		dataset_drop_last_variables(1, dset);
+		dataset_drop_last_variables(dset, 1);
 	    }
 	    return 1;
 	}
@@ -2737,8 +2871,8 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 
     /* common stuff for adding a var */
     strcpy(dset->varname[dbv], sinfo->varname);
-    strcpy(VARLABEL(dset, dbv), sinfo->descrip);
-    COMPACT_METHOD(dset, dbv) = method;
+    series_set_label(dset, dbv, sinfo->descrip);
+    series_set_compact_method(dset, dbv, method);
     get_db_padding(sinfo, dset, &pad1, &pad2);
 
     if (pad1 > 0) {
@@ -3085,7 +3219,7 @@ get_daily_compact_params (CompactMethod default_method,
     *any_sop = (default_method == COMPACT_SOP)? 1 : 0;
 
     for (i=1; i<dset->v; i++) {
-	CompactMethod method = COMPACT_METHOD(dset, i);
+	CompactMethod method = series_get_compact_method(dset, i);
 
 	if (method != default_method && method != COMPACT_NONE) {
 	    *all_same = 0;
@@ -3132,7 +3266,7 @@ get_global_compact_params (int compfac, int startmin, int endmin,
 		*any_eop = 1;
 	    }
 	} else {
-	    method = COMPACT_METHOD(dset, i);
+	    method = series_get_compact_method(dset, i);
 	    if (method != default_method && method != COMPACT_NONE) {
 		get_startskip_etc(compfac, startmin, endmin, dset->n, 
 				  method, &startskip, &n);
@@ -3434,7 +3568,7 @@ static int weekly_dataset_to_monthly (DATASET *dset,
     }
 
     mset.v = dset->v;
-    err = allocate_Z(&mset);
+    err = allocate_Z(&mset, 0);
     if (err) {
 	return err;
     }
@@ -3584,7 +3718,7 @@ static int daily_dataset_to_monthly (DATASET *dset,
     err = shorten_the_constant(dset->Z, nm);
 
     for (i=1; i<dset->v && !err; i++) {
-	method = COMPACT_METHOD(dset, i);
+	method = series_get_compact_method(dset, i);
 	if (method == COMPACT_NONE) {
 	    method = default_method;
 	}
@@ -3635,7 +3769,7 @@ static int insert_missing_hidden_obs (DATASET *dset, int nmiss)
     int i, s, t, skip;
     int err = 0;
 
-    err = dataset_add_observations(nmiss, dset, OPT_NONE);
+    err = dataset_add_observations(dset, nmiss, OPT_NONE);
     if (err) {
 	return err;
     }
@@ -3851,8 +3985,10 @@ int compact_data_set (DATASET *dset, int newpd,
 	double *x;
 
 	if (!all_same) {
-	    if (COMPACT_METHOD(dset, i) != COMPACT_NONE) {
-		this_method = COMPACT_METHOD(dset, i);
+	    CompactMethod m_i = series_get_compact_method(dset, i);
+
+	    if (m_i != COMPACT_NONE) {
+		this_method = m_i;
 	    }
 
 	    startskip = compfac - (startmin % compfac) + 1;
@@ -3959,7 +4095,7 @@ int expand_data_set (DATASET *dset, int newpd, int interpol)
     newn = mult * dset->n;
     nadd = newn - oldn;
 
-    err = dataset_add_observations(nadd, dset, OPT_NONE);
+    err = dataset_add_observations(dset, nadd, OPT_NONE);
     if (err) {
 	goto bailout;
     }
