@@ -844,16 +844,19 @@ static void fill_op_model (MODEL *pmod, const int *list,
     mle_criteria(pmod, 0);
     pmod->rsq = pmod->adjrsq = NADBL;
 
-    gretl_model_allocate_params(pmod, npar);
+    gretl_model_allocate_param_names(pmod, npar);
 
     if (pmod->errcode == 0) {
+	char tmp[16];
+
 	for (i=0; i<nx; i++) {
 	    v = OC->list[i+2];
-	    strcpy(pmod->params[i], dset->varname[v]);
+	    gretl_model_set_param_name(pmod, i, dset->varname[v]);
 	}
 	s = 1;
 	for (i=nx; i<npar; i++) {
-	    sprintf(pmod->params[i], "cut%d", s++);
+	    sprintf(tmp, "cut%d", s++);
+	    gretl_model_set_param_name(pmod, i, tmp);
 	}
     }
 
@@ -1191,7 +1194,7 @@ static int check_for_missing_dummies (MODEL *pmod, int *dlist)
 
 static int ordered_depvar_check (int v, const DATASET *dset)
 {
-    if (!var_is_discrete(dset, v) && 
+    if (!series_is_discrete(dset, v) && 
 	!gretl_is_oprobit_ok(dset->t1, dset->t2, dset->Z[v])) {
 	gretl_errmsg_sprintf(_("The variable '%s' is not discrete"),
 			     dset->varname[v]);
@@ -1281,7 +1284,7 @@ static MODEL ordered_estimate (int *list, DATASET *dset, int ci,
 
     if (dset->v > orig_v) {
 	/* clean up any automatically-added dummies */
-	dataset_drop_last_variables(dset->v - orig_v, dset);
+	dataset_drop_last_variables(dset, dset->v - orig_v);
     }
 
     set_model_id(&model);
@@ -2029,7 +2032,7 @@ static void mnl_finish (mnl_info *mnl, MODEL *pmod,
 	pmod->ess = pmod->sigma = NADBL;
 	pmod->fstt = pmod->rsq = pmod->adjrsq = NADBL;
 
-	gretl_model_allocate_params(pmod, mnl->npar);
+	gretl_model_allocate_param_names(pmod, mnl->npar);
     }
 
     if (!pmod->errcode) {
@@ -2038,7 +2041,7 @@ static void mnl_finish (mnl_info *mnl, MODEL *pmod,
 	for (i=0; i<mnl->n; i++) {
 	    for (j=0; j<mnl->k; j++) {
 		vj = pmod->list[j+2];
-		strcpy(pmod->params[k++], dset->varname[vj]);
+		gretl_model_set_param_name(pmod, k++, dset->varname[vj]);
 	    }
 	}
 	mn_logit_yhat(pmod, mnl, yvals);
@@ -3130,7 +3133,7 @@ static int make_logistic_depvar (DATASET *dset, int dv,
     int t, v = dset->v;
     int err;
 
-    err = dataset_add_series(1, dset);
+    err = dataset_add_series(dset, 1);
 
     if (!err) {
 	for (t=0; t<dset->n; t++) {
@@ -3266,7 +3269,7 @@ MODEL logistic_model (const int *list, double lmax,
 	set_model_id(&lmod);
     }
 
-    dataset_drop_last_variables(1, dset);
+    dataset_drop_last_variables(dset, 1);
     free(llist);
     
     return lmod;
