@@ -19,7 +19,7 @@
 
 #include "libgretl.h"
 #include "matrix_extra.h"
-#include "gretl_scalar.h"
+#include "uservar.h"
 #include "forecast.h"
 #include "var.h"
 #include "system.h"
@@ -386,9 +386,9 @@ static const int *model_xlist (MODEL *pmod)
 
 static int has_depvar_lags (MODEL *pmod, const DATASET *dset)
 {
-    const char *yname;
+    const char *yname, *parent;
     const int *xlist;
-    int i, vi;
+    int i, vi, lag;
 
     xlist = model_xlist(pmod);
     if (xlist == NULL) {
@@ -399,8 +399,9 @@ static int has_depvar_lags (MODEL *pmod, const DATASET *dset)
 
     for (i=1; i<=xlist[0]; i++) {
         vi = xlist[i];
-	if (dset->varinfo[vi]->lag > 0 &&
-	    !strcmp(yname, dset->varinfo[vi]->parent)) {
+	lag = series_get_lag(dset, vi);
+	parent = series_get_parent_name(dset, vi);
+	if (lag > 0 && parent != NULL && !strcmp(yname, parent)) {
 	    return 1;
 	}
     }
@@ -424,7 +425,7 @@ static int process_lagged_depvar (MODEL *pmod,
 {
     const int *xlist = NULL;
     int *dvlags = NULL;
-    int anylags;
+    int lag, anylags;
     int err = 0;
 
     anylags = has_depvar_lags(pmod, dset);
@@ -441,16 +442,17 @@ static int process_lagged_depvar (MODEL *pmod,
     if (dvlags == NULL) {
 	err = E_ALLOC;
     } else {
-	const char *yname;
+	const char *yname, *parent;
 	int i, vi;
 
 	yname = gretl_model_get_depvar_name(pmod, dset);
 
 	for (i=1; i<=xlist[0]; i++) {
             vi = xlist[i];
-	    if (dset->varinfo[vi]->lag > 0 &&
-		!strcmp(yname, dset->varinfo[vi]->parent)) {
-		dvlags[i-1] = dset->varinfo[vi]->lag;
+	    lag = series_get_lag(dset, vi);
+	    parent = series_get_parent_name(dset, vi);
+	    if (lag > 0 && parent != NULL && !strcmp(yname, parent)) {
+		dvlags[i-1] = lag;
 	    } else {
 		dvlags[i-1] = 0;
 	    }
@@ -762,7 +764,7 @@ static int nls_fcast (Forecast *fc, const MODEL *pmod,
     }
 
     if (dset->v > fcv) {
-	err = dataset_drop_last_variables(dset->v - fcv, dset);
+	err = dataset_drop_last_variables(dset, dset->v - fcv);
     }
 
     dset->t1 = oldt1;
@@ -1370,7 +1372,6 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	}
 	if (xlist[0] == 1 && xlist[1] == 0) {
 	    /* just a const, no ARMAX */
-	    mu = pmod->coeff[0];
 	    if (phi0 != NULL) {
 		mu = 1.0;
 		for (i=1; i<=px; i++) {
@@ -1405,7 +1406,7 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	ar_smax = fc->t2;
 	ma_smax = pmod->t2;
     } else if (fc->method == FC_DYNAMIC) {
-	ar_smax = max(fc->t1 - 1, p);
+	ar_smax = max(fc->t1 - 1, p - 1);
 	ma_smax = max(fc->t1 - 1, q);
     } else {
 	ar_smax = pmod->t2;
@@ -1422,7 +1423,11 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	double yh = 0.0;
 	int miss = 0;
 
-	DPRINTF(("\n *** Doing forecast for obs %d\n", t));
+#if AR_DEBUG
+	char obsstr[OBSLEN];
+	ntodate(obsstr, t, dset);
+#endif
+	DPRINTF(("\n *** Doing forecast for obs %d (%s)\n", t, obsstr));
 
 	/* contribution of const and/or independent variables */
 
@@ -1466,7 +1471,7 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	    }
 	}
 
-	DPRINTF((" Xb contribution = %g\n", yh));
+	DPRINTF((" Xb contribution: yh = %g\n", yh));
 
 	/* AR contribution (incorporating any differencing) */
 
@@ -1496,6 +1501,8 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 			yh += phi[i] * (yval - Xb[s]);
 		    }
 		} else if (!regarma && !na(mu)) {
+		    DPRINTF(("    !regarma && !na(mu): yh += %g * (%g - %g)\n",
+			     phi[i], yval, mu)); 
 		    yh += phi[i] * (yval - mu);
 		} else {
 		    yh += phi[i] * yval;
@@ -2875,7 +2882,7 @@ static int add_fcast_to_dataset (FITRESID *fr, const char *vname,
 	if (check_varname(vname)) {
 	    err = E_DATA;
 	} else {
-	    err = dataset_add_series(1, dset);
+	    err = dataset_add_series(dset, 1);
 	    if (!err) {
 		strcpy(dset->varname[v], vname);
 	    }
@@ -2889,7 +2896,7 @@ static int add_fcast_to_dataset (FITRESID *fr, const char *vname,
 	    dset->Z[v][t] = fr->fitted[t];
 	}
 
-	strcpy(VARLABEL(dset, v), _("predicted values"));
+	series_set_label(dset, v, _("predicted values"));
 
 	if (gretl_messages_on()) {
 	    if (v < oldv) {
@@ -3416,13 +3423,13 @@ void forecast_options_for_model (MODEL *pmod, const DATASET *dset,
     }
 }
 
-static int y_lag (int v, int parent, const DATASET *dset)
+static int y_lag (int v, int parent_id, const DATASET *dset)
 {
-    if (dset->varinfo[v]->transform == LAGS) {
-	int pv = series_index(dset, dset->varinfo[v]->parent);
+    if (series_get_transform(dset, v) == LAGS) {
+	int pv = series_get_parent_id(dset, v);
 
-	if (pv == parent) {
-	    return dset->varinfo[v]->lag;
+	if (pv == parent_id) {
+	    return series_get_lag(dset, v);
 	}
     }
 
