@@ -22,7 +22,6 @@
 #include "genparse.h"
 #include "libset.h"
 #include "monte_carlo.h"
-#include "gretl_string_table.h"
 #include "gretl_panel.h"
 #include "gretl_fft.h"
 #include "estim_private.h"
@@ -839,11 +838,12 @@ int movavg_series (const double *x, double *y, const DATASET *dset,
 }
 
 int seasonally_adjust_series (const double *x, double *y, 
-			      DATASET *dset, int tramo)
+			      DATASET *dset, int tramo,
+			      int use_log)
 {
     void *handle;
     int (*adjust_series) (const double *, double *, 
-			  const DATASET *, int);
+			  const DATASET *, int, int);
     int t1 = dset->t1;
     int t2 = dset->t2;
     int T, err = 0;
@@ -885,7 +885,7 @@ int seasonally_adjust_series (const double *x, double *y,
 	dset->t1 = t1;
 	dset->t2 = t2;
 
-	err = (*adjust_series) (x, y, dset, tramo);
+	err = (*adjust_series) (x, y, dset, tramo, use_log);
 
 	dset->t1 = save_t1;
 	dset->t2 = save_t2;
@@ -2372,7 +2372,7 @@ int dummy (DATASET *dset, int center)
     for (vi=0; vi<ndums; vi++) {
 	make_dummy_name_and_label(vi + 1, dset, center, vname, vlabel);
 	di = series_index(dset, vname);
-	if (di >= dset->v || strcmp(vlabel, VARLABEL(dset, di))) {
+	if (di >= dset->v || strcmp(vlabel, series_get_label(dset, di))) {
 	    nnew++;
 	} else if (vi == 0) {
 	    di0 = di;
@@ -2391,7 +2391,7 @@ int dummy (DATASET *dset, int center)
 	return -1;
     }
 
-    if (dataset_add_series(ndums, dset)) {
+    if (dataset_add_series(dset, ndums)) {
 	gretl_errmsg_set(_("Out of memory!"));
 	return 0;
     }
@@ -2399,7 +2399,7 @@ int dummy (DATASET *dset, int center)
     for (vi=1, di = di0; vi<=ndums; vi++, di++) {
 	make_dummy_name_and_label(vi, dset, center, vname, vlabel);
 	strcpy(dset->varname[di], vname);
-	strcpy(VARLABEL(dset, di), vlabel);
+	series_set_label(dset, di, vlabel);
     }
 
     if (dataset_is_daily(dset)) {
@@ -2412,7 +2412,7 @@ int dummy (DATASET *dset, int center)
 
 	for (vi=1, di = di0; vi<=ndums; vi++, di++) {
 	    for (t=0; t<dset->n; t++) {
-		xx = date(t, dset->pd, dset->sd0) + .1;
+		xx = date_as_double(t, dset->pd, dset->sd0) + .1;
 		yy = (int) xx;
 		pp = (int) (mm * (xx - yy) + 0.5);
 		dx = (pp == vi)? 1.0 : 0.0;
@@ -2460,7 +2460,7 @@ int dummy (DATASET *dset, int center)
 
 int panel_dummies (DATASET *dset, gretlopt opt, PRN *prn)
 {
-    char vname[16];
+    char vname[16], label[MAXLABEL];
     int vi, t, yy, pp, mm;
     int orig_v = dset->v;
     int ndum, nnew;
@@ -2493,7 +2493,7 @@ int panel_dummies (DATASET *dset, gretlopt opt, PRN *prn)
 	}
     }
 
-    if (nnew > 0 && dataset_add_series(nnew, dset)) {
+    if (nnew > 0 && dataset_add_series(dset, nnew)) {
 	return E_ALLOC;
     }
 
@@ -2518,12 +2518,12 @@ int panel_dummies (DATASET *dset, gretlopt opt, PRN *prn)
 	}
 
 	strcpy(dset->varname[dnum], vname);
-	sprintf(VARLABEL(dset, dnum), 
-		_("%s = 1 if %s is %d, 0 otherwise"), vname, 
+	sprintf(label, _("%s = 1 if %s is %d, 0 otherwise"), vname, 
 		_("period"), vi);
+	series_set_label(dset, dnum, label);
 
 	for (t=0; t<dset->n; t++) {
-	    xx = date(t, dset->pd, dset->sd0);
+	    xx = date_as_double(t, dset->pd, dset->sd0);
 	    yy = (int) xx;
 	    pp = (int) (mm * (xx - yy) + 0.5);
 	    dset->Z[dnum][t] = (pp == vi)? 1.0 : 0.0;
@@ -2545,9 +2545,9 @@ int panel_dummies (DATASET *dset, gretlopt opt, PRN *prn)
 	}	
 
 	strcpy(dset->varname[dnum], vname);
-	sprintf(VARLABEL(dset, dnum), 
-		_("%s = 1 if %s is %d, 0 otherwise"), vname, 
+	sprintf(label, _("%s = 1 if %s is %d, 0 otherwise"), vname, 
 		_("unit"), vi);
+	series_set_label(dset, dnum, label);
 
 	for (t=0; t<dset->n; t++) {
 	    dset->Z[dnum][t] = (t >= dmin && t < dmax)? 1 : 0;
@@ -2580,12 +2580,12 @@ int gen_unit (DATASET *dset)
 
     i = series_index(dset, "unit");
 
-    if (i == dset->v && dataset_add_series(1, dset)) {
+    if (i == dset->v && dataset_add_series(dset, 1)) {
 	return E_ALLOC;
     }
 
     strcpy(dset->varname[i], "unit");
-    strcpy(VARLABEL(dset, i), _("cross-sectional unit index"));
+    series_set_label(dset, i, _("cross-sectional unit index"));
 
     for (t=0; t<dset->n; t++) {
 	if (t % dset->pd == 0) {
@@ -2654,27 +2654,27 @@ static void make_panel_time_var (double *x, const DATASET *dset)
 
 int gen_time (DATASET *dset, int tm)
 {
-    int i, t;
+    int v, t;
 
-    i = series_index(dset, (tm)? "time" : "index");
+    v = series_index(dset, (tm)? "time" : "index");
 
-    if (i == dset->v && dataset_add_series(1, dset)) {
+    if (v == dset->v && dataset_add_series(dset, 1)) {
 	return E_ALLOC;
     }
 
     if (tm) {
-	strcpy(dset->varname[i], "time");
-	strcpy(VARLABEL(dset, i), _("time trend variable"));
+	strcpy(dset->varname[v], "time");
+	series_set_label(dset, v, _("time trend variable"));
     } else {
-	strcpy(dset->varname[i], "index");
-	strcpy(VARLABEL(dset, i), _("data index variable"));
+	strcpy(dset->varname[v], "index");
+	series_set_label(dset, v, _("data index variable"));
     }
     
     if (tm && dset->structure == STACKED_TIME_SERIES) {
-	make_panel_time_var(dset->Z[i], dset);
+	make_panel_time_var(dset->Z[v], dset);
     } else {
 	for (t=0; t<dset->n; t++) {
-	    dset->Z[i][t] = (double) (t + 1);
+	    dset->Z[v][t] = (double) (t + 1);
 	}
     }
 
@@ -2705,12 +2705,12 @@ int gen_wkday (DATASET *dset)
 
     i = series_index(dset, "weekday");
 
-    if (i == dset->v && dataset_add_series(1, dset)) {
+    if (i == dset->v && dataset_add_series(dset, 1)) {
 	return E_ALLOC;
     }
 
     strcpy(dset->varname[i], "weekday");
-    strcpy(VARLABEL(dset, i), _("day of week (1 = Monday)"));
+    series_set_label(dset, i, _("day of week (1 = Monday)"));
     
     for (t=0; t<dset->n; t++) {
 	ntodate(datestr, t, dset);
@@ -3000,7 +3000,7 @@ int genr_fit_resid (const MODEL *pmod, DATASET *dset,
     x = get_fit_or_resid(pmod, dset, idx, vname, vlabel, &err);
 
     if (!err) {
-	err = dataset_add_allocated_series(x, dset);
+	err = dataset_add_allocated_series(dset, x);
     }
 
     if (err) {
@@ -3009,7 +3009,7 @@ int genr_fit_resid (const MODEL *pmod, DATASET *dset,
 	int v = dset->v - 1;
 
 	strcpy(dset->varname[v], vname);
-	strcpy(VARLABEL(dset, v), vlabel);
+	series_set_label(dset, v, vlabel);
     }
 
     return err;
@@ -3101,11 +3101,11 @@ int get_t_from_obs_string (const char *s, const DATASET *dset)
     int t;
 
     if (*s == '"') {
-	char obs[16];
+	char obs[OBSLEN+2];
 	int err = 0;
 
 	*obs = '\0';
-	strncat(obs, s, 15);
+	strncat(obs, s, OBSLEN+1);
 	gretl_unquote(obs, &err);
 	t = dateton(obs, dset);
     } else {
@@ -3199,12 +3199,7 @@ int check_declarations (char ***pS, parser *p)
     }
 
     for (i=0; i<n && !p->err; i++) {
-	if (gretl_is_series(S[i], p->dset) ||
-	    gretl_is_scalar(S[i]) ||
-	    gretl_is_bundle(S[i]) ||
-	    get_matrix_by_name(S[i]) ||
-	    get_list_by_name(S[i]) ||
-	    get_string_by_name(S[i])) {
+	if (gretl_type_from_name(S[i], p->dset)) {
 	    /* variable already exists */
 	    exists = 1;
 	    p->err = E_DATA;
@@ -3222,7 +3217,7 @@ int check_declarations (char ***pS, parser *p)
 	} else if (!badname) {
 	    gretl_errmsg_set(_("Invalid declaration"));
 	}
-	free_strings_array(S, n);
+	strings_array_free(S, n);
     } else {
 	*pS = S;
     }
@@ -3947,7 +3942,7 @@ gretl_matrix *multi_xcf (const void *px, int xtype,
     int Ty, nx = 1;
     int i, j;
 
-    if (xtype == LVEC) {
+    if (xtype == LIST) {
 	xlist = px;
 	nx = xlist[0];
 	if (nx < 1) {
