@@ -20,8 +20,8 @@
 /* lexer module for 'genr' and related commands */
 
 #include "genparse.h"
-#include "usermat.h"
 #include "gretl_func.h"
+#include "uservar.h"
 #include "gretl_string_table.h"
 
 #include <glib.h>
@@ -53,10 +53,11 @@ struct str_table {
 };
 
 struct str_table consts[] = {
-    { CONST_PI,    "pi" },
+    { CONST_PI,    "$pi" },
     { CONST_NA,    "NA" },
+    { CONST_INF,   "$inf" },
     { CONST_WIN32, "WIN32" },
-    { CONST_EPS,   "macheps" },
+    { CONST_EPS,   "$macheps" },
     { 0,        NULL }
 };
 
@@ -88,6 +89,7 @@ struct str_table dvars[] = {
     { R_VERSION,   "$version" },
     { R_ERRNO,     "$error" },
     { R_SEED,      "$seed" },
+    { R_HUGE,      "$huge" },
     { 0,           NULL },
 };
 
@@ -294,7 +296,6 @@ struct str_table funcs[] = {
     { F_NRMAX,    "NRmax" },
     { F_OBSNUM,   "obsnum" },
     { F_ISSERIES, "isseries" },
-    { F_ISSCALAR, "isscalar" },
     { F_ISLIST,   "islist" },
     { F_ISSTRING, "isstring" },
     { F_ISNULL,   "isnull" },
@@ -306,6 +307,7 @@ struct str_table funcs[] = {
     { F_CRIT,     "critical" },
     { F_RANDGEN,  "randgen" },
     { F_MRANDGEN, "mrandgen" },
+    { F_RANDGEN1, "randgen1" },
     { F_URCPVAL,  "urcpval" },
     { F_VALUES,   "values" },
     { F_UNIQ,     "uniq" },
@@ -385,6 +387,10 @@ struct str_table funcs[] = {
     { F_SIMANN,   "simann" },     /* simulated annealing */
     { F_LOESS,    "loess" },
     { F_FREQ,     "freq" },
+    { F_GHK,      "ghk" },
+    { F_HALTON,   "halton" },
+    { F_ISNAN,    "isnan" },
+    { F_TYPESTR,  "typestr" },
     { 0,          NULL }
 };
 
@@ -408,6 +414,13 @@ int const_lookup (const char *s)
 	}
     }
 
+    /* backward comatibility: to be removed eventually */
+    if (!strcmp(s, "pi")) {
+	return CONST_PI;
+    } else if (!strcmp(s, "macheps")) {
+	return CONST_EPS;
+    }
+
     return 0;
 }
 
@@ -720,7 +733,7 @@ void context_error (int c, parser *p)
 
 static char *get_quoted_string (parser *p)
 {
-    int n = parser_charpos(p, '"');
+    int n = parser_gretl_charpos(p, '"');
     char *s = NULL;
 
     if (n >= 0) {
@@ -784,7 +797,7 @@ NODE *obs_node (parser *p)
     int special = 0;
     int t = -1;
 
-    close = charpos(']', s);
+    close = gretl_charpos(']', s);
 
 #if LDEBUG
     fprintf(stderr, "obs_node: s='%s', ch='%c', close=%d\n", 
@@ -798,7 +811,7 @@ NODE *obs_node (parser *p)
 	pprintf(p->prn, _("Unmatched '%c'\n"), '[');
 	p->err = E_PARSE;
     } else if (*s == '"' && close < OBSLEN + 2 &&
-	       charpos('"', s+1) == close - 2) {
+	       gretl_charpos('"', s+1) == close - 2) {
 	/* quoted observation label? */
 	strncat(word, s, close);
 	special = 1;
@@ -832,14 +845,15 @@ NODE *obs_node (parser *p)
 
 static void look_up_string_variable (const char *s, parser *p)
 {
-    const char *val = get_string_by_name(s + 1);
+    char *val = get_string_by_name(s + 1);
 
     if (val != NULL) {
-	p->idstr = gretl_strdup(s);
+	p->idstr = gretl_strdup(s + 1);
 	if (p->idstr == NULL) {
 	    p->err = E_ALLOC;
 	} else {
-	    p->sym = VSTR;
+	    p->uval = val;
+	    p->sym = USTR;
 	}
     } else {
 	undefined_symbol_error(s, p);
@@ -869,16 +883,14 @@ int is_gretl_accessor (const char *s)
 
 static void look_up_dollar_word (const char *s, parser *p)
 {
-    p->idnum = dvar_lookup(s);
-    if (p->idnum > 0) {
+    if ((p->idnum = dvar_lookup(s)) > 0) {
 	p->sym = DVAR;
+    } else if ((p->idnum = const_lookup(s)) > 0) {
+	p->sym = CON;
+    } else if ((p->idnum = mvar_lookup(s)) > 0) {
+	p->sym = MVAR;
     } else {
-	p->idnum = mvar_lookup(s);
-	if (p->idnum > 0) {
-	    p->sym = MVAR;
-	} else {
-	    undefined_symbol_error(s, p);
-	}
+	undefined_symbol_error(s, p);
     }
 
 #if LDEBUG
@@ -918,35 +930,42 @@ static void look_up_word (const char *s, parser *p)
 	    if (p->idnum > 0) {
 		p->sym = DUM;
 	    } else {
+		GretlType vtype = 0;
+		char *bstr;
+
 		if ((p->idnum = current_series_index(p->dset, s)) >= 0) {
-		    p->sym = USERIES;
+		    p->sym = UVEC;
+		    p->idstr = gretl_strdup(s);
 		} else if (!strcmp(s, "time")) {
 		    p->sym = DUM;
 		    p->idnum = DUM_TREND;
-		} else if (gretl_is_scalar(s)) {
-		    p->sym = USCALAR;
-		    p->idstr = gretl_strdup(s);
-		} else if (get_matrix_by_name(s)) {
-		    p->sym = UMAT;
-		    p->idstr = gretl_strdup(s);
-		} else if (gretl_is_bundle(s)) {
-		    p->sym = BUNDLE;
+		} else if ((p->uval = user_var_get_value_and_type(s, &vtype)) != NULL) {
+		    if (vtype == GRETL_TYPE_DOUBLE) {
+			p->sym = UNUM;
+		    } else if (vtype == GRETL_TYPE_MATRIX) {
+			p->sym = UMAT;
+		    } else if (vtype == GRETL_TYPE_BUNDLE) {
+			p->sym = BUNDLE;
+		    } else if (vtype == GRETL_TYPE_STRING) {
+			p->sym = USTR;
+		    } else if (vtype == GRETL_TYPE_LIST) {
+			p->sym = ULIST;
+		    } else if (vtype == GRETL_TYPE_STRING) {
+			p->sym = USTR;
+		    }
 		    p->idstr = gretl_strdup(s);
+		} else if ((bstr = get_built_in_string_by_name(s))) {
+		    /* FIXME should use $-accessors? */
+		    p->sym = STR;
+		    p->idstr = gretl_strdup(bstr);
 		} else if (gretl_get_object_by_name(s)) {
 		    p->sym = UOBJ;
 		    p->idstr = gretl_strdup(s);
-		} else if (get_list_by_name(s)) {
-		    p->sym = LIST;
-		    p->idstr = gretl_strdup(s);
 		} else if (get_user_function_by_name(s)) {
 		    p->sym = UFUN;
 		    p->idstr = gretl_strdup(s);
-		} else if (gretl_is_string(s)) {
-		    p->sym = VSTR;
-		    p->idstr = gretl_strdup(s);
-		} else if (p->targ == LIST &&
-			   varname_match_any(p->dset, s)) {
-		    p->sym = LIST;
+		} else if (p->targ == LIST && varname_match_any(p->dset, s)) {
+		    p->sym = WLIST;
 		    p->idstr = gretl_strdup(s);
 		} else if (!strcmp(s, "t")) {
 		    /* if "t" has not been otherwise defined, treat it
@@ -977,7 +996,7 @@ static void look_up_word (const char *s, parser *p)
 #define could_be_matrix(t) (model_data_matrix(t) || \
 			    model_data_matrix_builder(t) || \
 			    t == M_UHAT || t == M_YHAT || \
-			    t == R_TEST_STAT || R_TEST_PVAL)
+			    t == R_TEST_STAT || t == R_TEST_PVAL)
 
 static void word_check_next_char (parser *p)
 {
@@ -988,7 +1007,7 @@ static void word_check_next_char (parser *p)
 
     if (p->ch == '(') {
 	/* series (lag) or function */
-	if (p->sym == USERIES) {
+	if (p->sym == UVEC) {
 	    if (p->idnum == p->lh.v) {
 		p->flags |= P_AUTOREG;
 	    }
@@ -1012,9 +1031,15 @@ static void word_check_next_char (parser *p)
 		   could_be_matrix(p->idnum)) {
 	    /* slice of $ matrix */
 	    p->sym = DMSL;
-	} else if (p->sym == USERIES) {
+	} else if (p->sym == UVEC) {
 	    /* observation from series */
 	    p->sym = OBS;
+	} else if (p->sym == ULIST) {
+	    /* element of list */
+	    p->sym = LISTELEM;
+	} else if (p->sym == MVAR && model_data_list(p->idnum)) {
+	    /* element of accessor list */
+	    p->sym = MLISTELEM;
 	} else if (p->sym == BUNDLE) {
 	    /* object from bundle */
 	    p->sym = BOBJ;
@@ -1033,7 +1058,7 @@ static void word_check_next_char (parser *p)
 	    p->err = E_PARSE;
 	}	    
     } else if (p->ch == '.' && isalpha(*p->point)) {
-	if (p->sym == LIST) {
+	if (p->sym == ULIST) {
 	    p->sym = LISTVAR;
 	} else {
 	    p->err = E_PARSE;
@@ -1100,6 +1125,7 @@ static void getword (parser *p)
     if ((*word == '$' && word[1]) || !strcmp(word, "obs")) {
 	look_up_dollar_word(word, p);
     } else if (*word == '@') {
+	/* do we actually want to do this? */
 	look_up_string_variable(word, p);
     } else if (*word == '$' && word[1] == '\0' && p->ch == '[') {
 	p->sym = BUNDLE;
@@ -1436,10 +1462,18 @@ void lex (parser *p)
 	    } else if (p->ch == '>') {
 		p->sym = B_DOTGT;
 		parser_getc(p);
+		if (p->ch == '=') {
+		    p->sym = B_DOTGTE;
+		    parser_getc(p);
+		}
 		return;
 	    } else if (p->ch == '<') {
 		p->sym = B_DOTLT;
 		parser_getc(p);
+		if (p->ch == '=') {
+		    p->sym = B_DOTLTE;
+		    parser_getc(p);
+		}
 		return;
 	    } else if (p->ch == '.') {
 		p->sym = B_ELLIP;
@@ -1516,26 +1550,32 @@ const char *getsymb (int t, const parser *p)
 	return "MDEF";
     } else if (t == FARGS) {
 	return "FARGS";
-    } else if (t == LIST) {
+    } else if (t == LIST || t == ULIST || t == WLIST) {
 	return "LIST";
     } else if (t == OVAR) {
 	return "OVAR";
-    } else if (t == VSTR) {
-	return "VSTR";
+    } else if (t == USTR) {
+	return "USTR";
     } else if (t == EMPTY) {
 	return "EMPTY";
     } else if (t == LISTVAR) {
 	return "LISTVAR";
     } else if (t == BOBJ) {
 	return "BOBJ";
+    } else if (t == LISTELEM) {
+	return "LISTELEM";
+    } else if (t == VEC) {
+	return "VEC";
+    } else if (t == MAT) {
+	return "MAT";
     }
 
     if (p != NULL) {
 	if (t == NUM) {
 	    return fromdbl(p->xval); 
-	} else if (t == USERIES) {
+	} else if (t == UVEC) {
 	    return p->dset->varname[p->idnum];
-	} else if (t == USCALAR) {
+	} else if (t == UNUM) {
 	    return p->idstr;
 	} else if (t == BUNDLE) {
 	    return p->idstr;
@@ -1554,7 +1594,35 @@ const char *getsymb (int t, const parser *p)
 	} else if (t == STR) {
 	    return p->idstr;
 	}
-    } 
+    } else {
+	if (t == NUM) {
+	    return "NUM"; 
+	} else if (t == UVEC) {
+	    return "UVEC";
+	} else if (t == UNUM) {
+	    return "UNUM";
+	} else if (t == BUNDLE) {
+	    return "BUNDLE";
+	} else if (t == UMAT) {
+	    return "UMAT";
+	} else if (t == UOBJ) {
+	    return "UOBJ";
+	} else if (t == CON) {
+	    return "CON";
+	} else if (t == DUM) {
+	    return "DUM";
+	} else if (t == DVAR) {
+	    return "DVAR";
+	} else if (t == MVAR) {
+	    return "MVAR";
+	} else if (t == UFUN) {
+	    return "UFUN";
+	} else if (t == RFUN) {
+	    return "RFUN";
+	} else if (t == STR) {
+	    return "STR";
+	}
+    }	
 
     switch (t) {
     case B_ASN:
@@ -1627,6 +1695,10 @@ const char *getsymb (int t, const parser *p)
 	return ".>";
     case B_DOTLT: 
 	return ".<";
+    case B_DOTGTE: 
+	return ".>=";
+    case B_DOTLTE: 
+	return ".<=";
     case B_KRON: 
 	return "**";
     case B_HCAT: 
@@ -1634,7 +1706,7 @@ const char *getsymb (int t, const parser *p)
     case B_VCAT: 
 	return "|";
     case B_LCAT:
-	return " ";
+	return "LCAT";
     case P_COM: 
 	return ",";
     case P_DOT: 
