@@ -103,6 +103,7 @@ static int maybe_record_lag_info (parser *p)
     const char *s = p->input;
     int n = strlen(p->lh.name);
     char vname[VNAMELEN];
+    char fmt[16];
     int lag;
 
     if (!strncmp(s, "genr ", 5)) {
@@ -120,16 +121,17 @@ static int maybe_record_lag_info (parser *p)
 	s += strspn(s, " ");
     }
 
-    if (sscanf(s, "%15[^ ()](%d)", vname, &lag) == 2) {
+    sprintf(fmt, "%%%d[^ ()](%%d)", VNAMELEN-1);
+
+    if (sscanf(s, fmt, vname, &lag) == 2) {
 	s = strchr(s, ')');
 	if (s != NULL && string_is_blank(s + 1)) {
 	    int pv = series_index(p->dset, vname);
 
 	    if (pv < p->dset->v) {
-		strcpy(p->dset->varinfo[p->lh.v]->parent, 
-		   p->dset->varname[pv]);
-		p->dset->varinfo[p->lh.v]->transform = LAGS;
-		p->dset->varinfo[p->lh.v]->lag = -lag;
+		series_set_parent(p->dset, p->lh.v, p->dset->varname[pv]);
+		series_set_transform(p->dset, p->lh.v, LAGS);
+		series_set_lag(p->dset, p->lh.v, -lag);
 	    }
 	}
     }
@@ -182,11 +184,11 @@ static void gen_write_label (parser *p, int oldv)
 	strncat(tmp, src, MAXLABEL - 1);
     }
 
-    strcpy(VARLABEL(p->dset, p->lh.v), tmp);
-    p->dset->varinfo[p->lh.v]->flags |= VAR_GENERATED;
+    series_set_label(p->dset, p->lh.v, tmp);
+    series_set_flag(p->dset, p->lh.v, VAR_GENERATED);
 
 #if GDEBUG
-    fprintf(stderr, "varlabel: '%s'\n", VARLABEL(p->dset, p->lh.v));
+    fprintf(stderr, "varlabel: '%s'\n", series_get_label(p->dset, p->lh.v));
 #endif
 }
 
@@ -227,7 +229,6 @@ int function_from_string (const char *s)
 static const char *reswords[] = {
     /* constants */
     "const",
-    "CONST",
     "pi",
     "NA",
     "null",
@@ -307,8 +308,11 @@ static int try_for_listvar (const DATASET *dset, const char *s)
 {
     char vname[VNAMELEN];
     char lname[VNAMELEN];
+    char fmt[16];
 
-    if (sscanf(s, "%15[^.].%15s", lname, vname) == 2) {
+    sprintf(fmt, "%%%d[^.].%%%ds", VNAMELEN-1, VNAMELEN-1);
+
+    if (sscanf(s, fmt, lname, vname) == 2) {
 	int *list = get_list_by_name(lname);
 
 	if (list != NULL) {
@@ -380,8 +384,8 @@ int series_index (const DATASET *dset, const char *varname)
 	       that was passed as an argument.
 	    */
 	    for (i=1; i<dset->v; i++) { 
-		if (fd == STACK_LEVEL(dset, i) &&
-		    !var_is_listarg(dset, i) && 
+		if (fd == series_get_stack_level(dset, i) &&
+		    !series_is_listarg(dset, i) && 
 		    strcmp(dset->varname[i], s) == 0) {
 		    ret = i;
 		    break;
@@ -404,7 +408,7 @@ int current_series_index (const DATASET *dset, const char *vname)
 {
     int v = -1;
 
-    if (dset != NULL && vname != NULL) {
+    if (dset != NULL && vname != NULL && *vname != '\0') {
 	v = series_index(dset, vname);
 	if (v >= dset->v) {
 	    v = -1;
@@ -800,16 +804,13 @@ int *generate_list (const char *s, DATASET *dset, int *err)
 
     if (!*err) {
 	if (p.ret->t == LIST) {
-	    const int *nlist = get_list_by_name(p.ret->v.str);
+	    const int *nlist = p.ret->v.ivec;
 
 	    if (nlist == NULL) {
 		*err = E_DATA;
 	    } else {
 		ret = gretl_list_copy(nlist);
 	    }
-	} else if (p.ret->t == LVEC) {
-	    ret = p.ret->v.ivec;
-	    p.ret->v.ivec = NULL;
 	} else {
 	    *err = E_TYPES;
 	}
@@ -870,8 +871,8 @@ parser *genr_compile (const char *s, DATASET *dset,
 int execute_genr (parser *p, DATASET *dset, PRN *prn)
 {
 #if GDEBUG
-    fprintf(stderr, "\n*** execute_genr: p=%p, LHS='%s', Z=%p\n", 
-	    (void *) p, p->lh.name, (void *) dset->Z);
+    fprintf(stderr, "\n*** execute_genr: p=%p, LHS='%s', Z=%p, prn=%p\n", 
+	    (void *) p, p->lh.name, (void *) dset->Z, (void *) prn);
 #endif
 
     realgen(NULL, p, dset, prn, P_EXEC);
