@@ -20,9 +20,8 @@
 /* shared private header for all 'genr' related modules */
 
 #include "libgretl.h"
-#include "usermat.h"
+#include "uservar.h"
 #include "gretl_func.h"
-#include "gretl_scalar.h"
 #include "gretl_bundle.h"
 
 #define GENDEBUG 0
@@ -60,6 +59,8 @@ enum {
               B_DOTEQ,
               B_DOTGT,
   /* 30 */    B_DOTLT,
+	      B_DOTGTE,
+	      B_DOTLTE,
               B_DOTASN,
               B_KRON,     /* Kronecker product */
               B_HCAT,     /* horizontal concatenation */
@@ -67,9 +68,9 @@ enum {
 	      B_LCAT,     /* list concatentation */
 	      B_LDIV,     /* matrix left division */
 	      B_ELLIP,    /* list-generating ellipsis */
-	      B_JOIN,     /* list-joining with separator */
+  /* 40 */    B_JOIN,     /* list-joining with separator */
 	      OP_MAX,     /* SEPARATOR: end of binary operators */
-  /* 40 */    G_LPR,      /* grouping: left paren */
+              G_LPR,      /* grouping: left paren */
 	      G_RPR,      /* right paren */
               G_LBR,      /* left bracket */
               G_RBR,      /* right bracket */
@@ -77,19 +78,18 @@ enum {
 	      G_RCB,      /* right curly bracket */  
 	      P_COM,	  /* punctuation: comma */
 	      P_DOT,	  /* period */
-	      P_SEMI,	  /* semi-colon */
+  /* 50 */    P_SEMI,	  /* semi-colon */
 	      P_COL,	  /* colon */
-  /* 50 */    PUNCT_MAX,  /* SEPARATOR: end of grouping and punctuation marks */
+              PUNCT_MAX,  /* SEPARATOR: end of grouping and punctuation marks */
 	      CON,	  /* named constant */
 	      DUM,	  /* dummy variable */
-              USCALAR,	  /* user variable, scalar */
-              USERIES,    /* user variable, series */
-	      UMAT,	  /* user-defined matrix */
+              UNUM,	  /* user variable, named scalar */
+              UVEC,       /* user variable, named series */
+	      UMAT,	  /* user variable, named matrix */
+	      ULIST,      /* user variable, named list */
 	      UOBJ,	  /* user-defined object (e.g. model) */
-	      NUM,	  /* scalar, evaluated */
+  /* 60 */    NUM,	  /* scalar, evaluated */
 	      VEC,	  /* series, evaluated */
-	      IVEC,	  /* vector of integers, evaluated */
-  /* 60 */    LVEC,       /* list, as array of ints */
 	      MAT,	  /* matrix, evaluated */
 	      OBS,	  /* observation from a series */
               MSL,	  /* matrix plus subspec */
@@ -98,28 +98,32 @@ enum {
 	      MSL2,	  /* unevaluated matrix subspec */
 	      MSPEC,	  /* evaluated matrix subspec */
 	      SUBSL,	  /* row or column component of MSPEC */
-	      MDEF,	  /* explicit matrix definition {...} */
-  /* 70 */    LAG,        /* variable plus lag length */	  
+  /* 70 */    MDEF,	  /* explicit matrix definition {...} */
+              LAG,        /* variable plus lag length */	  
 	      DVAR,	  /* $ "dataset" variable (mostly scalar or series) */
 	      MVAR,	  /* $ model var (scalar, series, or matrix) */
               OVAR,	  /* object variable: variable "under" an object */
-              LIST,	  /* reference to named list */
-	      LISTVAR,    /* variable in list */
+              LIST,	  /* list, evaluated */
+	      LISTVAR,    /* variable in list, dot syntax */
+	      LISTELEM,   /* list member, [...] syntax) */
+	      MLISTELEM,  /* accessor list member, [...] syntax) */
 	      STR,	  /* string */
-	      BUNDLE,     /* gretl bundle (hash table) */
-	      BOBJ,       /* object inside a bundle */
+  /* 80 */    BUNDLE,     /* gretl bundle (hash table) */
+              BOBJ,       /* object inside a bundle */
 	      FARGS,	  /* set of n function arguments */
-  /* 80 */    EMPTY,      /* "null" */
+              EMPTY,      /* "null" */
 	      ABSENT,
 	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
               EROOT,	  /* dummy root for (...) expression */
               UFUN,	  /* user-defined function */
 	      RFUN,       /* GNU R function */
-              VSTR,       /* string variable */
+	      USTR,       /* string variable */
+  /* 90 */    WLIST,      /* wildcard list spec */
+	      IVEC,       /* array of ints, not a varlist */
               INC,   
-	      DEC,
+              DEC,
 	      QUERY,
-  /* 90 */    EOT,	  /* end of transmission */
+              EOT,	  /* end of transmission */
 	      UNK 
 };
 
@@ -211,7 +215,6 @@ enum {
     F_RANK, 
     F_OBSNUM,
     F_ISSERIES,
-    F_ISSCALAR,
     F_ISLIST,	 
     F_ISSTRING,
     F_ISNULL,
@@ -224,6 +227,7 @@ enum {
     F_URCPVAL,
     F_RANDGEN,
     F_MRANDGEN,
+    F_RANDGEN1,
     F_VALUES,
     F_UNIQ,
     F_NULLSPC,
@@ -260,6 +264,8 @@ enum {
     F_NGETENV,
     F_PSHRINK,
     F_FREQ,
+    F_ISNAN,
+    F_TYPESTR,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -350,6 +356,7 @@ enum {
     F_EIGSOLVE,
     F_NADARWAT,
     F_SIMANN,
+    F_HALTON,
     F3_MAX,       /* SEPARATOR: end of three-arg functions */
     F_BKFILT,
     F_MOLS,
@@ -362,12 +369,14 @@ enum {
     F_KSIMUL,
     F_NRMAX,
     F_LOESS,
+    F_GHK,
     FN_MAX,	  /* SEPARATOR: end of n-arg functions */
 };
 
 enum {
     CONST_PI = 1,
     CONST_NA,
+    CONST_INF,
     CONST_WIN32,
     CONST_EPS
 };
@@ -396,7 +405,7 @@ enum {
 /* function with multiple args, character code for first arg */
 #define char0_func(s) (s == F_PVAL || s == F_CDF || s == F_INVCDF || \
 		       s == F_CRIT || s == F_RANDGEN || s == F_PDF ||	\
-		       s == F_BESSEL || s == F_MRANDGEN)
+		       s == F_BESSEL || s == F_MRANDGEN || s == F_RANDGEN1)
 
 /* functions taking a string arg in last position */
 #define string_last_func(s) (s == F_FDJAC || s == F_BFGSMAX || \
@@ -413,7 +422,7 @@ enum {
 
 #define evalb2(s) (binary_op(s) || func2_symb(s) || s == MSL || \
                    s == MSL2 || s == SUBSL || s == LAG || \
-                   s == OBS || s == BOBJ)
+                   s == OBS || s == BOBJ || s == LISTELEM)
 
 #define b1sym(s) (unary_op(s) || func1_symb(s) || funcn_symb(s) || \
                   s == G_LPR || s == EROOT)
@@ -429,9 +438,6 @@ enum {
 
 #define bnsym(s) (s == MDEF || s == FARGS)
 
-#define freestr(s) (s == STR || s == USCALAR || s == UMAT || s == UOBJ || \
-                    s == LIST || s == VSTR)
-
 #define bare_data_type(s) (s > PUNCT_MAX && s < DTYPE_MAX)
 
 #define closing_sym(s) (s == G_RPR || s == G_RBR || s == G_RCB)
@@ -494,7 +500,8 @@ enum {
 struct node {
     short t;       /* type indentifier */
     char flags;    /* AUX_NODE etc., see above */
-    int vnum;      /* associated variable ID number */
+    int vnum;      /* associated series ID number */
+    char *vname;   /* associated variable name */
     union val v;   /* value (of whatever type) */
 };
 
@@ -568,6 +575,7 @@ struct parser_ {
     double xval;
     int idnum;
     char *idstr;
+    void *uval;
     int err;
 };
 
@@ -578,10 +586,11 @@ struct parser_ {
 int parser_getc (parser *p);
 void parser_ungetc (parser *p);
 void parser_advance (parser *p, int n);
-int parser_charpos (parser *p, int c);
+int parser_gretl_charpos (parser *p, int c);
 int parser_next_nonspace_char (parser *p);
 void parser_print_input (parser *p);
 void lex (parser *s);
+NODE *new_node (int t);
 NODE *expr (parser *s);
 NODE *newdbl (double x);
 NODE *obs_node (parser *p);
