@@ -24,8 +24,6 @@
 
 #include <stdio.h>
 
-#define GRAPH_NO_DATA -999
-
 typedef enum {
     GPT_IMPULSES       = 1 << 0,  /* use impulses for plotting */
     GPT_LINES          = 1 << 1,  /* force use of lines for plotting */
@@ -168,7 +166,8 @@ typedef enum {
 				c == PLOT_FREQ_GAMMA)
 
 #define set_png_output(p) (p->flags |= GPT_PNG_OUTPUT)
-#define get_png_output(p) (p->flags & GPT_PNG_OUTPUT) 
+#define get_png_output(p) (p->flags & GPT_PNG_OUTPUT)
+#define unset_png_output(p) (p->flags &= ~GPT_PNG_OUTPUT) 
     
 const char *get_gretl_png_term_line (PlotType ptype, GptFlags flags);
 
@@ -202,6 +201,8 @@ void print_plot_bounding_box_request (FILE *fp);
 
 PlotType plot_type_from_string (const char *str);
 
+void plot_get_scaled_dimensions (int *width, int *height, double scale);
+
 int gnuplot_make_graph (void);
 
 int graph_written_to_file (void);
