@@ -18,12 +18,12 @@
  */
 
 #include "libgretl.h"
+#include "libset.h"
 #include "gretl_foreign.h"
 
 #include <glib.h>
 
 #ifdef USE_RLIB
-# include "libset.h"
 # include <Rinternals.h> /* for SEXP and friends */
 #endif
 
@@ -64,7 +64,7 @@ enum {
 
 static void destroy_foreign (void)
 {
-    free_strings_array(foreign_lines, foreign_n_lines);
+    strings_array_free(foreign_lines, foreign_n_lines);
     foreign_lines = NULL;
     foreign_started = 0;
     foreign_n_lines = 0;
@@ -221,7 +221,7 @@ static char *win32_dotpath (void)
 {
     gchar *dotpath = g_strdup(gretl_dotdir());
 
-    charsub(dotpath, '\\', '/');
+    gretl_charsub(dotpath, '\\', '/');
     return dotpath;
 }
 
@@ -572,10 +572,12 @@ int write_gretl_ox_file (const char *buf, gretlopt opt, const char **pfname)
 static int write_data_for_stata (const DATASET *dset,
 				 FILE *fp)
 {
-    const char *save_na = get_csv_na_string();
+    char save_na[8];
     gchar *sdata;
     int err;
 
+    *save_na = '\0';
+    strncat(save_na, get_csv_na_string(), 7);
     set_csv_na_string(".");
     sdata = g_strdup_printf("%sstata.csv", gretl_dotdir());
     err = write_data(sdata, NULL, dset, OPT_C, 0);
