@@ -31,11 +31,11 @@ static char *numeric_locale = NULL;
 /**
  * gretl_push_c_numeric_locale:
  *
- * Description:  Saves the current %LC_NUMERIC locale and sets it to "C"
+ * Description: Saves the current %LC_NUMERIC locale and sets it to "C".
  * This way you can safely read write floating point numbers all in the
- * same format.  You should make sure that code between
- * gretl_push_c_numeric_locale() and gretl_pop_c_numeric_locale()
- * doesn't do any setlocale calls or locale may end up in a strange setting.
+ * same format, using '.' as the decimal character.  You should make sure 
+ * that code between gretl_push_c_numeric_locale() and gretl_pop_c_numeric_locale()
+ * doesn't do any setlocale calls, or locale may end up in a strange setting.
  * Also make sure to always pop the C numeric locale after you've pushed it.
  * The calls can be nested.
  **/
@@ -198,7 +198,7 @@ void set_gretl_charset (void)
 	char *p;
 
 	strncat(gretl_charset, charset, 31);
-	lower(gretl_charset);
+	gretl_lower(gretl_charset);
 	p = strstr(gretl_charset, "iso");
 	if (p != NULL) {
 	    char numstr[6] = {0};
@@ -708,7 +708,7 @@ set_locale_with_workaround (int langid, const char *lcode)
 	    
 	    langstr = lang_string_from_id(langid);
 	    sscanf(langstr, "%s", tmp);
-	    lower(tmp);
+	    gretl_lower(tmp);
 	    gretl_setenv("LANGUAGE", tmp);
 	}
     }
