@@ -59,10 +59,6 @@ typedef struct matrix_info_ matrix_info;
  * gretl_matrix:
  * @rows: number of rows in matrix
  * @cols: number of columns
- * @t1: observation number corresponding to first row (if
- * applicable, otherwise 0)
- * @t2: observation number corresponding to last row (if
- * applicable, otherwise 0)
  * @val: flat array of double-precision values
  *
  * The basic libgretl matrix type; #gretl_vector is an alias
@@ -73,6 +69,7 @@ typedef struct gretl_matrix_ {
     int rows;
     int cols;
     double *val;
+    /*< private >*/
     matrix_info *info;
 } gretl_matrix;
 
@@ -149,8 +146,8 @@ typedef struct gretl_matrix_block_ gretl_matrix_block;
  */
 
 #define gretl_vector_get_length(v) ((v == NULL)? 0 : \
-                                    (v->cols == 1)? v->rows : \
-                                    (v->rows == 1)? v->cols : 0)
+				    ((v)->cols == 1)? (v)->rows :	\
+				    ((v)->rows == 1)? (v)->cols : 0)
 
 /**
  * gretl_vector_alloc:
@@ -218,6 +215,10 @@ int gretl_matrix_realloc (gretl_matrix *m, int rows, int cols);
 
 void gretl_matrix_init (gretl_matrix *m);
 
+void gretl_matrix_init_full (gretl_matrix *m,
+			     int rows, int cols,
+			     double *val);
+
 gretl_matrix *gretl_matrix_replace (gretl_matrix **pa, 
 				    gretl_matrix *b);
 
