@@ -58,7 +58,6 @@ struct INTERNAL_PATHS {
     char plotfile[MAXLEN];
     char libpath[MAXLEN];
     char binbase[MAXLEN];
-    char ratsbase[MAXLEN];
     char helpfile[MAXLEN];
     char cmd_helpfile[MAXLEN];
     char cli_helpfile[MAXLEN];
@@ -272,7 +271,7 @@ static FILE *win32_mktemp (char *tmpl, const char *mode)
 
 /**
  * gretl_mktemp:
- * @template: template for filename; must end with "XXXXXX".
+ * @pattern: template for filename; must end with "XXXXXX".
  * @mode: e.g. "w" for text use or "wb" for binary mode.
  *
  * A wrapper for the combination of mkstemp() and fdopen(),
@@ -284,7 +283,7 @@ static FILE *win32_mktemp (char *tmpl, const char *mode)
  * Returns: file pointer, or %NULL on failure.
  */
 
-FILE *gretl_mktemp (char *template, const char *mode)
+FILE *gretl_mktemp (char *pattern, const char *mode)
 {
     FILE *fp = NULL;
     int fd = -1;
@@ -292,9 +291,9 @@ FILE *gretl_mktemp (char *template, const char *mode)
     gretl_error_clear();
 
 #ifdef G_OS_WIN32
-    fp = win32_mktemp(template, mode);
+    fp = win32_mktemp(pattern, mode);
 #else
-    fd = mkstemp(template); 
+    fd = mkstemp(pattern); 
 #endif
 
     if (errno != 0) {
@@ -305,7 +304,7 @@ FILE *gretl_mktemp (char *template, const char *mode)
 
 #if 0
     fprintf(stderr, "gretl_mktemp: name='%s', fd=%d, fp=%p\n",
-	    template, fd, (void *) fp);
+	    pattern, fd, (void *) fp);
 #endif
 
     return fp;
@@ -667,11 +666,23 @@ static gzFile gretl_try_gzopen (const char *fname, const char *mode)
 
 int gretl_chdir (const char *path)
 {
+#ifdef WIN32
+    char *ptmp = NULL;
+    int len = strlen(path);
+#endif
     gchar *pconv = NULL;
     int err;
 
     gretl_error_clear();
 
+#ifdef WIN32
+    if (len > 1 && path[len - 1] == '\\' && path[len - 2] != ':') {
+	/* trim trailing slash for non-root dir */
+	ptmp = gretl_strndup(path, len - 1);
+	path = ptmp;
+    }
+#endif
+
     err = maybe_recode_path(path, stdio_use_utf8, &pconv);
 
     if (!err) {
@@ -686,6 +697,10 @@ int gretl_chdir (const char *path)
     if (errno != 0) {
 	gretl_errmsg_set_from_errno("chdir");
     }
+
+#ifdef WIN32
+    free(ptmp);
+#endif
     
     return err;
 }
@@ -1246,8 +1261,8 @@ static char *fname_strstr (char *fname, char *dname)
     *lfname = *ldname = '\0';
     strncat(lfname, fname, MAXLEN - 1);
     strncat(ldname, dname, MAXLEN - 1);
-    lower(lfname);
-    lower(ldname);
+    gretl_lower(lfname);
+    gretl_lower(ldname);
     return strstr(lfname, ldname);
 #else
     return strstr(fname, dname);
@@ -1318,7 +1333,7 @@ static int shelldir_open_dotfile (char *fname, char *orig)
  * 
  * Returns: an array of plausible search paths, depending on the 
  * @type of search. The array should be freed when you are done 
- * with it, using free_strings_array().
+ * with it, using strings_array_free().
  */
 
 char **get_plausible_search_dirs (int type, int *n_dirs)
@@ -1473,7 +1488,7 @@ char *gretl_function_package_get_path (const char *name,
 	closedir(dir);
     }
 
-    free_strings_array(dirs, n_dirs);
+    strings_array_free(dirs, n_dirs);
 
     if (*path != '\0') {
 	ret = gretl_strdup(path);
@@ -1766,7 +1781,7 @@ int getopenfile (const char *line, char *fname, gretlopt opt)
     fullname = gretl_addpath(fname, script);
 
     if (fullname != NULL && (opt & OPT_S)) {
-	int spos = slashpos(fname);
+	int spos = gretl_slashpos(fname);
 
 	if (spos) {
 	    *current_dir = '\0';
@@ -1841,12 +1856,17 @@ static void set_gretl_binbase (const char *path)
 /* This should be called after we're fairly confident that we
    have the dotdir setting right */
 
-static int set_tramo_x12a_dirs (void)
+static int set_extra_dot_paths (void)
 {
     char dirname[MAXLEN];
     size_t n;
     int err = 0;
 
+    /* the personal function package directory */
+    *dirname = '\0';
+    build_path(dirname, paths.dotdir, "functions", NULL);
+    gretl_mkdir(dirname);
+
     *paths.tramodir = '\0';
     *paths.x12adir = '\0';
 
@@ -1854,6 +1874,7 @@ static int set_tramo_x12a_dirs (void)
     return 0;
 #endif
 
+    *dirname = '\0';
     strcpy(dirname, paths.dotdir);
     n = strlen(dirname);
 
@@ -2175,11 +2196,6 @@ const char *gretl_binbase (void)
     return paths.binbase;
 }
 
-const char *gretl_ratsbase (void)
-{
-    return paths.ratsbase;
-}
-
 const char *gretl_tramo (void)
 {
     return paths.tramo;
@@ -2245,7 +2261,7 @@ const char *gretl_current_dir (void)
 
 void gretl_set_current_dir (const char *s)
 {
-    int spos = slashpos(s);
+    int spos = gretl_slashpos(s);
 
     if (spos) {
 	*current_dir = '\0';
@@ -2570,7 +2586,7 @@ static int initialize_dotdir (void)
 	*paths.tramodir = '\0';
     } else {
 	/* these paths depend on dotdir */
-	err = set_tramo_x12a_dirs();
+	err = set_extra_dot_paths();
     }
 
     return err;
@@ -2610,8 +2626,7 @@ static int maybe_transcribe_path (char *targ, char *src,
 
    gretldir
    gnuplot (but not on MS Windows)
-   tramo, x12a, rbinpath, rlibpath, oxlpath, octpath,
-   ratsbase, dbhost
+   tramo, x12a, rbinpath, rlibpath, oxlpath, octpath, dbhost
 
    * paths.workdir is updated via the separate working directory
      dialog
@@ -2630,8 +2645,7 @@ int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt)
 	ndelta++;
     }
     
-    /* databases, native and RATS */
-    maybe_transcribe_path(paths.ratsbase, cpaths->ratsbase, 1);
+    /* native databases */
     maybe_transcribe_path(paths.dbhost, cpaths->dbhost, 0);
 
 #ifndef WIN32
@@ -2691,8 +2705,6 @@ static void load_default_path (char *targ)
 
     if (targ == paths.workdir) {
 	load_default_workdir(targ);
-    } else if (targ == paths.ratsbase) {
-	strcpy(targ, "f:\\"); 
     } else if (targ == paths.dbhost) {
 	strcpy(targ, "ricardo.ecn.wfu.edu");
     } else if (targ == paths.x12a) {
@@ -2744,8 +2756,6 @@ static void load_default_path (char *targ)
 {
     if (targ == paths.workdir) {
 	load_default_workdir(targ);
-    } else if (targ == paths.ratsbase) {
-	strcpy(targ, "/mnt/dosc/userdata/rats/oecd/");
     } else if (targ == paths.dbhost) {
 	strcpy(targ, "ricardo.ecn.wfu.edu");
     } else if (targ == paths.gnuplot) {
@@ -2832,9 +2842,6 @@ static void copy_paths_with_fallback (ConfigPaths *cpaths)
     /* working directory */
     path_init(paths.workdir, cpaths->workdir, 1);
 
-    /* RATS databases */
-    path_init(paths.ratsbase, cpaths->ratsbase, 1);
-
     /* database server */
     path_init(paths.dbhost, cpaths->dbhost, 0);
 
@@ -3208,7 +3215,7 @@ int cli_read_rc (void)
 	    *val = '\0';
 	    /* get the string that follows " = " */ 
 	    strncat(val, line + strlen(key) + 3, MAXLEN - 1);
-	    chopstr(val); 
+	    gretl_chopstr(val); 
 	    if (!strcmp(key, "gretldir")) {
 		strncat(cpaths.gretldir, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "gnuplot")) {
@@ -3220,8 +3227,6 @@ int cli_read_rc (void)
 	    } else if (!strcmp(key, "usecwd")) {
 		usecwd = rc_bool(val);
 		libset_set_bool(USE_CWD, usecwd);
-	    } else if (!strcmp(key, "ratsbase")) {
-		strncat(cpaths.ratsbase, val, MAXLEN - 1);
 	    } else if (!strcmp(key, "dbhost")) {
 		strncat(cpaths.dbhost, val, 32 - 1);
 	    } else if (!strcmp(key, "dbproxy")) {
