@@ -21,11 +21,10 @@
 #include "system.h"
 #include "var.h"
 #include "objstack.h"
-#include "usermat.h"
+#include "uservar.h"
 #include "matrix_extra.h"
 #include "gretl_restrict.h"
 #include "bootstrap.h"
-#include "gretl_scalar.h"
 #include "gretl_func.h"
 #include "gretl_bfgs.h"
 #include "cmd_private.h"
@@ -778,8 +777,8 @@ static int pick_apart (gretl_restriction *r, const char *s,
 		       const DATASET *dset)
 {
     const char *junk;
-    char s1[16] = {0};
-    char s2[16] = {0};
+    char s1[VNAMELEN] = {0};
+    char s2[VNAMELEN] = {0};
     char *targ = s1;
     int i, j, k;
     int err = 0;
@@ -790,8 +789,8 @@ static int pick_apart (gretl_restriction *r, const char *s,
 
     *eq = *bnum = -1;
 
-    k = charpos(']', s);
-    if (k <= 0 || k > 30) {
+    k = gretl_charpos(']', s);
+    if (k <= 0 || k > 2*(VNAMELEN-1)) {
 	return E_PARSE;
     }
     
@@ -808,7 +807,7 @@ static int pick_apart (gretl_restriction *r, const char *s,
 	    targ = s2;
 	    j = 0;
 	} else if (!isspace(s[i])) {
-	    if (j == 15) {
+	    if (j == VNAMELEN-1) {
 		return E_PARSE;
 	    }
 	    targ[j++] = s[i];
@@ -1512,7 +1511,7 @@ static int read_matrix_line (const char *s, gretl_restriction *rset)
 
     s++;
     while (isspace((unsigned char) *s)) s++;
-    if (sscanf(s, "%15s", mname) != 1) {
+    if (gretl_scan_varname(s, mname) != 1) {
 	return E_PARSE;
     }
 
@@ -1918,7 +1917,7 @@ static int print_restricted_estimates (MODEL *pmod,
 
     se = malloc(nc * sizeof *se);
     if (se == NULL) {
-	free_strings_array(names, nc);
+	strings_array_free(names, nc);
 	return E_ALLOC;
     }
 
@@ -1926,7 +1925,11 @@ static int print_restricted_estimates (MODEL *pmod,
 
     for (i=0; i<nc; i++) {
 	v = gretl_matrix_get(S, i, i);
-	se[i] = (v > 1.0e-16)? sqrt(v) : 0.0;
+	if (fabs(v) < 1.0e-16) {
+	    se[i] = 0.0;
+	} else {
+	    se[i] = sqrt(v);
+	}
 	gretl_model_get_param_name(pmod, dset, i, names[i]);
     }
 
@@ -1937,7 +1940,7 @@ static int print_restricted_estimates (MODEL *pmod,
     pprintf(prn, "  %s = %.*g\n", _("Standard error of the regression"), 
 	    GRETL_DIGITS, sqrt(s2));
     
-    free_strings_array(names, nc);
+    strings_array_free(names, nc);
     free(se);
 
     return 0;
@@ -2040,6 +2043,7 @@ static int save_restricted_model (ExecState *state,
 	for (i=0; i<rmod->ncoeff; i++) {
 	    rmod->coeff[i] = b->val[i];
 	}
+	gretl_model_set_int(rmod, "restricted", 1);
 	err = gretl_model_write_vcv(rmod, S);
     }
 
@@ -2074,12 +2078,20 @@ static int save_restricted_model (ExecState *state,
 	    double tmp;
 
 	    rmod->rsq = 1.0 - (rmod->ess / rmod->tss);
-	    tmp = rmod->tss * rmod->dfd;
-	    rmod->adjrsq = 1 - (rmod->ess * (rmod->nobs - 1) / tmp);
-	    tmp = rmod->dfd / (double) rmod->dfn;
-	    rmod->fstt = tmp * (rmod->tss - rmod->ess) / rmod->ess;
-	    if (rmod->fstt < 0) {
+	    if (rmod->rsq < 0.0) {
+		rmod->rsq = 0.0;
+	    }
+	    if (rmod->dfn == 0) {
+		rmod->adjrsq = 0.0;
 		rmod->fstt = NADBL;
+	    } else {
+		tmp = rmod->tss * rmod->dfd;
+		rmod->adjrsq = 1 - (rmod->ess * (rmod->nobs - 1) / tmp);
+		tmp = rmod->dfd / (double) rmod->dfn;
+		rmod->fstt = tmp * (rmod->tss - rmod->ess) / rmod->ess;
+		if (rmod->fstt < 0) {
+		    rmod->fstt = NADBL;
+		}
 	    }
 	}
 
@@ -2092,7 +2104,6 @@ static int save_restricted_model (ExecState *state,
 	gretl_model_free(rmod);
     } else {
 	set_model_id(rmod);
-	gretl_model_set_int(rmod, "restricted", 1);
 	gretl_exec_state_set_model(state, rmod);
     } 
 
