@@ -22,30 +22,40 @@
 
 typedef struct _gretl_string_table gretl_string_table;
 
-gretl_string_table *gretl_string_table_new (int *err);
+gretl_string_table *gretl_string_table_new (const int *list);
 
-gretl_string_table *string_table_new_from_cols_list (int *list);
+int gretl_string_table_index (gretl_string_table *gst, const char *s, 
+			      int col, int addcol, PRN *prn);
 
-int gretl_string_table_index (gretl_string_table *st, const char *s, int col,
-			      int addcol, PRN *prn);
-
-int gretl_string_table_print (gretl_string_table *st, DATASET *dset,
+int gretl_string_table_print (gretl_string_table *gst, DATASET *dset,
 			      const char *fname, PRN *prn);
 
-void gretl_string_table_destroy (gretl_string_table *st);
+int gretl_string_table_save (gretl_string_table *gst, DATASET *dset);
 
-void gretl_string_table_add_extra (gretl_string_table *st, PRN *prn);
+void gretl_string_table_destroy (gretl_string_table *gst);
 
-int gretl_string_table_reset_column_id (gretl_string_table *st, 
+void gretl_string_table_add_extra (gretl_string_table *gst, PRN *prn);
+
+int gretl_string_table_reset_column_id (gretl_string_table *gst, 
 					int oldid, int newid);
 
+series_table *series_table_new (char **strs, int n_strs);
+
+void series_table_destroy (series_table *st);
+
+double series_table_get_value (series_table *st, const char *s);
+
+const char *series_table_get_string (series_table *st, double val);
+
+int series_table_add_string (series_table *st, const char *s);
+
+const char **series_table_get_strings (series_table *st, int *n_strs);
+
 void gretl_insert_builtin_string (const char *name, const char *s);
 
-int save_named_string (const char *name, const char *s, PRN *prn);
+char *get_built_in_string_by_name (const char *name);
 
-int add_string_as (const char *s, const char *name);
-
-const char *get_string_by_name (const char *name);
+void builtin_strings_cleanup (void);
 
 int process_string_command (const char *line, 
 			    double ***pZ, DATASET *dset,
@@ -53,18 +63,6 @@ int process_string_command (const char *line,
 
 int substitute_named_strings (char *line, int *subst);
 
-int gretl_is_string (const char *sname);
-
-int is_user_string (const char *sname);
-
-void saved_strings_cleanup (void);
-
-int delete_saved_string (const char *name, PRN *prn);
-
-void destroy_user_strings (void);
-
-int destroy_saved_strings_at_level (int d);
-
 char *gretl_getenv (const char *key, int *defined, int *err);
 
 char *retrieve_date_string (int t, const DATASET *dset, int *err);
