@@ -23,14 +23,14 @@
 #include "objstack.h"
 #include "cmd_private.h"
 #include "libset.h"
-#include "usermat.h"
+#include "uservar.h"
 #include "gretl_panel.h"
-#include "gretl_scalar.h"
 #include "gretl_bundle.h"
 #include "gretl_string_table.h"
 #include "gretl_xml.h"
 #include "forecast.h"
 #include "kalman.h"
+#include "gretl_www.h"
 
 #ifdef USE_RLIB
 # include "gretl_foreign.h"
@@ -46,7 +46,7 @@
 static void gretl_tests_cleanup (void);
 
 /**
- * date:
+ * date_as_double:
  * @t: observation number (zero-based).
  * @pd: data periodicity or frequency.
  * @sd0: floating point representation of starting date.
@@ -54,7 +54,7 @@ static void gretl_tests_cleanup (void);
  * Returns: the date corresponding to @t, as a double-precision number.
  */
 
-double date (int t, int pd, double sd0)
+double date_as_double (int t, int pd, double sd0)
 {
     int ysd = (int) sd0, yy, pp, yp;
     int p10 = 10;
@@ -220,6 +220,10 @@ int gretl_isconst (int t1, int t2, const double *x)
 	t1++;
     }
 
+    if (t1 >= t2) {
+        return 0;
+    }
+
     for (t=t1+1; t<=t2; t++) {
 	if (na(x[t])) {
 	    continue;
@@ -1033,6 +1037,11 @@ int set_obs (const char *line, DATASET *dset, gretlopt opt)
 	sd0 = dot_atof(stobs);
     }
 
+    if (structure == TIME_SERIES && (pd == 1 || pd == 4 || pd == 12)) {
+	/* force use of regular time-series obs labels */
+	dataset_destroy_obs_markers(dset);
+    }
+
     dset->pd = pd;
     dset->structure = structure;
     dset->sd0 = sd0;
@@ -1206,26 +1215,16 @@ int varnum_from_string (const char *str, DATASET *dset)
  * @dset: dataset information.
  * 
  * Returns: non-zero if @s is the name of an existing
- * series, matrix, scalar, list or string variable,
+ * series, matrix, scalar, list, string or bundle,
  * otherwise 0.
  */
 
 GretlType gretl_type_from_name (const char *s, const DATASET *dset)
 {
-    if (gretl_is_series(s, dset)) {
+    if (dset != NULL && gretl_is_series(s, dset)) {
 	return GRETL_TYPE_SERIES;
-    } else if (get_matrix_by_name(s) != NULL) {
-	return GRETL_TYPE_MATRIX;
-    } else if (gretl_is_scalar(s)) {
-	return GRETL_TYPE_DOUBLE;
-    } else if (get_list_by_name(s)) {
-	return GRETL_TYPE_LIST;
-    } else if (get_string_by_name(s)) {
-	return GRETL_TYPE_STRING;
-    } else if (gretl_is_bundle(s)) {
-	return GRETL_TYPE_BUNDLE;
     } else {
-	return GRETL_TYPE_NONE;
+	return user_var_get_type_by_name(s);
     }
 }
 
@@ -1571,13 +1570,16 @@ static int maybe_delete_bundle_value (const char *s, PRN *prn)
 {
     char bname[VNAMELEN];
     char key[VNAMELEN];
+    char fmt[16];
     int err = 0;
 
-    if (sscanf(s, "%15[^[][%15[^]]", bname, key) == 2) {
+    sprintf(fmt, "%%%d[^[][%%%d[^]]", VNAMELEN-1, VNAMELEN-1);
+
+    if (sscanf(s, fmt, bname, key) == 2) {
 	gretl_bundle *bundle;
 	const char *s;
 
-	bundle = get_gretl_bundle_by_name(bname);
+	bundle = get_bundle_by_name(bname);
 	if (bundle == NULL) {
 	    err = E_UNKVAR;
 	} else {
@@ -1611,20 +1613,14 @@ int gretl_delete_var_by_name (const char *s, PRN *prn)
     }
 
     if (object_is_function_arg(s)) {
-	gretl_errmsg_sprintf("The variable %s is read-only", s);
+	gretl_errmsg_sprintf(_("The variable %s is read-only"), s);
 	return E_DATA;
     }
 
     if (!strcmp(s, "kalman")) {
-	err = delete_kalman(prn);	
-    } else if (gretl_is_scalar(s)) {
-	err = gretl_scalar_delete(s, prn);
-    } else if (get_matrix_by_name(s)) {
-	err = user_matrix_destroy_by_name(s, prn);
-    } else if (get_string_by_name(s)) {
-	err = delete_saved_string(s, prn);
-    } else if (gretl_is_bundle(s)) {
-	err = gretl_bundle_delete_by_name(s, prn);
+	err = delete_kalman(prn);
+    } else if (gretl_is_user_var(s)) {
+	err = user_var_delete_by_name(s, prn);
     } else {
 	err = maybe_delete_bundle_value(s, prn);
     } 
@@ -1756,20 +1752,14 @@ void libgretl_session_cleanup (int mode)
 {
     gretl_saved_objects_cleanup();
 
-    if (mode != SESSION_CLEAR_OTHER) {
-	/* trash dataset-related items */
-	gretl_transforms_cleanup();
-	gretl_lists_cleanup();
-	gretl_tests_cleanup();
-	gretl_plotx(NULL);
-    }
+    /* trash dataset-related items */
+    gretl_transforms_cleanup();
+    gretl_lists_cleanup();
+    gretl_tests_cleanup();
+    gretl_plotx(NULL);
 
     if (mode != SESSION_CLEAR_DATASET) {
-	/* trash non-dataset items */
-	destroy_user_scalars();
-	destroy_user_matrices();
-	destroy_user_bundles();
-	destroy_user_strings();
+	destroy_user_vars();
     }
 }
 
@@ -1793,13 +1783,14 @@ void libgretl_cleanup (void)
     libset_cleanup();
     gretl_command_hash_cleanup();
     gretl_function_hash_cleanup();
-    saved_strings_cleanup();
     lapack_mem_free();
     forecast_matrix_cleanup();
     option_flags_cleanup();
     kalman_cleanup();
     gnuplot_cleanup();
     bufgets_cleanup();
+    gretl_www_cleanup();
+    builtin_strings_cleanup();
 
 #ifdef USE_RLIB
     gretl_R_cleanup();
