@@ -34,8 +34,7 @@
 
 enum {
     SESSION_CLEAR_ALL,
-    SESSION_CLEAR_DATASET,
-    SESSION_CLEAR_OTHER
+    SESSION_CLEAR_DATASET
 };
 
 void libgretl_init (void);
@@ -44,7 +43,7 @@ void libgretl_session_cleanup (int mode);
 
 void libgretl_cleanup (void);
 
-double date (int t, int pd, double sd0);
+double date_as_double (int t, int pd, double sd0);
 
 /* checks on variables */
 
