@@ -163,7 +163,7 @@ int read_rc_string (FILE *fp, const char *key, char *value)
 	n = strlen(keystr);
 
 	while (fgets(line, sizeof line, fp) && !ret) {
-	    chopstr(line);
+	    gretl_chopstr(line);
 	    if (!strncmp(line, keystr, n)) {
 		strcpy(value, line + n);
 		ret = 1;
@@ -213,9 +213,6 @@ void cli_read_registry (char *callname)
     /* user's working directory */
     cli_read_gretl_var("userdir", cpaths.workdir, fp, HKEY_CURRENT_USER);
 
-    /* base path for RATS databases */
-    cli_read_gretl_var("ratsbase", cpaths.ratsbase, fp, HKEY_CURRENT_USER);
-
     /* path to X-12-ARIMA */
     done = read_rc_string(fp[0], "x12a", cpaths.x12a);
     if (!done) {
@@ -447,6 +444,15 @@ char *appdata_path (void)
     return win_special_path(CSIDL_APPDATA);
 }
 
+#if 0 /* not yet? */
+
+char *local_appdata_path (void)
+{
+    return win_special_path(CSIDL_LOCAL_APPDATA);
+}
+
+#endif
+
 char *mydocs_path (void)
 {
     return win_special_path(CSIDL_PERSONAL);
