@@ -23,9 +23,8 @@
 #include "gretl_xml.h"
 #include "gretl_panel.h"
 #include "gretl_func.h"
-#include "usermat.h"
-#include "gretl_scalar.h"
 #include "gretl_bundle.h"
+#include "gretl_string_table.h"
 #include "dbread.h"
 
 #include <sys/types.h>
@@ -816,8 +815,6 @@ int *gretl_xml_node_get_list (xmlNodePtr node, xmlDocPtr doc, int *err)
 	list = NULL;
     }
 
-    fprintf(stderr, "returning list = %p\n", (void *) list);
-
     return list;
 }
 
@@ -1169,7 +1166,7 @@ char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
     }
 
     if (S != NULL && *err) {
-	free_strings_array(S, n);
+	strings_array_free(S, n);
 	S = NULL;
     }
 
@@ -1225,9 +1222,9 @@ int gretl_xml_child_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
  * on failure.
  */
 
-static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc, 
-					  char **colnames, char **rownames,
-					  int *err)
+gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc, 
+				   char **colnames, char **rownames,
+				   int *err)
 {
     gretl_matrix *m = NULL;
     xmlChar *tmp;
@@ -1534,6 +1531,22 @@ int gretl_write_matrix_as_gdt (const char *fname,
     return err;
 }
 
+static int string_table_count (const DATASET *dset,
+			       const int *list,
+			       int nvars)
+{
+    int i, v, n = 0;
+
+    for (i=1; i<=nvars; i++) {
+	v = savenum(list, i);
+	if (series_has_string_table(dset, v)) {
+	    n++;
+	}
+    }
+
+    return n;
+}
+
 #define GDT_DIGITS 17
 
 /**
@@ -1567,7 +1580,7 @@ int gretl_write_gdt (const char *fname, const int *list,
     void *handle = NULL;
     int (*show_progress) (long, long, int) = NULL;
     long sz = 0L;
-    int i, t, v, nvars;
+    int i, t, v, nvars, ntabs;
     int have_markers, in_c_locale = 0;
     int uerr = 0;
     int err = 0;
@@ -1693,6 +1706,9 @@ int gretl_write_gdt (const char *fname, const int *list,
     }
 
     for (i=1; i<=nvars; i++) {
+	const char *vstr;
+	int vprop;
+
 	v = savenum(list, i);
 	gretl_xml_encode_to_buf(xmlbuf, dset->varname[v], sizeof xmlbuf);
 
@@ -1702,8 +1718,9 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    fprintf(fp, "<variable name=\"%s\"", xmlbuf);
 	}
 
-	if (*VARLABEL(dset, v)) {
-	    uerr = gretl_xml_encode_to_buf(xmlbuf, VARLABEL(dset, v), sizeof xmlbuf);
+	vstr = series_get_label(dset, v);
+	if (*vstr) {
+	    uerr = gretl_xml_encode_to_buf(xmlbuf, vstr, sizeof xmlbuf);
 	    if (!uerr) {
 		if (gz) {
 		    gzprintf(fz, "\n label=\"%s\"", xmlbuf);
@@ -1713,8 +1730,9 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    }
 	} 
 
-	if (*DISPLAYNAME(dset, v)) {
-	    uerr = gretl_xml_encode_to_buf(xmlbuf, DISPLAYNAME(dset, v), sizeof xmlbuf);
+	vstr = series_get_display_name(dset, v);
+	if (*vstr) {
+	    uerr = gretl_xml_encode_to_buf(xmlbuf, vstr, sizeof xmlbuf);
 	    if (!uerr) {
 		if (gz) {
 		    gzprintf(fz, "\n displayname=\"%s\"", xmlbuf);
@@ -1724,8 +1742,9 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    }
 	}
 
-	if (*PARENT(dset, v)) {
-	    uerr = gretl_xml_encode_to_buf(xmlbuf, PARENT(dset, v), sizeof xmlbuf);
+	vstr = series_get_parent_name(dset, v);
+	if (vstr != NULL) {
+	    uerr = gretl_xml_encode_to_buf(xmlbuf, vstr, sizeof xmlbuf);
 	    if (!uerr) {
 		if (gz) {
 		    gzprintf(fz, "\n parent=\"%s\"", xmlbuf);
@@ -1735,8 +1754,9 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    }
 	}
 
-	if (dset->varinfo[i]->transform != 0) {
-	    const char *tr = gretl_command_word(dset->varinfo[i]->transform);
+	vprop = series_get_transform(dset, v);
+	if (vprop != 0) {
+	    const char *tr = gretl_command_word(vprop);
 
 	    if (gz) {
 		gzprintf(fz, "\n transform=\"%s\"", tr);
@@ -1745,16 +1765,18 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    }
 	}
 
-	if (dset->varinfo[i]->lag != 0) {
+	vprop = series_get_lag(dset, v);
+	if (vprop != 0) {
 	    if (gz) {
-		gzprintf(fz, "\n lag=\"%d\"", dset->varinfo[i]->lag);
+		gzprintf(fz, "\n lag=\"%d\"", vprop);
 	    } else {
-		fprintf(fp, "\n lag=\"%d\"", dset->varinfo[i]->lag);
+		fprintf(fp, "\n lag=\"%d\"", vprop);
 	    }
-	}	    
+	}
 
-	if (COMPACT_METHOD(dset, v) != COMPACT_NONE) {
-	    const char *meth = compact_method_to_string(COMPACT_METHOD(dset, v));
+	vprop = series_get_compact_method(dset, v);
+	if (vprop != COMPACT_NONE) {
+	    const char *meth = compact_method_to_string(vprop);
 
 	    if (gz) {
 		gzprintf(fz, "\n compact-method=\"%s\"", meth);
@@ -1763,7 +1785,7 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    }
 	} 
 
-	if (var_is_discrete(dset, v)) {
+	if (series_is_discrete(dset, v)) {
 	    alt_puts("\n discrete=\"true\"", fp, fz);
 	}	    
 
@@ -1817,7 +1839,46 @@ int gretl_write_gdt (const char *fname, const int *list,
 	}
     }
 
-    alt_puts("</observations>\n</gretldata>\n", fp, fz);
+    alt_puts("</observations>\n", fp, fz);
+
+    if ((ntabs = string_table_count(dset, list, nvars)) > 0) {
+	const char **strs;
+	int j, n_strs;
+
+	if (gz) {
+	    gzprintf(fz, "<string-tables count=\"%d\">\n", ntabs);
+	} else {
+	    fprintf(fp, "<string-tables count=\"%d\">\n", ntabs);
+	}
+
+	for (i=1; i<=nvars; i++) {
+	    v = savenum(list, i);
+	    if (!series_has_string_table(dset, v)) {
+		continue;
+	    }
+	    strs = series_get_string_vals(dset, v, &n_strs);
+	    gretl_xml_encode_to_buf(xmlbuf, dset->varname[v], sizeof xmlbuf);
+	    if (gz) {
+		gzprintf(fz, "<valstrings owner=\"%s\" count=\"%d\">", xmlbuf,
+			 n_strs);
+	    } else {
+		fprintf(fp, "<valstrings owner=\"%s\" count=\"%d\">", xmlbuf,
+			n_strs);
+	    }
+	    for (j=0; j<n_strs; j++) {
+		gretl_xml_encode_to_buf(xmlbuf, strs[j], sizeof xmlbuf);
+		if (gz) {
+		    gzprintf(fz, "\"%s\" ", xmlbuf);
+		} else {
+		    fprintf(fp, "\"%s\" ", xmlbuf);
+		}
+	    }
+	    alt_puts("</valstrings>\n", fp, fz);
+	}
+	alt_puts("</string-tables>\n", fp, fz);
+    }
+
+    alt_puts("</gretldata>\n", fp, fz);
 
  cleanup: 
 
@@ -1903,35 +1964,35 @@ static int process_varlist (xmlNodePtr node, DATASET *dset)
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "label");
 	    if (tmp != NULL) {
-		transcribe_string(VARLABEL(dset, i), (char *) tmp, MAXLABEL);
+		series_set_label(dset, i, (char *) tmp);
 		free(tmp);
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "displayname");
 	    if (tmp != NULL) {
-		var_set_display_name(dset, i, (char *) tmp);
+		series_set_display_name(dset, i, (char *) tmp);
 		free(tmp);
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "parent");
 	    if (tmp != NULL) {
-		strcpy(dset->varinfo[i]->parent, (char *) tmp); 
+		series_set_parent(dset, i, (char *) tmp); 
 		free(tmp);
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "transform");
 	    if (tmp != NULL) {
 		int ci = gretl_command_number((char *) tmp);
 
-		dset->varinfo[i]->transform = ci; 
+		series_set_transform(dset, i, ci);
 		free(tmp);
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "lag");
 	    if (tmp != NULL) {
-		dset->varinfo[i]->lag = atoi((char *) tmp); 
+		series_set_lag(dset, i, atoi((char *) tmp));
 		free(tmp);
 	    }
 
 	    tmp = xmlGetProp(cur, (XUC) "compact-method");
 	    if (tmp != NULL) {
-		COMPACT_METHOD(dset, i) = compact_string_to_int((char *) tmp);
+		series_set_compact_method(dset, i, compact_string_to_int((char *) tmp));
 		free(tmp);
 	    }
 
@@ -1944,20 +2005,6 @@ static int process_varlist (xmlNodePtr node, DATASET *dset)
 	    }
 	    tmp = xmlGetProp(cur, (XUC) "role");
 	    if (tmp != NULL) {
-#if 0 /* FIXME old datafiles? */
-		if (!strcmp((char *) tmp, "scalar")) {
-		    char *val = (char *) xmlGetProp(cur, (XUC) "value");
-		    
-		    if (val) {
-			double xx = atof(val);
-
-			free(val);
-			(*pZ)[i] = malloc(sizeof ***pZ);
-			(*pZ)[i][0] = xx;
-			set_var_scalar(dset, i, 1);
-		    }
-		}
-#endif
 		free(tmp);
 	    }
 	    i++;
@@ -2160,6 +2207,89 @@ static int process_observations (xmlDocPtr doc, xmlNodePtr node,
     return err;
 }
 
+static int owner_id (const DATASET *dset, const char *s)
+{
+    int i;
+
+    for (i=1; i<dset->v; i++) {
+	if (!strcmp(s, dset->varname[i])) {
+	    return i;
+	}
+    }
+
+    return -1;
+}
+
+static int process_string_tables (xmlDocPtr doc, xmlNodePtr node, 
+				  DATASET *dset)
+{
+    xmlNodePtr cur;
+    xmlChar *tmp;
+    int ntabs = 0;
+    int err = 0;
+
+    tmp = xmlGetProp(node, (XUC) "count");
+    if (tmp == NULL) {
+	return E_DATA;
+    } 
+
+    if (sscanf((char *) tmp, "%d", &ntabs) != 1) {
+	err = E_DATA;
+    }
+
+    free(tmp);
+
+    if (!err) {
+	cur = node->xmlChildrenNode;
+	while (cur && xmlIsBlankNode(cur)) {
+	    cur = cur->next;
+	}
+	if (cur == NULL) {
+	    err = E_DATA;
+	}
+    }
+
+    if (err) {
+	return err;
+    }
+
+    while (cur != NULL && !err) {
+        if (!xmlStrcmp(cur->name, (XUC) "valstrings")) {
+	    xmlChar *owner = xmlGetProp(cur, (XUC) "owner");
+	    series_table *st;
+	    char **strs = NULL;
+	    int v = 0, n_strs = 0;
+
+	    if (owner == NULL) {
+		err = E_DATA;
+	    } else {
+		v = owner_id(dset, (const char *) owner);
+		if (v <= 0) {
+		    err = E_DATA;
+		}
+	    }
+	    if (!err) {
+		strs = gretl_xml_get_strings_array(cur, doc, &n_strs, 
+						   0, &err);
+	    }
+	    if (!err) {
+		st = series_table_new(strs, n_strs);
+		if (st == NULL) {
+		    strings_array_free(strs, n_strs);
+		    err = E_ALLOC;
+		} else {
+		    series_attach_string_table(dset, v, st);
+		}
+	    }
+	    free(owner);
+	}	   
+ 
+	cur = cur->next;
+    }
+
+    return err;
+}
+
 static double get_gdt_version (xmlNodePtr node)
 {
     xmlChar *tmp = xmlGetProp(node, (XUC) "version");
@@ -2232,11 +2362,11 @@ static int xml_get_startobs (xmlNodePtr node, double *sd0, char *stobs,
     int err = 0;
 
     if (tmp != NULL) {
-	char obstr[16];
+	char obstr[OBSLEN];
 
 	obstr[0] = '\0';
-	strncat(obstr, (char *) tmp, 15);
-	charsub(obstr, ':', '.');
+	strncat(obstr, (char *) tmp, OBSLEN - 1);
+	gretl_charsub(obstr, ':', '.');
 	
 	if (strchr(obstr, '/') != NULL && caldata) {
 	    long ed = get_epoch_day((char *) tmp);
@@ -2304,11 +2434,13 @@ static int xml_get_endobs (xmlNodePtr node, char *endobs, int caldata)
 
 static int lag_from_label (int v, const DATASET *dset, int *lag)
 {
-    const char *test = VARLABEL(dset, v);
-    char pm, vname[VNAMELEN];
+    const char *test = series_get_label(dset, v);
+    char pm, fmt[20], vname[VNAMELEN];
     int pv = 0;
 
-    if (sscanf(test, "= %15[^(](t %c %d)", vname, &pm, lag) == 3) {
+    sprintf(fmt, "= %%%d[^(](t %%c %%d)", VNAMELEN - 1);
+
+    if (sscanf(test, fmt, vname, &pm, lag) == 3) {
 	pv = series_index(dset, vname);
 	pv = (pv < dset->v)? pv : 0;
     }
@@ -2318,7 +2450,7 @@ static int lag_from_label (int v, const DATASET *dset, int *lag)
 
 static int dummy_child_from_label (int v, const DATASET *dset)
 {
-    const char *test = VARLABEL(dset, v);
+    const char *test = series_get_label(dset, v);
     char vname[VNAMELEN];
     double val;
     int pv = 0;
@@ -2334,25 +2466,23 @@ static int dummy_child_from_label (int v, const DATASET *dset)
 
 static void record_transform_info (DATASET *dset, double version)
 {
-    VARINFO *vinfo;
     int i, p, pv;
 
     for (i=1; i<dset->v; i++) {
-	vinfo = dset->varinfo[i];
-	if (vinfo->transform == LAGS) {
+	if (series_get_transform(dset, i) == LAGS) {
 	    /* already handled */
 	    continue;
 	}
 	pv = lag_from_label(i, dset, &p);
 	if (pv > 0) {
-	    strcpy(vinfo->parent, dset->varname[pv]);
-	    vinfo->transform = LAGS;
-	    vinfo->lag = p;
+	    series_set_parent(dset, i, dset->varname[pv]);
+	    series_set_transform(dset, i, LAGS);
+	    series_set_lag(dset, i, p);
 	} else if (version < 1.1) {
 	    pv = dummy_child_from_label(i, dset);
 	    if (pv > 0) {
-		strcpy(vinfo->parent, dset->varname[pv]);
-		vinfo->transform = DUMMIFY;
+		series_set_parent(dset, i, dset->varname[pv]);
+		series_set_transform(dset, i, DUMMIFY);
 	    }
 	}
     }
@@ -2380,13 +2510,13 @@ static long get_filesize (const char *fname)
 
 static int remedy_empty_data (DATASET *dset)
 {
-    int err = dataset_add_series(1, dset);
+    int err = dataset_add_series(dset, 1);
 
     if (!err) {
 	int t;
 
 	strcpy(dset->varname[1], "index");
-	strcpy(VARLABEL(dset, 1), _("index variable"));
+	series_set_label(dset, 1, _("index variable"));
 	for (t=0; t<dset->n; t++) {
 	    dset->Z[1][t] = (double) (t + 1);
 	}
@@ -2513,6 +2643,14 @@ int gretl_read_gdt (const char *fname, DATASET *dset,
 	    } else {
 		gotobs = 1;
 	    }
+	} else if (!xmlStrcmp(cur->name, (XUC) "string-tables")) {
+	    if (!gotvars) {
+		gretl_errmsg_set(_("Variables information is missing"));
+		err = 1;
+	    }
+	    if (process_string_tables(doc, cur, tmpset)) {
+		err = 1;
+	    }	    
 	}
 	if (!err) {
 	    cur = cur->next;
@@ -2643,127 +2781,6 @@ static char *gretl_xml_get_doc_type (const char *fname, int *err)
     return ret;
 }
 
-int load_user_matrix_file (const char *fname) 
-{
-    xmlDocPtr doc = NULL;
-    xmlNodePtr cur = NULL;
-    gretl_matrix *m;
-    char *colnames;
-    char *rownames;
-    char *name;
-    int err = 0;
-
-    err = gretl_xml_open_doc_root(fname, "gretl-matrices", &doc, &cur);
-    if (err) {
-	return err;
-    }
-
-    cur = cur->xmlChildrenNode;
-    while (cur != NULL && !err) {
-        if (!xmlStrcmp(cur->name, (XUC) "gretl-matrix")) {
-	    name = (char *) xmlGetProp(cur, (XUC) "name");
-	    if (name == NULL) {
-		err = 1;
-	    } else {
-		colnames = rownames = NULL;
-		m = xml_get_user_matrix(cur, doc, &colnames, 
-					&rownames, &err);
-		if (m != NULL) {
-		    err = user_matrix_add(m, name);
-		    if (!err && colnames != NULL) {
-			umatrix_set_names_from_string(m, colnames, 0);
-		    }
-		    if (!err && rownames != NULL) {
-			umatrix_set_names_from_string(m, rownames, 1);
-		    }		    
-		}
-		free(colnames);
-		free(rownames);
-		free(name);
-	    }
-	}
-	cur = cur->next;
-    }
-
-    if (doc != NULL) {
-	xmlFreeDoc(doc);
-    }
-
-    return err;
-}
-
-int load_user_bundle_file (const char *fname)
-{
-    xmlDocPtr doc = NULL;
-    xmlNodePtr cur = NULL;
-    char *name;
-    int err = 0;
-
-    err = gretl_xml_open_doc_root(fname, "gretl-bundles", &doc, &cur);
-    if (err) {
-	return err;
-    }
-
-    gretl_push_c_numeric_locale();
-
-    cur = cur->xmlChildrenNode;
-    while (cur != NULL && !err) {
-        if (!xmlStrcmp(cur->name, (XUC) "gretl-bundle")) {
-	    name = (char *) xmlGetProp(cur, (XUC) "name");
-	    if (name == NULL) {
-		err = 1;
-	    } else {
-		err = load_bundle_from_xml(cur, doc, name);
-		free(name);
-	    }
-	}
-	cur = cur->next;
-    }
-
-    gretl_pop_c_numeric_locale();
-
-    if (doc != NULL) {
-	xmlFreeDoc(doc);
-    }
-
-    return err;
-}
-
-int load_user_scalars_file (const char *fname) 
-{
-    xmlDocPtr doc = NULL;
-    xmlNodePtr cur = NULL;
-    char *name, *val;
-    int err = 0;
-
-    err = gretl_xml_open_doc_root(fname, "gretl-scalars", &doc, &cur);
-    if (err) {
-	return err;
-    }
-
-    cur = cur->xmlChildrenNode;
-    while (cur != NULL && !err) {
-        if (!xmlStrcmp(cur->name, (XUC) "gretl-scalar")) {
-	    name = (char *) xmlGetProp(cur, (XUC) "name");
-	    val = (char *) xmlGetProp(cur, (XUC) "value");
-	    if (name == NULL || val == NULL) {
-		err = 1;
-	    } else {
-		err = gretl_scalar_add(name, dot_atof(val));
-	    }
-	    free(name);
-	    free(val);
-	}
-	cur = cur->next;
-    }
-
-    if (doc != NULL) {
-	xmlFreeDoc(doc);
-    }
-
-    return err;
-}
-
 /* This is called in response to the "include" command in
    the CLI program, the GUI program, and in interact.c,
    if we detect that the named file is XML.
@@ -2781,10 +2798,8 @@ int load_user_XML_file (const char *fname)
 
     if (!strcmp(rootname, "gretl-functions")) {
 	err = load_function_package_by_filename(fname);
-    } else if (!strcmp(rootname, "gretl-matrices")) {
-	err = load_user_matrix_file(fname);
-    } else if (!strcmp(rootname, "gretl-scalars")) {
-	err = load_user_scalars_file(fname);
+    } else {
+	err = E_DATA;
     }
 
     free(rootname);
