@@ -18,13 +18,20 @@
  */
 
 #include "libgretl.h"
-#include "usermat.h"
+#include "uservar.h"
 #include "gretl_func.h"
 #include "matrix_extra.h"
 #include "libset.h"
-#include "gretl_scalar.h"
 #include "kalman.h"
 
+/**
+ * SECTION:kalman
+ * @short_description: The Kalman filter
+ * @title: Kalman
+ * @include: gretl/libgretl.h, gretl/kalman.h
+ *
+ */
+
 #define KDEBUG 0
 
 /* notation (quasi-Hamilton):
@@ -250,11 +257,11 @@ void kalman_free (kalman *K)
 	    }
 	}
 
-	free_strings_array(K->mnames, K_MMAX);
+	strings_array_free(K->mnames, K_MMAX);
     }    
 
     if (K->matcalls != NULL) {
-	free_strings_array(K->matcalls, NMATCALLS);
+	strings_array_free(K->matcalls, NMATCALLS);
     }
 
     if (K->cross != NULL) {
@@ -2036,7 +2043,7 @@ attach_input_matrix (kalman *K, const char *s, int i,
     if (i <= K_R && strchr(s, '(')) {
 	/* we have a function call? */
 	err = add_matrix_fncall(K, s, i, mname, &m);
-    } else if (sscanf(s, "%15s", mname) == 1) {
+    } else if (gretl_scan_varname(s, mname) == 1) {
 	m = get_matrix_by_name(mname);
     } else {
 	err = E_PARSE;
@@ -2418,7 +2425,7 @@ static const char *kalman_matrix_name (int sym)
     return "matrix";
 }
 
-/**
+/*
  * kalman_parse_line:
  * @line: "kalman" to start, "end kalman" to end; otherwise
  * this string should contain a matrix specification on the 
@@ -3179,8 +3186,8 @@ gretl_matrix *kalman_arma_smooth (kalman *K, int *err)
  * @err: location to receive error code.
  * 
  * Runs a filtering pass followed by a backward, smoothing pass. 
- * At present the @Uname argument is experimental and a bodge: it will 
- * not actually do anything unless @Pname is left null.
+ * At present the @pU argument is experimental and a bodge: it will 
+ * not actually do anything unless @pP is left NULL.
  *
  * Returns: matrix containing the smoothed estimate of the
  * state, or NULL on error.
@@ -3310,7 +3317,7 @@ gretl_matrix *kalman_smooth (kalman *K,
     return S;
 }
 
-/**
+/*
  * user_kalman_smooth:
  * @Pname: name of matrix in which to retrieve the MSE of the
  * smoothed state (or NULL if this is not required).
@@ -3378,8 +3385,8 @@ gretl_matrix *user_kalman_smooth (const char *Pname,
 }
 
 /* See the account in Koopman, Shephard and Doornik, Econometrics
-   Journal, 1999, section 4.2, regarding the initialization of the
-   state under simulation.
+   Journal, 1999 (volume 2, pp. 113-166), section 4.2, regarding
+   the initialization of the state under simulation.
 */
 
 static int sim_state_0 (kalman *K, const gretl_matrix *V)
@@ -3516,7 +3523,7 @@ static int kalman_simulate (kalman *K,
     return err;
 }
 
-/**
+/*
  * user_kalman_simulate:
  * @V: artificial disturbance to state.
  * @W: artificial disturbance to observation.
@@ -3631,7 +3638,7 @@ gretl_matrix *user_kalman_simulate (const gretl_matrix *V,
     return Y;
 }
 
-/**
+/*
  * user_kalman_get_loglik:
  * 
  * Retrieves the log-likelhood calculated via the last run of 
@@ -3651,7 +3658,7 @@ double user_kalman_get_loglik (void)
     }
 }
 
-/**
+/*
  * user_kalman_get_matrix:
  * @idx: identifier for matrix.
  * @err: location to receive error code.
@@ -3691,7 +3698,7 @@ gretl_matrix *user_kalman_get_matrix (int idx, int *err)
     return m;
 }
 
-/**
+/*
  * user_kalman_get_s2:
  * 
  * Retrieves the scale factor, \hat{\sigma}^2, calculated 
@@ -3711,7 +3718,7 @@ double user_kalman_get_s2 (void)
     }
 }
 
-/**
+/*
  * user_kalman_get_time_step:
  * 
  * Retrieves the time step, t, from the current run of a
