@@ -22,10 +22,10 @@
 #include "libgretl.h"
 #include "libset.h"
 #include "usermat.h"
+#include "uservar.h"
 #include "matrix_extra.h"
 #include "gretl_string_table.h"
 #include "gretl_func.h"
-#include "gretl_scalar.h"
 
 #include <unistd.h>
 #include <errno.h>
@@ -81,6 +81,7 @@ struct robust_opts {
 struct set_vars_ {
     int flags;
     unsigned int seed;          /* for PRNG */
+    double conv_huge;           /* conventional value for $huge */
     int horizon;                /* for VAR impulse responses */ 
     int bootrep;                /* bootstrap replications */
     double nls_toler;           /* NLS convergence criterion */
@@ -133,7 +134,8 @@ struct set_vars_ {
 			   !strcmp(s, BFGS_RSTEP) || \
 			   !strcmp(s, DPDSTYLE))
 
-#define libset_double(s) (!strcmp(s, BFGS_TOLER) || \
+#define libset_double(s) (!strcmp(s, CONV_HUGE) || \
+			  !strcmp(s, BFGS_TOLER) || \
 			  !strcmp(s, BFGS_MAXGRAD) || \
 			  !strcmp(s, BHHH_TOLER) || \
 			  !strcmp(s, NLS_TOLER) || \
@@ -375,6 +377,7 @@ static void state_vars_copy (set_vars *sv)
     sv->flags &= ~STATE_LOOPING;
 
     sv->seed = state->seed;
+    sv->conv_huge = state->conv_huge;
     sv->horizon = state->horizon;
     sv->bootrep = state->bootrep;
     sv->loop_maxiter = state->loop_maxiter;
@@ -411,6 +414,7 @@ static void state_vars_init (set_vars *sv)
     sv->flags = STATE_ECHO_ON | STATE_MSGS_ON | STATE_WARN_ON | 
 	STATE_HALT_ON_ERR | STATE_SKIP_MISSING;
     sv->seed = 0;
+    sv->conv_huge = 1.0e100;
     sv->horizon = UNSET_INT;
     sv->bootrep = 1000;
     sv->nls_toler = NADBL;
@@ -868,7 +872,7 @@ void set_xsect_hccme (const char *s)
     scpy = gretl_strdup(s);
 
     if (scpy != NULL) {
-	lower(scpy);
+	gretl_lower(scpy);
 	parse_hc_variant(scpy);
 	free(scpy);
     }
@@ -883,7 +887,7 @@ void set_tseries_hccme (const char *s)
     scpy = gretl_strdup(s);
 
     if (scpy != NULL) {
-	lower(scpy);
+	gretl_lower(scpy);
 	if (parse_hc_variant(scpy) == 0) {
 	    libset_set_bool(FORCE_HC, 1);
 	} else {
@@ -913,7 +917,7 @@ void set_garch_robust_vcv (const char *s)
     scpy = gretl_strdup(s);
 
     if (scpy != NULL) {
-	lower(scpy);
+	gretl_lower(scpy);
 	if (!strcmp(s, "qml")) {
 	    state->garch_robust_vcv = ML_QML;
 	} else if (!strcmp(s, "bw")) {
@@ -952,7 +956,7 @@ static int set_line_width (const char *s0, const char *s1,
     if (w < 0 || w > 32) {
 	err = E_DATA;
     } else {
-	var_set_linewidth(dset, v, w);
+	series_set_linewidth(dset, v, w);
 	pprintf(prn, _("Line width for %s = %d\n"), 
 		dset->varname[v], w);
     }
@@ -969,7 +973,7 @@ static int set_initvals (const char *s, PRN *prn)
     /* skip past "set initvals" */
     s += 12;
 
-    if (sscanf(s, "%15s", mname) != 1 || !strcmp(mname, "auto")) {
+    if (gretl_scan_varname(s, mname) != 1 || !strcmp(mname, "auto")) {
 	gretl_matrix_free(state->initvals);
 	state->initvals = NULL;
     } else {
@@ -1001,7 +1005,7 @@ static int set_matmask (const char *s, const DATASET *dset,
     /* skip past "set matrix_mask" */
     s += 15;
 
-    if (sscanf(s, "%15s", vname) != 1 || !strcmp(vname, "null")) {
+    if (gretl_scan_varname(s, vname) != 1 || !strcmp(vname, "null")) {
 	gretl_matrix_free(state->matmask);
 	state->matmask = NULL;
     } else {
@@ -1070,7 +1074,7 @@ static int set_shelldir (const char *s)
 	gretl_insert_builtin_string("shelldir", state->shelldir);
     } else if (*s == '"') {
 	s++;
-	len = charpos('"', s);
+	len = gretl_charpos('"', s);
 	if (len <= 0) {
 	    err = E_PARSE;
 	} 
@@ -1300,6 +1304,7 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_int(LOOP_MAXITER, prn, opt);
     libset_print_bool(MAX_VERBOSE, prn, opt);
     libset_print_int(BFGS_VERBSKIP, prn, opt);
+    libset_print_double(CONV_HUGE, prn, opt);
     libset_print_bool(MESSAGES, prn, opt);
     libset_print_bool(WARNINGS, prn, opt);
     libset_print_int(GRETL_DEBUG, prn, opt);
@@ -1532,7 +1537,7 @@ int execute_set_line (const char *line, DATASET *dset,
 	    return 0;
 	}	    
 
-	lower(setarg);
+	gretl_lower(setarg);
 
 	if (libset_boolvar(setobj)) {
 	    if (!strcmp(setobj, SHELL_OK)) {
@@ -1643,6 +1648,12 @@ double libset_get_double (const char *key)
 	} else {
 	    return state->nadarwat_trim;
 	}
+    } else if (!strcmp(key, CONV_HUGE)) {
+	if (na(state->conv_huge)) {
+	    return 1.0e100;
+	} else {
+	    return state->conv_huge;
+	}
     } else {
 	fprintf(stderr, "libset_get_double: unrecognized "
 		"variable '%s'\n", key);	
@@ -1690,6 +1701,8 @@ int libset_set_double (const char *key, double val)
 	state->bfgs_maxgrad = val;
     } else if (!strcmp(key, NADARWAT_TRIM)) {
 	state->nadarwat_trim = val;
+    } else if (!strcmp(key, CONV_HUGE)) {
+	state->conv_huge = val;
     } else {
 	fprintf(stderr, "libset_set_double: unrecognized "
 		"variable '%s'\n", key);	
@@ -1800,7 +1813,6 @@ static int intvar_min_max (const char *s, int *min, int *max,
 	*var = &state->optim;
     } else if (!strcmp(s, GRETL_DEBUG)) {
 	*min = 0;
-	*max = 4;
 	*var = &gretl_debug;
     } else {
 	fprintf(stderr, "libset_set_int: unrecognized "
