@@ -710,6 +710,8 @@ gretl_matrix_data_subset_special (const int *list,
  * gretl_dataset_from_matrix:
  * @m: source matrix.
  * @list: list of columns (1-based) to include, or NULL.
+ * @opt: may include OPT_B to attempt "borrowing" of data;
+ * may include OPT_N to use plain numbers as variable names.
  * @err: location to receive error code.
  *
  * Creates a gretl dataset from matrix @m, either using the
@@ -722,6 +724,7 @@ gretl_matrix_data_subset_special (const int *list,
 
 DATASET *gretl_dataset_from_matrix (const gretl_matrix *m, 
 				    const int *list,
+				    gretlopt opt,
 				    int *err)
 {
     DATASET *dset = NULL;
@@ -741,16 +744,34 @@ DATASET *gretl_dataset_from_matrix (const gretl_matrix *m,
 	for (i=1; i<=list[0]; i++) {
 	    col = list[i];
 	    if (col < 1 || col > nv) {
-		fprintf(stderr, "column %d: out of bounds\n", col);
+		gretl_errmsg_sprintf("Variable number %d is out of bounds", col);
 		*err = E_DATA;
 		break;
-	    } 
+	    } else if (opt & OPT_B) {
+		/* try borrowing? */
+		for (t=0; t<T; t++) {
+		    x = gretl_matrix_get(m, t, col-1);
+		    if (xna(x)) {
+			opt = OPT_NONE;
+			break;
+		    }
+		}
+	    }
 	}
 	nv = list[0];
-    } 
+    } else if (opt & OPT_B) {
+	int N = T * nv;
+
+	for (i=0; i<N; i++) {
+	    if (xna(m->val[i])) {
+		opt = OPT_NONE;
+		break;
+	    }
+	}
+    }
 
     if (!*err) {
-	dset = create_auxiliary_dataset(nv + 1, T);
+	dset = create_auxiliary_dataset(nv + 1, T, opt);
 	if (dset == NULL) {
 	    *err = E_ALLOC;
 	}
@@ -764,12 +785,21 @@ DATASET *gretl_dataset_from_matrix (const gretl_matrix *m,
 
     for (i=1; i<=nv; i++) {
 	col = (list != NULL)? list[i] - 1 : i - 1;
-	for (t=0; t<T; t++) {
-	    x = gretl_matrix_get(m, t, col);
-	    dset->Z[i][t] = x;
+	if (opt & OPT_B) {
+	    dset->Z[i] = m->val + T * col;
+	} else {
+	    for (t=0; t<T; t++) {
+		x = gretl_matrix_get(m, t, col);
+		if (xna(x)) {
+		    x = NADBL;
+		}
+		dset->Z[i][t] = x;
+	    }
 	}
 	if (names != NULL) {
 	    strcpy(dset->varname[i], names[col]);
+	} else if (opt & OPT_N) {
+	    sprintf(dset->varname[i], "%d", col + 1);
 	} else {
 	    sprintf(dset->varname[i], "col%d", col + 1);
 	}
@@ -898,6 +928,28 @@ int gretl_plotfit_matrices (const double *yvar, const double *xvar,
     return err;
 }
 
+static int skip_matrix_comment (FILE *fp, int *err)
+{
+    int c = fgetc(fp);
+    int ret = 0;
+
+    if (c == '#') {
+	ret = 1;
+	while (c != '\n' && c != EOF) {
+	    c = fgetc(fp);
+	}
+    } else {
+	ungetc(c, fp);
+    }
+
+    if (c == EOF) {
+	fprintf(stderr, "reached premature end of file\n");
+	*err = E_DATA;
+    }
+
+    return ret;
+}
+
 /**
  * gretl_matrix_read_from_text:
  * @fname: name of text file.
@@ -926,17 +978,25 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
 	return NULL;
     }
 
-    n = fscanf(fp, "%d %d\n", &r, &c);
-
-    if (n < 2 || r <= 0 || c <= 0) {
-	*err = E_DATA;
-	fclose(fp);
-	return NULL;
+    /* skip any leading comment lines starting with '#' */
+    while (!*err && skip_matrix_comment(fp, err)) {
+	;
     }
 
-    A = gretl_matrix_alloc(r, c);
-    if (A == NULL) {
-	*err = E_ALLOC;
+    if (!*err) {
+	n = fscanf(fp, "%d %d\n", &r, &c);
+	if (n < 2 || r <= 0 || c <= 0) {
+	    fprintf(stderr, "error reading rows, cols\n");
+	    *err = E_DATA;
+	} else {
+	    A = gretl_matrix_alloc(r, c);
+	    if (A == NULL) {
+		*err = E_ALLOC;
+	    }
+	}
+    }
+
+    if (*err) {
 	fclose(fp);
 	return NULL;
     }
@@ -1083,13 +1143,17 @@ static int max_label_length (const char **names, int n)
 
 static void 
 real_matrix_print_to_prn (const gretl_matrix *m, const char *msg, 
-			  int packed, int errout, int plain,
-			  const char **heads, PRN *prn)
+			  int plain, const char **colnames, 
+			  PRN *prn)
 {
     const char **rownames = NULL;
     char numstr[32];
     double x;
-    int i, j, llen = 0;
+    int strwidth = 12;
+    int rnamelen = 0;
+    int cmax = 0;
+    int cpad = 2;
+    int i, j;
 
     if (prn == NULL) {
 	return;
@@ -1104,6 +1168,8 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 	return;
     }
 
+    /* @plain != 0 means skip the header stuff */
+
     if (msg != NULL && *msg != '\0' && !plain) {
 	pprintf(prn, "%s (%d x %d)", msg, m->rows, m->cols);
 	if (gretl_matrix_is_dated(m)) {
@@ -1116,72 +1182,48 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 	}
     }
 
-    if (heads == NULL) {
-	heads = gretl_matrix_get_colnames(m);
+    cmax = max_numchars(m);
+    if (cmax > 5) {
+	cmax = 0;
+    }    
+
+    if (colnames == NULL) {
+	colnames = gretl_matrix_get_colnames(m);
     }
 
-    if (!packed) {
-	rownames = gretl_matrix_get_rownames(m);
-	if (rownames != NULL) {
-	    llen = max_label_length(rownames, m->rows);
-	}
+    rownames = gretl_matrix_get_rownames(m);
+    if (rownames != NULL) {
+	rnamelen = max_label_length(rownames, m->rows);
     }
 
-    if (heads != NULL) {
+    if (colnames != NULL) {
 	if (rownames != NULL) {
-	    bufspace(llen + 1, prn);
+	    bufspace(rnamelen + 1, prn);
 	}
 	for (j=0; j<m->cols; j++) {
-	    pprintf(prn, "%12.12s ", heads[j]);
+	    pprintf(prn, "%*.*s ", strwidth, strwidth, colnames[j]);
 	}
 	pputc(prn, '\n');
     }
 
-    if (packed) {
-	int v, n;
-
-	v = gretl_vector_get_length(m);
-	if (v == 0) {
-	    pputs(prn, " not a packed matrix\n");
-	    return;
+    for (i=0; i<m->rows; i++) {
+	if (rownames != NULL) {
+	    pprintf(prn, "%*s ", rnamelen, rownames[i]);
 	}
-
-	n = (sqrt(1.0 + 8.0 * v) - 1.0) / 2.0;
-
-	for (i=0; i<n; i++) {
-	    for (j=0; j<n; j++) {
-		x = m->val[ijton(i, j, n)];
-		make_numstr(numstr, x);
-		pprintf(prn, "%12s ", numstr);
-	    }
-	    pputc(prn, '\n');
-	}
-    } else {
-	int cmax = 0;
-
-	if (heads == NULL) {
-	    cmax = max_numchars(m);
-	    if (cmax > 5) {
-		cmax = 0;
-	    }
-	} 
-
-	for (i=0; i<m->rows; i++) {
-	    if (rownames != NULL) {
-		pprintf(prn, "%*s ", llen, rownames[i]);
-	    }
-	    for (j=0; j<m->cols; j++) {
-		x = gretl_matrix_get(m, i, j);
-		if (cmax) {
-		    sprintf(numstr, "%g", x);
-		    pprintf(prn, "%*s ", cmax + 2, numstr);
-		} else {
-		    make_numstr(numstr, x);
-		    pprintf(prn, "%12s ", numstr);
+	for (j=0; j<m->cols; j++) {
+	    x = gretl_matrix_get(m, i, j);
+	    if (cmax > 0) {
+		if (colnames != NULL) {
+		    bufspace(strwidth - cmax - cpad, prn);
 		}
+		sprintf(numstr, "%g", x);
+		pprintf(prn, "%*s ", cmax + cpad, numstr);
+	    } else {
+		make_numstr(numstr, x);
+		pprintf(prn, "%*s ", strwidth, numstr);
 	    }
-	    pputc(prn, '\n');
 	}
+	pputc(prn, '\n');
     }
 
     pputc(prn, '\n');
@@ -1199,7 +1241,7 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 void 
 gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn)
 {
-    real_matrix_print_to_prn(m, msg, 0, 0, 0, NULL, prn);
+    real_matrix_print_to_prn(m, msg, 0, NULL, prn);
 }
 
 /**
@@ -1218,7 +1260,7 @@ void gretl_matrix_print_with_col_heads (const gretl_matrix *m,
 					const char **heads,
 					PRN *prn)
 {
-    real_matrix_print_to_prn(m, title, 0, 0, 0, heads, prn);
+    real_matrix_print_to_prn(m, title, 0, heads, prn);
 }
 
 static void maybe_print_col_heads (const gretl_matrix *m, 
@@ -1298,7 +1340,7 @@ void gretl_matrix_print_with_format (const gretl_matrix *m,
     }
 
     if (gretl_is_null_matrix(m) || fmt == NULL || *fmt == '\0') {
-	real_matrix_print_to_prn(m, NULL, 0, 0, 1, NULL, prn);
+	real_matrix_print_to_prn(m, NULL, 1, NULL, prn);
     } else {
 	const char **rownames = NULL;
 	int llen = 0, intcast = 0;
