@@ -27,10 +27,11 @@
 #include "var.h"
 #include "objstack.h"
 #include "gretl_func.h"
-#include "gretl_scalar.h"
+#include "uservar.h"
 #include "flow_control.h"
 #include "system.h"
 #include "genparse.h"
+#include "gretl_string_table.h"
 
 #include <time.h>
 #include <unistd.h>
@@ -59,7 +60,8 @@ enum loop_types {
 typedef struct {
     int lineno;    /* location: line number in loop */
     int n;         /* number of repetitions */
-    int *list;     /* list of vars to print */
+    int nvars;     /* number of variables */
+    char **names;  /* names of vars to print */
     bigval *sum;   /* running sum of values */
     bigval *ssq;   /* running sum of squares */
     double *xbak;  /* previous values */
@@ -86,11 +88,13 @@ typedef struct {
 } LOOP_MODEL;
 
 typedef struct {
-    int lineno;       /* location: line number in loop */ 
-    int n;            /* number of observations */
-    char *fname;      /* filename for output */
-    gretlopt opt;     /* formatting option */
-    DATASET *dset;    /* temporary data storage */
+    int lineno;     /* location: line number in loop */ 
+    int n;          /* number of observations */
+    int nvars;      /* number of variables to store */
+    char **names;   /* names of vars to print */
+    char *fname;    /* filename for output */
+    gretlopt opt;   /* formatting option */
+    DATASET *dset;  /* temporary data storage */
 } LOOP_STORE;
 
 enum loop_flags {
@@ -198,7 +202,8 @@ static int
 make_dollar_substitutions (char *str, int maxlen,
 			   const LOOPSET *loop,
 			   const DATASET *dset,
-			   int *subst);
+			   int *subst,
+			   gretlopt opt);
 
 #define LOOP_BLOCK 32
 
@@ -227,7 +232,8 @@ int gretl_execute_loop (void)
    that value.  
 */
 
-static double controller_get_val (controller *clr, 
+static double controller_get_val (controller *clr,
+				  LOOPSET *loop,
 				  DATASET *dset, 
 				  int *err)
 {
@@ -238,12 +244,43 @@ static double controller_get_val (controller *clr,
 	    gretl_errmsg_sprintf(_("'%s': not a scalar"), clr->vname);
 	} 
     } else if (clr->expr != NULL) {
-	clr->val = generate_scalar(clr->expr, dset, err);
+	int done = 0;
+
+	if (strchr(clr->expr, '@')) {
+	    /* the expression needs string substitution? */
+	    int subst = 0;
+	    char expr[32];
+
+	    *expr = '\0';
+	    strncat(expr, clr->expr, 31);
+	    *err = substitute_named_strings(expr, &subst);
+	    if (!*err && subst) {
+		clr->val = generate_scalar(expr, dset, err);
+		done = 1;
+	    }
+	}
+	if (!done && !*err && strchr(clr->expr, '$')) {
+	    /* the expression needs dollar substitution? */
+	    int subst = 0;
+	    char expr[32];
+
+	    *expr = '\0';
+	    strncat(expr, clr->expr, 31);
+	    *err = make_dollar_substitutions(expr, 31, loop, dset, 
+					     &subst, OPT_T);
+	    if (!*err && subst) {
+		clr->val = generate_scalar(expr, dset, err);
+		done = 1;
+	    }	    
+	}
+	if (!*err && !done) {
+	    clr->val = generate_scalar(clr->expr, dset, err);
+	}
     }
 
 #if LOOP_DEBUG
-    fprintf(stderr, "controller_get_val: vname='%s', returning %g\n", 
-	    clr->vname, clr->val);
+    fprintf(stderr, "controller_get_val: vname='%s', expr='%s', val=%g, err=%d\n", 
+	    clr->vname, clr->expr, clr->val, *err);
 #endif
 
     return clr->val;
@@ -353,14 +390,12 @@ int ok_in_loop (int c)
 
     if (c == CORRGM ||
 	c == CUSUM ||
-	c == DATA ||
 	c == EQNPRINT ||
 	c == FUNC ||
 	c == HURST ||
 	c == INCLUDE ||
 	c == LEVERAGE ||
 	c == NULLDATA ||
-	c == OPEN ||
 	c == RMPLOT ||
 	c == RUN ||
 	c == SETMISS ||
@@ -370,6 +405,10 @@ int ok_in_loop (int c)
 	return 0;
     }
 
+    if (c == OPEN && gretl_in_gui_mode()) {
+	return 0;
+    }
+
     return 1;
 }
 
@@ -403,6 +442,8 @@ static void loop_store_init (LOOP_STORE *lstore)
 {
     lstore->lineno = -1;
     lstore->n = 0;
+    lstore->nvars = 0;
+    lstore->names = NULL;
     lstore->fname = NULL;
     lstore->opt = OPT_NONE;
     lstore->dset = NULL;
@@ -506,7 +547,7 @@ static void gretl_loop_destroy (LOOPSET *loop)
     free(loop->models);
 
     if (loop->eachstrs != NULL) {
-	free_strings_array(loop->eachstrs, loop->itermax);
+	strings_array_free(loop->eachstrs, loop->itermax);
     }
 
     if (loop->lmodels != NULL) {
@@ -538,7 +579,7 @@ static void gretl_loop_destroy (LOOPSET *loop)
     }
 
     if (loop->flags & LOOP_DELVAR) {
-	gretl_scalar_delete(loop->idxname, NULL);
+	user_var_delete_by_name(loop->idxname, NULL);
     }
 
     free(loop);
@@ -566,7 +607,7 @@ static int parse_as_while_loop (LOOPSET *loop, const char *s)
 }
 
 static int loop_attach_index_var (LOOPSET *loop, const char *vname,
-				  const DATASET *dset)
+				  DATASET *dset)
 {
     int err = 0;
 
@@ -574,7 +615,16 @@ static int loop_attach_index_var (LOOPSET *loop, const char *vname,
 	strcpy(loop->idxname, vname);
 	gretl_scalar_set_value(vname, loop->init.val);
     } else {
-	err = gretl_scalar_add_with_check(vname, loop->init.val, dset);
+	char genline[64];
+	
+	if (na(loop->init.val)) {
+	    sprintf(genline, "scalar %s = NA", vname);
+	} else {
+	    gretl_push_c_numeric_locale();
+	    sprintf(genline, "scalar %s = %g", vname, loop->init.val);
+	    gretl_pop_c_numeric_locale();
+	}
+	err = generate(genline, dset, OPT_Q, NULL);
 	if (!err) {
 	    strcpy(loop->idxname, vname);
 	    loop->flags |= LOOP_DELVAR;
@@ -791,7 +841,7 @@ static int loop_list_refresh (LOOPSET *loop, const DATASET *dset)
 
 	strcpy(lname, loop->listname);
 	err = make_dollar_substitutions(lname, VNAMELEN, loop, 
-					dset, NULL);
+					dset, NULL, OPT_T);
 	if (!err) {
 	    list = get_list_by_name(lname);
 	} 
@@ -800,7 +850,7 @@ static int loop_list_refresh (LOOPSET *loop, const DATASET *dset)
     }
 
     if (loop->eachstrs != NULL) {
-	free_strings_array(loop->eachstrs, loop->itermax);
+	strings_array_free(loop->eachstrs, loop->itermax);
 	loop->eachstrs = NULL;
     }
 
@@ -822,12 +872,14 @@ static int loop_list_refresh (LOOPSET *loop, const DATASET *dset)
 
 static int find_list_in_parentage (LOOPSET *loop, const char *s)
 {
-    char lname[VNAMELEN];
+    char fmt[16], lname[VNAMELEN];
     int i;
 
+    sprintf(fmt, "list %%%d[^ =]", VNAMELEN-1);
+
     while ((loop = loop->parent) != NULL) {
 	for (i=0; i<loop->n_cmds; i++) {
-	    if (sscanf(loop->cmds[i].line, "list %15[^ =]", lname)) {
+	    if (sscanf(loop->cmds[i].line, fmt, lname)) {
 		if (!strcmp(lname, s)) {
 		    return 1;
 		}
@@ -897,10 +949,13 @@ each_strings_from_list_of_vars (LOOPSET *loop, const DATASET *dset,
 	list = varname_match_list(dset, s, &err);
     } else {
 	char vn1[VNAMELEN], vn2[VNAMELEN];
+	char fmt[16];
+	
+	gretl_delchar(' ', s);
+	sprintf(fmt, "%%%d[^.]..%%%ds", VNAMELEN-1, VNAMELEN-1);
 	
-	delchar(' ', s);
 
-	if (sscanf(s, "%15[^.]..%15s", vn1, vn2) != 2) {
+	if (sscanf(s, fmt, vn1, vn2) != 2) {
 	    err = E_PARSE;
 	} else {
 	    int v1 = current_series_index(dset, vn1);
@@ -931,7 +986,7 @@ each_strings_from_list_of_vars (LOOPSET *loop, const DATASET *dset,
 		vi = list[i];
 		loop->eachstrs[i-1] = gretl_strdup(dset->varname[vi]);
 		if (loop->eachstrs[i-1] == NULL) {
-		    free_strings_array(loop->eachstrs, list[0]);
+		    strings_array_free(loop->eachstrs, list[0]);
 		    loop->eachstrs = NULL;
 		    err = E_ALLOC;
 		}
@@ -947,7 +1002,7 @@ each_strings_from_list_of_vars (LOOPSET *loop, const DATASET *dset,
 }
 
 static int
-parse_as_each_loop (LOOPSET *loop, const DATASET *dset, char *s)
+parse_as_each_loop (LOOPSET *loop, DATASET *dset, char *s)
 {
     char ivar[VNAMELEN] = {0};
     int done = 0;
@@ -964,7 +1019,7 @@ parse_as_each_loop (LOOPSET *loop, const DATASET *dset, char *s)
     fprintf(stderr, "parse_as_each_loop: s = '%s'\n", s);
 #endif 
 
-    if (sscanf(s, "%15s", ivar) != 1) {
+    if (gretl_scan_varname(s, ivar) != 1) {
 	return E_PARSE;
     } 
 
@@ -1004,7 +1059,7 @@ parse_as_each_loop (LOOPSET *loop, const DATASET *dset, char *s)
 
 	    loop->eachstrs[i] = gretl_strndup(s, len);
 	    if (loop->eachstrs[i] == NULL) {
-		free_strings_array(loop->eachstrs, nf);
+		strings_array_free(loop->eachstrs, nf);
 		loop->eachstrs = NULL;
 		err = E_ALLOC;
 	    } else {
@@ -1094,6 +1149,7 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
 				 DATASET *dset)
 {
     char lvar[VNAMELEN], rvar[VNAMELEN], op[VNAMELEN];
+    char fmt[32];
     int err = 0;
 
     /* skip preliminary string */
@@ -1112,15 +1168,18 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
     fprintf(stderr, "parse_first_loopline: '%s'\n", s);
 #endif
 
+    sprintf(fmt, "%%%d[^= ] = %%%d[^.]..%%%ds", VNAMELEN-1, VNAMELEN-1, 
+	    VNAMELEN-1);
+
     if (!strncmp(s, "foreach ", 8)) {
 	err = parse_as_each_loop(loop, dset, s + 8);
-    } else if (sscanf(s, "%15[^= ] = %15[^.]..%15s", lvar, op, rvar) == 3) {
+    } else if (sscanf(s, fmt, lvar, op, rvar) == 3) {
 	err = parse_as_indexed_loop(loop, dset, lvar, op, rvar);
     } else if (!strncmp(s, "for", 3)) {
 	err = parse_as_for_loop(loop, s + 4);
     } else if (!strncmp(s, "while", 5)) {
 	err = parse_as_while_loop(loop, s + 6);
-    } else if (sscanf(s, "%15s", lvar) == 1) {
+    } else if (gretl_scan_varname(s, lvar) == 1) {
 	err = parse_as_count_loop(loop, dset, lvar);
     } else {
 	printf("parse_first_loopline: failed on '%s'\n", s);
@@ -1128,6 +1187,10 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
 	err = 1;
     }
 
+#if LOOP_DEBUG
+    fprintf(stderr, "parse_first_loopline: returning %d\n", err);
+#endif
+
     return err;
 }
 
@@ -1429,14 +1492,15 @@ static void loop_print_free (LOOP_PRINT *lprn)
 {
     int i;
 
-    for (i=0; i<lprn->list[0]; i++) {
+    for (i=0; i<lprn->nvars; i++) {
 	mpf_clear(lprn->sum[i]);
 	mpf_clear(lprn->ssq[i]);
     }
 
+    strings_array_free(lprn->names, lprn->nvars);
+
     free(lprn->sum);
     free(lprn->ssq);
-    free(lprn->list);
     free(lprn->xbak);
     free(lprn->diff);
     free(lprn->na);
@@ -1448,7 +1512,7 @@ static void loop_print_zero (LOOP_PRINT *lprn, int started)
 
     lprn->n = 0;
 
-    for (i=0; i<lprn->list[0]; i++) { 
+    for (i=0; i<lprn->nvars; i++) { 
 	if (started) {
 	    mpf_set_d(lprn->sum[i], 0.0);
 	    mpf_set_d(lprn->ssq[i], 0.0);
@@ -1462,40 +1526,43 @@ static void loop_print_zero (LOOP_PRINT *lprn, int started)
     }
 }
 
-static void loop_print_init (LOOP_PRINT *lprn, int lno)
-{
-    lprn->lineno = lno;
-    lprn->list = NULL;
-    lprn->sum = NULL;
-    lprn->ssq = NULL;
-    lprn->xbak = NULL;
-    lprn->diff = NULL;
-    lprn->na = NULL;
-}
-
 /* allocate and initialize @lprn, based on the number of
-   elements in @list */
+   elements in @namestr */
 
-static int loop_print_start (LOOP_PRINT *lprn, const int *list)
+static int loop_print_start (LOOP_PRINT *lprn, const char *namestr)
 {
-    int k = list[0];
+    int i, nv;
 
-    lprn->list = gretl_list_copy(list);
-    if (lprn->list == NULL) return 1;
+    lprn->names = gretl_string_split(namestr, &lprn->nvars);
+    if (lprn->names == NULL) {
+	return E_ALLOC;
+    }
 
-    lprn->sum = malloc(k * sizeof *lprn->sum);
+    nv = lprn->nvars;
+
+    for (i=0; i<nv; i++) {
+	if (!gretl_is_scalar(lprn->names[i])) {
+	    gretl_errmsg_sprintf(_("'%s': not a scalar"), lprn->names[i]);
+	    strings_array_free(lprn->names, lprn->nvars);
+	    lprn->names = NULL;
+	    lprn->nvars = 0;
+	    return E_DATA;
+	}
+    }
+
+    lprn->sum = malloc(nv * sizeof *lprn->sum);
     if (lprn->sum == NULL) goto cleanup;
 
-    lprn->ssq = malloc(k * sizeof *lprn->ssq);
+    lprn->ssq = malloc(nv * sizeof *lprn->ssq);
     if (lprn->ssq == NULL) goto cleanup;
 
-    lprn->xbak = malloc(k * sizeof *lprn->xbak);
+    lprn->xbak = malloc(nv * sizeof *lprn->xbak);
     if (lprn->xbak == NULL) goto cleanup;
 
-    lprn->diff = malloc(k * sizeof *lprn->diff);
+    lprn->diff = malloc(nv * sizeof *lprn->diff);
     if (lprn->diff == NULL) goto cleanup;
 
-    lprn->na = malloc(k);
+    lprn->na = malloc(nv);
     if (lprn->na == NULL) goto cleanup;
 
     loop_print_zero(lprn, 0);
@@ -1504,14 +1571,35 @@ static int loop_print_start (LOOP_PRINT *lprn, const int *list)
 
  cleanup:
 
-    free(lprn->list);
+    strings_array_free(lprn->names, lprn->nvars);
+    lprn->names = NULL;
+    lprn->nvars = 0;
+    
     free(lprn->sum);
     free(lprn->ssq);
     free(lprn->xbak);
     free(lprn->diff);
     free(lprn->na);
 
-    return 1;
+    lprn->sum = NULL;
+    lprn->ssq = NULL;
+    lprn->xbak = NULL;
+    lprn->diff = NULL;
+    lprn->na = NULL;
+
+    return E_ALLOC;
+}
+
+static void loop_print_init (LOOP_PRINT *lprn, int lno)
+{
+    lprn->lineno = lno;
+    lprn->nvars = 0;
+    lprn->names = NULL;
+    lprn->sum = NULL;
+    lprn->ssq = NULL;
+    lprn->xbak = NULL;
+    lprn->diff = NULL;
+    lprn->na = NULL;
 }
 
 static LOOP_PRINT *get_loop_print_by_line (LOOPSET *loop, int lno, int *err)
@@ -1543,6 +1631,11 @@ static void loop_store_free (LOOP_STORE *lstore)
 {
     destroy_dataset(lstore->dset);
     lstore->dset = NULL;
+
+    strings_array_free(lstore->names, lstore->nvars);
+    lstore->nvars = 0;
+    lstore->names = NULL;
+
     free(lstore->fname);
     lstore->fname = NULL;
 
@@ -1551,63 +1644,71 @@ static void loop_store_free (LOOP_STORE *lstore)
     lstore->opt = OPT_NONE;
 }
 
-static int loop_store_set_filename (LOOPSET *loop, const char *fname,
+static int loop_store_set_filename (LOOP_STORE *lstore, 
+				    const char *fname,
 				    gretlopt opt)
 {
     if (fname == NULL || *fname == '\0') {
 	return E_ARGS;
     }
 
-    loop->store.fname = gretl_strdup(fname);
-    if (loop->store.fname == NULL) {
+    lstore->fname = gretl_strdup(fname);
+    if (lstore->fname == NULL) {
 	return E_ALLOC;
     }
 
     if (opt == OPT_NONE) {
-	opt = data_save_opt_from_suffix(loop->store.fname);
+	opt = data_save_opt_from_suffix(lstore->fname);
     }
 
-    loop->store.opt = opt;    
+    lstore->opt = opt;    
 
     return 0;
 }
 
 /* check, allocate and initialize loop data storage */
 
-static int loop_store_start (LOOPSET *loop, const int *list, 
+static int loop_store_start (LOOPSET *loop, const char *names, 
 			     const char *fname, gretlopt opt)
 {
+    LOOP_STORE *lstore = &loop->store;
     int i, n, err = 0;
 
-    if (list == NULL || list[0] == 0) {
+    if (names == NULL || *names == '\0') {
 	gretl_errmsg_set("'store' list is empty");
 	return E_DATA;
     }
 
-    err = loop_store_set_filename(loop, fname, opt);
+    lstore->names = gretl_string_split(names, &lstore->nvars);
+    if (lstore->names == NULL) {
+	return E_ALLOC;
+    }
+
+    err = loop_store_set_filename(lstore, fname, opt);
     if (err) {
 	return err;
     }
 
     n = (loop->itermax > 0)? loop->itermax : DEFAULT_NOBS;
 
-    loop->store.dset = create_auxiliary_dataset(list[0] + 1, n);
-    if (loop->store.dset == NULL) {
+    lstore->dset = create_auxiliary_dataset(lstore->nvars + 1, n, 0);
+    if (lstore->dset == NULL) {
 	return E_ALLOC;
     }
     
 #if LOOP_DEBUG
     fprintf(stderr, "loop_store_init: created sZ, v = %d, n = %d\n",
-	    loop->store.dset->v, loop->store.dset->n);
+	    lstore->dset->v, lstore->dset->n);
 #endif
 
-    for (i=1; i<=list[0] && !err; i++) {
-	const char *s = gretl_scalar_get_name(list[i]);
+    for (i=0; i<lstore->nvars && !err; i++) {
+	const char *s = lstore->names[i];
 
-	if (s == NULL) {
+	if (!gretl_is_scalar(s)) {
+	    gretl_errmsg_sprintf(_("'%s': not a scalar"), s);
 	    err = E_DATA;
 	} else {
-	    strcpy(loop->store.dset->varname[i], s);
+	    strcpy(lstore->dset->varname[i+1], s);
 	}
     }
 
@@ -1615,40 +1716,40 @@ static int loop_store_start (LOOPSET *loop, const int *list,
 }
 
 static int loop_store_update (LOOPSET *loop, int lno,
-			      const int *list, const char *fname,
+			      const char *names, const char *fname,
 			      gretlopt opt)
 {
+    LOOP_STORE *lstore = &loop->store;
     int i, t, err = 0;
 
-    if (loop->store.lineno >= 0 && loop->store.lineno != lno) {
+    if (lstore->lineno >= 0 && lstore->lineno != lno) {
 	gretl_errmsg_set("Only one 'store' command is allowed in a "
 			 "progressive loop");
 	return E_DATA;
     }
 
-    if (loop->store.dset == NULL) {
+    if (lstore->dset == NULL) {
 	/* not started yet */
-	err = loop_store_start(loop, list, fname, opt);
+	err = loop_store_start(loop, names, fname, opt);
 	if (err) {
 	    return err;
 	}
     }
 
-    loop->store.lineno = lno;
-    t = loop->store.n;
+    lstore->lineno = lno;
+    t = lstore->n;
 
-    if (t >= loop->store.dset->n) {
-	if (extend_loop_dataset(&loop->store)) {
+    if (t >= lstore->dset->n) {
+	if (extend_loop_dataset(lstore)) {
 	    return E_ALLOC;
 	}
     }
 
-    for (i=1; i<=list[0]; i++) {
-	loop->store.dset->Z[i][t] = 
-	    gretl_scalar_get_value_by_index(list[i]);
+    for (i=0; i<lstore->nvars; i++) {
+	lstore->dset->Z[i+1][t] = gretl_scalar_get_value(lstore->names[i]);
     }
 
-    loop->store.n += 1;
+    lstore->n += 1;
 
     return 0;
 }
@@ -1821,16 +1922,15 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
    allocation first.
 */
 
-static int loop_print_update (LOOP_PRINT *lprn, const int *list) 
+static int loop_print_update (LOOP_PRINT *lprn, const char *names) 
 {
     mpf_t m;
-    int j, vj;
     double x;
-    int err = 0;
+    int i, err = 0;
 
-    if (lprn->list == NULL) {
+    if (lprn->names == NULL) {
 	/* not started yet */
-	err = loop_print_start(lprn, list);
+	err = loop_print_start(lprn, names);
 	if (err) {
 	    return err;
 	}
@@ -1838,24 +1938,23 @@ static int loop_print_update (LOOP_PRINT *lprn, const int *list)
 
     mpf_init(m);
     
-    for (j=0; j<list[0]; j++) {
-	if (lprn->na[j]) {
+    for (i=0; i<lprn->nvars; i++) {
+	if (lprn->na[i]) {
 	    continue;
 	}
-	vj = list[j+1];
-	x = gretl_scalar_get_value_by_index(vj);
+	x = gretl_scalar_get_value(lprn->names[i]);
 	if (na(x)) {
-	    lprn->na[j] = 1;
+	    lprn->na[i] = 1;
 	    continue;
 	}
 	mpf_set_d(m, x); 
-	mpf_add(lprn->sum[j], lprn->sum[j], m);
+	mpf_add(lprn->sum[i], lprn->sum[i], m);
 	mpf_mul(m, m, m);
-	mpf_add(lprn->ssq[j], lprn->ssq[j], m);
-	if (!na(lprn->xbak[j]) && realdiff(x, lprn->xbak[j])) {
-	    lprn->diff[j] = 1;
+	mpf_add(lprn->ssq[i], lprn->ssq[i], m);
+	if (!na(lprn->xbak[i]) && realdiff(x, lprn->xbak[i])) {
+	    lprn->diff[i] = 1;
 	}
-	lprn->xbak[j] = x;
+	lprn->xbak[i] = x;
     }
 
     mpf_clear(m);
@@ -2055,6 +2154,7 @@ static void print_loop_coeff (const DATASET *dset,
 			      int i, PRN *prn)
 {
     char pname[VNAMELEN];
+    char tmp[NAMETRUNC];
     mpf_t c1, c2, m, sd1, sd2;
     unsigned long ln = lmod->n;
 
@@ -2095,7 +2195,8 @@ static void print_loop_coeff (const DATASET *dset,
     }
 
     gretl_model_get_param_name(lmod->model0, dset, i, pname);
-    pprintf(prn, "%*s", VNAMELEN - 1, pname);
+    maybe_trim_varname(tmp, pname);
+    pprintf(prn, "%*s", 15, tmp); /* FIXME length */
     pprintf(prn, "%#14g %#14g %#14g %#14g\n", mpf_get_d(c1), mpf_get_d(sd1), 
 	    mpf_get_d(c2), mpf_get_d(sd2));
 
@@ -2141,7 +2242,8 @@ static void loop_model_print (LOOP_MODEL *lmod, const DATASET *dset,
 static void loop_print_print (LOOP_PRINT *lprn, PRN *prn)
 {
     bigval mean, m, sd;
-    int i, vi, n;
+    int len, maxlen = 7;
+    int i, n;
     const char *s;
 
     if (lprn == NULL) {
@@ -2150,19 +2252,31 @@ static void loop_print_print (LOOP_PRINT *lprn, PRN *prn)
 
     n = lprn->n;
 
-    pprintf(prn, _("Statistics for %d repetitions\n"), n); 
-    pputs(prn, "    ");
-    pputs(prn, _("   Variable     mean         std. dev.\n"));
-
     mpf_init(mean);
     mpf_init(m);
     mpf_init(sd);
+
+    for (i=0; i<lprn->nvars; i++) {
+	len = strlen(lprn->names[i]);
+	if (len > maxlen) {
+	    maxlen = len;
+	}
+    }
+
+    pprintf(prn, _("Statistics for %d repetitions\n"), n); 
+    pputc(prn, '\n');
+    bufspace(maxlen + 1, prn);
+
+    len = get_utf_width(_("mean"), 14);
+    pprintf(prn, "%*s ", len, _("mean"));
+
+    len = get_utf_width(_("std. dev"), 14);
+    pprintf(prn, "%*s\n", len, _("std. dev"));
     
-    for (i=0; i<lprn->list[0]; i++) {
-	vi = lprn->list[i+1];
-	s = gretl_scalar_get_name(vi);
+    for (i=0; i<lprn->nvars; i++) {
+	s = lprn->names[i];
 	if (lprn->na[i]) {
-	    pprintf(prn, "%*s", VNAMELEN - 1, s);
+	    pprintf(prn, "%*s", maxlen + 1, s);
 	    pprintf(prn, "%14s %14s\n", "NA   ", "NA   ");
 	    continue;
 	}
@@ -2180,7 +2294,7 @@ static void loop_print_print (LOOP_PRINT *lprn, PRN *prn)
 		mpf_set_d(sd, 0.0);
 	    }
 	}
-	pprintf(prn, "%*s", VNAMELEN - 1, s);
+	pprintf(prn, "%*s", maxlen + 1, s);
 	pprintf(prn, "%#14g %#14g\n", mpf_get_d(mean), mpf_get_d(sd));
     }
 
@@ -2443,13 +2557,13 @@ static int top_of_loop (LOOPSET *loop, DATASET *dset)
     if (is_list_loop(loop)) {
 	err = loop_list_refresh(loop, dset);
     } else if (loop->type == INDEX_LOOP) {
-	loop->init.val = controller_get_val(&loop->init, dset, &err);
+	loop->init.val = controller_get_val(&loop->init, loop, dset, &err);
     } else if (loop->type == FOR_LOOP) {
 	forloop_init(loop, dset, &err);
     }
 
     if (!err && (loop->type == COUNT_LOOP || indexed_loop(loop))) {
-	loop->final.val = controller_get_val(&loop->final, dset, &err);
+	loop->final.val = controller_get_val(&loop->final, loop, dset, &err);
 	if (na(loop->init.val) || na(loop->final.val)) {
 	    gretl_errmsg_set(_("error evaluating loop condition"));
 	    fprintf(stderr, "loop: got NA for init and/or final value\n");
@@ -2514,14 +2628,16 @@ static int
 make_dollar_substitutions (char *str, int maxlen,
 			   const LOOPSET *loop,
 			   const DATASET *dset,
-			   int *subst)
+			   int *subst,
+			   gretlopt opt)
 {
     int err = 0;
 
-    /* if maxlen == VNAMELEN we're just processing a list name, at the top
-       of a loop */
+    /* if (opt & OPT_T) we're just processing a variable name, at the top
+       of a loop, so we can skip to the "parentage" bit 
+    */
 
-    if (maxlen != VNAMELEN && (indexed_loop(loop) || loop->type == FOR_LOOP)) {
+    if (!(opt & OPT_T) && (indexed_loop(loop) || loop->type == FOR_LOOP)) {
 	err = substitute_dollar_targ(str, maxlen, loop, dset, subst);
     }
 
@@ -2850,7 +2966,8 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 	    int subst = 0;
 
 #if LOOP_DEBUG
-	    fprintf(stderr, " j=%d, line='%s'\n", j, line);
+	    fprintf(stderr, " j=%d, line='%s', compiled = %d\n", j, line,
+	            genr_compiled(loop, j));
 #endif
 	    strcpy(errline, line);
 
@@ -2881,7 +2998,7 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 
 	    if (!loop_cmd_nodol(loop, j) && strchr(line, '$')) {
 		err = make_dollar_substitutions(line, MAXLINE, loop, 
-						dset, &subst);
+						dset, &subst, OPT_NONE);
 		if (err) {
 		    break;
 		} else if (!subst) {
@@ -2984,10 +3101,10 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		       loop_is_progressive(loop)) {
 		lprn = get_loop_print_by_line(loop, j, &err);
 		if (!err) {
-		    loop_print_update(lprn, cmd->list);
+		    err = loop_print_update(lprn, cmd->extra);
 		}
 	    } else if (cmd->ci == STORE && loop_is_progressive(loop)) {
-		err = loop_store_update(loop, j, cmd->list, cmd->param,
+		err = loop_store_update(loop, j, cmd->extra, cmd->param,
 					cmd->opt);
 	    } else if (loop_is_progressive(loop) && not_ok_in_progloop(cmd->ci)) {
 		gretl_errmsg_sprintf(_("%s: not implemented in 'progressive' loops"),
@@ -3008,7 +3125,7 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 			err = execute_genr(genr, dset, prn);
 		    }
 		}
-	    } else if (cmd->ci == DELEET) {
+	    } else if (cmd->ci == DELEET && !(cmd->opt & OPT_F)) {
 		err = loop_delete_object(cmd, prn);
 	    } else {
 		err = gretl_cmd_exec(s, dset);
@@ -3018,6 +3135,12 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		}
 	    }
 
+	    if (err && (cmd->flags & CMD_CATCH)) {
+		set_gretl_errno(err);
+		cmd->flags ^= CMD_CATCH;
+		err = 0;
+	    }
+
 	} /* end execution of commands within loop */
 
 	if (err) {
