@@ -24,14 +24,12 @@
 
 #include "libgretl.h" 
 #include "libset.h"
-#include "usermat.h"
+#include "uservar.h"
 #include "matrix_extra.h"
 #include "gretl_func.h"
 #include "nlspec.h"
 #include "cmd_private.h"
 #include "estim_private.h"
-#include "gretl_scalar.h"
-#include "gretl_string_table.h"
 #include "gretl_bfgs.h"
 #include "tsls.h"
 
@@ -534,12 +532,13 @@ static int push_scalar_coeff (nlspec *s, double x)
    either a scalar or a vector
 */
 
-static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
+static int nlspec_push_param (nlspec *s, const char *name, 
+			      GretlType type, char *deriv)
 {
     parm *params, *p;
     int np = s->nparam;
     int err;
-    
+
     params = realloc(s->params, (np + 1) * sizeof *params);
     if (params == NULL) {
 	return E_ALLOC;
@@ -549,7 +548,7 @@ static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
 
     p->name[0] = '\0';
     strncat(p->name, name, VNAMELEN - 1);
-    p->type = GRETL_TYPE_DOUBLE;
+    p->type = type;
     p->dtype = GRETL_TYPE_NONE;
     p->deriv = deriv;
     p->dnum = 0;
@@ -565,7 +564,7 @@ static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
     s->params = params;
     s->nparam = np + 1;
 
-    if (gretl_is_scalar(name)) {
+    if (type == GRETL_TYPE_DOUBLE) {
 	err = push_scalar_coeff(s, gretl_scalar_get_value(name));
     } else {
 	gretl_matrix *m = get_matrix_by_name(name);
@@ -574,8 +573,6 @@ static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
 #if NLS_DEBUG
 	fprintf(stderr, "vector param: m = %p, k = %d\n", (void *) m, k);
 #endif
-
-	p->type = GRETL_TYPE_MATRIX;
 	p->vec = m;
 	p->nc = k;
 	err = push_vec_coeffs(s, m, k);
@@ -591,19 +588,21 @@ static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
    if so return 0, else return E_DATATYPE.
 */
 
-static int check_param_name (const char *name)
+static int check_param_name (const char *name, GretlType *type)
 {
     int err = 0;
 
     if (gretl_is_scalar(name)) {
-	; /* OK */
+	*type = GRETL_TYPE_DOUBLE; /* OK */
     } else {
 	gretl_matrix *m = get_matrix_by_name(name);
 
 	if (m == NULL || gretl_vector_get_length(m) == 0) {
 	    gretl_errmsg_sprintf(_("'%s': expected a scalar or vector"), name);
 	    err = E_DATATYPE;
-	} 
+	} else {
+	    *type = GRETL_TYPE_MATRIX;
+	}
     }
 
     return err;
@@ -669,13 +668,14 @@ nlspec_add_params_from_line (nlspec *s, const char *str)
 
     for (i=0; i<nf && !err; i++) {
 	char *name = gretl_word_strdup(str, &str, OPT_S, &err);
+	GretlType type = 0;
 
 	if (!err) {
-	    err = check_param_name(name);
+	    err = check_param_name(name, &type);
 	}
 
 	if (!err) {
-	    err = nlspec_push_param(s, name, NULL);
+	    err = nlspec_push_param(s, name, type, NULL);
 	}
 
 	free(name);
@@ -688,6 +688,37 @@ nlspec_add_params_from_line (nlspec *s, const char *str)
     return err;
 }
 
+/* note: this function handles scalar params only */
+
+int real_nlspec_add_param_list (nlspec *spec, int np, 
+				double *vals, char **names, 
+				gretlopt opt)
+{
+    int i, err = 0;
+
+    if (spec->params != NULL || np == 0) {
+	return E_DATA;
+    }
+
+    for (i=0; i<np && !err; i++) {
+	if (opt & OPT_A) {
+	    /* doing internal auxiliary NLS (e.g. arma init) */
+	    err = add_auxiliary_scalar(names[i], vals[i]);
+	} else {
+	    err = gretl_scalar_add(names[i], vals[i]);
+	}
+	if (!err) {
+	    err = nlspec_push_param(spec, names[i], GRETL_TYPE_DOUBLE, NULL);
+	}
+    }
+
+    if (err) {
+	nlspec_destroy_arrays(spec);
+    } 
+
+    return err;
+}
+
 /**
  * nlspec_add_param_list:
  * @spec: nls specification.
@@ -696,7 +727,8 @@ nlspec_add_params_from_line (nlspec *s, const char *str)
  * @names: array of parameter names.
  *
  * Adds to @spec a list of (scalar) parameters to be estimated.
- * For an example of use see arma.c in the gretl plugin directory.
+ * For an example of use see nls_example.c in the gretl extra
+ * subdirectory.
  *
  * Returns: 0 on success, non-zero error code on error.
  */
@@ -704,24 +736,16 @@ nlspec_add_params_from_line (nlspec *s, const char *str)
 int nlspec_add_param_list (nlspec *spec, int np, double *vals,
 			   char **names)
 {
-    int i, err = 0;
+    return real_nlspec_add_param_list(spec, np, vals, names,
+				      OPT_NONE);
 
-    if (spec->params != NULL || np == 0) {
-	return E_DATA;
-    }
+}
 
-    for (i=0; i<np && !err; i++) {
-	err = gretl_scalar_add(names[i], vals[i]);
-	if (!err) {
-	    err = nlspec_push_param(spec, names[i], NULL);
-	}
-    }
-
-    if (err) {
-	nlspec_destroy_arrays(spec);
-    } 
-
-    return err;
+int aux_nlspec_add_param_list (nlspec *spec, int np, double *vals,
+			       char **names)
+{
+    return real_nlspec_add_param_list(spec, np, vals, names,
+				      OPT_A);
 }
 
 /* update the 'external' values of scalars or matrices using
@@ -792,7 +816,7 @@ static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
 	return E_DF;
     }
 
-    dset = create_auxiliary_dataset(nvar, nobs);
+    dset = create_auxiliary_dataset(nvar, nobs, 0);
     if (dset == NULL) {
 	return E_ALLOC;
     }
@@ -1768,7 +1792,7 @@ static MODEL GNR (nlspec *spec, DATASET *dset, PRN *prn)
 
     /* number of variables = 1 (const) + 1 (depvar) + spec->ncoeff
        (derivatives) */
-    gdset = create_auxiliary_dataset(spec->ncoeff + 2, dset->n);
+    gdset = create_auxiliary_dataset(spec->ncoeff + 2, dset->n, 0);
     if (gdset == NULL) {
 	gretl_model_init(&gnr);
 	gnr.errcode = E_ALLOC;
@@ -2554,6 +2578,7 @@ int nlspec_add_param_with_deriv (nlspec *spec, const char *s)
     const char *p = s;
     char *name = NULL;
     char *deriv = NULL;
+    GretlType type = 0;
     int err = 0;
 
     if (spec->ci == GMM) {
@@ -2572,10 +2597,10 @@ int nlspec_add_param_with_deriv (nlspec *spec, const char *s)
 	return E_PARSE;
     }
 
-    err = check_param_name(name);
+    err = check_param_name(name, &type);
     
     if (!err) {
-	err = nlspec_push_param(spec, name, deriv);
+	err = nlspec_push_param(spec, name, type, deriv);
 	if (err) {
 	    free(deriv);
 	    deriv = NULL;
@@ -3174,7 +3199,7 @@ static MODEL real_nl_model (nlspec *spec, DATASET *dset,
 #endif
 
     if (!dset->auxiliary) {
-	dataset_drop_last_variables(dset->v - origv, dset);
+	dataset_drop_last_variables(dset, dset->v - origv);
     }
 
     if (!(opt & OPT_A) && !nlmod.errcode) {
@@ -3362,7 +3387,7 @@ static int ivreg_oc_setup (nlspec *spec, const int *ilist,
     int i, err;
 
     /* add GMM residual */
-    err = dataset_add_series(1, dset);
+    err = dataset_add_series(dset, 1);
 
     if (!err) {
 	*rv = v;
@@ -3432,7 +3457,7 @@ static int ivreg_set_params (nlspec *spec, MODEL *pmod)
 
     err = nlspec_add_param_list(spec, np, pmod->coeff, names);
 
-    free_strings_array(names, np);
+    strings_array_free(names, np);
 
     return err;
 }
@@ -3562,8 +3587,8 @@ MODEL ivreg_via_gmm (const int *list, DATASET *dset, gretlopt opt)
     free(ilist);
     clear_model(&olsmod);
 
-    dataset_drop_last_variables(dset->v - orig_v, dset);
-    user_matrix_destroy_by_name(IVREG_WEIGHTNAME, NULL);
+    dataset_drop_last_variables(dset, dset->v - orig_v);
+    user_var_delete_by_name(IVREG_WEIGHTNAME, NULL);
 
     return model;
 }
@@ -3815,7 +3840,7 @@ int nls_boot_calc (const MODEL *pmod, DATASET *dset,
 
     clear_nlspec(spec);
 
-    dataset_drop_last_variables(dset->v - origv, dset);
+    dataset_drop_last_variables(dset, dset->v - origv);
 
     /* restore original y */
     if (orig_y != NULL) {
