@@ -18,7 +18,7 @@
  */
 
 #include "libgretl.h"
-#include "gretl_scalar.h"
+#include "uservar.h"
 
 #include <errno.h>
 
@@ -101,6 +101,9 @@ struct gretl_option gretl_opts[] = {
     { APPEND,   OPT_T, "time-series", 0 },
     { APPEND,   OPT_R, "rowoffset", 2 },
     { APPEND,   OPT_C, "coloffset", 2 },
+    { APPEND,   OPT_F, "fixed-cols", 2 },
+    { APPEND,   OPT_M, "rowmask", 2 },
+    { APPEND,   OPT_L, "cols", 2 },
     { APPEND,   OPT_S, "sheet", 2 },
     { ARBOND,   OPT_A, "asymptotic", 0 },
     { ARBOND,   OPT_D, "time-dummies", 0 },
@@ -124,10 +127,10 @@ struct gretl_option gretl_opts[] = {
     { BXPLOT,   OPT_L, "outliers", 2 },
     { BXPLOT,   OPT_P, "panel", 0 },
     { BXPLOT,   OPT_U, "output", 2 },
+    { BXPLOT,   OPT_X, "matrix", 2 },
     { BXPLOT,   OPT_Z, "factorized", 0 },
     { CHOW,     OPT_D, "dummy", 0 },
     { CLEAR,    OPT_D, "dataset", 0 },
-    { CLEAR,    OPT_O, "other", 0 },
     { COINT,    OPT_E, "test-down", 0 },
     { COINT,    OPT_N, "nc", 0 },
     { COINT,    OPT_R, "ctt", 0 },     
@@ -150,8 +153,10 @@ struct gretl_option gretl_opts[] = {
     { CORRGM,   OPT_G, "gui-graph", 0 },
     { CUSUM,    OPT_R, "squares", 0 },
     { DATA,     OPT_O, "odbc", 0 },
+    { DATA,     OPT_M, "rowmask", 2 },
     { DATAMOD,  OPT_P, "preserve", 0 },
     { DELEET,   OPT_D, "db", 0 },
+    { DELEET,   OPT_F, "force", 0 },
     { DELEET,   OPT_L, "list", 0 },
     { DIFFTEST, OPT_G, "sign", 0 },
     { DIFFTEST, OPT_R, "rank-sum", 0 },
@@ -205,6 +210,7 @@ struct gretl_option gretl_opts[] = {
     { FREQ,     OPT_N, "nbins", 2 },
     { FREQ,     OPT_M, "min", 2 },
     { FREQ,     OPT_W, "binwidth", 2 },
+    { FREQ,     OPT_X, "matrix", 2 },
     { FUNDEBUG, OPT_C, "continue", 0 },
     { FUNDEBUG, OPT_N, "next", 0 },
     { FUNDEBUG, OPT_Q, "quit", 0 },
@@ -227,6 +233,7 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_B, "cubic-fit", 0 },
     { GNUPLOT,  OPT_E, "semilog-fit", 0 },
     { GNUPLOT,  OPT_M, "with-impulses", 1 },
+    { GNUPLOT,  OPT_P, "with-lp", 1 },
     { GNUPLOT,  OPT_S, "suppress-fitted", 0 },
     { GNUPLOT,  OPT_T, "time-series", 0 },
     { GNUPLOT,  OPT_Z, "dummy", 0 },
@@ -253,6 +260,13 @@ struct gretl_option gretl_opts[] = {
     { IVREG,    OPT_W, "weights", 2 },
     { IVREG,    OPT_X, "no-tests", 0 },
     { IVREG,    OPT_C, "cluster", 2 },
+    { JOIN,     OPT_I, "ikey", 2 },
+    { JOIN,     OPT_O, "okey", 2 },
+    { JOIN,     OPT_F, "filter", 2 },
+    { JOIN,     OPT_A, "aggr", 2 },
+    { JOIN,     OPT_D, "data", 2 },
+    { JOIN,     OPT_R, "rev", 2 },
+    { JOIN,     OPT_V, "verbose", 0 },
     { KALMAN,   OPT_C, "cross", 0 },
     { KALMAN,   OPT_D, "diffuse", 0 },
     { KPSS,     OPT_T, "trend", 0 },
@@ -264,6 +278,9 @@ struct gretl_option gretl_opts[] = {
     { LEVINLIN, OPT_T, "ct", 0 },
     { MAKEPKG,  OPT_I, "index", 0 },
     { MAKEPKG,  OPT_T, "translations", 0 },
+    { MARKERS,  OPT_D, "delete", 0 },
+    { MARKERS,  OPT_F, "from-file", 2 },
+    { MARKERS,  OPT_T, "to-file", 2 },
     { MODTEST,  OPT_A, "autocorr", 0 },
     { MODTEST,  OPT_B, "breusch-pagan", 0 },
     { MODTEST,  OPT_C, "comfac", 0 },
@@ -275,6 +292,7 @@ struct gretl_option gretl_opts[] = {
     { MODTEST,  OPT_R, "robust", 0 },
     { MODTEST,  OPT_W, "white", 0 },
     { MODTEST,  OPT_X, "white-nocross", 0 },
+    { LABELS,   OPT_D, "delete", 0 },
     { LABELS,   OPT_F, "from-file", 2 },
     { LABELS,   OPT_T, "to-file", 2 },
     { LOGISTIC, OPT_M, "ymax", 2 },
@@ -333,13 +351,15 @@ struct gretl_option gretl_opts[] = {
     { OMIT,     OPT_W, "test-only", 0 },
     { OPEN,     OPT_B, "progress-bar", 0 },
     { OPEN,     OPT_D, "drop-empty", 0 },
-    { OPEN,     OPT_F, "cols", 2 },    
+    { OPEN,     OPT_F, "fixed-cols", 2 },    
     { OPEN,     OPT_O, "odbc", 0 },
     { OPEN,     OPT_P, "preserve", 0 },
     { OPEN,     OPT_R, "rowoffset", 2 },
     { OPEN,     OPT_C, "coloffset", 2 },
     { OPEN,     OPT_S, "sheet", 2 },
     { OPEN,     OPT_W, "www", 0 },
+    { OPEN,     OPT_L, "cols", 2 },
+    { OPEN,     OPT_M, "rowmask", 2 },
     { OUTFILE,  OPT_A, "append", 0 },
     { OUTFILE,  OPT_C, "close", 0 },
     { OUTFILE,  OPT_W, "write", 0 },
@@ -413,7 +433,6 @@ struct gretl_option gretl_opts[] = {
     { SETOBS,   OPT_T, "time-series", 0 },
     { SETOBS,   OPT_X, "cross-section", 0 },
     { SETOBS,   OPT_N, "special-time-series", 0 },
-    { SETOBS,   OPT_M, "labels", 2 },
     { SMPL,     OPT_B, "balanced", 0 },
     { SMPL,     OPT_C, "contiguous", 0 },
     { SMPL,     OPT_F, "full", 0 },
@@ -438,8 +457,8 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_E, "comment", 2 },
     { STORE,    OPT_I, "decimal-comma", 0 },
     { SUMMARY,  OPT_B, "by", 2 },
-    { SUMMARY,  OPT_M, "matrix", 2 },
     { SUMMARY,  OPT_S, "simple", 0 },
+    { SUMMARY,  OPT_X, "matrix", 2 },
     { SYSTEM,   OPT_I, "iterate", 0 },
     { SYSTEM,   OPT_V, "verbose", 0 },
     { TOBIT,    OPT_L, "llimit", 2 },
@@ -496,6 +515,27 @@ int cluster_option_ok (int ci)
     return 0;
 }
 
+int matrix_data_option (int ci, gretlopt opt)
+{
+    if (opt & OPT_X) {
+	int i, found = 0;
+
+	for (i=0; gretl_opts[i].ci; i++) {
+	    if (gretl_opts[i].ci == ci) {
+		if (gretl_opts[i].o == OPT_X &&
+		    !strcmp(gretl_opts[i].longopt, "matrix")) {
+		    return 1;
+		}
+		found = 1;
+	    } else if (found) {
+		break;
+	    }
+	}
+    }
+
+    return 0;
+}
+
 /* this function is used in compiling the gretl reference
    manual */
 
@@ -514,7 +554,7 @@ char **get_all_option_strings (int *pn)
 	for (i=0; i<n; i++) {
 	    optstrs[i] = gretl_strdup(gretl_opts[i].longopt);
 	    if (optstrs[i] == NULL) {
-		free_strings_array(optstrs, n);
+		strings_array_free(optstrs, n);
 		optstrs = NULL;
 		break;
 	    }
@@ -919,7 +959,9 @@ double get_optval_double (int ci, gretlopt opt)
  *
  * Returns: the integer ancillary value currently 
  * associated with option @opt for command @ci, if any,
- * otherwise 0 (with a non-zero value written to @err).
+ * otherwise 0. A non-zero value written to @err if
+ * such a value is required for the option in question
+ * but is not present.
  */
 
 int get_optval_int (int ci, gretlopt opt, int *err)
@@ -940,7 +982,7 @@ int get_optval_int (int ci, gretlopt opt, int *err)
 		ret = (int) x;
 	    }
 	}
-    } else if (status == 2) {
+    } else if (status == 2 && err != NULL) {
 	/* parameter is required */
 	*err = E_DATA;
     }
@@ -948,7 +990,7 @@ int get_optval_int (int ci, gretlopt opt, int *err)
     return ret;
 }
 
-/* called via GUI */
+/* below: called via GUI */
 
 /**
  * set_optval_double:
@@ -1254,10 +1296,15 @@ gretlopt get_gretl_options (char *line, int *err)
 	*err = 0;
     }
 
-    if (strlen(line) < 2 || *line == '#') {
+    if (strlen(line) < 2 || *line == '#' ||
+	strchr(line, '-') == NULL) {
 	return 0;
     }
 
+#if 0
+    fprintf(stderr, "get_gretl_options: '%s'\n", line);
+#endif
+
     get_cmdword(line, cmdword);
 
     if (!strcmp(cmdword, "catch")) {
