@@ -95,6 +95,7 @@ GPT_SPEC *plotspec_new (void)
     spec->auxdata = NULL;
     spec->markers = NULL;
     spec->n_markers = 0;
+    spec->scale = 1.0;
     spec->labeled = NULL;
     spec->ptr = NULL;
     spec->bars = NULL;
@@ -153,11 +154,11 @@ void plotspec_destroy (GPT_SPEC *spec)
     }
 
     if (spec->literal != NULL) {
-	free_strings_array(spec->literal, spec->n_literal);
+	strings_array_free(spec->literal, spec->n_literal);
     }
 
     if (spec->markers != NULL) {
-	free_strings_array(spec->markers, spec->n_markers);
+	strings_array_free(spec->markers, spec->n_markers);
     }
 
     if (spec->labeled != NULL) {
@@ -1066,6 +1067,12 @@ int plotspec_print (GPT_SPEC *spec, FILE *fp)
 	}
     }
 
+    if (spec->scale != 1.0) {
+	gretl_push_c_numeric_locale();
+	fprintf(fp, "# scale = %.1f", spec->scale);
+	gretl_pop_c_numeric_locale();
+    }
+
     if (!mono) {
 	write_styles_from_plotspec(spec, fp);
     }
@@ -1333,7 +1340,9 @@ int plotspec_print (GPT_SPEC *spec, FILE *fp)
 	    }
 	}
 
-	if (line->width != 1) {
+	if (line->width == 1 && spec->scale > 1.0) {
+	    fprintf(fp, " lw 2");
+	} else if (line->width != 1) {
 	    fprintf(fp, " lw %d", line->width);
 	}
 
