@@ -25,7 +25,7 @@
 #include "forecast.h"
 #include "gretl_func.h"
 #include "matrix_extra.h"
-#include "gretl_scalar.h"
+#include "uservar.h"
 #include "gretl_bundle.h"
 #include "gretl_string_table.h"
 
@@ -343,17 +343,21 @@ void print_freq_test (const FreqDist *freq, PRN *prn)
 /**
  * print_freq:
  * @freq: gretl frequency distribution struct.
+ * @varno: ID number of the series in question.
+ * @dset: pointer to dataset.
  * @prn: gretl printing struct.
  *
  * Print frequency distribution to @prn.
  */
 
-void print_freq (const FreqDist *freq, PRN *prn)
+void print_freq (const FreqDist *freq, int varno, const DATASET *dset, 
+		 PRN *prn)
 {
     int i, k, nlw, K;
     int total, valid, missing;
     char word[64];
     double f, cumf = 0;
+    const char **labels = NULL;
 
     if (freq == NULL) {
 	return;
@@ -373,11 +377,25 @@ void print_freq (const FreqDist *freq, PRN *prn)
 	return;
     } 
 
+    if (varno > 0 && dset != NULL && series_has_string_table(dset, varno)) {
+	int n_labels;
+
+	labels = series_get_string_vals(dset, varno, &n_labels);
+	if (n_labels != freq->numbins) {
+	    labels = NULL;
+	}
+    }
+
     if (freq->discrete) {
 	pputs(prn, _("\n          frequency    rel.     cum.\n\n"));
 
 	for (k=0; k<=K; k++) {
-	    sprintf(word, "%4g", freq->midpt[k]);
+	    if (labels != NULL) {
+		*word = '\0';
+		gretl_utf8_strncat(word, labels[k], 8);
+	    } else {
+		sprintf(word, "%4g", freq->midpt[k]);
+	    }
 	    pputs(prn, word);
 	    nlw = 10 - strlen(word);
 	    bufspace(nlw, prn);
@@ -517,6 +535,9 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
     double ymin = 1.0e-7;
     double pearson = 0.0;
     int i, j;
+    char lbl[64];
+    int has_clabels = (tab->clabels!=NULL);
+    int has_rlabels = (tab->rlabels!=NULL);
 
     if (*tab->rvarname != '\0' && *tab->cvarname != '\0') {
 	pputc(prn, '\n');
@@ -527,18 +548,38 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 	pputs(prn, "\n       ");
     }
 
+    if(has_rlabels){
+	pputs(prn, "    ");
+    }
+
     for (j=0; j<c; j++) {
-	pprintf(prn, "[%4g]", tab->cval[j]);
-    } 
+	if (!has_clabels) {
+	    pprintf(prn, "[%4g]", tab->cval[j]);
+	} else {
+	    *lbl = '\0';
+	    gretl_utf8_strncat(lbl, tab->clabels[j], 8);
+	    pprintf(prn, "[%8s]", lbl);
+	}
+    }
 
     pprintf(prn,"  %s\n  \n", _("TOT."));
 
     for (i=0; i<r; i++) {
 
 	if (tab->rtotal[i] > 0) {
-	    pprintf(prn, "[%4g] ", tab->rval[i]);
+	    if (!has_rlabels) {
+		pprintf(prn, "[%4g] ", tab->rval[i]);
+	    } else {
+		*lbl = '\0';
+		gretl_utf8_strncat(lbl, tab->rlabels[i], 8);
+		pprintf(prn, "[%8s] ", lbl);
+	    }
 
 	    for (j=0; j<c; j++) {
+		if (has_clabels) {
+		    pputs(prn, "    ");
+		}
+
 		if (tab->ctotal[j]) {
 		    if (tab->f[i][j] || (opt & OPT_Z)) {
 			if (opt & (OPT_C | OPT_R)) {
@@ -579,8 +620,14 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 
     pputc(prn, '\n');
     pputs(prn, _("TOTAL  "));
+    if(has_rlabels){
+	pputs(prn, "    ");
+    }
 
     for (j=0; j<c; j++) {
+	if (has_clabels) {
+	    pputs(prn, "    ");
+	}
 	if (opt & OPT_R) {
 	    x = 100.0 * tab->ctotal[j] / tab->n;
 	    pprintf(prn, "%5.1f%%", x);
@@ -1071,8 +1118,7 @@ static void outxx (double x, int ci, int wid, PRN *prn)
 
 static int vmat_maxlen (VMatrix *vmat)
 {
-    int len, maxlen = 0;
-    int i;
+    int i, len, maxlen = 0;
 
     for (i=0; i<vmat->dim; i++) {
 	len = strlen(vmat->names[i]);
@@ -1099,7 +1145,7 @@ static int vmat_maxlen (VMatrix *vmat)
 void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 {
     register int i, j;
-    int nf, li2, p, k, idx, ij2;
+    int n, nf, li2, p, k, idx, ij2;
     int maxlen = 0;
     int fwidth = 14;
     int fields = 5;
@@ -1124,8 +1170,13 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 	/* print the varname headings */
 	for (j=1; j<=p; ++j)  {
 	    s = vmat->names[j + nf - 1];
-	    bufspace(fwidth - strlen(s), prn);
-	    pputs(prn, s); 
+	    n = strlen(s);
+	    if (n > fwidth - 1) {
+		pprintf(prn, " %.*s~", fwidth - 2, s);
+	    } else {
+		bufspace(fwidth - n, prn);
+		pputs(prn, s);
+	    } 
 	}
 	pputc(prn, '\n');
 
@@ -1136,7 +1187,12 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 		outxx(vmat->vec[idx], vmat->ci, fwidth, prn);
 	    }
 	    if (fwidth < 15) pputc(prn, ' ');
-	    pprintf(prn, " %s\n", vmat->names[j]);
+	    n = strlen(vmat->names[j]);
+	    if (n > fwidth - 1) {
+		pprintf(prn, " %.*s~\n", fwidth - 2, vmat->names[j]);
+	    } else {
+		pprintf(prn, " %s\n", vmat->names[j]);
+	    }
 	}
 
 	/* print upper triangular part of matrix */
@@ -1148,7 +1204,12 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 		outxx(vmat->vec[idx], vmat->ci, fwidth, prn);
 	    }
 	    if (fwidth < 15) pputc(prn, ' ');
-	    pprintf(prn, " %s\n", vmat->names[ij2]);
+	    n = strlen(vmat->names[ij2]);
+	    if (n > fwidth - 1) {
+		pprintf(prn, " %.*s~\n", fwidth - 2, vmat->names[ij2]);
+	    } else {
+		pprintf(prn, " %s\n", vmat->names[ij2]);
+	    }	    
 	}
 	pputc(prn, '\n');
     }
@@ -1584,6 +1645,55 @@ void print_obs_marker (int t, const DATASET *dset, int len, PRN *prn)
     real_print_obs_marker(t, dset, len, 1, prn);
 }
 
+char *maybe_trim_varname (char *targ, const char *src)
+{
+    int srclen = strlen(src);
+
+    if (srclen < NAMETRUNC) {
+	strcpy(targ, src);
+    } else {
+	const char *p = strrchr(src, '_');
+
+	*targ = '\0';
+
+	if (p != NULL && isdigit(*(p+1)) && strlen(p) < 4) {
+	    /* preserve lag identifier? */
+	    int snip = srclen - NAMETRUNC + 2;
+	    int fore = p - src;
+
+	    strncat(targ, src, fore - snip);
+	    strncat(targ, "~", 1);
+	    strncat(targ, p, strlen(p));
+	} else {
+	    strncat(targ, src, NAMETRUNC - 2);
+	    strncat(targ, "~", 1);
+	}
+    }
+
+    return targ;
+}
+
+int max_namelen_in_list (const int *list, const DATASET *dset)
+{
+    int i, vi, ni, n = 0;
+
+    for (i=1; i<=list[0]; i++) {
+	vi = list[i];
+	if (vi >= 0 && vi < dset->v) {
+	    ni = strlen(dset->varname[list[i]]);
+	    if (ni > n) {
+		n = ni;
+	    }
+	}
+    }
+
+    if (n >= NAMETRUNC) {
+	n = NAMETRUNC - 1;
+    }
+
+    return n;
+}
+
 /**
  * varlist:
  * @dset: data information struct.
@@ -1605,7 +1715,7 @@ void varlist (const DATASET *dset, PRN *prn)
     }
 
     for (i=0; i<dset->v; i++) {
-	if (STACK_LEVEL(dset, i) == level) {
+	if (series_get_stack_level(dset, i) == level) {
 	    len = strlen(dset->varname[i]);
 	    if (len > maxlen) {
 		maxlen = len;
@@ -1616,6 +1726,8 @@ void varlist (const DATASET *dset, PRN *prn)
 
     if (maxlen < 9) {
 	nv = 5;
+    } else if (maxlen > 20) {
+	nv = 1;
     } else if (maxlen > 13) {
 	nv = 3;
     }
@@ -1624,7 +1736,7 @@ void varlist (const DATASET *dset, PRN *prn)
 
     j = 1;
     for (i=0; i<dset->v; i++) {
-	if (level > 0 && STACK_LEVEL(dset, i) != level) {
+	if (level > 0 && series_get_stack_level(dset, i) != level) {
 	    continue;
 	}
 	pprintf(prn, "%3d) %-*s", i, maxlen + 2, dset->varname[i]);
@@ -1702,7 +1814,7 @@ static void print_listed_objects (const char *s,
 	    gretl_matrix_print_to_prn(m, name, prn);
 	} else if ((list = get_list_by_name(name)) != NULL) {
 	    print_varlist(name, list, dset, prn);
-	} else if ((b = get_gretl_bundle_by_name(name)) != NULL) {
+	} else if ((b = get_bundle_by_name(name)) != NULL) {
 	    gretl_bundle_print(b, prn);
 	} else if ((p = get_string_by_name(name)) != NULL) {
 	    pprintf(prn, "%s\n", p);	
@@ -1811,9 +1923,11 @@ static int print_by_obs (int *list,
     double x;
     int err = 0;
 
-    pmax = get_pmax_array(list, dset);
-    if (pmax == NULL) {
-	return E_ALLOC;
+    if (!(opt & OPT_S)) {
+	pmax = get_pmax_array(list, dset);
+	if (pmax == NULL) {
+	    return E_ALLOC;
+	}
     }
 
     if (opt & OPT_N) {
@@ -1853,7 +1967,14 @@ static int print_by_obs (int *list,
 		x = dset->Z[blist[i]][t];
 		if (na(x)) {
 		    bufspace(colwidth, prn);
-		} else { 
+		} else if (opt & OPT_S) {
+		    const char *s = series_get_string_val(dset, blist[i], t);
+
+		    if (s != NULL) {
+			bufspace(6, prn);
+			pputs(prn, s);
+		    }
+		} else {
 		    bufprintnum(buf, x, pmax[j-1], gprec, colwidth);
 		    pputs(prn, buf);
 		}
@@ -1943,7 +2064,7 @@ int printdata (const int *list, const char *mstr,
     /* at this point plist should have something in it */
     if (plist == NULL) {
 	return E_ALLOC;
-    } 
+    }
 
     if (gretl_list_has_separator(plist)) {
 	err = adjust_print_list(plist, &screenvar, opt);
@@ -1967,6 +2088,9 @@ int printdata (const int *list, const char *mstr,
     }
 
     if (opt & OPT_O) {
+	if (plist[0] == 1 && series_has_string_table(dset, plist[1])) {
+	    opt |= OPT_S;
+	}
 	err = print_by_obs(plist, dset, opt, screenvar, prn);
     } else {
 	err = print_by_var(plist, dset, prn);
@@ -2862,9 +2986,10 @@ char *bufgets (char *s, size_t size, const char *buf)
 	    break;
 	}
 	if (i == size - 1) {
-	    fprintf(stderr, "bufgets: line too long: max %d characters\n", 
+	    fprintf(stderr, "*** bufgets: line too long: max %d characters\n", 
 		    (int) size);
 	    s[i] = '\0';
+	    fprintf(stderr, " '%.16s...'\n", s);
 	    break;
 	}
     }
