@@ -21,6 +21,7 @@
 
 #define GRETL_DIGITS 6
 #define GRETL_MP_DIGITS 12
+#define NAMETRUNC 18
 
 void session_time (PRN *prn);
 
@@ -34,7 +35,8 @@ void gui_logo (PRN *prn);
 
 void text_print_model_confints (const CoeffIntervals *cf, PRN *prn);
 
-void print_freq (const FreqDist *freq, PRN *prn);
+void print_freq (const FreqDist *freq, int varno, const DATASET *dset, 
+		 PRN *prn);
 
 void print_freq_test (const FreqDist *freq, PRN *prn);
 
@@ -53,6 +55,10 @@ int max_obs_marker_length (const DATASET *dset);
 
 void print_obs_marker (int t, const DATASET *dset, int len, PRN *prn);
 
+char *maybe_trim_varname (char *targ, const char *src);
+
+int max_namelen_in_list (const int *list, const DATASET *dset);
+
 void varlist (const DATASET *dset, PRN *prn);
 
 void maybe_list_vars (const DATASET *dset, PRN *prn);
