@@ -22,9 +22,8 @@
 #include "libgretl.h"
 #include "libset.h"
 #include "gretl_func.h"
-#include "gretl_string_table.h"
-#include "gretl_scalar.h"
 #include "matrix_extra.h"
+#include "uservar.h"
 
 #include <errno.h>
 
@@ -189,7 +188,7 @@ static int gen_arg_val (const char *s, DATASET *dset,
 	if (type == GRETL_TYPE_DOUBLE) {
 	    *scalar = 1;
 	    *px = gretl_scalar_get_value(name);
-	    gretl_scalar_delete(name, NULL);
+	    user_var_delete_by_name(name, NULL);
 	} else if (type == GRETL_TYPE_MATRIX) {
 	    gretl_matrix *m = get_matrix_by_name(name);
 
@@ -197,7 +196,7 @@ static int gen_arg_val (const char *s, DATASET *dset,
 	    if (*pm == NULL) {
 		err = E_ALLOC;
 	    }
-	    user_matrix_destroy_by_name(name, NULL);
+	    user_var_delete_by_name(name, NULL);
 	} else if (type == GRETL_TYPE_SERIES) {
 	    *pv = current_series_index(dset, name);
 	} else {
@@ -535,7 +534,7 @@ static int print_arg (char **pfmt, char **pargs, DATASET *dset,
     if (free_m) {
 	gretl_matrix_free(m);
     } else if (free_v) {
-	dataset_drop_last_variables(1, dset);
+	dataset_drop_last_variables(dset, 1);
     }
     
     return err;
@@ -623,6 +622,45 @@ static int split_printf_line (const char *s, char *targ, int *sp,
     return 0;
 }
 
+#define SPRINTF_COMPAT 1
+
+static user_var *get_stringvar_target (const char *targ, 
+				       DATASET *dset,
+				       PRN *prn,
+				       int *err)
+{
+    user_var *uvar = NULL;
+
+    if (*targ == '\0') {
+	*err = E_PARSE;
+    } else {
+	uvar = get_user_var_by_name(targ);
+	if (uvar != NULL && 
+	    user_var_get_type(uvar) != GRETL_TYPE_STRING) {
+	    *err = E_TYPES;
+	    return NULL;
+	}
+#if SPRINTF_COMPAT
+	if (uvar == NULL) {
+	    char genline[64];
+
+	    sprintf(genline, "string %s = \"\"", targ);
+	    *err = generate(genline, dset, OPT_NONE, prn);
+	    if (!*err) {
+		uvar = get_user_var_of_type_by_name(targ, GRETL_TYPE_STRING);
+		
+	    }
+	}
+#endif	
+	if (uvar == NULL) {
+	    gretl_errmsg_sprintf(_("%s: not a string variable"), targ);
+	    *err = E_DATA;
+	}
+    }
+
+    return uvar;
+}
+
 /* supports both printf and sprintf */
 
 static int real_do_printf (const char *line, DATASET *dset, 
@@ -632,6 +670,7 @@ static int real_do_printf (const char *line, DATASET *dset,
     char targ[VNAMELEN];
     char *format = NULL;
     char *args = NULL;
+    user_var *uvar = NULL;
     int sp, err = 0;
 
     gretl_error_clear();
@@ -643,10 +682,13 @@ static int real_do_printf (const char *line, DATASET *dset,
 	return err;
     }
 
-    if (sp && *targ == '\0') {
-	/* sprintf: we need a target stringvar */
-	return E_PARSE;
-    }    
+    if (sp) {
+	/* sprintf: we need a target string variable */
+	uvar = get_stringvar_target(targ, dset, inprn, &err);
+	if (err) {
+	    return err;
+	}
+    }
 
     /* Even for printf we'll buffer the output locally in 
        case there's an error part way through the printing.
@@ -692,7 +734,13 @@ static int real_do_printf (const char *line, DATASET *dset,
 	const char *buf = gretl_print_get_buffer(prn);
 
 	if (sp) {
-	    err = save_named_string(targ, buf, inprn);
+	    char *tmp = gretl_strdup(buf);
+
+	    if (tmp == NULL) {
+		err = E_ALLOC;
+	    } else {
+		user_var_replace_value(uvar, tmp);
+	    }
 	} else {
 	    pputs(inprn, buf);
 	}
@@ -726,17 +774,6 @@ int do_printf (const char *line, DATASET *dset, PRN *prn)
 
 /* below: sscanf apparatus */
 
-static int sscanf_target_var (const char *vname, int *err)
-{
-    if (gretl_is_scalar(vname)) {
-	return 0;
-    } else {
-	gretl_errmsg_set(_("sscanf: numerical target must be scalar"));
-	*err = E_DATA;
-	return 0;
-    }
-}
-
 struct bracket_scan {
     int reverse;
     char *chrs;
@@ -909,17 +946,20 @@ scan_string (const char *targ, char **psrc, int width,
     }
 
     if (targ != NULL) {
-	if (!is_user_string(targ)) {
+	user_var *uvar;
+
+	uvar = get_user_var_of_type_by_name(targ, GRETL_TYPE_STRING);
+
+	if (uvar == NULL) {
 	    gretl_errmsg_sprintf(_("%s: not a string variable"), targ);
-	    err = E_UNKVAR;
+	    err = E_DATA;
 	} else if (n > 0) {
 	    char *conv = gretl_strndup(*psrc, n);
 
 	    if (conv == NULL) {
 		err = E_ALLOC;
 	    } else {
-		err = save_named_string(targ, conv, NULL);
-		free(conv);
+		user_var_replace_value(uvar, conv);
 		*psrc += n;
 	    }
 	    if (!err) {
@@ -933,23 +973,15 @@ scan_string (const char *targ, char **psrc, int width,
     return err;
 }
 
-static int scan_scalar (const char *targ, char **psrc,
+static int scan_scalar (char *targ, char **psrc,
 			int fc, int width, DATASET *dset, 
 			int *ns)
 {
     char *endp = NULL;
-    int v = 0;
     long k = 0;
     double x = 0;
     int err = 0;
 
-    if (targ != NULL) {
-	v = sscanf_target_var(targ, &err);
-	if (err) {
-	    return err;
-	}
-    }
-
     errno = 0;
 
     if (fc == 'd') {
@@ -978,11 +1010,15 @@ static int scan_scalar (const char *targ, char **psrc,
     }
 
     if (!errno && endp != *psrc) {
-	if (v == 0) {
-	    gretl_scalar_set_value(targ, (fc == 'd')? k : x);
-	} else if (v > 0) {
-	    dset->Z[v][0] = (fc == 'd')? k : x;
+	char *genline;
+
+	if (fc == 'd') {
+	    genline = gretl_strdup_printf("%s=%ld", targ, k);
+	} else {
+	    genline = gretl_strdup_printf("%s=%.16g", targ, x);
 	}
+	err = generate(genline, dset, OPT_P, NULL);
+	free(genline);
 	*ns += 1;
     }
 
@@ -1165,7 +1201,7 @@ static char *get_literal_or_stringvar (const char **src, int *err)
 	s++;
 	n = gretl_namechar_spn(s);
     } else {
-	n = charpos(',', s);
+	n = gretl_charpos(',', s);
 	if (n == gretl_namechar_spn(s)) {
 	    svar = 1;
 	}
