@@ -25,6 +25,13 @@
 #include "../../cephes/libprob.h"
 
 #include <errno.h>
+#include <libset.h>
+
+#if defined(_OPENMP) && defined(USE_OPENMP)
+# include <omp.h>
+#endif
+
+#define NORM_CDF_MAX 0.9999999999999999
 
 /**
  * SECTION:pvalues
@@ -1002,7 +1009,7 @@ double normal_cdf (double x)
 	y = NADBL;
     } else if (y == 1.0) {
 	/* do we want to do this? */
-	y = 0.9999999999999999;
+	y = NORM_CDF_MAX;
     }
 
     return y;
@@ -1186,8 +1193,182 @@ double invmills (double x)
     return ret;
 }
 
+#define GENZ_BVN 1
+
+#if GENZ_BVN
+
 /**
- * bvnorm_cdf:
+ * genz04:
+ * @rho: correlation coefficient.
+ * @limx: abscissa value, first Gaussian r.v.
+ * @limy: abscissa value, second Gaussian r.v.
+ *
+ * Based on FORTRAN code by Alan Genz, with minor adaptations.
+ * Original source at 
+ * http://www.math.wsu.edu/faculty/genz/software/fort77/tvpack.f
+ * No apparent license.
+ *
+ * The algorithm is from Drezner and Wesolowsky (1989), 'On the
+ * Computation of the Bivariate Normal Integral', Journal of
+ * Statist. Comput. Simul. 35 pp. 101-107, with major modifications
+ * for double precision, and for |R| close to 1.
+ *
+ * Returns: for (x, y) a bivariate standard Normal rv with correlation
+ * coefficient @rho, the joint probability that (x < @limx) and (y < @limy),
+ * or #NADBL on failure.
+ */
+
+static double genz04 (double rho, double limx, double limy)
+{
+    double w[10], x[10];
+    double absrho = fabs(rho);
+    double h, k, hk, bvn, hs, asr;
+    double a, b, as, d1, bs, c, d, tmp;
+    double sn, xs, rs;
+    int i, lg, j;
+
+    if (absrho < 0.3) {
+
+	w[0] = .1713244923791705;
+	w[1] = .3607615730481384;
+	w[2] = .4679139345726904;
+
+	x[0] = -.9324695142031522;
+	x[1] = -.6612093864662647;
+	x[2] = -.238619186083197;
+
+	lg = 3;
+    } else if (absrho < 0.75) {
+
+	w[0] = .04717533638651177;
+	w[1] = .1069393259953183;
+	w[2] = .1600783285433464;
+	w[3] = .2031674267230659;
+	w[4] = .2334925365383547;
+	w[5] = .2491470458134029;
+
+	x[0] = -.9815606342467191;
+	x[1] = -.904117256370475;
+	x[2] = -.769902674194305;
+	x[3] = -.5873179542866171;
+	x[4] = -.3678314989981802;
+	x[5] = -.1252334085114692;
+
+	lg = 6;
+    } else {
+
+	w[0] = .01761400713915212;
+	w[1] = .04060142980038694;
+	w[2] = .06267204833410906;
+	w[3] = .08327674157670475;
+	w[4] = .1019301198172404;
+	w[5] = .1181945319615184;
+	w[6] = .1316886384491766;
+	w[7] = .1420961093183821;
+	w[8] = .1491729864726037;
+	w[9] = .1527533871307259;
+
+	x[0] = -.9931285991850949;
+	x[1] = -.9639719272779138;
+	x[2] = -.9122344282513259;
+	x[3] = -.8391169718222188;
+	x[4] = -.7463319064601508;
+	x[5] = -.636053680726515;
+	x[6] = -.5108670019508271;
+	x[7] = -.3737060887154196;
+	x[8] = -.2277858511416451;
+	x[9] = -.07652652113349733;
+
+	lg = 10;
+    }
+
+    h = -limx;
+    k = -limy;
+    hk = h * k;
+    bvn = 0.0;
+
+    if (absrho < 0.925) {
+	hs = (h * h + k * k) / 2;
+	asr = asin(rho);
+	for (i=0; i<lg; i++) {
+	    for (j=0; j<=1; j++) {
+		sn = sin(asr * (1 + (2*j-1)*x[i]) / 2);
+		bvn += w[i] * exp((sn * hk - hs) / (1 - sn * sn));
+	    }
+	}
+	bvn = bvn * asr / (2 * M_2PI); 
+
+	d1 = -h;
+	bvn += normal_cdf(d1) * normal_cdf(-k);
+    } else {
+	if (rho < 0.0) {
+	    k = -k;
+	    hk = -hk;
+	}
+
+	as = (1 - rho) * (1 + rho);
+	a = sqrt(as);
+	bs = (h - k) * (h - k);
+	c = (4 - hk) / 8;
+	d = (12 - hk) / 16;
+	asr = -(bs / as + hk) / 2;
+	if (asr > -100.0) {
+	    bvn = a * exp(asr) * (1 - c * (bs - as) * (1 - d * bs / 5)
+				  / 3 + c * d * as * as / 5);
+	}
+
+	if (-hk < 100.0) {
+	    b = sqrt(bs);
+	    d1 = -b / a;
+	    /*
+	      Note: the condition below was not in the original
+	      FORTRAN code this was ripped off from. Without it, there
+	      are a few problems for rho very near -1; the version
+	      below seems to work ok. Could it be a problem with
+	      normal_cdf in the left tail?
+	    */
+	    if (d1 > -12.0) {
+		bvn -= exp(-hk / 2) * SQRT_2_PI * normal_cdf(d1) * b * 
+		    (1 - c * bs * (1 - d * bs / 5) / 3);
+	    }
+	}
+
+	a /= 2;
+
+	for (i=0; i<lg; i++) {
+	    for (j=0; j<=1; j++) {
+		d1 = a * (1 + (2*j-1)*x[i]);
+		xs = d1 * d1;
+		rs = sqrt(1 - xs);
+		asr = -(bs / xs + hk) / 2;
+		if (asr > -100.0) {
+		    tmp = exp(-hk * (1 - rs) / ((rs + 1) * 2)) / 
+			rs - (c * xs * (d * xs + 1) + 1);
+		    bvn += a * w[i] * exp(asr) * tmp;
+		}
+	    }
+	}
+	
+	bvn = -bvn / M_2PI;
+
+	if (rho > 0.0) {
+	    bvn += normal_cdf((h > k) ? -h : -k);
+	} else {
+	    bvn = -bvn;
+	    if (k > h) {
+		bvn = bvn + normal_cdf(k) - normal_cdf(h);
+	    }
+	}
+    }
+
+    /* sanity check */
+    return (bvn < 0) ? 0 : bvn;
+}
+
+#else
+
+/**
+ * drezner78:
  * @rho: correlation coefficient.
  * @a: abscissa value, first Gaussian r.v.
  * @b: abscissa value, second Gaussian r.v.
@@ -1195,12 +1376,15 @@ double invmills (double x)
  * Ripped and adapted from Gnumeric, with a bug corrected for the case
  * (a * b < 0) && (rho < 0).
  *
+ * The algorithm is from Drezner (1978), 'Computation of the Bivariate
+ * Normal Integral', Mathematics of Computation, volume 32, number 141.
+ *
  * Returns: for (x, y) a bivariate standard Normal rv with correlation
  * coefficient @rho, the joint probability that (x < @a) and (y < @b), or
  * #NADBL on failure.
  */
 
-double bvnorm_cdf (double rho, double a, double b)
+static double drezner78 (double rho, double a, double b)
 {
     static const double x[] = {0.24840615, 0.39233107, 0.21141819, 
 			       0.03324666, 0.00082485334};
@@ -1210,28 +1394,6 @@ double bvnorm_cdf (double rho, double a, double b)
     double a1, b1, den;
     int i, j;
 
-    if (fabs(rho) > 1) {
-	return NADBL;
-    }	
-
-    if (rho == 0.0) {
-	/* joint prob is just the product of the marginals */
-	return normal_cdf(a) * normal_cdf(b);
-    }
-
-    if (rho == 1.0) {
-	/* the two variables are in fact the same */
-	return normal_cdf(a<b ? a : b);
-    }
-    
-    if (rho == -1.0) {
-	/* the two variables are perfectly negatively correlated: 
-	   P(x<a, y<b) = P((x<a) && (x>b)) = P(x \in (b,a))
-	*/
-	ret = (a<=b) ? 0 : normal_cdf(a)-normal_cdf(b);
-	return ret;
-    }
-    
     den = sqrt(2.0 * (1 - rho * rho));
 
     a1 = a / den;
@@ -1272,6 +1434,387 @@ double bvnorm_cdf (double rho, double a, double b)
     return ret;
 }
 
+#endif /* bvnorm variants */
+
+/**
+ * bvnorm_cdf:
+ * @rho: correlation coefficient.
+ * @a: abscissa value, first Gaussian r.v.
+ * @b: abscissa value, second Gaussian r.v.
+ *
+ * Returns: for (x, y) a bivariate standard Normal rv with correlation
+ * coefficient @rho, the joint probability that (x < @a) and (y < @b), or
+ * #NADBL on failure.
+ */
+
+double bvnorm_cdf (double rho, double a, double b)
+{
+    if (fabs(rho) > 1) {
+	return NADBL;
+    }	
+
+    if (rho == 0.0) {
+	/* joint prob is just the product of the marginals */
+	return normal_cdf(a) * normal_cdf(b);
+    }
+
+    if (rho == 1.0) {
+	/* the two variables are in fact the same */
+	return normal_cdf(a < b ? a : b);
+    }
+    
+    if (rho == -1.0) {
+	/* the two variables are perfectly negatively correlated: 
+	   P(x<a, y<b) = P((x<a) && (x>b)) = P(x \in (b,a))
+	*/
+	return (a <= b) ? 0 : normal_cdf(a) - normal_cdf(b);
+    }
+
+#if GENZ_BVN
+    return genz04(rho, a, b);
+#else 
+    return drezner78(rho, a, b);
+#endif
+}
+
+/*
+  C  Lower triangular Cholesky factor of \Sigma, m x m
+  A  Lower bound of rectangle, m x 1
+  B  Upper bound of rectangle, m x 1
+  U  Random variates, m x r
+*/
+
+static double GHK_1 (const gretl_matrix *C, 
+		     const gretl_matrix *A, 
+		     const gretl_matrix *B, 
+		     const gretl_matrix *U,
+		     gretl_matrix *TA,
+		     gretl_matrix *TB,
+		     gretl_matrix *WGT,
+		     gretl_matrix *TT,
+		     double huge)
+{
+    int m = C->rows; /* Dimension of the multivariate normal */
+    int r = U->cols; /* Number of repetitions */
+    double P, den = gretl_matrix_get(C, 0, 0);
+    double ui, x, z, cjk, tki;
+    int i, j, k;
+
+    z = A->val[0];
+    TA->val[0] = (z == -huge) ? 0 : ndtr(z / den);
+    z = B->val[0];
+    TB->val[0] = (z == huge) ? 1 : ndtr(z / den);
+    
+    for (i=1; i<r; i++) {
+	TA->val[i] = TA->val[0];
+	TB->val[i] = TB->val[0];
+    }
+
+    /* form WGT = TB - TA */
+    gretl_matrix_copy_values(WGT, TB);
+    gretl_matrix_subtract_from(WGT, TA);
+    gretl_matrix_zero(TT);
+
+    for (i=0; i<r; i++) {
+	ui = gretl_matrix_get(U, 0, i);
+	x = TB->val[i] - ui * (TB->val[i] - TA->val[i]);
+	gretl_matrix_set(TT, 0, i, ndtri(x));
+    }
+
+    for (j=1; j<m; j++) {
+	den = gretl_matrix_get(C, j, j);
+
+	for (i=0; i<r; i++) {
+	    if (WGT->val[i] == 0) {
+		/* If WGT[i] ever comes to be zero, it cannot in
+		   principle be modified by the code below; in fact,
+		   however, running through the computations
+		   regardless may produce a NaN (since 0 * NaN = NaN).
+		   This becomes more likely for large dimension @m.
+		*/
+		continue;
+	    }
+	    x = 0.0;
+	    for (k=0; k<j; k++) {
+		cjk = gretl_matrix_get(C, j, k);
+		tki = gretl_matrix_get(TT, k, i);
+		x += cjk * tki;
+	    }
+
+	    if (A->val[j] == -huge) {
+		TA->val[i] = 0.0;
+	    } else {
+		TA->val[i] = ndtr((A->val[j] - x) / den);
+	    }
+
+	    if (B->val[j] == huge) {
+		TB->val[i] = 1.0;
+	    } else {
+		TB->val[i] = ndtr((B->val[j] - x) / den);
+	    }
+
+	    /* component j draw */
+	    ui = gretl_matrix_get(U, j, i);
+	    x = TB->val[i] - ui * (TB->val[i] - TA->val[i]);
+	    gretl_matrix_set(TT, j, i, ndtri(x));
+	}
+
+	/* accumulate weight */
+	gretl_matrix_subtract_from(TB, TA);
+	for (i=0; i<r; i++) {
+	    WGT->val[i] *= TB->val[i];
+	}
+    }
+
+    P = 0.0;
+    for (i=0; i<r; i++) {
+	P += WGT->val[i];
+    }
+    P /= r;
+
+    if (P < 0.0 || P > 1.0) {
+	fprintf(stderr, "*** ghk error: P = %g\n", P);
+	P = 0.0/0.0; /* force a NaN */
+    }	
+
+    return P;
+}
+
+/**
+ * gretl_GHK:
+ * @C: Cholesky decomposition of covariance matrix, lower triangular,
+ * m x m.
+ * @A: Lower bounds, n x m; in case a lower bound is minus infinity
+ * this should be represented as -1.0E10.
+ * @B: Upper bounds, n x m; in case an upper bound is plus infinity
+ * this should be represented as +1.0E10.
+ * @U: Uniform random matrix, m x r.
+ *
+ * Computes the GHK (Geweke, Hajivassiliou, Keane) approximation to 
+ * the multivariate normal distribution function for n observations 
+ * on m variates, using r draws. 
+ *
+ * Returns: an n x 1 vector of probabilities.
+ */
+
+#if defined(_OPENMP) && defined(USE_OPENMP)
+
+gretl_matrix *gretl_GHK (const gretl_matrix *C,
+			 const gretl_matrix *A,
+			 const gretl_matrix *B,
+			 const gretl_matrix *U,
+			 int *err)
+{
+    gretl_matrix_block *Bk = NULL;
+    gretl_matrix *P = NULL;
+    gretl_matrix *Ai, *Bi;
+    gretl_matrix *TA, *TB;
+    gretl_matrix *WT, *TT;
+    double huge;
+    int dim, nobs, ndraws;
+    int ierr, ghk_err = 0;
+    int ABok, pzero;
+    int i, j;
+
+    if (gretl_is_null_matrix(C) ||
+	gretl_is_null_matrix(A) ||
+	gretl_is_null_matrix(B) ||
+	gretl_is_null_matrix(U)) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (A->rows != B->rows ||
+	A->cols != B->cols ||
+	C->rows != A->cols ||
+	C->cols != A->cols ||
+	U->rows != A->cols) {
+	*err = E_NONCONF;
+	return NULL;
+    }
+
+    huge = libset_get_double(CONV_HUGE);
+
+    dim = C->rows;
+    nobs = A->rows;
+    ndraws = U->cols;
+
+    P = gretl_matrix_alloc(nobs, 1);
+    if (P == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    set_cephes_hush(1);
+
+#pragma omp parallel if (nobs>256) private(i,j,Bk,Ai,Bi,TA,TB,WT,TT,ABok,pzero,ierr)
+    {
+	Bk = gretl_matrix_block_new(&Ai, dim, 1,
+				    &Bi, dim, 1,
+				    &TA, 1, ndraws,
+				    &TB, 1, ndraws,
+				    &WT, 1, ndraws,
+				    &TT, dim, ndraws,
+				    NULL);
+	if (Bk == NULL) {
+	    ierr = E_ALLOC;
+	    goto omp_end;
+	} else {
+	    ierr = 0;
+	}
+
+        #pragma omp for
+	for (i=0; i<nobs; i++) {
+	    ABok = 1; pzero = 0;
+
+	    for (j=0; j<dim && !ierr; j++) {
+		Ai->val[j] = gretl_matrix_get(A, i, j);
+		Bi->val[j] = gretl_matrix_get(B, i, j);
+		ABok = !(isnan(Ai->val[j]) || isnan(Bi->val[j]));
+
+		if (!ABok) {
+		    /* If there are any NaNs in A or B, there's no
+		       point in continuing
+		    */
+		    P->val[i] = 0.0/0.0; /* NaN */
+		    break;
+		} else if (Bi->val[j] < Ai->val[j]) {
+		    gretl_errmsg_sprintf("ghk: inconsistent bounds: B[%d,%d] < A[%d,%d]",
+					 i+1, j+1, i+1, j+1);
+		    ierr = E_DATA;
+		} else if (Bi->val[j] == Ai->val[j]) {
+		    P->val[i] = 0.0;
+		    pzero = 1;
+		    break;
+		}
+	    }
+	    if (!ierr && !pzero && ABok) {
+		P->val[i] = GHK_1(C, Ai, Bi, U, TA, TB, WT, TT, huge);
+	    }
+	}
+
+    omp_end:
+
+	if (ierr) {
+	    ghk_err = ierr;
+	}
+
+	gretl_matrix_block_destroy(Bk);
+    }
+
+    set_cephes_hush(0);
+
+    if (ghk_err) {
+	*err = ghk_err;
+	gretl_matrix_free(P);
+	P = NULL;
+    }
+
+    return P;
+}
+
+#else /* non-MP version */
+
+gretl_matrix *gretl_GHK (const gretl_matrix *C,
+			 const gretl_matrix *A,
+			 const gretl_matrix *B,
+			 const gretl_matrix *U,
+			 int *err)
+{
+    gretl_matrix *P = NULL;
+    gretl_matrix_block *Bk = NULL;
+    gretl_matrix *Ai, *Bi;
+    gretl_matrix *TA, *TB;
+    gretl_matrix *WT, *TT;
+    double huge;
+    int dim, nobs, ndraws;
+    int i, j;
+
+    if (gretl_is_null_matrix(C) ||
+	gretl_is_null_matrix(A) ||
+	gretl_is_null_matrix(B) ||
+	gretl_is_null_matrix(U)) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (A->rows != B->rows ||
+	A->cols != B->cols ||
+	C->rows != A->cols ||
+	C->cols != A->cols ||
+	U->rows != A->cols) {
+	*err = E_NONCONF;
+	return NULL;
+    }
+
+    huge = libset_get_double(CONV_HUGE);
+
+    dim = C->rows;
+    nobs = A->rows;
+    ndraws = U->cols;
+
+    if (!*err) {
+	Bk = gretl_matrix_block_new(&Ai, dim, 1,
+				    &Bi, dim, 1,
+				    &TA, 1, ndraws,
+				    &TB, 1, ndraws,
+				    &WT, 1, ndraws,
+				    &TT, dim, ndraws,
+				    NULL);
+	if (Bk == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	P = gretl_matrix_alloc(nobs, 1);
+	if (P == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    set_cephes_hush(1);
+
+    for (i=0; i<nobs && !*err; i++) {
+	int ABok = 1, pzero = 0;
+
+	for (j=0; j<dim && !*err; j++) {
+	    Ai->val[j] = gretl_matrix_get(A, i, j);
+	    Bi->val[j] = gretl_matrix_get(B, i, j);
+	    ABok = !(isnan(Ai->val[j]) || isnan(Bi->val[j]));
+
+	    if (!ABok) {
+		/* If there are any NaNs in A or B, there's no
+		   point in continuing
+		*/
+		P->val[i] = 0.0/0.0; /* NaN */
+		break;
+	    } else if (Bi->val[j] < Ai->val[j]) {
+		gretl_errmsg_sprintf("ghk: inconsistent bounds: B[%d,%d] < A[%d,%d]",
+				     i+1, j+1, i+1, j+1);
+		*err = E_DATA;
+		gretl_matrix_free(P);
+		P = NULL;
+	    } else if (Bi->val[j] == Ai->val[j]) {
+		P->val[i] = 0.0;
+		pzero = 1;
+		break;
+	    }
+	}
+	if (!*err && !pzero && ABok) {
+	    P->val[i] = GHK_1(C, Ai, Bi, U, TA, TB, WT, TT, huge);
+	}
+    }
+
+    set_cephes_hush(0);
+
+    gretl_matrix_block_destroy(Bk);
+
+    return P;
+}
+
+#endif /* OPENMP or not */
+
 /**
  * gamma_cdf:
  * @s1: first parameter.
@@ -2426,6 +2969,17 @@ gretl_matrix *gretl_get_random_matrix (char st, const double *parm,
     return m;
 }
 
+double gretl_get_random_scalar (char st, const double *parm,
+				int *err)
+{
+    double x;
+
+    *err = gretl_fill_random_array(&x, 0, 0, st,
+				   parm, NULL, NULL);
+
+    return x;
+}
+
 static int 
 print_pv_string (double x, double p, PRN *prn)
 {
@@ -2640,7 +3194,7 @@ int batch_pvalue (const char *str, DATASET *dset, PRN *prn)
 	}
     }
 
-    free_strings_array(S, n);
+    strings_array_free(S, n);
 
     if (!err) {
 	pv = generate_scalar(line, dset, &err);
