@@ -919,6 +919,57 @@ void gretl_rand_uniform (double *a, int t1, int t2)
     }
 }
 
+double gretl_rand_gamma_one (double shape, double scale)
+{
+    double k = shape;
+    double d, c, x, v, u, dv;
+
+    if (shape <= 0 || scale <= 0) {
+	return NADBL;
+    }
+
+    if (shape < 1) {
+	k = shape + 1.0;
+    }
+
+    d = k - 1.0/3;
+    c = 1.0 / sqrt(9*d);    
+    
+    while (1) {
+	x = gretl_one_snormal();
+	v = pow(1 + c*x, 3);
+	if (v > 0.0) {
+	    dv = d * v;
+	    u = gretl_rand_01();
+	    /* apply squeeze */
+	    if (u < 1 - 0.0331 * pow(x, 4) ||
+		log(u) < 0.5*x*x + d*(1-v+log(v))) {
+		break;
+	    } 
+	}
+    }
+    if (shape < 1) {
+	u = gretl_rand_01();
+	dv *= pow(u, 1/shape);
+    }
+
+    return dv * scale;
+}
+
+#define USE_MT_GAMMA 1
+
+/* Marsaglia-Tsang, "A Simple Method for Generating Gamma Variables",
+   ACM Transactions on Mathematical Software, Vol. 26, No. 3,
+   September 2000, Pages 363­372.
+
+   (1) Setup: d=a-1/3, c=1/sqrt(9*d).
+   (2) Generate v=(1+c*x)^3 with x normal; repeat if v <= 0.
+   (3) Generate uniform U.
+   (4) If U < 1-0.0331*x^4 return d*v.
+   (5) If log(U) < 0.5*x^2+d*(1-v+log(v)) return d*v.
+   (6) Go to step 2.
+*/   
+
 /**
  * gretl_rand_gamma:
  * @a: target array.
@@ -936,6 +987,45 @@ void gretl_rand_uniform (double *a, int t1, int t2)
 int gretl_rand_gamma (double *a, int t1, int t2, 
 		      double shape, double scale) 
 {
+#if USE_MT_GAMMA
+    double k = shape;
+    double d, c, x, v, u, dv;
+    int t;
+
+    if (shape <= 0 || scale <= 0) {
+	return E_DATA;
+    }
+
+    if (shape < 1) {
+	k = shape + 1.0;
+    }
+
+    d = k - 1.0/3;
+    c = 1.0 / sqrt(9*d);    
+    
+    for (t=t1; t<=t2; t++) {
+	while (1) {
+	    x = gretl_one_snormal();
+	    v = pow(1 + c*x, 3);
+	    if (v > 0.0) {
+		dv = d * v;
+		u = gretl_rand_01();
+		/* apply squeeze */
+		if (u < 1 - 0.0331 * pow(x, 4) ||
+		    log(u) < 0.5*x*x + d*(1-v+log(v))) {
+		    break;
+		} 
+	    }
+	}
+	if (shape < 1) {
+	    u = gretl_rand_01();
+	    dv *= pow(u, 1/shape);
+	}
+	a[t] = dv * scale;
+    }
+
+    return 0;
+#else
     double *U = NULL;
     double e = 2.718281828459045235;
     double delta, dinv = 0, d1 = 0;
@@ -1021,6 +1111,7 @@ int gretl_rand_gamma (double *a, int t1, int t2,
     free(U);
 	
     return 0;
+#endif /* gamma variants */
 }
 
 /**
@@ -1347,3 +1438,69 @@ unsigned int gretl_rand_int (void)
     }
 }
 
+static double halton (int i, int base)
+{
+    double f = 1.0 / base;
+    double h = 0.0;
+
+    while (i > 0) {
+	h += f * (i % base);
+	i = floor(i / base);
+	f /= base;
+    }
+
+    return h;
+}
+
+/**
+ * halton_matrix:
+ * @m: number of rows (sequences); the maximum is 40.
+ * @r: number of columns (elements in each sequence).
+ * @offset: the number of initial values to discard.
+ * @err: location to receive error code.
+ *
+ * Returns: an @m x @r matrix containing @m Halton
+ * sequences. The sequences are contructed using the first 
+ * @m primes, and the first @offset elements of each sequence 
+ * are discarded.
+ */
+
+gretl_matrix *halton_matrix (int m, int r, int offset, int *err)
+{
+    const int bases[] = {
+	2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31,
+	37, 41, 43, 47, 53, 59, 61, 67, 71, 73,
+	79, 83, 89, 97, 101, 103, 107, 109, 113,
+	127, 131, 137, 139, 149, 151, 157, 163,
+	167, 173, 179, 181
+    };
+    gretl_matrix *H;
+    double hij;
+    int i, j, k, n;
+
+    if (m > 40 || offset < 0) {
+	*err = E_DATA;
+	return NULL;
+    }
+	
+    H = gretl_matrix_alloc(m, r);
+    if (H == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+    
+    /* we'll discard the first @offset elements */
+    n = r + offset;
+
+    for (i=0; i<m; i++) {
+	j = 0;
+	for (k=1; k<n; k++) {
+	    hij = halton(k, bases[i]);
+	    if (k >= offset) {
+		gretl_matrix_set(H, i, j++, hij);
+	    }
+	}
+    }
+
+    return H;
+}
