@@ -103,7 +103,7 @@ double dot_atof (const char *s)
 }
 
 /**
- * dotpos:
+ * gretl_dotpos:
  * @str: the string to examine.
  *
  * Returns: the integer position of the last "." within @str,
@@ -111,7 +111,7 @@ double dot_atof (const char *s)
  * ends with a (backward or forward) slash.
  */
 
-int dotpos (const char *str)
+int gretl_dotpos (const char *str)
 { 
     int i, p = 0;
 
@@ -131,14 +131,14 @@ int dotpos (const char *str)
 }
 
 /**
- * slashpos:
+ * gretl_slashpos:
  * @str: the string to examine.
  *
  * Returns: the integer position of the last #SLASH within @str,
  * or 0 in case a #SLASH is not found.
  */
 
-int slashpos (const char *str)
+int gretl_slashpos (const char *str)
 { 
     int i, p = 0;
 
@@ -156,7 +156,7 @@ int slashpos (const char *str)
 }
 
 /**
- * delchar:
+ * gretl_delchar:
  * @c: the character to delete.
  * @str: the string from which to delete @c.
  *
@@ -165,7 +165,7 @@ int slashpos (const char *str)
  * Returns: the possibly modified string.
  */
 
-char *delchar (int c, char *str)
+char *gretl_delchar (int c, char *str)
 {
     int i, j;
 
@@ -243,7 +243,7 @@ char *gretl_unquote (char *str, int *err)
 }
 
 /**
- * charpos:
+ * gretl_charpos:
  * @c: the character to look for.
  * @s: the string to examine.
  *
@@ -251,7 +251,7 @@ char *gretl_unquote (char *str, int *err)
  * found.
  */
 
-int charpos (char c, const char *s)
+int gretl_charpos (char c, const char *s)
 {
     int i = 0;
 
@@ -266,7 +266,7 @@ int charpos (char c, const char *s)
 }
 
 /**
- * charsub:
+ * gretl_charsub:
  * @str: the string to operate on.
  * @find: the character to replace.
  * @repl: the replacement character.
@@ -276,7 +276,7 @@ int charpos (char c, const char *s)
  * Returns: the (possibly modified) string.
  */
 
-char *charsub (char *str, char find, char repl)
+char *gretl_charsub (char *str, char find, char repl)
 {
     char *p = str;
 
@@ -445,7 +445,7 @@ int ends_with_backslash (const char *s)
 }
 
 /**
- * lower:
+ * gretl_lower:
  * @str: the string to transform.
  *
  * Converts any upper case characters in @str to lower case.
@@ -453,7 +453,7 @@ int ends_with_backslash (const char *s)
  * Returns: the possibly modified string.
  */
 
-char *lower (char *str)
+char *gretl_lower (char *str)
 {
     char *p = str;
 
@@ -779,7 +779,7 @@ char **gretl_string_split (const char *s, int *n)
 	k = strcspn(s, " ");
 	word = gretl_strndup(s, k);
 	if (word == NULL) {
-	    free_strings_array(S, m);
+	    strings_array_free(S, m);
 	    return NULL;
 	}
 	S[i] = word;
@@ -847,7 +847,7 @@ char **gretl_string_split_quoted (const char *s, int *n, int *err)
 	substr = gretl_strndup(s, len);
 	if (substr == NULL) {
 	    *err = E_ALLOC;
-	    free_strings_array(S, m);
+	    strings_array_free(S, m);
 	    return NULL;
 	}
 	S[i] = substr;
@@ -872,7 +872,7 @@ char **gretl_string_split_quoted (const char *s, int *n, int *err)
 char *gretl_trunc (char *str, size_t n)
 {
     if (n < strlen(str)) {
-	str[n] = 0;
+	str[n] = '\0';
     }
 
     return str;
@@ -902,19 +902,6 @@ int gretl_namechar_spn (const char *s)
     return ret;
 }
 
-/**
- * clear:
- * @str: the string to clear.
- * @len: the length of the string to be cleared.
- *
- * Sets all bytes in @str to 0.
- */
-
-void clear (char *str, int len)
-{
-    memset(str, 0, len);
-}
-
 /**
  * count_fields:
  * @s: the string to process.
@@ -981,7 +968,7 @@ char *shift_string_left (char *str, size_t move)
 }
 
 /**
- * chopstr:
+ * gretl_chopstr:
  * @str: the string to process.
  *
  * Removes both leading and trailing space from a string.
@@ -989,7 +976,7 @@ char *shift_string_left (char *str, size_t move)
  * Returns: the possibly modified string.
  */
 
-char *chopstr (char *str)
+char *gretl_chopstr (char *str)
 {
     int i, n = strspn(str, " \t");
 
@@ -1024,7 +1011,7 @@ char *chopstr (char *str)
 
 char *switch_ext (char *targ, const char *src, const char *ext)
 {
-    int i = dotpos(src);
+    int i = gretl_dotpos(src);
 
     if (targ != src) {
         strncpy(targ, src, i);
@@ -1074,6 +1061,26 @@ char *switch_ext_new (const char *src, const char *ext)
     return ret;
 }
 
+static int ends_in_comment (const char *s, int n)
+{
+    int i, quoted = 0;
+
+    /* the '#' character is inert (only) if it appears
+       within a string literal */
+
+    for (i=n; i>1; i--) {
+	if (s[i] == '"') {
+	    quoted = !quoted;
+	} else if (!quoted && s[i] == '#') {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+#define LINE_CONT(c) (c == '\\' || c == ',' || c == '(')
+
 /**
  * top_n_tail:
  * @str: the string to process.
@@ -1090,6 +1097,8 @@ char *switch_ext_new (const char *src, const char *ext)
  * was found, otherwise 0.
  */
 
+#include <assert.h>
+
 int top_n_tail (char *str, size_t maxlen, int *err)
 {
     int i, n, cont = 0;
@@ -1108,6 +1117,7 @@ int top_n_tail (char *str, size_t maxlen, int *err)
     for (i=n; i>=0; i--) {
 	if (isspace((unsigned char) str[i])) {
 	    str[i] = 0;
+	    n--;
 	} else {
 	    break;
 	}
@@ -1122,28 +1132,25 @@ int top_n_tail (char *str, size_t maxlen, int *err)
 	       str[i] == '?' ||
 	       str[i] == (char) 0xC2 ||
 	       str[i] == (char) 0xA0) {
+	    n--;
 	    i++;
 	}
 	if (i > 0) {
 	    shift_string_left(str, i);
 	}
 
-	/* does this line start a comment? */
 	if (*str == '#' || !strncmp(str, "/*", 2)) {
-	    ; /* leave well alone */
-	} else {
-	    /* replace backslash, if present */
-	    n = strlen(str) - 1;
-	    if (n >= 0) {
-		if (str[n] == '\\') {
-		    str[n] = ' ';
-		    cont = 1;
-		} else if (str[n] == ',') {
-		    cont = 1;
-		} else if (str[n] == '(') {
-		    cont = 1;
-		}
-	    }
+	    ; /* the line starts a comment: leave well alone */
+	} else if (n >= 0 && LINE_CONT(str[n])) {
+	    /* register line continuation characters at the end of
+	       the line, but only if not preceded by the comment
+	       character '#' (unquoted)
+	    */
+	    cont = !ends_in_comment(str, n - 1);
+	    if (cont && str[n] == '\\') {
+		/* replace backslash */
+		str[n] = ' ';
+	    }		    
 	}
     }
 
@@ -1324,25 +1331,6 @@ char *space_to_score (char *s)
     return s;
 }
 
-/**
- * safecpy:
- * @targ: target or output string (must be pre-allocated).
- * @src: source or input string.
- * @n: maximum length of target string.
- *
- * Copies at most @n characters from @src to @targ, and ensures that
- * @targ[@n] is a NUL byte.
- * 
- * Returns: the output string.
- */
-
-char *safecpy (char *targ, const char *src, int n)
-{
-    *targ = 0;
-    strncat(targ, src, n);
-    return targ;
-}
-
 /**
  * strings_array_new:
  * @nstrs: number of strings in array.
@@ -1485,7 +1473,7 @@ char **strings_array_realloc_with_length (char ***pS,
     }
 
     if (newn <= 0) {
-	free_strings_array(*pS, oldn);
+	strings_array_free(*pS, oldn);
 	*pS = NULL;
 	return NULL;
     }
@@ -1498,7 +1486,7 @@ char **strings_array_realloc_with_length (char ***pS,
 
     S = realloc(*pS, newn * sizeof *S);
     if (S == NULL) {
-	free_strings_array(*pS, oldn);
+	strings_array_free(*pS, oldn);
 	*pS = NULL;
 	return NULL;
     }
@@ -1653,7 +1641,7 @@ int strings_array_cmp (char **strs1, char **strs2, int n)
 }
 
 /**
- * free_strings_array:
+ * strings_array_free:
  * @strs: array of allocated strings.
  * @nstrs: number of strings in array.
  *
@@ -1661,7 +1649,7 @@ int strings_array_cmp (char **strs1, char **strs2, int n)
  * Checks that @strs is not NULL before proceeding.
  */
 
-void free_strings_array (char **strs, int nstrs)
+void strings_array_free (char **strs, int nstrs)
 {
     int i;
 
@@ -1697,13 +1685,14 @@ char *get_obs_string (char *obs, int t, const DATASET *dset)
  * obs_str_to_double:
  * @obs: string representation of observation number.
  *
- * Returns: the floating-point counterpart of @obs.
+ * Returns: the floating-point counterpart of @obs,
+ * or #NADBL on invalid input.
  */
 
 double obs_str_to_double (const char *obs)
 {
-    char tmp[OBSLEN];
-    char *p;
+    char *p, *test, tmp[OBSLEN];
+    double ret;
 
     strcpy(tmp, obs);
     p = tmp;
@@ -1715,7 +1704,17 @@ double obs_str_to_double (const char *obs)
 	p++;
     }
 
-    return dot_atof(tmp);
+    errno = 0;
+
+    gretl_push_c_numeric_locale();
+    ret = strtod(tmp, &test);
+    gretl_pop_c_numeric_locale();
+ 
+    if (*test != '\0' || errno == ERANGE) {
+	ret = NADBL;
+    }
+
+    return ret;
 }
 
 /**
@@ -1755,9 +1754,9 @@ char *colonize_obs (char *obs)
 void modify_date_for_csv (char *s, int pd)
 {
     if (pd == 4) {
-	charsub(s, ':', 'Q');
+	gretl_charsub(s, ':', 'Q');
     } else {
-	charsub(s, ':', 'M');
+	gretl_charsub(s, ':', 'M');
     }
 }
 
@@ -1981,10 +1980,12 @@ char *make_varname_unique (char *vname, int v, DATASET *dset)
     size_t n = strlen(vname);
     int i, j, conflict;
 
-    if (n > 7) {
-	n = 7;
+    if (n > 15) {
+	n = 15;
     }
 
+ tryagain:
+
     for (j=0; j<26; j++) {
 	conflict = 0;
 	for (i=1; i<dset->v && !conflict; i++) {
@@ -2000,6 +2001,11 @@ char *make_varname_unique (char *vname, int v, DATASET *dset)
 	}
     }
 
+    if (conflict && n < 18) {
+	n++;
+	goto tryagain;
+    }
+
     return vname;
 }
 
@@ -2201,6 +2207,8 @@ void get_column_widths (const char **strs, int *widths, int n)
  * Works just like strncat(), except that it ensures that we
  * don't end up with an incomplete UTF-8 character preceding
  * the terminating NUL byte.
+ *
+ * Returns: the destination string.
  */
 
 char *gretl_utf8_strncat (char *dest, const char *src, size_t n)
@@ -2221,3 +2229,71 @@ char *gretl_utf8_strncat (char *dest, const char *src, size_t n)
 
     return strncat(dest, src, b0);
 }
+
+/**
+ * gretl_utf8_strncat_trim:
+ * @dest: destination string.
+ * @src: source string.
+ * @n: maximum number of bytes to append.
+ *
+ * The same as gretl_utf8_strncat(), except that any leading and/or
+ * trailing white space is trimmed from @dest.
+ *
+ * Returns: the destination string.
+ */
+
+char *gretl_utf8_strncat_trim (char *dest, const char *src, size_t n)
+{
+    const char *p;
+    size_t b, b0 = 0;
+    int i;
+
+    src += strspn(src, " \t\r\n");
+    p = src;
+
+    while (p && *p) {
+	p = g_utf8_next_char(p);
+	if (p) {
+	    b = p - src;
+	    if (b > n) {
+		break;
+	    }
+	    b0 = b;
+	}
+    }
+
+    strncat(dest, src, b0);
+
+    n = strlen(dest);
+
+    for (i=n-1; i>=0; i--) {
+	if (isspace(dest[i]) || dest[i] == '\r') {
+	    dest[i] = '\0';
+	} else {
+	    break;
+	}
+    }
+
+    return dest;
+}
+
+/**
+ * gretl_scan_varname:
+ * @src: source string.
+ * @targ: target string.
+ *
+ * Performs sscanf() on @src, using a conversion specifier
+ * which allows for writing up to VNAMELEN-1 bytes into
+ * @targ. The latter must therefore be at least VNAMELEN
+ * bytes long.
+ *
+ * Returns: the return value from sscanf().
+ */
+
+int gretl_scan_varname (const char *src, char *targ)
+{
+    char fmt[8];
+
+    sprintf(fmt, "%%%ds", VNAMELEN-1);
+    return sscanf(src, fmt, targ);
+}
