@@ -51,13 +51,13 @@ double dot_atof (const char *s);
 
 void set_atof_point (char c);
  
-int dotpos (const char *str);
+int gretl_dotpos (const char *str);
 
-int slashpos (const char *str);
+int gretl_slashpos (const char *str);
 
-char *delchar (int c, char *str);
+char *gretl_delchar (int c, char *str);
 
-int charpos (char c, const char *s);
+int gretl_charpos (char c, const char *s);
 
 int ends_with_backslash (const char *s);
 
@@ -86,17 +86,15 @@ char **gretl_string_split_quoted (const char *s, int *n, int *err);
 
 char *gretl_str_expand (char **orig, const char *add, const char *sep);
 
-char *charsub (char *str, char find, char repl);
+char *gretl_charsub (char *str, char find, char repl);
 
 char *comma_separate_numbers (char *s);
 
 char *shift_string_left (char *str, size_t move);
 
-char *lower (char *str);
+char *gretl_lower (char *str);
 
-void clear (char *str, int len);
-
-char *chopstr (char *str);
+char *gretl_chopstr (char *str);
 
 char *switch_ext (char *targ, const char *src, const char *ext);
 
@@ -112,8 +110,6 @@ char *compress_spaces (char *s);
 
 char *space_to_score (char *s);
 
-char *safecpy (char *targ, const char *src, int n);
-
 char **strings_array_new (int nstrs);
 
 char **strings_array_realloc_with_length (char ***pS, 
@@ -131,7 +127,7 @@ int strings_array_sort (char ***pS, int *n, gretlopt opt);
 
 int strings_array_cmp (char **strs1, char **strs2, int n);
 
-void free_strings_array (char **strs, int nstrs);
+void strings_array_free (char **strs, int nstrs);
 
 char *get_obs_string (char *obs, int t, const DATASET *dset);
 
@@ -170,4 +166,8 @@ void get_column_widths (const char **strs, int *widths, int n);
 
 char *gretl_utf8_strncat (char *dest, const char *src, size_t n);
 
+char *gretl_utf8_strncat_trim (char *dest, const char *src, size_t n);
+
+int gretl_scan_varname (const char *src, char *targ);
+
 #endif /* STRUTILS_H */
