@@ -27,7 +27,7 @@
 #include "gretl_panel.h"
 #include "cmd_private.h"
 #include "dbread.h"
-#include "gretl_scalar.h"
+#include "uservar.h"
 #include "gretl_xml.h"
 
 #define SUBDEBUG 0
@@ -441,7 +441,7 @@ static int add_new_vars_to_full (DATASET *dset)
     for (i=V0; i<dset->v && !err; i++) {
 #if FULLDEBUG
 	fprintf(stderr, "adding to full: var %d (%s, level %d)\n",
-		i, dset->varname[i], STACK_LEVEL(dset, i));
+		i, dset->varname[i], series_get_stack_level(dset, i));
 #endif
 	fullset->Z[i] = malloc(N * sizeof **fullset->Z);
 	if (fullset->Z[i] == NULL) {
@@ -676,7 +676,7 @@ make_missing_mask (const int *list, const DATASET *dset, char *mask)
 	for (t=0; t<dset->n; t++) {
 	    mask[t] = 1;
 	    for (i=1; i<dset->v; i++) {
-		if (!var_is_hidden(dset, i) && na(dset->Z[i][t])) {
+		if (!series_is_hidden(dset, i) && na(dset->Z[i][t])) {
 		    mask[t] = 0;
 		    break;
 		}
@@ -732,7 +732,7 @@ static int mask_from_dummy (const char *s, const DATASET *dset,
     char dname[VNAMELEN] = {0};
     int dnum, err = 0;
 
-    sscanf(s, "%15s", dname);
+    gretl_scan_varname(s, dname);
     dnum = series_index(dset, dname);
 
     if (dnum == dset->v) {
@@ -1292,7 +1292,7 @@ restrict_sample_from_mask (char *mask, DATASET *dset, gretlopt opt)
     }
 
     /* set up the sub-sampled datainfo */
-    err = start_new_Z(subset, 1);
+    err = start_new_Z(subset, OPT_R);
     if (err) { 
 	free(subset);
 	return err;
@@ -1879,7 +1879,7 @@ int count_missing_values (const DATASET *dset, gretlopt opt,
     for (t=t1; t<=t2; t++) {
 	tmiss = 0;
 	for (i=1; i<dset->v; i++) {
-	    if (var_is_hidden(dset, i)) {
+	    if (series_is_hidden(dset, i)) {
 		continue;
 	    }
 	    if (na(dset->Z[i][t])) {
@@ -2028,7 +2028,7 @@ int add_dataset_to_model (MODEL *pmod, const DATASET *dset,
     }
 
     /* allocate auxiliary dataset */
-    pmod->dataset = create_auxiliary_dataset(maxv, sn);
+    pmod->dataset = create_auxiliary_dataset(maxv, sn, 0);
     if (pmod->dataset == NULL) {
 	return E_ALLOC;
     }
@@ -2262,7 +2262,7 @@ int data_report (const DATASET *dset, const char *fname, PRN *prn)
 
     for (i=1; i<dset->v; i++) {
 	pprintf(prn, "%*s  %s\n", VNAMELEN, dset->varname[i], 
-		VARLABEL(dset, i));
+		series_get_label(dset, i));
     }
 
     return 0;
