@@ -166,59 +166,16 @@ make_transform_label (char *label, const char *parent,
 }
 
 /**
- * is_standard_lag:
- * @v: ID number of variable to test.
- * @dset: dataset information.
- * @parent: location to receive ID number of parent variable,
- * or NULL.
- *
- * Returns: 1 if the variable @v is marked as being a
- * lag of some "parent" variable in the dataset,
- * otherwise 0.
- */
-
-int is_standard_lag (int v, const DATASET *dset, int *parent)
-{
-    int pv = 0, ret = 0;
-
-    if (dset == NULL || v <= 0 || v >= dset->v) {
-	return 0;
-    }
-
-    if (dset->varinfo[v]->transform == LAGS) {
-	pv = series_index(dset, dset->varinfo[v]->parent);
-	pv = (pv < dset->v)? pv : 0;
-	ret = dset->varinfo[v]->lag;
-    }
-
-    if (pv > 0) {
-	int n = strcspn(dset->varname[v], "_");
-
-	if (!strncmp(dset->varname[v], dset->varname[pv], n)) {
-	    if (parent != NULL) {
-		*parent = pv;
-	    }
-	} else {
-	    /* if the lag has a "custom" name, don't consider it
-	       a child to parent variable pv */
-	    ret = 0;
-	}
-    }
-
-    return ret;
-}
-
-/**
- * is_standard_lag_of:
+ * standard_lag_of:
  * @v: ID number of series to test.
- * @parent: potential parent series.
+ * @parent: ID of potential parent series.
  * @dset: dataset information.
  *
- * Returns: the lag order, if the series @v is marked as 
+ * Returns: the lag order of series @v, if it is marked as 
  * a lag of @parent, otherwise 0.
  */
 
-int is_standard_lag_of (int v, int parent, const DATASET *dset)
+int standard_lag_of (int v, int parent, const DATASET *dset)
 {
     int pv = 0, ret = 0;
 
@@ -226,10 +183,10 @@ int is_standard_lag_of (int v, int parent, const DATASET *dset)
 	return 0;
     }
 
-    if (dset->varinfo[v]->transform == LAGS) {
-	pv = series_index(dset, dset->varinfo[v]->parent);
+    if (series_get_transform(dset, v) == LAGS) {
+	pv = series_get_parent_id(dset, v);
 	if (pv == parent) {
-	    ret = dset->varinfo[v]->lag;
+	    ret = series_get_lag(dset, v);
 	}
     }
 
@@ -256,9 +213,8 @@ int is_standard_diff (int v, const DATASET *dset, int *parent)
 	return 0;
     }
 
-    if (dset->varinfo[v]->transform == DIFF) {
-	pv = series_index(dset, dset->varinfo[v]->parent);
-	pv = (pv < dset->v)? pv : 0;
+    if (series_get_transform(dset, v) == DIFF) {
+	pv = series_get_parent_id(dset, v);
 	if (pv > 0) {
 	    if (parent != NULL) {
 		*parent = pv;
@@ -270,45 +226,6 @@ int is_standard_diff (int v, const DATASET *dset, int *parent)
     return ret;
 }
 
-/**
- * is_dummy_child:
- * @v: ID number of variable to test.
- * @dset: dataset information.
- * @parent: location to receive ID number of parent variable,
- * or NULL.
- *
- * Returns: 1 if the variable @v is marked as being a dummy
- * variable that codes for a specific value of some "parent" 
- * variable in the dataset, otherwise 0.
- */
-
-int is_dummy_child (int v, const DATASET *dset, int *parent)
-{
-    int pv = dset->v;
-    int i = 0, ret = 0;
-
-    if (dset->varinfo[v]->transform == DUMMIFY) {
-	pv = series_index(dset, dset->varinfo[v]->parent);
-    } else if (!strncmp(dset->varname[v], "dt_", 3)) {
-	if (sscanf(dset->varname[v] + 3, "%d", &i) && i > 1) {
-	    pv = series_index(dset, "dt_1");
-	}
-    } else if (!strncmp(dset->varname[v], "du_", 3)) {
-	if (sscanf(dset->varname[v] + 3, "%d", &i) && i > 1) {
-	    pv = series_index(dset, "du_1");
-	}
-    }	
-
-    if (pv < dset->v) {
-	*parent = pv;
-	ret = 1;
-    } else {
-	*parent = 0;
-    }
-
-    return ret;
-}
-
 static void make_xp_varname (char *vname, int v1, int v2,
 			     const DATASET *dset,
 			     int len)
@@ -560,7 +477,7 @@ static int transform_handle_duplicate (int ci, int lag, int v,
     int ret = VARNAME_DUPLICATE;
     int t, ok = 0;
 
-    if (!strcmp(label, VARLABEL(dset, v))) {
+    if (!strcmp(label, series_get_label(dset, v))) {
 	/* labels identical, so OK? */
 	ok = 1;
     }
@@ -580,11 +497,11 @@ static int transform_handle_duplicate (int ci, int lag, int v,
 	    dset->Z[v][t] = x[t];
 	}
 	if (*label != '\0') {
-	    strcpy(VARLABEL(dset, v), label);
+	    series_set_label(dset, v, label);
 	}
-	dset->varinfo[v]->transform = ci;
-	dset->varinfo[v]->lag = lag;
-	dset->varinfo[v]->flags = 0;
+	series_set_transform(dset, v, ci);
+	series_set_lag(dset, v, lag);
+	series_zero_flags(dset, v);
 	ret = VAR_EXISTS_OK;
     }	
 
@@ -627,11 +544,11 @@ check_add_transform (int ci, int lag, int vnum, const double *x,
 	}
     } else {
 	/* no var of this name, working from scratch */
-	if (dataset_add_series(1, dset)) {
+	if (dataset_add_series(dset, 1)) {
 	    ret = VAR_ADD_FAILED;
 	} else {
 	    strcpy(dset->varname[vnum], vname);
-	    strcpy(VARLABEL(dset, vnum), label);
+	    series_set_label(dset, vnum, label);
 	    for (t=0; t<dset->n; t++) {
 		dset->Z[vnum][t] = x[t];
 	    }
@@ -643,13 +560,13 @@ check_add_transform (int ci, int lag, int vnum, const double *x,
 
 static int get_lag_ID (int srcv, int lag, const DATASET *dset)
 {
-    const char *vname = dset->varname[srcv];
-    VARINFO *vinfo;
-    int i;
+    const char *parent, *vname = dset->varname[srcv];
+    int i, vlag;
 
     for (i=1; i<dset->v; i++) {
-	vinfo = dset->varinfo[i];
-	if (vinfo->lag == lag && !strcmp(vinfo->parent, vname)) {
+	vlag = series_get_lag(dset, i);
+	parent = series_get_parent_name(dset, i);
+	if (vlag == lag && !strcmp(vname, parent)) {
 	    return i;
 	}
     }
@@ -753,18 +670,18 @@ static int get_transform (int ci, int v, int aux, double x,
 
     if (!err && vno > 0) {
 	if (ci == DUMMIFY) {
-	    strcpy(dset->varinfo[vno]->parent, dset->varname[v]);
-	    dset->varinfo[vno]->transform = DUMMIFY;
-	    set_var_discrete(dset, vno, 1);
+	    series_set_parent(dset, vno, dset->varname[v]);
+	    series_set_transform(dset, vno, DUMMIFY);
+	    series_set_discrete(dset, vno, 1);
 	} else if (ci == LAGS || ci == DIFF) {
-	    strcpy(dset->varinfo[vno]->parent, dset->varname[v]);
-	    dset->varinfo[vno]->transform = ci;
+	    series_set_parent(dset, vno, dset->varname[v]);
+	    series_set_transform(dset, vno, ci);
 	}
 
 	if (ci == LAGS) {
-	    dset->varinfo[vno]->lag = aux;
+	    series_set_lag(dset, vno, aux);
 	} else {
-	    dset->varinfo[vno]->lag = 0;
+	    series_set_lag(dset, vno, 0);
 	}
     }
 
@@ -1112,7 +1029,7 @@ transform_preprocess_list (int *list, const DATASET *dset, int f)
 	} else if (f == DUMMIFY) {
 	    ok = 0; /* reverse burden of proof */
 	    if (v > 0) {
-		if (var_is_discrete(dset, v)) {
+		if (series_is_discrete(dset, v)) {
 		    /* pre-approved */
 		    ok = 1;
 		} else if (gretl_isdiscrete(0, dset->n - 1, dset->Z[v])) {
@@ -1148,7 +1065,7 @@ transform_preprocess_list (int *list, const DATASET *dset, int f)
 	    }
 	}
 
-	free_strings_array(S, longnames);
+	strings_array_free(S, longnames);
     }
 
     return err;
@@ -1269,7 +1186,7 @@ int list_laggenr (int **plist, int order, DATASET *dset)
 	    if (lv > 0) {
 #if TRDEBUG > 1
 		fprintf(stderr, "lag var name '%s', label '%s'\n",
-			dset->varname[lv], VARLABEL(dset, lv));
+			dset->varname[lv], series_get_label(dset, lv));
 #endif
 		laglist[j++] = lv;
 		l0++;
@@ -1677,7 +1594,7 @@ int list_makediscrete (const int *list, DATASET *dset, gretlopt opt)
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
 	if (v > 0) {
-	    set_var_discrete(dset, v, disc);
+	    series_set_discrete(dset, v, disc);
 	}
     }
 
