@@ -59,8 +59,6 @@ int gretl_VAR_print_sigma (const GRETL_VAR *var, PRN *prn)
 #define VDC_ROW_MAX 5
 #define VDC_WIDTH  11
 
-#define VECM_WIDTH 13
-
 enum {
     IRF,
     VDC
@@ -201,7 +199,8 @@ static void VAR_info_end_table (PRN *prn)
     }
 }
 
-static int varprint_namelen (const GRETL_VAR *var, const DATASET *dset,
+static int varprint_namelen (const GRETL_VAR *var, 
+			     const DATASET *dset,
 			     int rmax, int block)
 {
     int len, maxlen = 0;
@@ -563,20 +562,24 @@ void print_Johansen_test_case (JohansenCode jcode, PRN *prn)
     }
 }
 
-static char *make_beta_vname (char *vname,
-			      const GRETL_VAR *v,
-			      const DATASET *dset,
-			      int i)
+char *vecm_beta_varname (char *vname,
+			 const GRETL_VAR *v,
+			 const DATASET *dset,
+			 int i)
 {
+    const char *src = "";
+
     if (i < v->neqns) {
-	strcpy(vname, dset->varname[v->ylist[i+1]]);
+	src = dset->varname[v->ylist[i+1]];
     } else if (auto_restr(v) && i == v->neqns) {
-	strcpy(vname, (jcode(v) == J_REST_CONST)? "const" : "trend");
+	src = (jcode(v) == J_REST_CONST)? "const" : "trend";
     } else if (v->rlist != NULL) {
 	int k = i - v->ylist[0] - auto_restr(v) + 1;
 
-	strcpy(vname, dset->varname[v->rlist[k]]);
-    } 
+	src = dset->varname[v->rlist[k]];
+    }
+
+    maybe_trim_varname(vname, src);
 
     return vname;
 }
@@ -589,7 +592,7 @@ static int max_beta_namelen (GRETL_VAR *v,
     int i, ni, n = 0;
 
     for (i=0; i<r; i++) {    
-	make_beta_vname(s, v, dset, i);
+	vecm_beta_varname(s, v, dset, i);
 	ni = strlen(s);
 	if (ni > n) {
 	    n = ni;
@@ -627,7 +630,7 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
     sprintf(namefmt, "%%-%ds", nwid);
 
     for (i=0; i<rows; i++) {
-	make_beta_vname(vname, jvar, dset, i);
+	vecm_beta_varname(vname, jvar, dset, i);
 	if (rtf) {
 	    pputs(prn, vname);
 	} else {
@@ -680,10 +683,12 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
     gretl_prn_newline(prn);
 
     for (i=0; i<rows; i++) {
-	sprintf(vname, "%s", dset->varname[jvar->ylist[i+1]]);
+	const char *src = dset->varname[jvar->ylist[i+1]];
+
 	if (rtf) {
-	    pputs(prn, vname);
+	    pputs(prn, src);
 	} else {
+	    maybe_trim_varname(vname, src);
 	    pprintf(prn, namefmt, vname);
 	}
 
@@ -719,52 +724,46 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
     gretl_prn_newline(prn);
 }
 
-static int max_vlen (const int *list, const DATASET *dset)
-{
-    int i, ni, n = 0;
-
-    for (i=1; i<=list[0]; i++) {
-	ni = strlen(dset->varname[list[i]]);
-	if (ni > n) {
-	    n = ni;
-	}
-    }
-
-    return n;
-}
-
 static void print_VECM_omega (GRETL_VAR *jvar, const DATASET *dset, PRN *prn)
 {
     int rtf = rtf_format(prn);
     int *list = jvar->ylist;
-    char s[32];
+    const char *src;
+    char vname[32] = {0};
+    int vwidth = 13;
     int w0, wi = 12;
     int i, j;
 
     pprintf(prn, "%s:\n", A_("Cross-equation covariance matrix"));
     gretl_prn_newline(prn);
 
-    w0 = max_vlen(list, dset) + 1;
+    w0 = max_namelen_in_list(list, dset) + 1;
+    if (w0 > vwidth) {
+	vwidth = w0;
+    }
 
     /* top row: names of Y variables */
 
     for (i=0; i<jvar->neqns; i++) {
-	sprintf(s, "%s", dset->varname[list[i+1]]);
+	src = dset->varname[list[i+1]];
+	if (plain_format(prn)) {
+	    maybe_trim_varname(vname, src);
+	}
 	if (i == 0) {
 	    if (rtf) {
-		pprintf(prn, "\t\t%s", s);
+		pprintf(prn, "\t\t%s", src);
 	    } else {
-		wi = strlen(s);
-		if (wi < VECM_WIDTH) wi = VECM_WIDTH;
-		pprintf(prn, "%*s", w0 + wi, s);
+		wi = strlen(vname);
+		if (wi < vwidth) wi = vwidth;
+		pprintf(prn, "%*s", w0 + wi, vname);
 	    }
 	} else {
 	    if (rtf) {
-		pprintf(prn, "\t%s", s);
+		pprintf(prn, "\t%s", src);
 	    } else {
-		wi = strlen(s) + 1;
-		if (wi < VECM_WIDTH) wi = VECM_WIDTH;
-		pprintf(prn, "%*s", wi, s);
+		wi = strlen(vname) + 1;
+		if (wi < vwidth) wi = vwidth;
+		pprintf(prn, "%*s", wi, vname);
 	    }
 	}
     }
@@ -774,21 +773,28 @@ static void print_VECM_omega (GRETL_VAR *jvar, const DATASET *dset, PRN *prn)
     /* subsequent rows: Y name plus values */
 
     for (i=0; i<jvar->neqns; i++) {
-	sprintf(s, "%s", dset->varname[list[i+1]]);
+	src = dset->varname[list[i+1]];
+	if (plain_format(prn)) {
+	    maybe_trim_varname(vname, src);
+	}
 	if (rtf) {
-	    pputs(prn, s);
-	    if (strlen(s) < 8) {
+	    pputs(prn, src);
+	    if (strlen(src) < 8) {
 		pputc(prn, '\t');
 	    }	    
 	} else {
-	    pprintf(prn, "%-*s ", w0, s);
+	    pprintf(prn, "%-*s ", w0, vname);
 	}
 	for (j=0; j<jvar->neqns; j++) {
 	    if (rtf) {
 		pprintf(prn, "\t%#.5g", gretl_matrix_get(jvar->S, i, j));
 	    } else {
-		wi = strlen(dset->varname[list[j+1]]);
-		if (wi < VECM_WIDTH - 1) wi = VECM_WIDTH - 1;
+		src = dset->varname[list[j+1]];
+		wi = strlen(src);
+		if (wi >= NAMETRUNC) {
+		    wi = NAMETRUNC - 1;
+		}
+		if (wi < vwidth - 1) wi = vwidth - 1;
 		pprintf(prn, "%#*.5g ", wi, gretl_matrix_get(jvar->S, i, j));
 	    }
 	}
@@ -963,13 +969,17 @@ static int max_Ftest_label_len (GRETL_VAR *var, const DATASET *dset,
     int i, v;
 
     for (i=0; i<var->neqns; i++) {
-	v = (var->models[i])->list[1];
+	v = var->models[i]->list[1];
 	len = strlen(dset->varname[v]);
 	if (len > maxnamelen) {
 	    maxnamelen = len;
 	}
     }
 
+    if (maxnamelen >= NAMETRUNC) {
+	maxnamelen = NAMETRUNC - 1;
+    }
+
     sprintf(s, _("All lags of %s"), "x");
     len1 = g_utf8_strlen(s, -1) + maxnamelen - 1;
 
@@ -1152,22 +1162,28 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
 	for (j=0; j<var->neqns; j++) {
 	    Fval = var->Fvals[k];
 	    if (!na(Fval)) {
+		const char *vname;
+
 		pv = snedecor_cdf_comp(nlags, dfd, Fval);
 		v = (var->models[j])->list[1];
+		vname = dset->varname[v];
 		if (tex) {
-		    pprintf(prn, A_("All lags of %s"), dset->varname[v]);
+		    pprintf(prn, A_("All lags of %s"), vname);
 		    pputs(prn, " & ");
 		    pprintf(prn, "$F(%d, %d) = %g$ & ", nlags, dfd, Fval);
 		    pprintf(prn, "[%.4f]\\\\\n", pv);
 		} else if (rtf) {
-		    sprintf(label, A_("All lags of %s"), dset->varname[v]);
+		    sprintf(label, A_("All lags of %s"), vname);
 		    llen = strlen(label);
 		    pputs(prn, label);
 		    bufspace(fwidth - llen, prn);
 		    pprintf(prn, "F(%d, %d) = %8.5g ", nlags, dfd, Fval);
 		    pprintf(prn, "[%.4f]\\par\n", pv);
 		} else {
-		    sprintf(label, _("All lags of %s"), dset->varname[v]);
+		    char tmp[NAMETRUNC];
+
+		    maybe_trim_varname(tmp, vname);
+		    sprintf(label, _("All lags of %s"), tmp);
 		    llen = g_utf8_strlen(label, -1);
 		    pputs(prn, label);
 		    bufspace(fwidth - llen, prn);
