@@ -905,7 +905,7 @@ static int dpd_wald_test (ddset *dpd)
     int cpos = dpd_const_pos(dpd);
     int k1, knd;
     int i, j, ri, rj;
-    int err;
+    int err = 0;
 
     /* total number of coefficients excluding any time dummies */
     knd = dpd->k - dpd->ndum;
@@ -975,20 +975,34 @@ static int dpd_wald_test (ddset *dpd)
 
     gretl_matrix_reuse(dpd->kmtmp, dpd->k, dpd->nz);
     gretl_matrix_reuse(dpd->kktmp, dpd->k, dpd->k);
+
+    if (err) {
+	fprintf(stderr, "dpd_wald_test failed: %s\n", 
+		errmsg_get_with_default(err));
+    }
     
     return err;
 }
 
 static int dpd_sargan_test (ddset *dpd)
 {
+    int save_rows, save_cols;
     gretl_matrix *Zu;
     int err = 0;
 
+    save_rows = gretl_matrix_rows(dpd->L1);
+    save_cols = gretl_matrix_cols(dpd->L1);
+
     Zu = gretl_matrix_reuse(dpd->L1, dpd->nz, 1);
     gretl_matrix_multiply(dpd->ZT, dpd->uhat, Zu);
     gretl_matrix_divide_by_scalar(dpd->A, dpd->effN);
     dpd->sargan = gretl_scalar_qform(Zu, dpd->A, &err);
 
+    if (!err && dpd->sargan < 0) {
+	dpd->sargan = NADBL;
+	err = E_NOTPD;
+    }
+
     if (!err && dpd->step == 1) {
 	/* allow for scale factor in H matrix */
 	if (dpd->flags & DPD_ORTHDEV) {
@@ -1018,7 +1032,12 @@ static int dpd_sargan_test (ddset *dpd)
     }
 #endif
 
-    gretl_matrix_reuse(dpd->L1, 1, dpd->nz);
+    gretl_matrix_reuse(dpd->L1, save_rows, save_cols);
+    
+    if (err) {
+	fprintf(stderr, "dpd_sargan_test failed: %s\n", 
+		errmsg_get_with_default(err));
+    }
 
     return err;
 }
@@ -1084,7 +1103,7 @@ static void make_asy_Hi (ddset *dpd, int i, gretl_matrix *H,
 static int dpd_ar_test (ddset *dpd)
 {
     double x, d0, d1, d2, d3;
-    gretl_matrix_block *B;
+    gretl_matrix_block *B = NULL;
     gretl_matrix *ui, *wi;
     gretl_matrix *Xi, *Zi;
     gretl_matrix *Hi, *ZU;
@@ -1092,12 +1111,16 @@ static int dpd_ar_test (ddset *dpd)
     gretl_matrix *Tmp;
     char *hmask = NULL;
     int HT, T = dpd->maxTi;
+    int save_rows, save_cols;
     int nz = dpd->nz;
     int asy, ZU_cols;
     int i, j, k, s, t;
     int nlags, m = 1;
     int err = 0;
 
+    save_rows = gretl_matrix_rows(dpd->Zi);
+    save_cols = gretl_matrix_cols(dpd->Zi);
+
     /* if non-robust and on first step, Hi is computed differently */
     if ((dpd->flags & DPD_TWOSTEP) || (dpd->flags & DPD_WINCORR)) {
 	asy = 0;
@@ -1107,7 +1130,8 @@ static int dpd_ar_test (ddset *dpd)
 	HT = dpd->T;
 	hmask = malloc(HT);
 	if (hmask == NULL) {
-	    return E_ALLOC;
+	    err = E_ALLOC;
+	    goto finish;
 	}
     }
 
@@ -1126,8 +1150,8 @@ static int dpd_ar_test (ddset *dpd)
 			       NULL);
 
     if (B == NULL) {
-	free(hmask);
-	return E_ALLOC;
+	err = E_ALLOC;
+	goto finish;
     }
 
     Zi = gretl_matrix_reuse(dpd->Zi, T, nz);
@@ -1288,7 +1312,7 @@ static int dpd_ar_test (ddset *dpd)
 
     if (!err) {
 	d2 *= -2.0;
-	/* d3 = w'X * var(\hat{\beta}) * X'w */
+	/* form w'X * var(\hat{\beta}) * X'w */
 	d3 = gretl_scalar_qform(wX, dpd->vbeta, &err);
     }
 
@@ -1296,8 +1320,8 @@ static int dpd_ar_test (ddset *dpd)
 	double z, den = d1 + d2 + d3;
 
 #if ADEBUG
-	fprintf(stderr, "ar_test: d0=%g, d1=%g, d2=%g, d3=%g, den=%g\n",
-		d0, d1, d2, d3, den);
+	fprintf(stderr, "ar_test: m=%d, d0=%g, d1=%g, d2=%g, d3=%g, den=%g\n",
+		m, d0, d1, d2, d3, den);
 #endif	
 	if (den <= 0) {
 	    err = E_NAN;
@@ -1322,7 +1346,14 @@ static int dpd_ar_test (ddset *dpd)
     free(hmask);
 
     /* restore original dimensions */
-    gretl_matrix_reuse(dpd->Zi, dpd->max_ni, dpd->nz);
+    gretl_matrix_reuse(dpd->Zi, save_rows, save_cols);
+
+    if (err) {
+	fprintf(stderr, "dpd_ar_test failed: %s\n", 
+		errmsg_get_with_default(err));
+    } else if (na(dpd->AR1) && na(dpd->AR1)) {
+	fprintf(stderr, "dpd_ar_test: no data\n");
+    }
 
     return err;
 }
@@ -1709,7 +1740,7 @@ static int dpd_finalize_model (MODEL *pmod, ddset *dpd,
     pmod->fstt = pmod->chisq = NADBL;
     pmod->lnL = NADBL;
   
-    gretl_model_allocate_params(pmod, dpd->k);
+    gretl_model_allocate_param_names(pmod, dpd->k);
     if (pmod->errcode) {
 	return pmod->errcode;
     }
@@ -1718,16 +1749,14 @@ static int dpd_finalize_model (MODEL *pmod, ddset *dpd,
 	j = 0;
 	if (dpd->laglist != NULL) {
 	    for (i=1; i<=dpd->laglist[0]; i++) {
-		pmod->params[j][0] = '\0';
 		sprintf(tmp, "%.10s(-%d)", dset->varname[dpd->yno], 
 			dpd->laglist[i]);
-		strncat(pmod->params[j++], tmp, 15);
+		gretl_model_set_param_name(pmod, j++, tmp);
 	    }	
 	} else {
 	    for (i=0; i<dpd->p; i++) {
-		pmod->params[j][0] = '\0';
 		sprintf(tmp, "%.10s(-%d)", dset->varname[dpd->yno], i+1);
-		strncat(pmod->params[j++], tmp, 15);
+		gretl_model_set_param_name(pmod, j++, tmp);
 	    }
 	}
     } else {
@@ -1736,29 +1765,29 @@ static int dpd_finalize_model (MODEL *pmod, ddset *dpd,
 	j = 0;
 	if (dpd->laglist != NULL) {
 	    for (i=1; i<=dpd->laglist[0]; i++) {
-		pmod->params[j][0] = '\0';
 		sprintf(tmp, "%c%.10s(-%d)", prefix, dset->varname[dpd->yno], 
 			dpd->laglist[i]);
-		strncat(pmod->params[j++], tmp, 15);
+		gretl_model_set_param_name(pmod, j++, tmp);
 	    }	
 	} else {
 	    for (i=0; i<dpd->p; i++) {
-		pmod->params[j][0] = '\0';
 		sprintf(tmp, "%c%.10s(-%d)", prefix, dset->varname[dpd->yno], i+1);
-		strncat(pmod->params[j++], tmp, 15);
+		gretl_model_set_param_name(pmod, j++, tmp);
 	    }
 	}
     }
 
     for (i=0; i<dpd->nx; i++) {
-	strcpy(pmod->params[j++], dset->varname[dpd->xlist[i+1]]);
+	gretl_model_set_param_name(pmod, j++, dset->varname[dpd->xlist[i+1]]);
     }
 
     for (i=0; i<dpd->ndum; i++) {
 	if (dpd->ifc) {
-	    sprintf(pmod->params[j++], "T%d", dpd->t1min + i + 2);
+	    sprintf(tmp, "T%d", dpd->t1min + i + 2);
+	    gretl_model_set_param_name(pmod, j++, tmp);
 	} else {
-	    sprintf(pmod->params[j++], "T%d", dpd->t1min + i + 1);
+	    sprintf(tmp, "T%d", dpd->t1min + i + 1);
+	    gretl_model_set_param_name(pmod, j++, tmp);
 	}
     }
 
@@ -1807,12 +1836,12 @@ static int dpd_finalize_model (MODEL *pmod, ddset *dpd,
 	if (!na(dpd->AR2)) {
 	    gretl_model_set_double(pmod, "AR2", dpd->AR2);
 	}
+	gretl_model_set_int(pmod, "sargan_df", dpd->nz - dpd->k);
 	if (!na(dpd->sargan)) {
-	    gretl_model_set_int(pmod, "sargan_df", dpd->nz - dpd->k);
 	    gretl_model_set_double(pmod, "sargan", dpd->sargan);
 	}
+	gretl_model_set_int(pmod, "wald_df", dpd->wdf[0]);
 	if (!na(dpd->wald[0])) {
-	    gretl_model_set_int(pmod, "wald_df", dpd->wdf[0]);
 	    gretl_model_set_double(pmod, "wald", dpd->wald[0]);
 	}
 	if (!na(dpd->wald[1])) {
@@ -2480,6 +2509,7 @@ static int parse_diag_info (int ci, const char *s, diag_info *d,
 			    const DATASET *dset)
 {
     char vname[VNAMELEN];
+    char fmt[16];
     int m1, m2;
     int err = 0;
 
@@ -2498,7 +2528,9 @@ static int parse_diag_info (int ci, const char *s, diag_info *d,
 	return E_PARSE;
     }
 
-    if (sscanf(s, "%15[^, ],%d,%d)", vname, &m1, &m2) != 3) {
+    sprintf(fmt, "%%%d[^, ],%%d,%%d)", VNAMELEN-1);
+
+    if (sscanf(s, fmt, vname, &m1, &m2) != 3) {
 	err = E_PARSE;
     } else {
 	int v = current_series_index(dset, vname);
