@@ -18,7 +18,7 @@
  */
 
 #include "libgretl.h"
-#include "gretl_scalar.h"
+#include "uservar.h"
 #include "arma_priv.h"
 
 #define AINIT_DEBUG 0
@@ -156,7 +156,7 @@ static int real_hr_arma_init (double *coeff, const DATASET *dset,
 	y = ainfo->y;
     } 
 
-    aset = create_auxiliary_dataset(pass1v + qtotal, ainfo->T);
+    aset = create_auxiliary_dataset(pass1v + qtotal, ainfo->T, 0);
     if (aset == NULL) {
 	return E_ALLOC;
     }
@@ -569,7 +569,7 @@ static int arma_init_add_dummies (arma_info *ainfo,
 	int origv = dset->v;
 	int j, v, nd = misslist[0];
 
-	err = dataset_add_series(nd, dset);
+	err = dataset_add_series(dset, nd);
 	if (!err) {
 	    for (i=1; i<=misslist[0]; i++) {
 		v = origv + i - 1;
@@ -1038,8 +1038,7 @@ static int arma_get_nls_model (MODEL *amod, arma_info *ainfo,
 
     if (!err) {
 	set_auxiliary_scalars();
-	err = nlspec_add_param_list(spec, nparam, parms, pnames);
-
+	err = aux_nlspec_add_param_list(spec, nparam, parms, pnames);
 	if (!err) {
 	    *amod = model_from_nlspec(spec, dset, nlsopt, prn);
 	    err = amod->errcode;
@@ -1056,7 +1055,7 @@ static int arma_get_nls_model (MODEL *amod, arma_info *ainfo,
 
     nlspec_destroy(spec);
     free(parms);
-    free_strings_array(pnames, nparam);
+    strings_array_free(pnames, nparam);
 
     return err;
 }
@@ -1158,7 +1157,7 @@ int ar_arma_init (double *coeff, const DATASET *dset,
 	maybe_set_yscale(ainfo);
     }
 
-    aset = create_auxiliary_dataset(av, ainfo->fullT);
+    aset = create_auxiliary_dataset(av, ainfo->fullT, 0);
     if (aset == NULL) {
 	return E_ALLOC;
     }
@@ -1247,7 +1246,7 @@ int arma_by_ls (const double *coeff, const DATASET *dset,
     int *arlist = NULL;
     int nonlin = 0;
 
-    aset = create_auxiliary_dataset(av, ainfo->T);
+    aset = create_auxiliary_dataset(av, ainfo->T, 0);
     if (aset == NULL) {
 	return E_ALLOC;
     }
