@@ -973,6 +973,7 @@ static int bp_do_maxlik (bp_container *bp, gretlopt opt, PRN *prn)
     double gradtol = 1.0e-05;
     int fncount, maxit = 1000;
     gretlopt maxopt = opt & OPT_V;
+    int quiet = opt & OPT_Q;
     double *theta;
     int err = 0;
 
@@ -988,12 +989,12 @@ static int bp_do_maxlik (bp_container *bp, gretlopt opt, PRN *prn)
 	err = BFGS_max(theta, bp->npar, maxit, crittol, &fncount, 
 		       &grcount, biprob_loglik, C_LOGLIK,
 		       biprob_score, bp, NULL,
-		       maxopt, prn);
+		       maxopt, quiet ? NULL : prn);
     } else {	
 	err = newton_raphson_max(theta, bp->npar, maxit, crittol, gradtol,
 				 &fncount, C_LOGLIK, biprob_loglik, 
 				 biprob_score, biprobit_hessian,
-				 bp, maxopt, prn);
+				 bp, maxopt, quiet ? NULL : prn);
     }
 
     free(theta);
@@ -1177,7 +1178,7 @@ static int biprobit_fill_model (MODEL *pmod, bp_container *bp,
     } else {
 	pmod->coeff = tmp;
 	pmod->ncoeff = npar;
-	err = gretl_model_allocate_params(pmod, npar);
+	err = gretl_model_allocate_param_names(pmod, npar);
     }
 
     if (err) {
@@ -1185,12 +1186,12 @@ static int biprobit_fill_model (MODEL *pmod, bp_container *bp,
     }
 
     for (i=0; i<bp->k1; i++) {
-	strcpy(pmod->params[i], dset->varname[bp->X1list[i+1]]);
+	gretl_model_set_param_name(pmod, i, dset->varname[bp->X1list[i+1]]);
 	pmod->coeff[i] = gretl_vector_get(bp->beta, i);
     }
 	
     for (i=0; i<bp->k2; i++) {
-	strcpy(pmod->params[i+bp->k1], dset->varname[bp->X2list[i+1]]);
+	gretl_model_set_param_name(pmod, i+bp->k1, dset->varname[bp->X2list[i+1]]);
 	pmod->coeff[i + bp->k1] = gretl_vector_get(bp->gama, i);
     }
 
