@@ -139,10 +139,7 @@ static int wf1_read_history (FILE *fp, unsigned pos,
     if (htxt != NULL) {
 	fseek(fp, hpos, SEEK_SET);
 	if (fread(htxt, 1, len, fp) == len) {
-	    char *targ = VARLABEL(dset, i);
-
-	    *targ = '\0';
-	    strncat(targ, htxt, MAXLABEL - 1);
+	    series_set_label(dset, i, htxt);
 	    fprintf(stderr, "%s\n", htxt);
 	}
 	free(htxt);
@@ -548,7 +545,7 @@ int wf1_get_data (const char *fname, DATASET *dset,
 	int nvtarg = newset->v - 1;
 
 	if (nvread < nvtarg) {
-	    dataset_drop_last_variables(nvtarg - nvread, newset);
+	    dataset_drop_last_variables(newset, nvtarg - nvread);
 	}
 
 	if (fix_varname_duplicates(newset)) {
