@@ -305,7 +305,7 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 		if (VTYPE_IS_NUMERIC(vtype) || vtype == VALUE_STRING) {
 		    if (i == 0) {
 			/* first column: write content to labels */
-			gretl_utf8_strncat(sheet->label[t], tmp, OBSLEN - 1);
+			gretl_utf8_strncat_trim(sheet->label[t], tmp, OBSLEN - 1);
 		    }
 		}
 
@@ -760,7 +760,7 @@ int gnumeric_get_data (const char *fname, int *list, char *sheetname,
 	if (blank_cols > 0) {
 	    fprintf(stderr, "Dropping %d apparently blank column(s)\n", 
 		    blank_cols);
-	    dataset_drop_last_variables(blank_cols, newset);
+	    dataset_drop_last_variables(newset, blank_cols);
 	}
 
 	if (missvals) {
