@@ -1056,20 +1056,20 @@ static int transcribe_heckit_params (MODEL *hm, h_container *HC, DATASET *dset)
     }
 
     if (!err) {
-	err = gretl_model_allocate_params(hm, npar);
+	err = gretl_model_allocate_param_names(hm, npar);
     }
 
     if (!err) {
 	for (i=0; i<kb; i++) {
-	    strcpy(hm->params[i], dset->varname[hm->list[i+2]]);
+	    gretl_model_set_param_name(hm, i, dset->varname[hm->list[i+2]]);
 	    fullcoeff[i] = gretl_vector_get(HC->beta, i);
 	}
 	
-	strcpy(hm->params[kb], "lambda");
+	gretl_model_set_param_name(hm, kb, "lambda");
 	fullcoeff[kb] = HC->lambda;
 
 	for (i=0; i<HC->ksel; i++) {
-	    strcpy(hm->params[i+kb+1], dset->varname[HC->Zlist[i+1]]);
+	    gretl_model_set_param_name(hm, i+kb+1, dset->varname[HC->Zlist[i+1]]);
 	    fullcoeff[i+kb+1] = gretl_vector_get(HC->gama, i);
 	}
     }
@@ -1524,7 +1524,7 @@ static MODEL heckit_init (h_container *HC, DATASET *dset)
 
     /* add the inverse Mills ratio to the dataset */
 
-    err = dataset_add_series(1, dset);
+    err = dataset_add_series(dset, 1);
     if (err) {
 	goto bailout;
     } 
@@ -1640,7 +1640,7 @@ MODEL heckit_estimate (const int *list, DATASET *dset,
     }
 
     if (dset->v > oldv) {
-	dataset_drop_last_variables(1, dset);
+	dataset_drop_last_variables(dset, 1);
     }
 
     h_container_destroy(HC);
