@@ -25,6 +25,7 @@
 #include "var.h"
 #include "johansen.h"
 #include "vartest.h"
+#include "varprint.h"
 #include "libset.h"
 #include "jprivate.h"
 
@@ -321,23 +322,6 @@ static void fix_xstr (char *s, int p)
     }
 }
 
-const char *beta_vname (const GRETL_VAR *v,
-			const DATASET *dset,
-			int i)
-{
-    if (i < v->neqns) {
-	return dset->varname[v->ylist[i+1]];
-    } else if (auto_restr(v) && i == v->neqns) {
-	return (jcode(v) == J_REST_CONST)? "const" : "trend";
-    } else if (v->rlist != NULL) {
-	int k = i - v->ylist[0] - auto_restr(v) + 1;
-
-	return dset->varname[v->rlist[k]];
-    } 
-
-    return "";
-}
-
 #define ABMIN 1.0e-15
 
 /* for cointegration test: print cointegrating vectors or adjustments,
@@ -351,7 +335,8 @@ static void print_beta_or_alpha (const GRETL_VAR *jvar, int k,
     gretl_matrix *c = (job == V_BETA)? jv->Beta : jv->Alpha;
     int rows = gretl_matrix_rows(c);
     int vnorm = libset_get_int(VECM_NORM);
-    char xstr[32];
+    char xstr[32], tmp[NAMETRUNC];
+    int n, namelen = 8;
     int i, j, row;
     double x, y;
 
@@ -370,7 +355,16 @@ static void print_beta_or_alpha (const GRETL_VAR *jvar, int k,
     }
 
     for (i=0; i<rows; i++) {
-	pprintf(prn, "%-10s", beta_vname(jvar, dset, i));
+	vecm_beta_varname(tmp, jvar, dset, i);
+	n = strlen(tmp);
+	if (n > namelen) {
+	    namelen = n;
+	}
+    }
+
+    for (i=0; i<rows; i++) {
+	vecm_beta_varname(tmp, jvar, dset, i);
+	pprintf(prn, "%-*s", namelen + 2, tmp);
 	for (j=0; j<k; j++) {
 	    x = gretl_matrix_get(c, i, j);
 	    if (rescale) {
@@ -452,6 +446,10 @@ static int print_long_run_matrix (const GRETL_VAR *jvar,
 {
     JohansenInfo *jv = jvar->jinfo;
     gretl_matrix *Pi;
+    char tmp[NAMETRUNC];
+    const char *vname;
+    int firstlen = 10;
+    int namelen;
     double x;
     int i, j;
 
@@ -460,28 +458,41 @@ static int print_long_run_matrix (const GRETL_VAR *jvar,
 	return E_ALLOC;
     }
 
+    namelen = max_namelen_in_list(jvar->ylist, dset);
+    if (firstlen <= namelen) {
+	firstlen = namelen + 1;
+    }
+    if (namelen < 12) {
+	namelen = 12;
+    }    
+
     gretl_matrix_multiply_mod(jv->Alpha, GRETL_MOD_NONE,
 			      jv->Beta, GRETL_MOD_TRANSPOSE,
 			      Pi, GRETL_MOD_NONE);
 
     pprintf(prn, "%s\n", _("long-run matrix (alpha * beta')"));
 
-    pprintf(prn, "%22s", dset->varname[jvar->ylist[1]]); 
+    vname = dset->varname[jvar->ylist[1]];
+    maybe_trim_varname(tmp, vname);
 
+    pprintf(prn, "%*s", firstlen + namelen, tmp); 
     for (j=1; j<Pi->cols; j++) {
-	pprintf(prn, "%13s", beta_vname(jvar, dset, j));
+	vecm_beta_varname(tmp, jvar, dset, j);
+	pprintf(prn, "%*s", namelen + 1, tmp);
     }
 
     pputc(prn, '\n');
 
     for (i=0; i<Pi->rows; i++) {
-	pprintf(prn, "%-10s", dset->varname[jvar->ylist[i+1]]);
+	vname = dset->varname[jvar->ylist[i+1]];
+	maybe_trim_varname(tmp, vname);
+	pprintf(prn, "%-*s", firstlen, tmp);
 	for (j=0; j<Pi->cols; j++) {
 	    x = gretl_matrix_get(Pi, i, j);
 	    if (fabs(x) < 0.5e-14) {
 		x = 0.0;
 	    }
-	    pprintf(prn, "%#12.5g ", x);
+	    pprintf(prn, "%#*.5g ", namelen, x);
 	}
 	pputc(prn, '\n');
     }
