@@ -371,15 +371,15 @@ transcribe_negbin_results (MODEL *pmod, negbin_info *nbinfo,
     }
 
     if (!err) {
-	err = gretl_model_allocate_params(pmod, nc);
+	err = gretl_model_allocate_param_names(pmod, nc);
 	if (!err) {
 	    int v;
 
 	    for (i=0; i<nbinfo->k; i++) {
 		v = pmod->list[i+2];
-		strcpy(pmod->params[i], dset->varname[v]);
+		gretl_model_set_param_name(pmod, i, dset->varname[v]);
 	    }
-	    strcpy(pmod->params[nc-1], "alpha");
+	    gretl_model_set_param_name(pmod, nc-1, "alpha");
 	}
     }
 
@@ -733,7 +733,7 @@ static int do_poisson (MODEL *pmod, offset_info *oinfo,
 	goto bailout;
     }
 
-    if (dataset_add_series(2, dset)) {
+    if (dataset_add_series(dset, 2)) {
 	pmod->errcode = E_ALLOC;
 	goto bailout;
     }
@@ -836,7 +836,7 @@ static int do_poisson (MODEL *pmod, offset_info *oinfo,
 
     clear_model(&tmpmod);
     free(local_list);
-    dataset_drop_last_variables(dset->v - origv, dset);
+    dataset_drop_last_variables(dset, dset->v - origv);
 
     dset->t1 = orig_t1;
     dset->t2 = orig_t2;
