@@ -355,7 +355,7 @@ static int recode_sav_string (char *targ, const char *src,
     *targ = '\0';
 
     if (g_utf8_validate(src, -1, NULL)) {
-	gretl_utf8_strncat(targ, src, maxlen);
+	gretl_utf8_strncat_trim(targ, src, maxlen);
     } else if (encoding >= 500 && encoding <= 9999) {
 	/* try Windows codepage? */
 	char cpage[8];
@@ -366,7 +366,7 @@ static int recode_sav_string (char *targ, const char *src,
 	conv = g_convert(src, -1, "UTF-8", cpage,
 			 NULL, &wrote, NULL);
 	if (conv != NULL) {
-	    gretl_utf8_strncat(targ, conv, maxlen);
+	    gretl_utf8_strncat_trim(targ, conv, maxlen);
 	    g_free(conv);
 	} else {
 	    err = E_DATA;
@@ -1547,7 +1547,7 @@ static int sav_read_observation (spss_data *sdat,
 		dset->Z[j][t] = ix;
 		v->n_ok_obs += 1;
 		if (t == 0) {
-		    set_var_discrete(dset, j, 1);
+		    series_set_discrete(dset, j, 1);
 		}
 	    } else {
 		dset->Z[j][t] = NADBL;
@@ -1579,6 +1579,7 @@ static int read_sav_data (spss_data *sdat, struct sysfile_header *hdr,
 			  DATASET *dset, PRN *prn)
 {
     double *tmp = NULL;
+    char label[MAXLABEL];
     int i, j, t, err = 0;
 
     /* temporary storage for one complete observation */
@@ -1594,10 +1595,11 @@ static int read_sav_data (spss_data *sdat, struct sysfile_header *hdr,
 	if (!CONTD(sdat, i)) {
 	    recode_sav_string(dset->varname[j], sdat->vars[i].name, 
 			      sdat->encoding, VNAMELEN - 1);
-	    recode_sav_string(VARLABEL(dset, j), sdat->vars[i].label,
+	    recode_sav_string(label, sdat->vars[i].label,
 			      sdat->encoding, MAXLABEL - 1);
+	    series_set_label(dset, j, label);
 	    if (has_value_labels(sdat, i)) {
-		set_var_discrete(dset, j, 1);
+		series_set_discrete(dset, j, 1);
 	    }
 	    j++;
 	}
@@ -1672,7 +1674,7 @@ static void free_labelset (spss_labelset *lset)
     if (lset != NULL) {
 	free(lset->varlist);
 	free(lset->vals);
-	free_strings_array(lset->labels, lset->nlabels);
+	strings_array_free(lset->labels, lset->nlabels);
 	free(lset);
     }
 }
@@ -1822,11 +1824,11 @@ static int maybe_add_string_table (spss_data *sdat)
 		    list[j++] = sdat->vars[i].gretl_index;
 		}
 	    }
-	    sdat->st = string_table_new_from_cols_list(list);
+	    sdat->st = gretl_string_table_new(list);
 	    free(list);
 	}
     } else if (sdat->nlabelsets > 0) {
-	sdat->st = gretl_string_table_new(&err);
+	sdat->st = gretl_string_table_new(NULL);
     }
 
     return err;
