@@ -673,7 +673,7 @@ static void copy_variable (DATASET *targ, int targv,
     }
 
     strcpy(targ->varname[targv], src->varname[srcv]);
-    strcpy(VARLABEL(targ, targv), VARLABEL(src, srcv));
+    series_set_label(targ, targv, series_get_label(src, srcv));
 }
 
 static void clear_tramo_files (const char *path, const char *vname)
@@ -716,6 +716,7 @@ static int add_series_from_file (const char *path, int src,
     FILE *fp;
     char line[128], sfname[MAXLEN];
     char varname[VNAMELEN], date[8];
+    char label[MAXLABEL];
     double x;
     int d, yr, per, err = 0;
     int t;
@@ -787,13 +788,13 @@ static int add_series_from_file (const char *path, int src,
 
     /* copy varname and label into place */
     strcpy(dset->varname[targv], varname);
-    sprintf(VARLABEL(dset, targv), _(tx_descrip_formats[src]), dset->varname[0]);
-
+    sprintf(label, _(tx_descrip_formats[src]), dset->varname[0]);
     if (request->prog == TRAMO_SEATS) {
-	strcat(VARLABEL(dset, targv), " (TRAMO/SEATS)");
+	strcat(label, " (TRAMO/SEATS)");
     } else {
-	strcat(VARLABEL(dset, targv), " (X-12-ARIMA)");
-    }	
+	strcat(label, " (X-12-ARIMA)");
+    }
+    series_set_label(dset, targv, label);
 
     for (t=0; t<dset->n; t++) {
 	dset->Z[targv][t] = NADBL;
@@ -1002,7 +1003,7 @@ static int write_tramo_file (const char *fname,
 
     gretl_push_c_numeric_locale();
 
-    x = date(dset->t1, dset->pd, dset->sd0);
+    x = date_as_double(dset->t1, dset->pd, dset->sd0);
     startyr = (int) x;
     sprintf(tmp, "%g", x);
     p = strchr(tmp, '.');
@@ -1058,7 +1059,7 @@ static int write_spc_file (const char *fname, const double *y,
 
     gretl_push_c_numeric_locale();
 
-    x = date(dset->t1, dset->pd, dset->sd0);
+    x = date_as_double(dset->t1, dset->pd, dset->sd0);
     startyr = (int) x;
     sprintf(tmp, "%g", x);
     p = strchr(tmp, '.');
@@ -1174,7 +1175,7 @@ static int save_vars_to_dataset (DATASET *dset,
 	}
     }
 
-    if (addvars > 0 && dataset_add_series(addvars, dset)) {
+    if (addvars > 0 && dataset_add_series(dset, addvars)) {
 	return E_ALLOC;
     }
 
@@ -1443,7 +1444,7 @@ int write_tx_data (char *fname, int varnum,
 	savescript = 1;
     } else {
 	/* create little temporary dataset */
-	tmpset = create_auxiliary_dataset(4, dset->n);
+	tmpset = create_auxiliary_dataset(4, dset->n, 0);
 	if (tmpset == NULL) {
 	    return E_ALLOC;
 	}
@@ -1468,7 +1469,7 @@ int write_tx_data (char *fname, int varnum,
 		       &request.xopt);
     } else { 
 	/* TRAMO, possibly plus SEATS */
-	lower(vname);
+	gretl_lower(vname);
 	gretl_trunc(vname, 8);
 	sprintf(fname, "%s%c%s", workdir, SLASH, vname);
 	/* next line: this also sets request->prog = TRAMO_ONLY if
@@ -1587,7 +1588,7 @@ int write_tx_data (char *fname, int varnum,
 }
 
 int adjust_series (const double *x, double *y, const DATASET *dset, 
-		   int tramo)
+		   int tramo, int use_log)
 {
     int prog = (tramo)? TRAMO_SEATS : X12A;
     int savelist[2] = {1, TX_SA};
@@ -1613,11 +1614,14 @@ int adjust_series (const double *x, double *y, const DATASET *dset,
     } 
 
     if (prog == X12A) { 
-	x12a_opts xopt = { 2, /* auto log transformation */
+	x12a_opts xopt = { 2, /* log transformation flag (2 == no) */
 			   0, /* don't correct for outliers */
 			   0  /* trading days correction */
 	};
 
+	if (use_log) {
+	    xopt.logtrans = 1;
+	}
 	if (dset->pd == 12) {
 	    xopt.trdays = 1;
 	}
